#!/bin/bash

BACKUP_STATUS="/tmp/acsbackup.status"
BACKUP_MSG="/tmp/acsbackup.msg"
BACKUP_DIR="/opt/agileacs_backup"
BACKUP_FILE_NAME="/tmp/acsbackup.file"
zip_password="tplinkacs"
#TIME_STAMP=""

source /opt/agileacs/bin/usb-util.sh
source /opt/agileacs/bin/log-util.sh

# (Start) /tmp/acsbackup.status: "start"
echo "" > $BACKUP_MSG
echo "start" > $BACKUP_STATUS

is_usb_available() {
	DEV_NAME=`mount | grep "$USB_MOUNT_POINT" | awk '{print $1}'`
	#echo "DEV_NAME [$DEV_NAME]"
	if [ "$DEV_NAME" == "" ]; then
		# (Log) Error: Cannot copy backup file to specific USB (no USB).
		sys_log_msg ERROR Backup_Restore "Cannot copy backup file to specific USB (no USB)."
		# (Message) /tmp/acsbackup.msg "Error: Cannot copy backup file to specific USB (no USB);
		echo "Error: Cannot copy backup file to specific USB (no USB)." > $BACKUP_MSG
		# (Status)
		echo "fail" > $BACKUP_STATUS

		killall -9 agileacs-backup-progress.sh
		exit 0
	fi
}

# 
# Check specified USB is ready or not (/media/usb0)
# /opt/agileacs/bin/agileacs-backup.sh <mount_point> <file_name>
#
USB_ID=""
DEV_STATUS="Read Only"
BACKUP_FILE=""
if [ $# -eq 2 ]; then
	# check format
	RET=`echo $1 | grep -E '/media\/usb[0-9]$'`
	if [ "RET" != "" ]; then
		USB_ID=`echo "$RET" | awk -F "/" '{print $3}'`
	fi
	USB_MOUNT_POINT=$1

	# Is specified USB ready

	check_cmd="lsblk -o model,name,fstype,size,label,RO,mountpoint -l | awk '/\/media\/$USB_ID/{print \$(NF-1)}'"
	RO=`eval $check_cmd`
	if [ "$RO" == "" ]; then
		# (Log) Error: The specified USB storage is not ready.
		sys_log_msg ERROR Backup_Restore "The specified USB storage is not ready."
		# (Message) /tmp/acsbackup.msg: "Error: The specified USB storage is not ready" or "Other message"
		echo "Error: The specified USB storage is not ready." > $BACKUP_MSG
		# (Status) /tmp/acsbackup.status: "fail"
		echo "fail" > $BACKUP_STATUS
		killall -9 agileacs-backup-progress.sh
		exit 0
	fi

	if [ "$RO" == "0" ]; then
		DEV_STATUS="Ready"
	fi
	# Backup file file name
	BACKUP_FILE=$2
else
	echo "Usage:"
	echo "	$0 <mount_point> <file_name>"
	echo ""
	exit 0
fi

$USB_CHECK_CMD

if [ "$DEV_STATUS" == "Ready" ]; then
	# (Status) /tmp/acsbackup.status: "prepare backup file"
	echo "prepare backup file" > $BACKUP_STATUS

	if [ -d $BACKUP_DIR ]; then
		echo "$BACKUP_DIR is existed (remove existed directory)"
		rm -rf $BACKUP_DIR
	fi
	echo "Create $BACKUP_DIR"
	mkdir -p $BACKUP_DIR
	cd $BACKUP_DIR
	pwd

	# Decrypt Database to Backup
	echo "DB decrypt"
	/opt/genieacs/bin/decrypt_backup.js > /dev/null 2>&1

	# DB lock
	#echo "DB lock"
	#/usr/bin/mongo < /opt/agileacs/bin/db-lock.js > /dev/null 2>&1

	# Backup DB & generate md5
	echo "mongodump --db genieacs"
	/usr/bin/mongodump -db genieacs --collection fs.chunks
	/usr/bin/mongodump -db genieacs --collection fs.files
	/usr/bin/mongodump -db genieacs --collection presets
	/usr/bin/mongodump -db genieacs --collection profiles

	echo "mongodump --db tplink"
	/usr/bin/mongodump -db tplink --collection syssetting
	/usr/bin/mongodump -db tplink --collection cpesetting
	/usr/bin/mongodump -db tplink --collection troubleshooting
	/usr/bin/mongodump -db tplink --collection notification
	/usr/bin/mongodump -db tplink --collection report_setting
	/usr/bin/mongodump -db tplink --collection devices_ext

	# DB unlock
	#echo "DB unlock"
	#/usr/bin/mongo < /opt/agileacs/bin/db-unlock.js > /dev/null 2>&1

	# Decrypt Database to Backup
	echo "DB encrypt"
	/opt/genieacs/bin/encrypt_process.js > /dev/null 2>&1

	# generate md5 (for DB backup)
	tar -zcf dbdump.tgz -C $BACKUP_DIR dump
	md5sum dbdump.tgz
	md5sum dbdump.tgz | awk '{print $1}' > $BACKUP_DIR/dbdump.md5

	# Backup SQLITE & generate md5
	tar -zcf sqldb.tgz /opt/genieacs-gui/db/development.sqlite3
	md5sum sqldb.tgz | awk '{print $1}' > $BACKUP_DIR/sqldb.md5
	
	# Backup Files & generate md5
	tar -zcf bk_file.tgz /opt/agileacs/pppoesetting
	md5sum bk_file.tgz | awk '{print $1}' > $BACKUP_DIR/bk_file.md5

	# Copy System Version information file
	cp /opt/genieacs/genieacs_version .

	# Pack and Encrypt backup data and generate md5
	zip -P $zip_password -r $BACKUP_FILE sqldb.tgz sqldb.md5 bk_file.md5 bk_file.tgz dbdump.md5 dbdump.tgz genieacs_version
	md5sum $BACKUP_FILE | awk '{print $1}' > $BACKUP_DIR/$BACKUP_FILE.md5

	# Is backup file size greater than 4Gbi
	BACKUP_SIZE=`ls -al $BACKUP_FILE | awk '{print $5}'`
	echo "BACKUP_SIZE [$BACKUP_SIZE]"

	if [ $BACKUP_SIZE -gt 4000000000 ]; then
		# Remove backup file
		cd -
		rm -rf $BACKUP_DIR

		# (Log)
		sys_log_msg ERROR Backup_Restore "The backup file $BACKUP_FILE size (${BACKUP_SIZE:0:-6} MB) is greater than 4GiB."
		# (Message)
		echo "Error: The backup file $BACKUP_FILE size (${BACKUP_SIZE:0:-6} MB) is greater than 4GiB." > $BACKUP_MSG
		# (Status)
		echo "fail" > $BACKUP_STATUS
		killall -9 agileacs-backup-progress.sh
		exit 0
	fi

	# Change BACKUP_SIZE to MB
	BACKUP_SIZE=${BACKUP_SIZE:0:-3}
	BACKUP_SIZE=$((BACKUP_SIZE + 1))

	echo "USB_MOUNT_POINT [$USB_MOUNT_POINT]"

	is_usb_available
	echo "DEV_NAME [$DEV_NAME]"

	# Is USB has free space for backup data?
	DEV_AVAILABLE_SIZE=`df | grep "$DEV_NAME"| awk '{print $4}'`
	echo "DEV_AVAILABLE_SIZE [$DEV_AVAILABLE_SIZE]"

	# Is USB has free space for backup file?
	if [ $BACKUP_SIZE -gt $DEV_AVAILABLE_SIZE ]; then
		# Remove backup file
		cd -
		rm -rf $BACKUP_DIR

		STR_LEN=${#BACKUP_SIZE}
		# (Log), (Message) and (Status)
		if [ $STR_LEN -gt 3 ]; then
			echo "Cannot store backup file $BACKUP_FILE (size: ${BACKUP_SIZE:0:-3} MB) on USB flash drive (No space left on device)"
			echo "Error: Cannot store backup file $BACKUP_FILE (size: ${BACKUP_SIZE:0:-3} MB) on USB flash drive (No space left on device)" > $BACKUP_MSG
			sys_log_msg ERROR Backup_Restore "Cannot store backup file $BACKUP_FILE (size: ${BACKUP_SIZE:0:-3} MB) on USB flash drive (No space left on device)"
		else
			echo "Cannot store backup file $BACKUP_FILE (size: ${BACKUP_SIZE} KB) on USB flash drive (No space left on device)"
			echo "Error: Cannot store backup file $BACKUP_FILE (size: ${BACKUP_SIZE} KB) on USB flash drive (No space left on device)" > $BACKUP_MSG
			sys_log_msg ERROR Backup_Restore "Cannot store backup file $BACKUP_FILE (size: ${BACKUP_SIZE} KB) on USB flash drive (No space left on device)"
		fi
		echo "fail" > $BACKUP_STATUS
		killall -9 agileacs-backup-progress.sh
		exit 0
	fi

	is_usb_available

	# Move encrypt data to USB
	echo "copy file to USB" > $BACKUP_STATUS
	echo "$BACKUP_FILE" > $BACKUP_FILE_NAME
	cp $BACKUP_FILE $USB_MOUNT_POINT
	chmod 644 $USB_MOUNT_POINT/$BACKUP_FILE

	# Is the MD5 of backup file and original file matched?
	md5sum $USB_MOUNT_POINT/$BACKUP_FILE | awk '{print $1}' > $USB_MOUNT_POINT/$BACKUP_FILE.md5
	RET=`diff $BACKUP_DIR/$BACKUP_FILE.md5 $USB_MOUNT_POINT/$BACKUP_FILE.md5`
	if [ "$RET" != "" ]; then
		# md5 not match
		echo "md5 not match"

		# Remove backup file
		cd -
		rm -rf $BACKUP_DIR
		rm -f $USB_MOUNT_POINT/$BACKUP_FILE
		rm -f $USB_MOUNT_POINT/$BACKUP_FILE.md5

		# (Log) Error: Cannot copy backup file to specific USB (copy operation failed).
		sys_log_msg ERROR Backup_Restore "Cannot copy backup file to specific USB (copy operation failed)."
		# (Message) /tmp/acsbackup.msg "Error: Cannot copy backup file to specific USB (copy operation failed);
		echo "Error: Cannot copy backup file to specific USB (copy operation failed)." > $BACKUP_MSG
		# (Status) /tmp/acsbackup.status: "fail"
		echo "fail" > $BACKUP_STATUS

		killall -9 agileacs-backup-progress.sh
		exit 0
	fi

	echo "Remove temp file"
	rm $USB_MOUNT_POINT/$BACKUP_FILE.md5
	cd -
	rm -rf $BACKUP_DIR 

	# Backup is complated
	echo "Backup is completed" > $BACKUP_MSG
	sys_log_msg INFO Backup_Restore "Backup is completed"
	echo "completed" > $BACKUP_STATUS
	rm -f $BACKUP_FILE_NAME
else
	echo "Error: USB storage fail (Read-only file system)" > $BACKUP_MSG
	# Log "No USB" or other message
	sys_log_msg ERROR Backup_Restore "USB storage fail (Read-only file system)"
	echo "fail" > $BACKUP_STATUS
fi

killall -9 agileacs-backup-progress.sh
exit 0

