#!/bin/bash

show_usage() {
	echo "agileacs-restore.sh <Mount Point> <Backup file> - Restore to specified backup file"
	echo "agileacs-restore.sh reset - Factory Default Restore"
}

remove_tmp_data() {
	echo "rm -rf $RESTORE_DIR"
	cd -
	rm -rf $RESTORE_DIR
}

# The restore file must include all files
# return 1: include all specific files
# return 0: doesnot include one of specific files
is_restore_file() {
	ret=1
	arr=("$@")
	for file in "${arr[@]}"; do
		if [ ! -f $file ]; then
			ret=0
			break;
		fi
	done
	echo "$ret"
}

RESTORE_STATUS="/tmp/acsrestore.status"
RESTORE_MSG="/tmp/acsrestore.msg"
RESTORE_DIR="/opt/agileacs_restore"
zip_password="tplinkacs"
# RESTORE_FILE (Default value is used for factory default)
RESTORE_FILE="/opt/agileacs/data/AgileACS_reset.bin"
RESTORE_FILE_NAME="AgileACS_reset.bin"

DB_LOCK="/usr/bin/mongo < /opt/agileacs/bin/db-lock.cmd"
DB_UNLOCK="/usr/bin/mongo < /opt/agileacs/bin/db-unlock.cmd"
DB_CLEAN="/usr/bin/mongo < /opt/agileacs/bin/db-clean.cmd"

source /opt/agileacs/bin/usb-util.sh
source /opt/agileacs/bin/log-util.sh


# Check specific file or operation
if [ $# -lt 1 ] || [ $# -gt 2 ]; then
	show_usage
	exit 0
fi

# (Start) /tmp/acsrestore.status
echo "start"
echo "" > $RESTORE_MSG
echo "start" > $RESTORE_STATUS

#$USB_CHECK_CMD
#
# Check specified USB is ready or not (/media/usb0)
# /opt/agileacs/bin/agileacs-restore.sh <Mount Point> <Backup file> - Restore to specified backup file
# /opt/agileacs/bin/agileacs-restore.sh reset - Factory Default Restore
#
USB_ID=""
DEV_STATUS="Read Only"
if [ $# -eq 2 ]; then
	# check format
	RET=`echo $1 | grep -E '/media\/usb[0-9]$'`
	if [ "RET" != "" ]; then
		USB_ID=`echo "$RET" | awk -F "/" '{print $3}'`
	fi
	USB_MOUNT_POINT=$1

	# Is specified USB ready

	check_cmd="lsblk -o model,name,fstype,size,label,RO,mountpoint -l | awk '/\/media\/$USB_ID/{print \$(NF-1)}'"
	RO=`eval $check_cmd`
	if [ "$RO" == "" ]; then
		# (Log) Error: The specified USB storage is not ready.
		sys_log_msg ERROR Backup_Restore "The specified USB storage is not ready."
		# (Message) /tmp/acsbackup.msg: "Error: The specified USB storage is not ready" or "Other message"
		echo "Error: The specified USB storage is not ready." > $RESTORE_MSG
		# (Status) /tmp/acsbackup.status: "fail"
		echo "fail" > $RESTORE_STATUS
		killall -9 agileacs-restore-progress.sh
		exit 0
	fi

	if [ "$RO" == "0" ]; then
		DEV_STATUS="Ready"
	fi
	# Backup file file name
	RESTORE_FILE="$1/$2"
	RESTORE_FILE_NAME=$2
else
	if [ "$1" != "reset" ]; then
		show_usage
		exit 0
	fi
fi


# Is USB ready?
#DEV_STATUS=`cat $USB_STATUS`
if [ "$1" == "reset" ] || [ "$DEV_STATUS" == "Ready" ]; then
	#if [ "$1" != "reset" ]; then
	#	RESTORE_FILE="/tmp/agileacs_usb.mount_point/$1"
	#	RESTORE_FILE_NAME=$1
	#fi
	echo "USB_ID [$USB_ID], DEV_STATUS [$DEV_STATUS], USB_MOUNT_POINT [$USB_MOUNT_POINT], RESTORE_FILE [$RESTORE_FILE], RESTORE_FILE_NAME [$RESTORE_FILE_NAME]"

	if [ ! -f $RESTORE_FILE ]; then
		msg="No specific backup file $RESTORE_FILE"
		echo "$msg"
		# (Log): "Error No specific backup file [$RESTORE_FILE]"
		sys_log_msg ERROR Backup_Restore "$msg"
		# (Message): /tmp/acsrestorage.msg: "Error No specific backup file [$RESTORE_FILE]"
		echo "$msg" > $RESTORE_MSG
		# (Status) /tmp/acsrestore.status: "fail"
		echo "fail" > $RESTORE_STATUS
		exit 0
	fi

	# Check file and restore space
	RESTORE_FILE_SIZE=`ls -al $RESTORE_FILE | awk '{print $5}'`
	echo "RESTORE_FILE_SIZE [$RESTORE_FILE_SIZE]"
	# Convert into KB
	FILE_SIZE_LEN=1
	if [ ${#RESTORE_FILE_SIZE} -gt 3 ]; then
		RESTORE_FILE_SIZE=${RESTORE_FILE_SIZE:0:-3}
		FILE_SIZE_LEN=${#RESTORE_FILE_SIZE}
	else
		# if file size less than 1KB
		RESTORE_FILE_SIZE=1
	fi

	FREE_SIZE=`df | awk '/\/var$/{print $4}'`
	REQUIRE_SIZE=$(($RESTORE_FILE_SIZE * 3))
	if [ $FREE_SIZE -lt $REQUIRE_SIZE ]; then
		msg=""
		if [ $FILE_SIZE_LEN -gt 0 ] && [ $FILE_SIZE_LEN -le 3 ]; then
			msg="Cannot restore backup file $RESTORE_FILE_NAME (size: $RESTORE_FILE_SIZE KB) on system (No space left on device)."
		elif [ $FILE_SIZE_LEN -gt 4 ] && [ $FILE_SIZE_LEN -le 9 ]; then
			msg="Cannot restore backup file $RESTORE_FILE_NAME (size: ${RESTORE_FILE_SIZE:0:-3} MB) on system (No space left on device)."
		else
			msg="Cannot restore backup file $RESTORE_FILE_NAME (size: $RESTORE_FILE_SIZE KB) on system (No space left on device)."
		fi
		echo "$msg"
		sys_log_msg ERROR Backup_Restore "$msg"
		echo "Error: $msg" > $RESTORE_MSG
		echo "fail" > $RESTORE_STATUS
		exit 0
	fi

	echo "backup file copying"
	echo "backup file copying" > $RESTORE_STATUS

	#echo "format checking" > $RESTORE_STATUS

	# Copy backup file to $RESTORE_DIR
	if [ -d $RESTORE_DIR ]; then
		echo "$RESTORE_DIR is existed (remove existed directory)"
		rm -rf $RESTORE_DIR
	fi
	echo "Create $RESTORE_DIR"
	mkdir -p $RESTORE_DIR
	echo "$$RESTORE_FILE_NAME" > /tmp/acsrestore.file
	RET=`cp $RESTORE_FILE $RESTORE_DIR 2>&1`
	# Check returnn information of cp command
	if [ "$RET" != "" ]; then
		ERR_CHECK=`echo "$RET" | grep "No space left on device"`
		if [ "ERR_CHECK" != "" ]; then
			echo "No space left on device"
		fi
		# (Log) Error: Cannot restore backup file ${filename} (size: XXXMB) on system (No space left on device).
		# (Message) /tmp/acsrestore.msg: "Error: Cannot restore backup file ${filename} (size: XXXMB) on system (No space left on device)."
		# (Status) /tmp/acsrestore.status: "fail"
		ERR_CHECK=`echo "$RET" | grep "error reading"`
		if [ "ERR_CHECK" != "" ]; then
			echo "Cannot read data from USB"
		fi
		remove_tmp_data
		exit 0
	fi
	cd $RESTORE_DIR
	pwd

	# Decrypt specific file
	unzip -P $zip_password $RESTORE_FILE_NAME > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		echo "Restore fail (backup file is not correct)"
		echo "Error: Restore fail (backup file is not correct)" > $RESTORE_MSG
		sys_log_msg ERROR Backup_Restore "Restore fail (backup file is not correct)"
		echo "fail" > $RESTORE_STATUS
		remove_tmp_data
		exit 0
	fi

	# Check required files
	fileArray=("$RESTORE_DIR/genieacs_version" "$RESTORE_DIR/dbdump.tgz" "$RESTORE_DIR/dbdump.md5" "$RESTORE_DIR/sqldb.tgz" "$RESTORE_DIR/sqldb.md5" "$RESTORE_DIR/bk_file.tgz" "$RESTORE_DIR/bk_file.md5")
	if [ "$1" == "reset" ]; then
		fileArray=("$RESTORE_DIR/dbdump.tgz" "$RESTORE_DIR/dbdump.md5" "$RESTORE_DIR/bk_file.tgz" "$RESTORE_DIR/bk_file.md5")
	fi
	if [ "$(is_restore_file "${fileArray[@]}")" == "0" ]; then
		echo "Restore fail (backup file is not correct)"
		echo "Error: Restore fail (backup file is not correct)" > $RESTORE_MSG
		sys_log_msg ERROR Backup_Restore "Restore fail (backup file is not correct)"
		echo "fail" > $RESTORE_STATUS
		remove_tmp_data
		exit 0
	fi

	# Check system version
	if [ "$1" != "reset" ]; then
		RET=`diff $RESTORE_DIR/genieacs_version /opt/genieacs/genieacs_version`
		if [ "$RET" != "" ]; then
			SYS_VERSION=`cat $RESTORE_DIR/genieacs_version`
			msg="The backup file cannot be used on this server. The Server version is not matched! The backup file is used for \"$SYS_VERSION\"."
			# (Log) Error: The backup file cannot be used on this server. The Server version is not matched! The backup file is used for "".
			sys_log_msg ERROR Backup_Restore "$msg"
			echo "Error: $msg" > $RESTORE_MSG
			echo "fail" > $RESTORE_STATUS
			remove_tmp_data
			exit 0
		fi
	fi

	# Check DB file
	MD5=`md5sum $RESTORE_DIR/dbdump.tgz | awk '{print $1}'`
	DB_MD5=`cat $RESTORE_DIR/dbdump.md5`
	if [ "$MD5" != "$DB_MD5" ]; then
		echo "Restore fail (backup file is not correct)"
		echo "Error: Restore fail (backup file is not correct)" > $RESTORE_MSG
		sys_log_msg ERROR Backup_Restore "Restore fail (backup file is not correct)"
		echo "fail" > $RESTORE_STATUS
		remove_tmp_data
		exit 0
	fi
        tar -zxf dbdump.tgz

	# Check backup file
	MD5=`md5sum $RESTORE_DIR/bk_file.tgz | awk '{print $1}'`
	BF_MD5=`cat $RESTORE_DIR/bk_file.md5`
	if [ "$MD5" != "$BF_MD5" ]; then
		echo "Restore fail (backup file is not correct)"
		echo "Error: Restore fail (backup file is not correct)" > $RESTORE_MSG
		sys_log_msg ERROR Backup_Restore "Restore fail (backup file is not correct)"
		echo "fail" > $RESTORE_STATUS
		remove_tmp_data
		exit 0
	fi

	# Check sqldb file
	MD5=`md5sum $RESTORE_DIR/sqldb.tgz | awk '{print $1}'`
	SQL_MD5=`cat $RESTORE_DIR/sqldb.md5`
	if [ "$MD5" != "$SQL_MD5" ]; then
		echo "Restore fail (backup file is not correct)"
		echo "Error: Restore fail (backup file is not correct)" > $RESTORE_MSG
		sys_log_msg ERROR Backup_Restore "Restore fail (backup file is not correct)"
		echo "fail" > $RESTORE_STATUS
		remove_tmp_data
		exit 0
	fi

	# (Status) /tmp/acsrestore.status: "data restoring"
	echo "data restorng" > $RESTORE_STATUS
	# Restore DB
	/usr/bin/mongo < /opt/agileacs/bin/db-clean.cmd
	/usr/bin/mongo 127.0.0.1:27018 < /opt/agileacs/bin/db-clean.cmd
	#/usr/bin/mongo < /opt/agileacs/bin/db-lock.js

	/usr/bin/mongorestore --db genieacs dump/genieacs
	/usr/bin/mongorestore --drop --db tplink --collection syssetting dump/tplink/syssetting.bson
	/usr/bin/mongorestore --drop --db tplink --collection cpesetting dump/tplink/cpesetting.bson
	/usr/bin/mongorestore --drop --db tplink --collection troubleshooting dump/tplink/troubleshooting.bson
	/usr/bin/mongorestore --drop --db tplink --collection notification dump/tplink/notification.bson
	/usr/bin/mongorestore --drop --db tplink --collection report_setting dump/tplink/report_setting.bson
	[ -f dump/tplink/devices_ext.bson ] && /usr/bin/mongorestore --drop --db tplink --collection devices_ext dump/tplink/devices_ext.bson

	#/usr/bin/mongo < /opt/agileacs/bin/db-unlock.js

	# Restoer sqldb file
	tar -zxf sqldb.tgz -C /tmp
	admin_pwd=$(sqlite3 /opt/genieacs-gui/db/development.sqlite3 "SELECT password FROM users WHERE id=1")
	sqlite3 /tmp/opt/genieacs-gui/db/development.sqlite3 "UPDATE users SET password='$admin_pwd' WHERE id=1"
	cp /tmp/opt/genieacs-gui/db/development.sqlite3 /opt/genieacs-gui/db/development.sqlite3

	# Restoer backup file
	rm /opt/agileacs/pppoesetting/*
        tar -zxf bk_file.tgz -C /

	# Remove monthly report rrd files
	rm /opt/agileacs/data/rrd/*

	# Apply system setting
	#[ -e /var/run/acsapps2.pid ] && kill -USR2 `cat /var/run/acsapps2.pid` >/dev/null 2>&1

	echo "Remove temp file"
	remove_tmp_data
	#cd -
	#rm -rf $RESTORE_DIR

	# Restore is complated
	echo "Restore is completed" > $RESTORE_MSG
	sys_log_msg INFO Backup_Restore "Restore is completed"
	echo "completed" > $RESTORE_STATUS
	rm -f /tmp/acsrestore.file

	sleep 10

	# (Status) /tmp/acsrestore.status: "apply setting"
	#echo "apply setting" > $RESTORE_STATUS

	# Apply license verifying
	/opt/agileacs/bin/acsutils2 license check serverinfo >> /dev/null 2>&1
	/opt/agileacs/bin/acsutils2 license restore >> /dev/null 2>&1

	# Apply system setting
	[ -e /var/run/acsapps2.pid ] && kill -USR2 `cat /var/run/acsapps2.pid` >/dev/null 2>&1
else
	echo $DEV_STATUS
	# (Log) Error: The specified USB storage is not ready
	sys_log_msg ERROR Backup_Restore "The specified USB storage is not ready."
	# (Message) /tmp/acsrestorage.msg: "Error: The specified USB storage is not ready"
	echo "Error: The specified USB storage is not ready." > $RESTORE_MSG
	# (Status) /tmp/acsrestore.status: "fail"
	echo "fail" > $RESTORE_STATUS
fi

killall -9 agileacs-restore-progress.sh
exit 0

