#!/bin/sh
UPGRADE_STATUS_FILE="/opt/agileacs/tmp/upgrade_status"
VERSION_FILE="/opt/genieacs/genieacs_version"
CHECK_VERSION_FILE="/opt/tmp/check_genieacs_version"
FEATURE_SET_FILE="/opt/agileacs/tmp/xj4j32l4ejvu045j6vup"
ACTIVATION_STATUS_FILE="/opt/agileacs/tmp/yk5k4325fkrv156k7rvq"
DEBUG_FILE="/var/log/genieacs/acsapps.log"
ENABLE_DEBUG="1"
PREFIX="check_upgrade_status"
RETRY=5
WAIT_INT=3
log_message() {
    [ "$ENABLE_DEBUG" = "1" ] &&
    {
        d=$(date +'%Y-%m-%dT%T.%3NZ')
        text="${d} [${PREFIX}]: $1"
        echo "${text}" >> $DEBUG_FILE
    }
}

get_service_status()
{
    servicename=$1
    sudo service $servicename status | grep active > /dev/null && echo "Pass" || echo "Fail"
    return
}

get_systemctl_status()
{
    servicename=$1
    sudo systemctl is-active $servicename | grep active > /dev/null && echo "Pass" || echo "Fail"
    return
}

is_genieacs_started()
{
    cnt=0
    while [ $cnt -lt $RETRY ]
    do
        sleep $WAIT_INT
        status=$(get_service_status genieacs)
		#log_message "status = $status (retry $cnt)"
        case "$status" in
            "Pass")
				#log_message "genieacs started"
                echo "$status (retry $cnt)"
                return
            ;;
        esac
        cnt=$(($cnt+1))
    done
	#log_message "genieacs start failed" 
    echo "$status (retry $cnt)"
    return
}

echo "After Upgrade Checking Service"
sleep 10
log_message " ======== This is package_version check_upgrade_status.sh Start ========"
ret=$(cat $UPGRADE_STATUS_FILE)
log_message "1. check upgrade status  ........................ $ret"

[ -e $VERSION_FILE ] || log_message "2. check current version  ........................ $VERSION_FILE not exist"
ret=$(cat $VERSION_FILE)
ret1=$(cat $CHECK_VERSION_FILE)
if [ "$ret" = "$ret1" ]; then
    log_message "2. check current version  ........................ $ret"
else
    log_message "2. check current version  ........................ $ret not match $ret1"
fi

ret=$(is_genieacs_started)
log_message "3. check genieacs started  ........................ $ret"

ret=$(get_service_status "nginx")
log_message "4. check nginx started  ........................ $ret"

ret=$(get_systemctl_status "mongodb.service")
log_message "5. check mongodb started  ........................ $ret"

ret=$(get_systemctl_status "mongodb_tmp.service")
log_message "6. check mongodb_tmp started  ........................ $ret"

ret=$(get_systemctl_status "agileacsdb.service")
log_message "7. check agileacsdb started  ........................ $ret"

ret=$(get_systemctl_status "agileacs_monitor.service")
log_message "8. check agileacs_monitor started  ........................ $ret"

ret=$(cat $FEATURE_SET_FILE)
log_message "9. check current feature set ........................ $ret"

[ "$(cat $ACTIVATION_STATUS_FILE)" = "1" ] && ret="complete" || ret="not complete"
log_message "10. check actviation status ........................ $ret"

log_message "======== This is package_version check_upgrade_status.sh Done ========"
echo "This is package_version check_upgrade_status.sh Done"
exit 0
