#!/bin/sh
#  Copyright(c) 2019-2023 TP-LINK Technologies Co.Ltd.
#
# file      genieacs_config_check.sh
# brief     shell script to check and unzip configration file
# details   /bin/sh /opt/genieacs/bin/genieacs_config_check.sh [confile file] [unzip dest]
# author    Sharon Lin
# version   1.0.0
# date      2Aug19
#
# history   \arg 1.0.0, 31July19, Sharon Lin, Create the file.


DEBUG_FILE="/var/log/genieacs/acsapps.log"
ZIP_PASSWORD="tplinkacs"
TMP_CONFIG_FOLDER="/tmp/tmpconfig"
CONFIG_FILE=$1
TMP_CONFIG_FILEINFO=$2

debug()
{
 date=`date +'%D %T'`
 echo "[$date]$@" >> ${DEBUG_FILE}
}

[ -z "${CONFIG_FILE}" ] && {
	debug "Error: No Config File assigned"
	exit 0
}
[ -e "${CONFIG_FILE}" ] || {
	debug "Error: Config File (${CONFIG_FILE}) not exist"
	exit 0
}
[ -z "${TMP_CONFIG_FILEINFO}" ] && {
	debug "Error: No Config Info File assigned"
	exit 0
}

[ -e ${TMP_CONFIG_FILEINFO} ] && rm -rf ${TMP_CONFIG_FILEINFO}
[ -e ${TMP_CONFIG_FOLDER} ] && rm -rf ${TMP_CONFIG_FOLDER}
mkdir ${TMP_CONFIG_FOLDER}
unzip -P ${ZIP_PASSWORD} ${CONFIG_FILE} -d ${TMP_CONFIG_FOLDER} > /dev/null 2>&1
[ -e ${TMP_CONFIG_FOLDER} ] || {
	debug "Error: unzip ${CONFIG_FILE} failed"
	exit 0
}
md5file="$(find ${TMP_CONFIG_FOLDER} -name 'md5sum')"
configfile="$(find ${TMP_CONFIG_FOLDER} -name '*.cfg')"
[ -z ${configfile} ] && {
	debug "Error: No Config DEB File Exist in ${CONFIG_FILE}"
	exit 0
}
md5file_sum="$(cat ${md5file} |awk '{print $1}')"
md5info_sum="$(md5sum ${configfile} |awk '{print $1}')"
if [ ${md5file_sum} = ${md5info_sum} ]; then
	cp ${configfile} ${TMP_CONFIG_FILEINFO}
	debug "MD5 Check Success"
    rm -rf ${TMP_CONFIG_FOLDER}
	echo "success"
else
	debug "MD5 Check Not Match"
	echo "Error: MD5 Check Not Match"
fi
exit 1


