#!/bin/sh

# /etc/fstab: static file system information.
#
# Use 'blkid' to print the universally unique identifier for a
# device; this may be used with UUID= as a more robust way to name devices
# that works even if disks are added and removed. See fstab(5).
#
# <file system> <mount point>   <type>  <options>       <dump>  <pass>
# / was on /dev/sda1 during installation
#UUID=badcfead-0811-4622-92c6-364ffc418c37 /               ext4    errors=remount-ro 0       1
#UUID=250bd23a-65c5-4c63-93e7-6e418520d761 none            swap    sw              0       0

BLK_INFO_FILE="/tmp/current_blk_info"
DF_INFO_FILE="/tmp/current_df_info"
ALL_BLK_INFO_FILE="/tmp/current_allblk_info"

get_hd_check_number()
{
	fstabUUID=`cat /etc/fstab | grep ext4 |grep errors=remount-ro |awk '{print $1}'|awk -F '=' '{print $2}'`
	#/dev/sda1: UUID="badcfead-0811-4622-92c6-364ffc418c37" TYPE="ext4" PARTUUID="998344a3-01"
	dev_number=`sudo /sbin/blkid |grep $fstabUUID |grep ext4 |wc -l`
	echo $dev_number
}

get_hd_info()
{
	log_str=""
	fstabUUID=`cat /etc/fstab | grep ext4 |grep errors=remount-ro |awk '{print $1}'|awk -F '=' '{print $2}'`
	#/dev/sda1: UUID="badcfead-0811-4622-92c6-364ffc418c37" TYPE="ext4" PARTUUID="998344a3-01"
	dev_number=`sudo /sbin/blkid |grep $fstabUUID |grep ext4 |wc -l`
	devs=`sudo /sbin/blkid |grep $fstabUUID |grep ext4 > $BLK_INFO_FILE`
	while read -r line; do
		dev_name=`echo $line | awk -F ':' '{printf $1}'`
		mount_path=`/bin/df |grep $dev_name | awk '{print $6}'`
		log_str="$log_str [dev:$dev_name mount_path:$mount_path]"
	done < "$BLK_INFO_FILE"
	echo "Current $dev_number UUID as $fstabUUID dev: $log_str"
}

get_hd_df_info()
{
	log_str=""
	log_str2=""
	devs=`/bin/df > $DF_INFO_FILE`
	while read -r line; do
		dev_name=`echo $line |awk '{printf $1}' | grep '/dev/'`
		if [ -n "$dev_name" ];then
				mount_path=`echo $line | awk '{print $6}'`
				hd_modelid=`sudo /sbin/udevadm info --query=all --name=$dev_name | grep ID_SERIAL= | awk -F '=' '{print $2}'`
				hd_serial=`sudo /sbin/udevadm info --query=all --name=$dev_name | grep ID_SERIAL_SHORT= | awk -F '=' '{print $2}'`
				log_str="$log_str [dev:$dev_name mount_path:$mount_path modelid:$hd_modelid serial:$hd_serial]"
		fi
	done < "$DF_INFO_FILE"
	echo "Current All DF HD: $log_str"
}

get_hd_blk_info()
{
	log_str=""
	hd_modelid=""
	hd_serial=""
	devs=`sudo /sbin/blkid > $ALL_BLK_INFO_FILE`
	while read -r line; do
		dev_name=`echo $line |awk -F ':' '{printf $1}' | grep '/dev/' `
		if [ -n "$dev_name" ];then
				tmp_hd_modelid=`sudo /sbin/udevadm info --query=all --name=$dev_name | grep ID_SERIAL= | awk -F '=' '{print $2}'`
				tmp_hd_serial=`sudo /sbin/udevadm info --query=all --name=$dev_name | grep ID_SERIAL_SHORT= | awk -F '=' '{print $2}'`
				if [ "$tmp_hd_modelid" != "$hd_modelid" -a "$tmp_hd_serial" != "$hd_serial" ];then
					mount_path=`/bin/df | grep $dev_name | awk '{print $6}'`
					hd_modelid="$tmp_hd_modelid"
					hd_serial="$tmp_hd_serial"
					[ -z "$mount_path" ] && mount_path="not in df"
					log_str="$log_str [dev:$dev_name mount_path:$mount_path modelid:$hd_modelid serial:$hd_serial]"
				fi
		fi
	done < "$ALL_BLK_INFO_FILE"
	echo "Current All BLKID: $log_str"
}


get_hd_devname()
{
	fstabUUID=`cat /etc/fstab | grep ext4 |grep errors=remount-ro |awk '{print $1}'|awk -F '=' '{print $2}'`
	#/dev/sda1: UUID="badcfead-0811-4622-92c6-364ffc418c37" TYPE="ext4" PARTUUID="998344a3-01"
	devs=`sudo /sbin/blkid |grep $fstabUUID |grep ext4 > $BLK_INFO_FILE`
	while read -r line; do
		dev_name=`echo $line | awk -F ':' '{printf $1}'`
		mount_path=`/bin/df |grep $dev_name | awk '{print $6}'`
		if [ -n "$mount_path" ];then
			if [ $mount_path = '/' ];then
				echo $dev_name
				break
			fi
		fi
	done < "$BLK_INFO_FILE"
}

action=$1
[ -z "$action" ] && echo "./genieacs_get_hd_info.sh [devname|number|devinfo|dfinfo|blkinfo]" && exit
if [ "$action" = "devname" ]; then
	get_hd_devname
elif [ "$action" = "number" ]; then
    get_hd_check_number
elif [ "$action" = "blkinfo" ]; then
    get_hd_blk_info
elif [ "$action" = "dfinfo" ]; then
    get_hd_df_info
elif [ "$action" = "devinfo" ]; then
    get_hd_info
else
	echo "Unknow action [$action]"
fi

exit 0
