#!/bin/bash

USB_STATUS="/tmp/agileacs_usb.status"
USB_CHECK_CMD="/opt/agileacs/bin/usb-storage.sh check"

USB_MOUNT_LOG="/tmp/usbmount.log"
MAXUSB=3
MAXUSB=$(($MAXUSB - 1))
MINSIZE=14

source /etc/usbmount/usbmount.conf

# Test if the first parameter is in the list given by the second
# parameter.
in_list()
{
	for v in $2; do
		[ "$1" != "$v" ] || return 0
	done
	return 1
}


usb_check() {
	# Clean related data/status
	OUTPUT_STR=""
	#echo -n "" > $USB_MOUNT_POINT
	if [ "$USB_MOUNT_POINT" != "" ]; then
		if [ -f $USB_MOUNT_POINT ]; then
			rm $USB_MOUNT_POINT
		fi
	fi

	# Get USB amount
	USB_AMOUNT=`cat /proc/mounts | grep "media/usb[0-9]" | awk '{ print $2 }' | uniq | awk '{ ++count } END { print count }'`
	[ "$USB_AMOUNT" = "" ] && USB_AMOUNT=0 # in case USB_AMOUNT is empty causes syntax error
	echo "Current USB Ammount: $USB_AMOUNT" >> $USB_MOUNT_LOG

	# Get USB amount and append the information into $USB_STATUS
	if [ $USB_AMOUNT -gt 0 ]; then
		OUTPUT_STR=$OUTPUT_STR"{"$'\n'
		OUTPUT_STR=$OUTPUT_STR"\"Msg\":\"USB Storage List\","$'\n'
		OUTPUT_STR=$OUTPUT_STR"\"MAXUSB\": $MAXUSB,"$'\n'
		OUTPUT_STR=$OUTPUT_STR"\"USB_AMOUNT\":$USB_AMOUNT,"$'\n'

		#
		# Get USB information and append related information into /tmp/agileacs_usb.status
		#
		usb_count=0
		valid_usb_amount=0
		count=0
		#while [ $count -lt $MAXUSB ]; do
		usb_list=`cat /proc/mounts | grep "/media/usb" | awk '{print $2}'`
		usb_list=`echo $usb_list | uniq`
		#while [ $count -lt $USB_AMOUNT ]; do
		for usb_mount in $usb_list; do
			usb_valid=0

			# Check mount point (if fail, remount usb via /opt/agileacs/bin/usbmount-monitor.sh)
			RET=`ls $usb_mount 2>&1 | grep "Transport endpoint is not connected"`
			if [ "$RET" != "" ]; then
				# Get Device
				DEV_NAME=`cat /proc/mounts | grep "$usb_mount" | awk '{print $1}'`
				echo "$DEV_NAME $usb_mount" > /tmp/acs_usbmount/remountusb
				sleep 3
			fi

			# Check format and size
			#fs_type=`cat /proc/mounts | grep "media/usb$usb_count" | awk '{ print $3 }'`
			fs_type=`cat /proc/mounts | grep "$usb_mount" | awk '{ print $3 }'`
			if in_list "$fs_type" "$FILESYSTEMS"; then
				usb_valid=$((usb_valid + 1))
			fi

			#usb_size=`lsblk -l | grep "media/usb$usb_count" | awk '{print $4}'`
			usb_size=`lsblk -l | grep "$usb_mount" | awk '{print $4}'`
			usb_size_unit=${usb_size:${#usb_size}-1:${#usb_size}}
			usb_size=${usb_size:0:-1}
			if [ "$usb_size_unit" == "T" ]; then
				size_T=`echo $usb_size | cut -d'.' -f 1`
				size_M=`echo $usb_size | cut -d'.' -f 2`

				usb_size=$((size_T * 1000 + size_M * 100))
			elif [ "$usb_size_unit" == "M" ]; then
				usb_size=$((usb_size / 1000))
			fi
			usb_size=`echo $usb_size | cut -d'.' -f 1`

			if [ $usb_size -ge $MINSIZE ]; then
				usb_valid=$((usb_valid + 1))
			fi

			if [ $usb_valid -ge 2 ]; then
				# USBStorage
				lsblk -o model,name,fstype,size,label,RO,mountpoint -l > /tmp/lsblk-output.txt
				#RET=`/opt/agileacs/bin/parse-lsblk-output.awk /tmp/lsblk-output.txt | grep "media/usb$usb_count"`
				RET=`/opt/agileacs/bin/parse-lsblk-output.awk /tmp/lsblk-output.txt | grep "$usb_mount"`

				# Data Formate: StorageNumber,USBStorage,Status,MountPoint,DevPath,FSType,Size,RO
				IFS=',' read -r -a array <<< "$RET"

				# Status
				usb_status="Ready"
				# Usage (Unit: 1Mb)
				#USED=`df -m | grep "media/usb$usb_count" | awk '{print $3}'`
				#SIZE=`df -m | grep "media/usb$usb_count" | awk '{print $2}'`
				USED=`df -m | grep "$usb_mount" | awk '{print $3}'`
				SIZE=`df -m | grep "$usb_mount" | awk '{print $2}'`
				# Percentage
				#PERCENTAGE=`df -m | grep "media/usb$usb_count" | awk '{print $5}'`
				PERCENTAGE=`df -m | grep "$usb_mount" | awk '{print $5}'`
				# Mount Point
				#USB_MOUNT="/media/usb$count"

				OUTPUT_STR=$OUTPUT_STR"\"Storage_$usb_count\":{"$'\n'
				OUTPUT_STR=$OUTPUT_STR"	\"USBStorage\":\"${array[1]}\","$'\n'
				OUTPUT_STR=$OUTPUT_STR"	\"Status\":\"${array[2]}\","$'\n'
				OUTPUT_STR=$OUTPUT_STR"	\"Usage\":\"($USED/$SIZE) [MB]\","$'\n'
				OUTPUT_STR=$OUTPUT_STR"	\"Percentage\":\"$PERCENTAGE\","$'\n'
				OUTPUT_STR=$OUTPUT_STR"	\"MountPoint\":\"${array[3]}\","$'\n'
				OUTPUT_STR=$OUTPUT_STR"	\"DevPath\":\"${array[4]}\","$'\n'
				OUTPUT_STR=$OUTPUT_STR"	\"FSType\":\"${array[5]}\","$'\n'
				OUTPUT_STR=$OUTPUT_STR"	\"ReadOnly\":${array[7]}"$'\n'
				OUTPUT_STR=$OUTPUT_STR"},"$'\n'

				usb_count=$((usb_count + 1))
				valid_usb_amount=$((valid_usb_amount + 1))
			fi

			count=$((count + 1))
		done

		OUTPUT_STR=$OUTPUT_STR"\"VALID_USB_AMOUNT\":$valid_usb_amount"$'\n'
		OUTPUT_STR=$OUTPUT_STR"}"$'\n'

		echo "$OUTPUT_STR" > $USB_STATUS

	else
		OUTPUT_STR=$OUTPUT_STR"{"$'\n'
		OUTPUT_STR=$OUTPUT_STR"\"Msg\":\"No USB flash drive\","$'\n'
		OUTPUT_STR=$OUTPUT_STR"\"USB_AMOUNT\":0"$'\n'
		OUTPUT_STR=$OUTPUT_STR"}"$'\n'

		echo "$OUTPUT_STR" > $USB_STATUS
	fi
}

#usb_check
