$(document).ready(function () {
    // Skip if Turbolinks preview
    if (document.documentElement.hasAttribute("data-turbolinks-preview"))
        return;
    if ($("#dialog_modal").length != 0) {
        var modal = document.getElementById("dialog_modal");
        var reload = 0;
        var percent = 0;
        var dataCount = 1;
        var PROGRESS_DURATION = 60000;
        var options = {
            size: 150,
            startAngle: -Math.PI,
            startColor: "#4acbd6",
            endColor: "#4acbd6",
            animateInterval: 30,
            animation: {
                duration: PROGRESS_DURATION,
                easing: "circleProgressEase",
                start: 0,
                end: 100
            }
        };

        $.easing.circleProgressEase = function (x, t, b, c, d) {
            if ((t /= d / 2) < 1)
                return c / 2 * t * t * t + b;
            return c / 2 * ((t -= 2) * t * t + 2) + b;
        };

        var s = options.size,
            r = s / 2,
            t = s / 14;

        var canvas = $("#canvas")[0];

        var ctx = canvas.getContext("2d");
        var lg = ctx.createLinearGradient(0, 0, s, 0);

        var deviceId = $('#dialog_modal_deviceid').text();
        var controllerId = $("#mesh_controllerId").text();
        var intervalTask, intervalAnimate, timeoutCancelDiagTask;
        var diagLockState = 0;
        var perSecondPercent = ((options.animation.end - options.animation.start) / (options.animation.duration / 1000)) / 100;
        var perIntervalPercent = perSecondPercent / (1000 / options.animateInterval);
        var url_diaglock = "";
        if (controllerId != "" && (location.search).includes("mesh")) {
            url_diaglock = location.pathname + "/diaglock?controllerId=" + controllerId;
        } else {
            url_diaglock = location.pathname + "/diaglock";
        }

        var confirm_title, confirm_message;
        
        // Only TR143/Wifi Scan/Mesh will start Progress bar
        initProgressBar = function() {
            // Task Type: Mesh/IP Ping Test/Trace Route Test/Upload Test/Download Test/UDP Echo Test/NS Lookup Test/WifiScan/Cancel/StopUDP
            $.ajax({
                url: url_diaglock,
                cache: false,
                success: function (data) {
                    if (data.locked == "yes") {
                        var type = diagnosticNameTransform(data.locked_name);
                        // storeFlash();
                        switch (type) {
                            case "Mesh":
                                startProgress(type, data.locked_message);
                                break;
                            case "UDP Echo Test":
                                PROGRESS_DURATION = parseInt(data.ttl) * 1000;
                            default:
                                startProgress(type);
                                break;
                        }
                    }
                },
                error: function (request, status, error) { }
            });
        }

        afterRemoveFaultyTask = function (task = null) {
            switch (task) {
                case "Cancel":
                    removeDiagTag(deviceId, task);
                    break;
                case "Mesh":
                    if ($("#mesh_controllerId").length != 0)
                        removeDiagTag($("#mesh_controllerId").text(), task);
                    break;
                case "UDP Echo Test":
                    cancelDiagTask(task);
                    resetProgressValue();
                    startProgress("StopUDP");
                    if (typeof diag_lock != 'undefined' && diag_lock != '') {
                        setTimeout(function () {
                            removeDiagTag(deviceId, task);
                        }, 3000);
                        return;
                    }
                    break;
                default:
                    cancelDiagTask(task);
                    resetProgressValue();
                    startProgress("Cancel");
                    break;
            }
        }

        removeFaultyTask = function (total, result, task = null) {
            if (total > 0) {
                for (let i = 0; i < total; i++) {
                    $.ajax({
                        url: result[i]["_id"] + "/task",
                        method: "DELETE",
                        cache: false,
                        success: function (data) {
                            if (task != null && i == total - 1) {
                                afterRemoveFaultyTask(task);
                            } else {
                                closeProgressBar(task);
                            }
                        },
                        error: function (request, status, error) {
                        }
                    });
                }
            } else {
                if (task != null) {
                    afterRemoveFaultyTask(task);
                } else {
                    closeProgressBar(task);
                }
            }
        }

        removeDiagTag = function (dev_id, task = null) {
            window.clearTimeout(timeoutCancelDiagTask);
            $.ajax({
                url: "/devices_show/" + dev_id + "/unlockDiag",
                cache: false,
                success: function (data) {
                    closeProgressBar(task);
                },
                error: function (request, status, error) {
                }
            });
        }

        getTasks = function (task = null) {
            $.ajax({
                url: deviceId + "/tasks",
                method: "GET",
                cache: false,
                success: function (data) {
                    dataCount = data["total"]
                if (data["fault"]) {
                    handleUnexpectedFault(data, task);
                } else if (dataCount != 0) {
                        continueDraw(task);
                    } else {
                        stopDraw(task);
                    }
                },
                error: function (request, status, error) { }
            });
        }

        getDiagLock = function (task = null) {
            $.ajax({
                url: url_diaglock,
                cache: false,
                success: function (data) {
                    if (task != "Cancel" && task != "StopUDP") {
                        switch (diagLockState) {
                            case 0:
                                if (data.locked == "yes") {
                                    if (task == "UDP Echo Test") {
                                        PROGRESS_DURATION = parseInt(data.ttl) * 1000;
                                        options.animation.duration = PROGRESS_DURATION;
                                    }
                                    diagLockState = 1;
                                    continueDraw(task);
                                }
                                break;
                            case 1:
                                if (!($('#btnCancelTasks').is(':visible')) && task != "WifiScan") {
                                    $('#btnCancelTasks').prop("hidden", false);
                                    $('.value').css("top", "46%");
                                }
                                if (data.locked == "no")
                                    diagLockState = -1;
                                break;
                        }
                    } else {
                        if (data.locked == "no")
                            diagLockState = -1;
                    }
                }
            });
        }

        continueDraw = function (task = null) {
            if (percent < 1) {
                if (percent <= 0.9) {
                    window.clearTimeout(intervalAnimate);
                    perSecondPercent = ((options.animation.end - parseInt(percent * 100)) / (options.animation.duration / 1000)) / 100;
                    perIntervalPercent = perSecondPercent / (1000 / options.animateInterval);
                    drawAnimated();
                } else if (percent > 0.9) {
                    window.clearTimeout(intervalAnimate);
                }
            } else {
                window.clearInterval(intervalTask);
                if (data["total"] != 0) {
                    removeFaultyTask(data["total"], data["result"]);
                } else {
                    closeProgressBar(task);
                }
            }
        }

        stopDraw = function (task = null) {
            window.clearInterval(intervalTask);
            window.clearTimeout(intervalAnimate);
            options.animation.duration = 1000;
            perSecondPercent = ((options.animation.end - parseInt(percent * 100)) / (options.animation.duration / 1000)) / 100;
            perIntervalPercent = perSecondPercent / (1000 / options.animateInterval);
            drawAnimated();
            setTimeout(() => {
                closeProgressBar(task);
            }, 1000);
        }

        draw = function (p) {
            // Clear frame
            ctx.clearRect(0, 0, s, s);

            // Draw background circle
            ctx.beginPath();
            ctx.arc(r, r, r, -Math.PI, Math.PI);
            ctx.arc(r, r, r - t, Math.PI, -Math.PI, true);
            ctx.closePath();
            ctx.fill(); // gray fill

            // Draw progress arc
            ctx.beginPath();
            ctx.arc(r, r, r, -Math.PI, -Math.PI + Math.PI * 2 * p);
            ctx.arc(r, r, r - t, -Math.PI + Math.PI * 2 * p, -Math.PI, true);
            ctx.closePath();
            ctx.save();
            ctx.clip();
            ctx.fillStyle = lg;
            ctx.fillRect(0, 0, s, s); // gradient fill
            ctx.restore();
        }

        drawAnimated = function () {
            intervalAnimate = window.setTimeout(() => {
                if (percent < 0.99 || dataCount == 0) {
                    percent += perIntervalPercent;
                    if (percent >= options.animation.end / 100) {
                        window.clearTimeout(intervalAnimate);
                        percent = options.animation.end / 100;
                        draw(percent);
                        $(canvas).trigger('circle-animation-progress', [percent, percent]);
                        $(canvas).trigger('circle-animation-end');
                    } else {
                        draw(percent);
                        $(canvas).trigger('circle-animation-progress', [percent, percent]);
                    }
                }
                drawAnimated();
            }, options.animateInterval);
        }

        startProgress = function (task = null, message = "") {
            init_confirm_dialog(task);
            getDiagLock(task);
            resetProgressValue();

            modal.style.display = "block";

            canvas.width = s;
            canvas.height = s;

            lg.addColorStop(0, options.startColor);
            lg.addColorStop(0, options.endColor);
            ctx.fillStyle = "rgba(255, 255, 255, 0.2)";

            drawAnimated();

            //animate numbers
            var valEl = $('.value');
            valEl.data('origVal', valEl.text());
            $(canvas).on('circle-animation-progress', function (e, progress) {
                valEl.text((parseInt(valEl.data('origVal') * progress)).toString() + "%")
            });

            progress_button = "Cancel"
            confirm_title = 'Cancel Task';
            confirm_message = 'Do you want to cancel the task?';
            if (task != null) {
                switch (task) {
                    case "WifiScan":
                        $('#btnCancelTasks').prop("hidden", true);
                        $('.value').css("top", "52%");
                        $('#showTasksName').text(task + " is running, please wait...");
                        break;
                    case "Cancel":
                        $('#showTasksName').text("Cancelling, please wait...");
                        $('#btnCancelTasks').prop("hidden", true);
                        $(".progress_bar").find("#canvas").prop("hidden", true);
                        $(".value").prop("hidden", true);
                        cancelDiagTaskTimeout(task);
                        break;
                    case "Mesh":
                        $('#btnCancelTasks').prop("hidden", true);
                        $('.value').css("top", "52%");
                        if (message == "" && $("#mesh_controllerId").text().length != 0)
                            $('#showTasksName').text(task + " (Controller ID: " + $("#mesh_controllerId").text() + ") is refreshing, please wait...");
                        else if (message != "")
                            $('#showTasksName').text(task + " (" + message + ") is refreshing, please wait...");
                        else
                            $('#showTasksName').text(task + " is refreshing, please wait...");
                        break;
                    default:
                        if (task == 'StopUDP') {
                            // UDP Echo Test only
                            cancelDiagTaskTimeout(task);
                            $('#btnCancelTasks').prop("hidden", true);
                            $(".progress_bar").find("#canvas").prop("hidden", true);
                            $(".value").prop("hidden", true);
                            $('#showTasksName').text("Stop and update the results, please wait...");
                        } else {
                            $('#btnCancelTasks').prop("hidden", true);
                            $('.value').css("top", "52%");
                            $('#showTasksName').text(task + " is running, please wait...");

                            if (task == "UDP Echo Test") {
                                // if ($("#udp_timeout").val() != "")
                                //     options.animation.duration = parseInt($("#udp_timeout").val()) * 1000;
                                if ($('.diagnostics').find("#type").val() == "UDP Echo Test") {
                                    progress_button = "Stop and View Result";
                                    confirm_message = 'Stop the task and view the results, are you sure?';
                                } else {
                                    progress_button = "Stop";
                                    confirm_message = 'Stop the task, are you sure?';
                                }
                                confirm_title = 'Stop Task';
                            }
                        }
                        break;
                }
            }

            $('#dialog_confirm').dialog('option', 'title', confirm_title);
            $('#btnCancelTasks').text(progress_button);
            $('#msg_confirm').text(confirm_message);

            diagLockState = 0;
            intervalTask = window.setInterval(() => {
                if (diagLockState != -1 && task != null) {
                    getDiagLock(task);
                } else {
                    getTasks(task);
                }
            }, 1000);
        }

        $('#btnCancelTasks').on('click', function (e) {
            e.preventDefault();
            if ($('#btnCancelTasks').text() == "Close") {
                cancelTask("Cancel");
            } else {
                $("#dialog_confirm").dialog("open");
            }
        });

        closeProgressBar = function (task = null) {
            window.clearTimeout(timeoutCancelDiagTask);
            if (task != null) {
                switch (task) {
                    case "Mesh":
                        refresh_page_without_preload();
                        break;
                    case "IP Ping Test":
                    case "Trace Route Test":
                    case "Upload Test":
                    case "Download Test":
                    case "NS Lookup Test":
                        if (typeof getResult != 'undefined' && getResult) {
                            getResult(task);
                        }
                        modal.style.display = "none";
                        $("#dialog_confirm").dialog("close");
                        break;
                    case "UDP Echo Test":
                    case "StopUDP":
                        modal.style.display = "none";
                        $("#dialog_confirm").dialog("close");
                        if (typeof getResult != 'undefined' && getResult) {
                            if ($('.diagnostics').find("#type").val() == "UDP Echo Test") {
                                setTimeout(function () {
                                    getResult("UDP Echo Test");
                                }, 2000);
                            } else {
                                getResult();
                            }

                        }
                        break;
                    case "Cancel":
                        if (typeof getResult != 'undefined' && getResult) {
                            getResult();
                        }
                    default:
                        modal.style.display = "none";
                        $("#dialog_confirm").dialog("close");
                        break;
                }
            } else {
                if (reload == 0) {
                    refresh_page_without_preload();
                    reload = 1;
                }
            }

        }

        storeFlash = function () {
            if ($('.flash').length) {
                sessionStorage.setItem("flashMessage", $('.flash').prop('outerHTML'));
                $('.flash').remove();

                if ((sessionStorage.getItem("flashMessage")).includes("Device is busy")) {
                    $('#showTasksName').text("Previous task is still running, please wait...")
                }
            }
        }

        showFlash = function () {
            const flashMessage = sessionStorage.getItem("flashMessage");
            if ( flashMessage != null) {
                $('.flash').remove();
                $('#content').prepend(flashMessage);
                sessionStorage.removeItem("flashMessage");
            }
        }

        cancelDiagTask = function (type) {
            if ($("#diagStartFlag").length != 0) {
                $("#diagStartFlag").val("0");
            }
            var data = [];
            switch (type) {
                case "IP Ping Test":
                    data = [{
                        "name": "setParameterValues",
                        "parameterValues": [],
                        "diagnostic": "IPPingDiagnostics",
                        "cancel": "true",
                        "DSCP": $('#ping_dscp').val()
                    }];
                    break;
                case "Trace Route Test":
                    data = [{
                        "name": "setParameterValues",
                        "parameterValues": [],
                        "diagnostic": "TraceRouteDiagnostics",
                        "cancel": "true",
                        "DSCP": $('#trace_dscp').val()
                    }];
                    break;
                case "Upload Test":
                    data = [{
                        "name": "setParameterValues",
                        "parameterValues": [],
                        "diagnostic": "UploadDiagnostics",
                        "cancel": "true",
                        "DSCP": $('#upload_dscp').val()
                    }];
                    break;
                case "Download Test":
                    data = [{
                        "name": "setParameterValues",
                        "parameterValues": [],
                        "diagnostic": "DownloadDiagnostics",
                        "cancel": "true",
                        "DSCP": $('#download_dscp').val()
                    }];
                    break;
                case "UDP Echo Test":
                    data = [{
                        "name": "setParameterValues",
                        "parameterValues": [],
                        "diagnostic": "UDPEchoConfig",
                        "cancel": "true",
                        "Enable": false
                    }];
                    break;
                case "NS Lookup Test":
                    data = [{
                        "name": "setParameterValues",
                        "parameterValues": [],
                        "diagnostic": "NSLookupDiagnostics",
                        "cancel": "true",
                        "Timeout": $('#ns_timeout').val()
                    }];
                    break;
                case "Mesh":
                    return
            }
            commitDiag(data);
        }

        diagnosticNameTransform = function (lockName) {
            var name = "";
            switch (lockName) {
                case "IPPingDiagnostics":
                    name = "IP Ping Test";
                    break;
                case "TraceRouteDiagnostics":
                    name = "Trace Route Test";
                    break;
                case "UploadDiagnostics":
                    name = "Upload Test";
                    break;
                case "DownloadDiagnostics":
                    name = "Download Test";
                    break;
                case "UDPEchoConfig":
                    name = "UDP Echo Test";
                    break;
                case "NSLookupDiagnostics":
                    name = "NS Lookup Test";
                    break;
                default:
                    name = lockName;
                    break;
            }
            return name;
        }

        resetProgressValue = function () {
            window.clearTimeout(timeoutCancelDiagTask);
            window.clearInterval(intervalTask);
            window.clearTimeout(intervalAnimate);
            $('.value').text("100");
            $(".progress_bar").find("#canvas").prop("hidden", false);
            $(".value").prop("hidden", false);
            reload = 0;
            percent = 0;
            dataCount = 1;
            options = {
                size: 150,
                startAngle: -Math.PI,
                startColor: "#4acbd6",
                endColor: "#4acbd6",
                animateInterval: 30,
                animation: {
                    duration: PROGRESS_DURATION,
                    easing: "circleProgressEase",
                    start: 0,
                    end: 100
                }
            };
            s = options.size;
            r = s / 2;
            t = s / 14;

            canvas = $("#canvas")[0];

            ctx = canvas.getContext("2d");
            lg = ctx.createLinearGradient(0, 0, s, 0);

            deviceId = $('#dialog_modal_deviceid').text();
            diagLockState = 0;
            perSecondPercent = ((options.animation.end - options.animation.start) / (options.animation.duration / 1000)) / 100;
            perIntervalPercent = perSecondPercent / (1000 / options.animateInterval);
        }

        cancelTask = function (task = null) {
            window.clearTimeout(timeoutCancelDiagTask);
            window.clearInterval(intervalTask);
            window.clearTimeout(intervalAnimate);
            if (sessionStorage.getItem("flashMessage") != null) {
                sessionStorage.removeItem("flashMessage");
            }
            $.ajax({
                url: url_diaglock,
                cache: false,
                success: function (dataDiaglock) {
                    var urlCancelTask = deviceId + "/tasks";
                    var type = diagnosticNameTransform(dataDiaglock.locked_name);
                    if (($("#showTasksName").text()).indexOf("Cancel") >= 0 || ($("#showTasksName").text()).indexOf("Stop") >= 0) {
                        type = "Cancel";
                    }
                    if (type == "Mesh") {
                        urlCancelTask = controllerId + "/tasks";
                    }
                    $.ajax({
                        url: urlCancelTask,
                        method: "GET",
                        cache: false,
                        success: function (dataCancelTask) {
                            if (dataCancelTask["fault"]) {
                                handleUnexpectedFault(dataCancelTask, task);
                            } else if (dataDiaglock.locked == "yes") {
                                removeFaultyTask(dataCancelTask["total"], dataCancelTask["result"], type);
                            } else {
                                window.clearTimeout(timeoutCancelDiagTask);
                                removeFaultyTask(dataCancelTask["total"], dataCancelTask["result"]);
                            }
                        },
                        error: function (request, status, error) {
                        }
                    });
                },
                error: function (request, status, error) {
                }
            });
        }

        handleUnexpectedFault = function(data, task) {
            if (task == null) return;
            let message = '<div class="flash error"> Task faulted </div>';
            // data = JSON.parse(data);
            if (data["fault"]["code"] && data["fault"]["message"]) {
                message = '<div class="flash error"> Task faulted (' + data["fault"]["code"] + ', ' + data["fault"]["message"] + ')';
                if (data["fault"]["provisions"]) {
                    message = message.concat('</br> Details: ');
                    for (i in data["fault"]["provisions"]) {
                        message = message.concat('</br>  ' + (parseInt(i) + 1).toString() + ". " + data["fault"]["provisions"][i]);
                    }
                }
                message = message.concat('</div>');
            }
            sessionStorage.setItem('flashMessage', message);
            removeFaultyTask(data["total"], data["result"]);
        }

        cancelDiagTaskTimeout = function(task = null) {
            timeoutCancelDiagTask = setTimeout(function() {
                switch (task) {
                    case "Cancel":
                        $('#showTasksName').text("Cancelling the task failed.");
                        break;
                    case "StopUDP":
                        $('#showTasksName').text("Stop the task failed.");
                        break;
                    default:
                        break;
                }
                $('#btnCancelTasks').prop("hidden", false);
                $('#btnCancelTasks').text("Close");
                $(".progress_bar").find("#canvas").prop("hidden", true);
                $(".value").prop("hidden", true);
            }, 30000);
        }

        init_confirm_dialog = function (task = null) {
            $(function () {
                $("#dialog_confirm").dialog({
                    autoOpen: false,
                    resizable: false,
                    height: "auto",
                    width: 400,
                    modal: true,
                    buttons: {
                        "Yes": function () {
                            $(this).dialog("close");
                            cancelTask(task);
                        },
                        "No": function () {
                            $(this).dialog("close");
                        }
                    }
                });
            });
        }

        refresh_page_without_preload = function () {
            let url = (window.location.href.split("?"))[0];
            let urlParams = get_url_parameter();
            let i = 0;
            let searchString = "";
            for (let key in urlParams) {
                if (key != "preload") {
                    if (i++)
                        searchString += "&"
                    searchString += key + "=" + urlParams[key]
                }
            }
            url = url + "?" + searchString;
            window.location.href = url;
        }

        get_url_parameter = function () {
            let pageUrl = window.location.search.substring(1);
            let urlVariables = pageUrl.split("&");
            let urlParameters = [];

            for (let i = 0; i < urlVariables.length; i++) {
                let param = urlVariables[i].split("=");
                urlParameters[param[0]] = param[1];
            }
            return urlParameters;
        }


        $(document).ready(function() {
            initProgressBar();
            showFlash();
        });

    }
});