# controller for /backup_restore
class BackupRestoreController < ApplicationController
  include Util
  include Loggers
  MODULE_NAME = '[Backup Restore]'.freeze()

  def index
    can?(:read, 'backup_restore') do
      create_usb_info()
      usb_selection_data()
      filelist_of_first_usb()
      @sys_version = check_status('/opt/genieacs/genieacs_version')
      @backup_status = check_status('/tmp/acsbackup.status')
      @restore_status = check_status('/tmp/acsrestore.status')
    end
  end

  def check
    can?(:read, 'backup_restore') do
      case params['check_action']
      when 'get_status', 'get_msg', 'get_progress'
        create_status_response(params['check_type'], params['check_action'].split('_')[1])
      when 'get_backup_file'
        create_filelist_response()
      end
    end
    render json: @check_status
  end

  def backup
    can?(:update, 'backup_restore') do
      if params['action'] == 'backup'
        agileACSLog(6, MODULE_NAME, 'Backup system configuration.')
        data = { 'action' => [params['mount_point'], params['backup_file_name']] }
        create_api_conn().post('/backup', ActiveSupport::JSON.encode(data), 'Content-Type' => 'application/json')
        # spawn("/usr/bin/nice -n19 /opt/agileacs/bin/agileacs-backup.sh #{params['mount_point']} #{params['backup_file_name']} &")
        # spawn("/opt/agileacs/bin/agileacs-backup-progress.sh #{params['mount_point']} &")
      end
    end
  end

  def restore
    can?(:update, 'backup_restore') do
      if params['action'] == 'restore'
        agileACSLog(6, MODULE_NAME, 'Restore system configuration.')
        data = { 'action' => [params['mount_point'], params['file_name']] }
        create_api_conn().post('/restore', ActiveSupport::JSON.encode(data), 'Content-Type' => 'application/json')
        # spawn("/usr/bin/nice -n19 /opt/agileacs/bin/agileacs-restore.sh #{params['mount_point']} #{params['file_name']} &")
        # spawn("/opt/agileacs/bin/agileacs-restore-progress.sh #{params['mount_point']} &")
      end
    end
  end

  def reset
    can?(:delete, 'backup_restore') do
      if params['action'] == 'reset'
        agileACSLog(6, MODULE_NAME, 'Factory Restore.')
        data = { 'action' => [params['action']] }
        # spawn('/usr/bin/nice -n19 /opt/agileacs/bin/agileacs-restore.sh reset &')
        create_api_conn().post('/restore', ActiveSupport::JSON.encode(data), 'Content-Type' => 'application/json')
        # Util.new.create_api_conn.post('/restore/reset', ActiveSupport::JSON.encode({ 'action': 'restore', 'filename': 'reset' }))
      end
    end
  end
end

def create_usb_info
  # type of res.body is string, need to use JSON.parse()
  usb_info = create_api_conn().get('/usbinfo').body
  # usb_info = `%x[/opt/agileacs/bin/usb-storage.sh get-usage]`.strip()
  @storage = JSON.parse('{}')
  @usb_status = 'No USB storage'
  return unless usb_info.include? 'VALID_USB_AMOUNT'

  @storage = JSON.parse(usb_info)
  @usb_status = @storage['Storage_0']['Status'] if @storage['VALID_USB_AMOUNT'].positive?
end

def usb_selection_data
  @usb_list = []
  idx = 0
  while @storage["Storage_#{idx}"]
    @usb_list.push(Array.new([@storage["Storage_#{idx}"]['USBStorage'], "Storage_#{idx}"]))
    idx += 1
  end
end

def filelist_of_first_usb
  @backup_files = !@storage['Storage_0'].nil? ? create_filelist_response(@storage['Storage_0']['MountPoint']) : ['']
  @total_pages = (@backup_files.length.to_f / 10).ceil
end

def check_status(filename)
  content = ''
  if File.file?(filename)
    File.open(filename).each do |line|
      content += line
    end
  end
  content.sub(/\n/, '')
end

def create_status_response(type, action)
  filename = type == 'backup' ? "/tmp/acsbackup.#{action}" : "/tmp/acsrestore.#{action}"
  @check_status = { 'status': check_status(filename) }.to_json
end

def create_filelist_response(mount_point = nil)
  data = { 'mount_point' => mount_point.nil? ? params['mount_point'] : mount_point }
  res = JSON.parse(create_api_conn().post('/backup/filelist', ActiveSupport::JSON.encode(data), 'Content-Type' => 'application/json').body)
  # backup_files = `%x[/opt/agileacs/bin/usb-storage.sh get-list #{params['mount_point']}]`.split("\n")
  @check_status = { 'backup_files': res['result'], 'total_pages': (res['result'].length.to_f / 10).ceil }.to_json
  res['result']
end
