class DevicesController < ApplicationController
  require 'net/http'
  require 'csv'
  include InstanceProfileApps
  include Log
  include Loggers
  include Models
  include DeviceFilters
  include DeviceTask

  def index
    can?(:read, 'devices') do
      @can_update = can?(:update, 'devices')
      @can_delete = can?(:delete, 'devices')

      @modelsAll = get_models()

      @filters = get_filters(1, Rails.configuration.devices_index_parameters)

      @query = {}
      @query = ActiveSupport::JSON.decode(URI.unescape(params['query'])) if params.key?('query')

      if request.format == Mime[:csv]
        index_csv()
        agileACSLog(6, '[Export Devices]', 'Export devices list succeed.')
        return
      end
      @total = 0
      @now = nil
      @devices = []
      begin
        http = create_api_conn()

        aggregate_pipeline = generate_aggregate_pipeline(params, true, true, false, Rails.configuration.devices_index_parameters, Rails.configuration.devices_summary_parameters)
        res = aggregate(http, 'devices', aggregate_pipeline, 'HEAD')
        @total = res[:total].to_i

        page_num = (@total / 10.0).ceil
        params[:page] = (@total / 10.0).ceil if page_num < params[:page].to_i
        aggregate_pipeline = generate_aggregate_pipeline(params, false, true, false, Rails.configuration.devices_index_parameters, Rails.configuration.devices_summary_parameters)
        res = aggregate(http, 'devices', aggregate_pipeline)
        @now = res[:timestamp]
        @devices = res[:result]
        @message = res[:message]
      rescue StandardError => e
        puts "exception occurs:\n #{e}"
        puts e.backtrace
        @total = 0
        @now = nil
        @devices = []
      end

      @appendix = APPENDIX
      interval = get_informinterval
      @online = interval['online']
      respond_to do |format|
        format.html # index.html.erb
        format.json { render json: @devices }
      end
    end
  end

  def edit_info
    ret = { 'action' => 'edit_info', 'result' => 'ok' }
    can?(:update, 'devices') do
      id = params[:_id]

      if !id.blank?
        data = { '_id' => id }

        extension_fields = [%w[homeId _homeId], %w[zoneId _zoneId], %w[cellPhone _cellPhone], %w[cpf _cpf]]
        extension_fields.each do |e|
          val = params[e[0].to_sym]
          data[e[1]] = val.blank? ? '' : val
        end

        res = nil
        http = create_api_conn()
        begin
          res = http.put('/devices/edit_info', ActiveSupport::JSON.encode(data), 'Content-Type' => 'application/json')
        rescue StandardError => e
          puts e
          res = nil
        end
        object = "[Device Detail][Setting]#{data['_id']}"

        if res && res.code.to_i == 200
          agileACSLog(6, object, "Device #{data['_id']} update info successfully. ['Home ID', 'Zone ID', 'Cell phone', 'CPF'] = ['#{data['_homeId']}', '#{data['_zoneId']}', '#{data['_cellPhone']}', '#{data['_cpf']}']")
          flash[:success] = "Device #{data['_id']} update info successfully."
        else
          agileACSLog(6, object, "Device #{data['_id']} update info failed. ['Home ID', 'Zone ID', 'Cell phone', 'CPF'] = ['#{data['_homeId']}', '#{data['_zoneId']}', '#{data['_cellPhone']}', '#{data['_cpf']}']")
          flash[:error] = "Device #{data['_id']} update info failed."
        end
      else
        ret.result = 'The given parameter is wrong.'
      end
    end

    render json: ret.to_json
  end

  def get_informinterval
    res = query_resource(create_api_conn(), 'devices/cpesetting', {})
    if res[:result][0].nil?
      cpesetting = {}
      cpesetting['informinterval'] = 86_400
      cpesetting['intervaloffset'] = 3600
    else
      cpesetting = res[:result][0]
    end

    online = cpesetting['informinterval'].to_i + cpesetting['intervaloffset'].to_i
    past = 7*24*60*60

    { 'online' => online, 'past' => past }
  end

  def destroy_select
    can?(:delete, 'devices') do
      http = create_api_conn()
      params[:devices_id].each do |id|
        object = "[Device Detail][Setting]#{id}"
        remove_device_from_license(id, http)
        res = remove_device(id, http)
        if res.code != '200'
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
          agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
        else
          flash[:success] = 'Successfully'
          agileACSLog(6, object, "Device #{id} deleted.")
        end
      end
    end
  end

  def index_csv

    aggregate_pipeline = generate_aggregate_pipeline(params, true, true, false, Rails.configuration.devices_index_parameters, Rails.configuration.devices_summary_parameters)

    aggregate_pipeline.delete_if do |element|
      if  element.key?('$project') || element.key?('$match')
        false
      else
        true
      end
    end

    response.headers['Content-Type'] = 'text/csv'

    title = []
    field = []
    Rails.configuration.devices_index_parameters.each do |label, definition|
      next if definition['field'] == 'Mesh' && !TpConfig.get_config('features-enable')

      title << label
      field << definition['field']
    end

    response.stream.write(title.to_csv())

    results = []
    begin
      results = aggregate(create_api_conn(), 'devices', aggregate_pipeline)
    rescue StandardError => e
      puts "exception occurs:\n #{e}"
      puts e.backtrace
      results = []
    end
    results.each do |k, v|
      next unless k.to_s == 'result'

      v.each do |item|
        line = []
        field.each do |f|
          val = '-'
          if item.include? f
            val = item[f]
            val = helpers.extract_ip(val) if f == 'IP'
            val = val.to_time.localtime.strftime('%FT%T.%3NZ') if f == '_lastInform'
            val = helpers.convert_to_xmpp_status(val) if f == 'XmppStatus'
            val = helpers.convert_to_mesh(val) if f == 'Mesh'
          else
            val = ''
            case f
            when 'XmppStatus'
              val = '-'
            when 'Mesh'
              val = 'N/A'
            end
          end
          line << val.to_s
        end
        response.stream.write(line.to_csv())
      end
    end
  ensure
    response.stream.close
    end
end
