class DevicesShowController < ApplicationController
  require 'net/http'
  require 'json'
  require 'csv'

  include ActionController::Live
  include DevicesShowLib2
  include DeviceTask
  include DiagLock
  include DiagUtil
  include Mesh
  include MeshDataType
  include Log
  include Util

  @@wifi_basic_scan = { :status => '<Empty>', :task_id => '', :total => '', :progress => 'start', :rawdata => nil, :diag_lock => '' }
  @@scan_count = 0
  @@last_scan_time = ''
  @@wifi_scan_task = { :get_parametervalue => '', :set_parametervalue => '', :get_bssdescentry => '' }

  helper_method :get_refresh_tab_params

  def update
    @is_tr181 = params.include?('isTr181') && params['isTr181'] == 'true' ? true : false
    if @is_tr181
      if !check_tab_valid(params[:tab], params[:id], create_api_conn(), true, params)
        flash[:warning] = 'You are not authorized to change values.'
        redirect_to(action: :show, tab: 'info', 'isTr181': params['isTr181'])
      else
        update_task(@is_tr181)
      end
    else
      update_task(@is_tr181)
    end
  end

  def diagresult
    can?(:read, 'devices') do
      id = params[:id]
      create_api_conn() do |http|
        res = query_resource(http, 'devices', { '_id' => id })
        raise ActiveRecord::RecordNotFound if res[:result].size != 1
        @is_tr181 = params.include?('isTr181') && params['isTr181'] == 'true' ? true : false
        @device = res[:result][0]
        @device_params = flatten_params(@device)
        diag_lock = get_diagnostic_lock(id)['result']
        if diag_lock.blank?
          get_result = get_diagnostic_result(@device_params, @is_tr181)
          get_result.each do |key, value|
            if key.to_s.include? 'DiagnosticsState'
              get_result[key] = 'Timeout' if value == '' || value == 'None' || value == 'Requested'
            end
          end
          get_result['NSLookupDiagnostics.DiagnosticsState'] = 'Timeout' unless get_result['NSLookupDiagnostics.DiagnosticsState']
          @diag_result = get_result.to_json
        else
          @diag_result = {}.to_json
        end
      end
    end
    # @diag_result = {}.to_json
    render json: @diag_result
  end

  # INFO: support TR-098 and TR-181
  def show
    can?(:read, 'devices') do
      id = params[:id]
      # the following 2 lines is for destroy task
      @tab = params[:tab]
      @device_id = params[:id]
      @is_tr181 = params.include?('isTr181') && params['isTr181'] == 'true' ? true : false
      need_preload = params.include?('preload') && params['preload'] == 'true' ? 1 : 0
      object = "[Device Detail][Refresh]#{params[:id]}"
      @device_state = 'reachable'
      @system_xmpp_enable = 'disable'
      return redirect_to('action': :show, 'tab': 'info', 'isTr181': params['isTr181'], 'preload': true) unless valid_tab?(@is_tr181)
# read privilege, for sub tab display
      @privilege_read = {
        'info' => can?(:read, '_devices-info'),
        'lan' => can?(:read, '_devices-lan'),
        'wan' => can?(:read, '_devices-wan'),
        'wlan' => can?(:read, '_devices-wireless'),
        'wlanscan' => can?(:read, '_devices-wifi_scan'),
        'mesh' => can?(:read, '_devices-mesh'),
        'diagnostics' => can?(:read, '_devices-diagnostics'),
        'tr_tree' => can?(:read, '_devices-tr_tree'),
        'log' => can?(:read, '_devices-log'),
        'actions' => can?(:read, '_devices-actions')
      }

      if params[:tab] == 'wlan'
        @privilege_others = {
          'create' => can?(:create, "_devices-wireless"),
          'update' => can?(:update, "_devices-wireless"),
          'delete' => can?(:delete, "_devices-wireless"),
          'updatescan' => can?(:update, "_devices-wifi_scan")
        }
      else
        @privilege_others = {
          'create' => can?(:create, "_devices-#{params[:tab]}"),
          'update' => can?(:update, "_devices-#{params[:tab]}"),
          'delete' => can?(:delete, "_devices-#{params[:tab]}")
        }
      end

      create_api_conn() do |http|
        case params['tab']
        when 'info'
          @device_state, @task_state = device_connected?(@is_tr181, CGI.escape(id), http, 1, object, need_preload)
          @system_xmpp_enable = system_xmpp_enable?(http)
        when 'diagnostics', 'wlan'
          @device_state, @task_state = device_connected?(@is_tr181, CGI.escape(id), http, 0, object)
        when 'mesh'
          @device_state, @task_state = device_connected?(@is_tr181, CGI.escape(id), http, 0, object)
          @db_project = get_mesh_data_project(DT_ALL)
        end

        res = query_resource(http, 'devices', { '_id' => id }, @db_project)
        raise ActiveRecord::RecordNotFound if res[:result].size != 1

        @now = res[:timestamp]
        @device = res[:result][0]
        @device.delete('Downloads') if @device.key?('Downloads')
        @device_params = flatten_params(@device) if params['tab'] != 'mesh'
        @files = query_resource(http, 'files', { 'metadata.model' => @device[@device.include?('InternetGatewayDevice') ? 'InternetGatewayDevice' : 'Device']['DeviceInfo']['ModelName']['_value'] }, nil, nil, 100)[:result]
        @tasks = query_resource(http, 'tasks', { 'device' => id }, nil, nil, nil, { 'timestamp' => 1 })[:result]
        @faults = {}
        query_resource(http, 'faults', { 'device' => id }) { |f| @faults[f['channel']] = f if f }
        @has_mesh_data = has_mesh_data(@device)

        case params['tab']
        when 'lan', 'wan', 'wlan'
          begin
            @item = get_items(http, params['tab'], @is_tr181)
            if @is_tr181
              @info = get_info(http, id, params['tab']) || {}
              if params['tab'] == 'wlan'
                process_wifi_scan(http, id, params, @is_tr181)
                return if @wifi_scan_render_result
              end
            else
              prepare98()
            end
          rescue StandardError
            @info = {}
            @item = []
          end
        when 'diagnostics'
          # Get diagnostic lock
          @diag_lock = get_diagnostic_lock(@device['_id'])['result']

          @wan_path = []
          @wan_name = []
          diag_list = ','

          if @is_tr181 # for 181
            @wan_path.push('')
            @wan_name.push('Default Route')

            d = DeviceEntry.new(http, @device['_id'], 'wan')
            wan_d = d.get_wan_interface_names
            wan_d.each do |v|
              @wan_name.push(v['name'])
              @wan_path.push(v['interface_path'])
            end

            @device_params.each do |v|
              next unless v['_path'].include?('Diagnostics')

              next unless /^\w+\.\w+\.\w+\.\w+$/.match(v['_path'])

              # IP Ping Test
              diag_list += 'IP Ping Test,' if v['_path'].include?('IPPing')
              # Trace Route Test
              diag_list += 'Trace Route Test,' if v['_path'].include?('TraceRoute')
              # Upload Test
              diag_list += 'Upload Test,' if v['_path'].include?('UploadDiagnostics')
              # Download Test
              diag_list += 'Download Test,' if v['_path'].include?('DownloadDiagnostics')
              # UDP Echo Test
              diag_list += 'UDP Echo Test,' if v['_path'].include?('UDPEchoConfig')
              # NS Lookup Test
              diag_list += 'NS Lookup Test,' if v['_path'].include?('NSLookupDiagnostics')
            end
            @diag_select_option = reorder_diagnostic_list(diag_list)
          else # for 98
            wan_enable_val = []
            @device_params.each do |v|
              if v['_path'].include?('WANIPConnection') || v['_path'].include?('WANPPPConnection') || v['_path'].include?('TP_Connection')
                if v['_path'].include?('.Name')
                  @wan_path = @wan_path.push(v['_path'].to_s.from(0).to(-6))
                  @wan_name = @wan_name.push(v['_value'])
                end
                wan_enable_val.push(v['_value']) if v['_path'].include?('.Enable')
              end
              next unless v['_path'].include?('Diagnostics') || v['_path'].include?('UDPEchoConfig')

              next unless /^\w+\.\w+$/.match(v['_path'])

              # IP Ping Test
              diag_list += 'IP Ping Test,' if v['_path'].include?('IPPingDiagnostics')
              # Trace Route Test
              diag_list += 'Trace Route Test,' if v['_path'].include?('TraceRouteDiagnostics')
              # Upload Test
              diag_list += 'Upload Test,' if v['_path'].include?('UploadDiagnostics')
              # Download Test
              diag_list += 'Download Test,' if v['_path'].include?('DownloadDiagnostics')
              # UDP Echo Test
              diag_list += 'UDP Echo Test,' if v['_path'].include?('UDPEchoConfig')
              # NS Lookup Test
              diag_list += 'NS Lookup Test,' if v['_path'].include?('NSLookupDiagnostics')
            end
            @diag_select_option = reorder_diagnostic_list(diag_list)
            # Remove interface which doesn't be enabled
            i = wan_enable_val.length - 1
            until i < 0
              unless wan_enable_val[i]
                @wan_path.delete_at(i)
                @wan_name.delete_at(i)
              end
              i -= 1
            end
            @wan_path = if @wan_path == [nil]
                          ['']
                        else
                          @wan_path.insert(0, '')
                        end
            @wan_name = if @wan_name == [nil]
                          ['Default Route']
                        else
                          @wan_name.insert(0, 'Default Route')
                        end
          end
        when 'log'
          get_log(id)
        when 'mesh'
          return redirect_to('action': :show, 'tab': 'info', 'isTr181': params['isTr181']) unless @has_mesh_data

          @mesh_topology = generate_mesh_topology(@device)
        end
      end

      respond_to do |format|
        format.html # show.html.erb
        format.json { render json: @device }
      end
    end
  end

  def get_info(http, device_id, tab)
    d = DeviceEntry.new(http, device_id, tab)
    res = d.get_entries
    res = res[0] if tab == 'lan'
    res
  end

  def search_hash(h, search, v = nil, tmp = [])
    if h.fetch(search, false)
      if v.nil?
        tmp.push(h[search])
      elsif h[search]['_value'] == v
        if (v == 'DHCP') || (v == 'Static')
          tmp.push(h[search]) if h[search]['_path'].match('WANIPConnection')
        else
          tmp.push(h[search])
        end
      end
    end

    h.each_key do |k|
      search_hash(h[k], search, v, tmp) if h[k].is_a? Hash
    end
    tmp
  end

  def get_node_by_path(path, hash)
    if path.blank?
      return nil if hash.nil?

      return { 'value' => hash['_value'], 'writable' => hash['_writable'] }
    end

    key = path.shift
    hash = hash[key] unless hash.nil?
    get_node_by_path(path, hash)
  end

  def get_info98(path, nodes, hash)
    tmp = {}
    nodes.each do |node|
      a = path.split('.')[0]

      if node == 'X_TP_VID' || node == 'X_TP_VLanEnabled' || node == 'X_TP_QosMark'
        a.pop
        a.pop
        a.push('WANEthernetLinkConfig')
      end

      if node == 'IPInterfaceIPAddress' || node == 'IPInterfaceSubnetMask'
        a.push('IPInterface')
        a.push('1')
      end

      a.push(node)

      if (ns = node.split('.')).length > 1
        a.pop
        ns.each { |n| a.push(n) }
      end

      value = get_node_by_path(a, hash)
      tmp[node] = value.nil? ? '' : value
    end

    tmp['path'] = path.join('.')
    tmp
  end

  def prepare98
    case params['tab']
    when 'lan'
      @lan_item = @item
      lan = search_hash(@device_params[0], 'IPRouters')
      lan_nodes = %w[IPInterfaceIPAddress IPInterfaceSubnetMask DHCPServerEnable MinAddress MaxAddress DHCPLeaseTime]
      @lan_info = []
      lan.each do |a|
        path = a['_path'].split('.')
        path.shift
        path.pop
        info = get_info98(path, lan_nodes, @device_params[0])
        @lan_info.push(info)
      end
    when 'wan'
      @wan_item = @item
      wan_static_addrType = search_hash(@device_params[0], 'AddressingType', 'Static')
      wan_dhcp_addrType = search_hash(@device_params[0], 'AddressingType', 'DHCP')
      wan_pppoe = search_hash(@device_params[0], 'TransportType', 'PPPoE')
      wan_l2tp = search_hash(@device_params[0], 'TransportType', 'L2TP')
      wan_pptp = search_hash(@device_params[0], 'TransportType', 'PPTP')

      static_nodes = %w[
        Name X_TP_FirewallEnabled X_TP_FullconeNATEnabled MaxMTUSize AddressingType NATEnabled
        SubnetMask DefaultGateway ExternalIPAddress DNSServers MaxMTUSize X_TP_VID X_TP_VLanEnabled
        X_TP_QosMark X_TP_IGMPProxyEnabled ConnectionStatus
      ]
      @wan_static_info = []
      wan_static_addrType.each do |a|
        path = a['_path'].split('.')
        path.shift
        path.pop
        info = get_info98(path, static_nodes, @device_params[0])
        @wan_static_info.push(info)
      end
      dhcp_nodes = %w[
        Name X_TP_FirewallEnabled X_TP_FullconeNATEnabled MaxMTUSize AddressingType NATEnabled SubnetMask
        DefaultGateway ExternalIPAddress DNSServers MaxMTUSize X_TP_VID X_TP_VLanEnabled X_TP_QosMark
        X_TP_Unicast X_TP_IGMPProxyEnabled ConnectionStatus DNSOverrideAllowed
      ]
      @wan_dhcp_info = []
      wan_dhcp_addrType.each do |a|
        path = a['_path'].split('.')
        path.shift
        path.pop
        info = get_info98(path, dhcp_nodes, @device_params[0])
        @wan_dhcp_info.push(info)
      end

      pppoe_nodes = %w[
        Name X_TP_FirewallEnabled NATEnabled Username Password PPPAuthenticationProtocol PPPoEServiceName
        MaxMRUSize ConnectionTrigger PPPoEACName ExternalIPAddress PPPLCPEcho DNSServers X_TP_VID X_TP_VLanEnabled
        X_TP_QosMark X_TP_UseStaticIP X_TP_IGMPProxyEnabled X_TP_FullconeNATEnabled ConnectionStatus SubnetMask
        DefaultGateway DNSOverrideAllowed
      ]
      @wan_pppoe_info = []
      wan_pppoe.each do |a|
        path = a['_path'].split('.')
        path.shift
        path.pop
        info = get_info98(path, pppoe_nodes, @device_params[0])
        @wan_pppoe_info.push(info)
      end

      l2tp_nodes = %w[
        Name Username Password ServerIpOrName ConnectionTrigger PPPAuthenticationProtocol MaxMRUSize AddressingType
        NATEnabled FullconeNATEnabled FirewallEnabled IGMPProxyEnabled DNSServersIP X_TP_VID X_TP_VLanEnabled
        X_TP_QosMark ExternalIPAddressIP SubnetMaskIP DefaultGatewayIP DNSServersIP ExternalIPAddress SubnetMask
        DefaultGateway DNSServers ConnectionStatus
      ]
      @wan_l2tp_info = []
      wan_l2tp.each do |a|
        path = a['_path'].split('.')
        path.shift
        path.pop
        info = get_info98(path, l2tp_nodes, @device_params[0])
        @wan_l2tp_info.push(info)
      end

      pptp_nodes = l2tp_nodes
      @wan_pptp_info = []
      wan_pptp.each do |a|
        path = a['_path'].split('.')
        path.shift
        path.pop
        info = get_info98(path, pptp_nodes, @device_params[0])
        @wan_pptp_info.push(info)
      end
    when 'wlan'
      @wifi_scan_supported = false
      @wlan_item = @item
      id = params[:id]
      if params['progress'] == 'scan_start' || params['progress'] == 'scan_refresh' || params['progress'] == 'scan_result'
        task_id = params['task_id']
        progress = params['progress']
        bandscan = params['bandscan']
        http = create_api_conn()
        object = "[Device Detail][Wi-Fi Scan]#{id}"
        if params['progress'] == 'scan_start' && task_id == ''
          cpe_status, task_status = device_connected?(@is_tr181, CGI.escape(id), http, 0, object)
          if cpe_status == 'reachable' and task_status == 0
            @@wifi_basic_scan[:diag_lock] = ''
            wifiscan_timeout = Rails.configuration.device_config['WiFiScanTimeout']   ? Rails.configuration.device_config['WiFiScanTimeout'].to_i  : 65
            save_diagnostic_lock(id, 'WifiScan', wifiscan_timeout)
            wifiscanbasic98(bandscan, id, task_id, progress)
          else
            @@wifi_basic_scan[:status] = '<Empty>'
            @@wifi_basic_scan[:task_id] = ''
            @@wifi_basic_scan[:total] = ''
            @@wifi_basic_scan[:progress] = 'off-line'
            @@wifi_basic_scan[:rawdata] = nil
            @@wifi_basic_scan[:diag_lock] = ''
            @@scan_count = 0
            @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
            render json: @scan_wifi_result
          end
          return
        end
        wifiscanbasic98(bandscan, id, task_id, progress)
        return
      else
        # default run no wi-fi scan progress
        @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
        @@scan_count = 0
        if params['progress'].nil?
          @scan_wifi_result['status'] = '<Empty>'
          @scan_wifi_result['total'] = ''
          @@wifi_scan_task = { 'get_parametervalue': '', 'set_parametervalue': '', 'get_bssdescentry': '' }
        end
      end
      wlan_2G = search_hash(@device_params[0], 'X_TP_Band', '2.4GHz')
      wlan_5G = search_hash(@device_params[0], 'X_TP_Band', '5GHz')
      wlan_nodes = %w[
        PossibleChannels TransmitPowerSupported WEPEncryptionLevel Channel AutoChannelEnable SSID Enable X_TP_Bandwidth TransmitPower X_TP_Band Standard X_TP_NeighbourScanEnabled
        BeaconType BasicEncryptionModes BasicAuthenticationMode WPAEncryptionModes IEEE11iEncryptionModes WPAAuthenticationMode
        IEEE11iAuthenticationMode X_TP_RadiusServerPort X_TP_RadiusServerIP X_TP_RadiusServerPassword WEPKeyIndex WPS.Enable
      ]
      @wlan_info_2G = []
      wlan_2G.each do |a|
        path = a['_path'].split('.')
        path.shift
        path.pop
        info = get_info98(path, wlan_nodes, @device_params[0])
        @wlan_info_2G.push(info)
        if info['X_TP_NeighbourScanEnabled'].is_a?(Hash) && info['X_TP_NeighbourScanEnabled'].key?('value')
          @wifi_scan_supported = true
        end
      end

      @wlan_info_5G = []
      wlan_5G.each do |a|
        path = a['_path'].split('.')
        path.shift
        path.pop
        info = get_info98(path, wlan_nodes, @device_params[0])
        @wlan_info_5G.push(info)
        if info['X_TP_NeighbourScanEnabled'].is_a?(Hash) && info['X_TP_NeighbourScanEnabled'].key?('value')
          @wifi_scan_supported = true
        end
      end
    end
  end

  def export_csv
    export
  end

  def destroy
    delete()
  end

  def process_wifi_scan(http, id, params, is_tr181)
    return if params['tab'] != 'wlan'

    begin
      obj = WifiScanEntry.new

      if params['progress'] == 'scan_start' && params['task_id'] == ''
        object = "[Device Detail][Wi-Fi Scan]#{id}"
        cpe_status, task_status = device_connected?(is_tr181, CGI.escape(id).to_s, http, 0, object)
        if cpe_status != 'reachable'
          set_wifi_scan_objects('off-line')
          render json: @scan_wifi_result
          return
        end
      end

      obj.process_wifi_scan(http, id, params, is_tr181)
      wifi_scan_params = obj.getParameters()
      unless wifi_scan_params.nil?
        @@wifi_basic_scan    = wifi_scan_params['@@wifi_basic_scan']
        @@scan_count         = wifi_scan_params['@@scan_count']
        @@last_scan_time     = wifi_scan_params['@@last_scan_time']
        @@wifi_scan_task     = wifi_scan_params['@@wifi_scan_task']
        @scan_wifi_result    = wifi_scan_params['@scan_wifi_result']
        @wifi_scan_supported = wifi_scan_params['@wifi_scan_supported']
        @wifi_scan_render_result = wifi_scan_params['render_result']
        flash = wifi_scan_params['@message']
        if wifi_scan_params['render_result']
          render json: @scan_wifi_result
          nil
        end
      end
    rescue StandardError => e
      puts "exception occurs:\n #{e}"
      puts e.backtrace
      set_wifi_scan_objects('exception')
    end
  end

  def set_wifi_scan_objects(type)
    case type
    when 'off-line'
      @@wifi_basic_scan = { status: '<Empty>', task_id: '', total: '', progress: 'start', rawdata: nil, diag_lock: '' }
      @@wifi_basic_scan[:status] = '<Empty>'
      @@wifi_basic_scan[:task_id] = ''
      @@wifi_basic_scan[:total] = ''
      @@wifi_basic_scan[:progress] = 'off-line'
      @@wifi_basic_scan[:rawdata] = nil
      @@scan_count = 0
      @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
    when 'exception'
      @@wifi_basic_scan = { status: '<Empty>', task_id: '', total: '', progress: 'start', rawdata: nil, diag_lock: '' }
      @@scan_count = 0
      @@last_scan_time = ''
      @@wifi_scan_task = { get_parametervalue: '', set_parametervalue: '', get_bssdescentry: '' }
      @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
      @wifi_scan_supported = false
    end
  end

  def get_tasks
    id = params[:id]
    http = create_api_conn()
    res = http.get("/devices/#{CGI.escape(id)}/tasks")
    tasks = JSON.parse(res.body)
    render json: tasks.to_json
  end

  def remove_faulty_task
    id = params[:id]
    http = create_api_conn()
    res = http.delete("/tasks/#{id}")
    render json: res.to_json
  end

  def diaglock
    res = { locked: 'no' }
    id = params[:id]
    controllerId = params[:controllerId]
    unless id.nil?
      resObj = get_diagnostic_lock(id, controllerId)
      diag_lock = resObj['result']
      diag_message = resObj['message']
      if diag_lock != ''
        res[:locked] = 'yes'
        res[:locked_name] = diag_lock
        res[:locked_message] = diag_message
        res[:ttl] = resObj['ttl'] unless resObj['ttl'].blank?
      end
    end
    render json: res.to_json
  end

  def unlockDiag
    res = { locked: 'no' }
    id = params[:id]
    unless id.nil?
      http = create_api_conn()
      diag_lock = destory_diagnostic_lock(id)
    end
    render json: res.to_json
  end

  # only support tab: lan, wan, wlan, wifi_scan, diag
  def get_refresh_tab_params(tab, input_root = 'Device')
    return Rails.configuration.device_info_parameters[input_root] if input_root.eql?('InternetGatewayDevice')

    ret = []
    case tab
    when 'lan', 'wan'
      ret = %w[
        Device.Ethernet Device.PPP Device.X_TP_FakeIntf Device.X_TP_L2TP Device.X_TP_PPTP
        Device.IP Device.InterfaceStack Device.DHCPv4 Device.NAT Device.DNS Device.Routing
      ]
    when 'wlan'
      ret = %w[
        Device.WiFi.Radio Device.WiFi.SSID Device.WiFi.AccessPoint Device.WiFi.X_TP_BandSteering.Enable
        Device.WiFi.NeighboringWiFiDiagnostic.DiagnosticsState
      ]
    when 'mesh_topology'
      ret = %w[
        Device.WiFi.MultiAP.APDevice Device.X_TP_RE_MAPPING Device.Hosts.Host
        Device.DeviceInfo.X_TP_DevName Device.X_TP_MESH_WIFI.WorkMode
      ]
    end
    ret.size.positive? ? ret : nil
  end

  # POST /tasks/:id/retry
  def retry_task
    can?(:update, 'devices') do
      http = create_api_conn()
      res = http.post("/tasks/#{params[:id]}/retry", nil)
      if res.code == '200'
        flash[:success] = 'Task updated'
      else
        flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      end

      redirect_to("/devices_show/#{CGI.escape(params['device_id'])}?isTr181=#{params['isTr181']}&tab=#{params['tab']}")
    end
  end

  # DELETE /tasks/:id
  def destroy_task
    can?(:delete, 'devices') do
      http = create_api_conn()
      res = http.delete("/tasks/#{params[:id]}", nil)
      if res.code == '200'
        flash[:success] = 'Task deleted'
      else
        flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      end

      redirect_to("/devices_show/#{CGI.escape(params['device_id'])}?isTr181=#{params['isTr181']}&tab=#{params['tab']}")
    end
  end

  def wifiscanbasic98(bandscan,dev_id,task_id,progress)
    http = create_api_conn()
    http.start()
    # wifi scan init ,default scan_count = 0
    if task_id == '' && progress == 'scan_start'
      @@scan_count = 0
      @@wifi_scan_task = { 'get_parametervalue': '', 'set_parametervalue': '', 'get_bssdescentry': '' }
    end
    if progress == 'scan_start' || progress == 'scan_result'
      @@scan_count += 1
    else
      @@scan_count += 5
    end
    # status : TimeOut
    if @@scan_count >= 60
      puts "timeout"
      http.finish()
      agileACSLog(4, "[Device Detail][Wi-Fi Scan]#{dev_id}", 'Wi-Fi scan timeout.')
      destorywifiscan98(dev_id, 'TimeOut', 'timeout')
      return
    end

    if task_id != '' # have task_id , query task is done?
      info = wifiscan_in_task_queue98(http,task_id,dev_id)
      if info == task_id
        @@wifi_basic_scan[:status] = 'In Progress'
        @@wifi_basic_scan[:task_id] = task_id
        @@wifi_basic_scan[:total] = ''
        @@wifi_basic_scan[:progress] = progress
        @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
        render json: @scan_wifi_result
        http.finish()
        return
      else
        @@wifi_scan_task.each { |k, v| @@wifi_scan_task[k] = '' if task_id == v }
        if progress == 'scan_start'
          progress = 'scan_refresh'
        elsif progress == 'scan_refresh'
          progress = 'scan_result'
        end
        task_id = ''
      end
    end

    if task_id == '' && progress == 'scan_start'
      # first click scan
      # ----------------------------------------------
      # refresh object X_TP_NeighbourScanEnabled
      # ----------------------------------------------
      refreshWlanScanParameter = { 'name': 'refreshObject', 'objectName': 'InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.X_TP_NeighbourScanEnabled' }
      if bandscan == '2.4GH'
        refreshWlanScanParameter[:objectName] = 'InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.X_TP_NeighbourScanEnabled'
      elsif bandscan == "5GHz"
        refreshWlanScanParameter[:objectName] = 'InternetGatewayDevice.LANDevice.1.WLANConfiguration.2.X_TP_NeighbourScanEnabled'
      end
      res_refresh = http.post("/devices/#{CGI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(refreshWlanScanParameter), 'Content-type' => 'application/json')
      res_refresh_task = JSON.parse(res_refresh.body)
      @@wifi_scan_task[:get_parametervalue] = res_refresh_task['_id']
      if res_refresh.code == '200' || res_refresh.code == '202' #keep 200 code , if else ...
        if res_refresh.message == 'Device is unreachable'
          flash[:warning] = res_refresh.message
          #status : <Empty> , progress : off-line
          http.finish()
          destorywifiscan98(dev_id, '<Empty>', 'off-line')
          return
        end
      else
        flash[:error] = "Unexpected error (#{res_refresh.code}): #{res_refresh.body}"
        http.finish()
        destorywifiscan98(dev_id, '<Empty>', 'off-line')
        return
      end
      # ----------------------------------------------
      # setParameterValues  X_TP_NeighbourScanEnabled = True ,start scan
      # ----------------------------------------------
      setWlanScanParameterEnable = { 'name': 'setParameterValues', 'parameterValues': '[[InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.X_TP_NeighbourScanEnabled,true]]' }
      if res_refresh.code == '200' || res_refresh.code == '202' # 202 : off-line or queue ,this case is task in queue
        if bandscan == '2.4GHz'
          setWlanScanParameterEnable[:parameterValues] = ActiveSupport::JSON.decode('[["InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.X_TP_NeighbourScanEnabled",true]]')
        elsif bandscan == '5GHz'
          setWlanScanParameterEnable[:parameterValues] = ActiveSupport::JSON.decode('[["InternetGatewayDevice.LANDevice.1.WLANConfiguration.2.X_TP_NeighbourScanEnabled",true]]')
        end
        starttime = Time.new
        res_scan = http.post("/devices/#{CGI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(setWlanScanParameterEnable), 'Content-type' => 'application/json')
        res_scan_task = JSON.parse(res_scan.body)
        endtime = Time.new
        @@scan_count += (endtime - starttime).to_i
        @@wifi_scan_task[:set_parametervalue] = res_scan_task['_id']
        if res_scan.code == '200' || res_scan.code == '202'
          if res_scan.message == 'Device is unreachable'
            flash[:warning] = res_scan.message
            #status : <Empty> , progress : off-line
            http.finish()
            destorywifiscan98(dev_id, '<Empty>', 'off-line')
            return
          end
          agileACSLog(6, "[Device Detail][Wi-Fi Scan]#{dev_id}", 'Start Wi-Fi scanning.')
        else
          flash[:error] = "Unexpected error (#{res_scan.code}): #{res_scan.body}"
          http.finish()
          destorywifiscan98(dev_id, '<Empty>', 'off-line')
          return
        end
        @@wifi_basic_scan[:status] = 'In Progress'
        @@wifi_basic_scan[:task_id] = res_scan_task['_id']
        @@wifi_basic_scan[:total] = ''
        @@wifi_basic_scan[:progress] = 'scan_start'
        @@wifi_basic_scan[:rawdata] = nil
        @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
        render json: @scan_wifi_result
      end
    elsif task_id == '' && progress == 'scan_refresh'
      info = wifiscanresult98(bandscan,dev_id) # get db last update time
      @@last_scan_time = info['_timestamp']
      refreshWlanScanResult = {:name => "refreshObject", :objectName => "InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.X_TP_BSSDescEntry" }
      if bandscan == "2.4GH"
        refreshWlanScanResult[:objectName] = "InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.X_TP_BSSDescEntry"
      elsif bandscan == "5GHz"
        refreshWlanScanResult[:objectName] = "InternetGatewayDevice.LANDevice.1.WLANConfiguration.2.X_TP_BSSDescEntry"
      end
      res = http.post("/devices/#{CGI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(refreshWlanScanResult), 'Content-type' => 'application/json')
      res_task = JSON.parse(res.body)
      @@wifi_scan_task[:get_bssdescentry] = res_task['_id']
      if res.code == '200' || res.code == '202'
        if res.message == "Device is unreachable"
          flash[:warning] = res.message
          #status : <Empty> , progress : off-line
          http.finish()
          destorywifiscan98(dev_id, "<Empty>", "off-line")
          return
        end
        agileACSLog(6, "[Device Detail][Wi-Fi Scan]#{dev_id}", "Get Wi-Fi scan results.")
      else
        flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        http.finish()
        destorywifiscan98(dev_id, "<Empty>", "off-line")
        return
      end
      @@wifi_basic_scan[:status] = "In Progress"
      @@wifi_basic_scan[:task_id] = res_task['_id']
      @@wifi_basic_scan[:total] = ""
      @@wifi_basic_scan[:progress] = "scan_refresh"
      @@wifi_basic_scan[:rawdata] = nil
      @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
      render json: @scan_wifi_result

    elsif task_id == "" && progress == "scan_result"
      # is DB redy and get result , if not send "scan_result"
      info = wifiscanresult98(bandscan,dev_id)
      getlastscantime = info['_timestamp']
      if @@last_scan_time != getlastscantime && info['_scanresult'][0][:bssid] != ""
        @@wifi_basic_scan[:status] = "Completed Successfully"
        @@wifi_basic_scan[:task_id] = ""
        @@wifi_basic_scan[:total] = info['_wificount'].to_s
        @@wifi_basic_scan[:progress] = "scan_final"
        @@wifi_basic_scan[:rawdata] = info['_scanresult']
        @@wifi_basic_scan[:pchannels] = @possible_channels

        if get_diagnostic_lock(dev_id)['result'] == "WifiScan"
          destory_diagnostic_lock(dev_id)
        end
        agileACSLog(6, "[Device Detail][Wi-Fi Scan]#{dev_id}", "Wi-Fi scan completed.")
        @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
        render json: @scan_wifi_result
      else
        @@wifi_basic_scan[:status] = "In Progress"
        @@wifi_basic_scan[:task_id] = @@wifi_scan_task[:get_bssdescentry]
        @@wifi_basic_scan[:total] = ""
        @@wifi_basic_scan[:progress] = "scan_result"
        @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
        render json: @scan_wifi_result
      end
    end
    http.finish()
  end

  def wifiscan_in_task_queue98(http,task_id,dev_id)
    info = ''
    tasks = query_resource(http, 'tasks', { 'device' => dev_id }, nil, nil, nil, { 'timestamp' => 1 })[:result]
    info = task_id if tasks.to_s.include?(task_id)
    info
  end

  def wifiscanresult98(band,dev_id)
    can?(:read, 'devices') do
      create_api_conn() do |http|
        res = query_resource(http, 'devices', { '_id' => dev_id })
        raise ActiveRecord::RecordNotFound if res[:result].size != 1
        scanentrycheck = res[:result][0]
        device_scan_params = flatten_params(scanentrycheck)
        includscanentry = '1.X_TP_BSSDescEntry'
        includscanentry = '2.X_TP_BSSDescEntry' if band == '5GHz'
        possible_channels_entry = 'PossibleChannels'
        tmp = { '_timestamp' => '' }
        wificounts = 0
        scanresult_json = []
        wifi_entry = { 'id': '', 'ssid': '', 'bssid': '', 'rssi': '', 'channel': '', 'security': '', 'encryption': '' }
        reset_possible_channels()
        for v in device_scan_params
          if v['_path'].include?(includscanentry)
            str_arr = v['_path'].split('.')
            if str_arr.last().gsub(/[^0-9]/, '') != '' && wificounts != 0
              wifi_entry[:id] = wificounts
              scanresult_json.push(wifi_entry)
              wifi_entry = { 'id': '', 'ssid': '', 'bssid': '', 'rssi': '', 'channel': '', 'security': '', 'encryption': '', 'X_TP_ExtensionChannel': '' }
            end
            case str_arr.last()
            when 'X_TP_BSSDescEntry'
              tmp['_timestamp'] = v['_timestamp']
            when 'SSID'
              wificounts += 1
              wifi_entry[:ssid] = v['_value'].to_s
            when 'BSSID'
              wifi_entry[:bssid] = v['_value'].to_s
            when 'RSSI'
              wifi_entry[:rssi] = v['_value'].to_s
            when 'Channel'
              wifi_entry[:channel] = v['_value'].to_s
            when 'AuthMode'
              wifi_entry[:security] = v['_value'].to_s
            when 'WepStatus'
              wifi_entry[:encryption] = v['_value'].to_s
            when 'OperatingChannelBandwidth'
              wifi_entry[:bandwidth] = v['_value'].to_s
            when 'X_TP_ExtensionChannel'
              wifi_entry[:X_TP_ExtensionChannel] = v['_value'].to_s
            end
          end

          if v['_path'].include?(possible_channels_entry)
            set_possible_channels(v['_value'])
          end
        end
        tmp['_wificount'] = wificounts
        wifi_entry[:id] = wificounts
        scanresult_json.push(wifi_entry)
        tmp['_scanresult'] = scanresult_json
        return tmp
      end
    end
  end

  def destorywifiscan98(dev_id, status,progress)
    http = create_api_conn()
    http.start()
    @@wifi_scan_task.each { |_, value| http.delete("/tasks/#{value}") if value != '' }
    destory_diagnostic_lock(dev_id) if get_diagnostic_lock(dev_id)['result'] == 'WifiScan'
    @@wifi_basic_scan[:status] = status
    @@wifi_basic_scan[:task_id] = ''
    @@wifi_basic_scan[:total] = ''
    @@wifi_basic_scan[:progress] = progress
    @@wifi_basic_scan[:rawdata] = nil
    http.finish()
    @@scan_count = 0
    @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
    render json: @scan_wifi_result
  end

  def flatten_params(params, prefix = '')
    output = []
    params.each do |n, v|
      next if ( n.start_with?('_') && !n.start_with?('__')) || v.is_a?(String)

      v['_path'] = "#{prefix}#{n}"
      output << v

      unless v.include?('_value')
        output += flatten_params(v, prefix ? "#{prefix}#{n}." : "#{n}.")
      end
    end
    output
  end
end
