class FaultsController < ApplicationController
  require 'net/http'
  require 'json'
  include Util
  include Profiles

  helper_method :update_profile_name, :update_fault_channel
  # GET /faults
  # GET /faults.json
  def index
    @application_list = profile_application_name()
    can?(:read, 'profiles') do
      skip = params.include?(:page) ? (Integer(params[:page]) - 1) * Rails.configuration.page_size : nil
      @query = if params.key?('query')
                 ActiveSupport::JSON.decode(URI.unescape(params['query']))
               else
                 {}
               end
      @filter = params['filter'] if params.key?('filter')
      res = query_resource(create_api_conn(), 'faults', @query, nil, skip, Rails.configuration.page_size)
      @faults = res[:result]
      @total = res[:total]
      @faults.each_with_index do |fault, _idx|
        fault_query = {}
        fault_params = {}
        fault_query = { '_id' => fault['device'] }
        fault_params['query'] = ActiveSupport::JSON.encode(fault_query)
        aggregate_pipeline = generate_aggregate_pipeline(fault_params, false, true, true, Rails.configuration.profiles_index_parameters, Rails.configuration.profiles_summary_parameters)
        http = create_api_conn()
        res = aggregate(http, 'devices', aggregate_pipeline)
        fault['isTr181'] = if res[:result].size != 1
                             false
                           else
                             res[:result][0]['isTr181']
                           end
      end
      respond_to do |format|
        format.html # index.html.erb
        format.json { render json: @faults }
      end
    end
  end

  # DELETE /faults/1
  # DELETE /faults/1.json
  def destroy
    can?(:delete, 'profiles') do
      http = create_api_conn()
      res = http.delete("/faults/#{URI.escape(params[:id])}", nil)
      if res.code == '200'
        flash[:success] = 'Fault deleted'
      else
        flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      end

      redirect_back(fallback_location: root_path)
    end
  end

  # POST /tasks/:id/retry
  def retry_task
    can?(:update, 'profiles') do
      http = create_api_conn()
      res = http.post("/tasks/#{params[:id]}/retry", nil)
      if res.code == '200'
        flash[:success] = 'Task updated'
      else
        flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      end

      redirect_back(fallback_location: root_path)
    end
  end

  # DELETE /tasks/1
  # DELETE /tasks/1.json
  def destroy_task
    can?(:delete, 'profiles') do
      http = create_api_conn()
      res = http.delete("/tasks/#{params[:id]}", nil)
      if res.code == '200'
        flash[:success] = 'Faulty task deleted'
      else
        flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      end

      redirect_back(fallback_location: root_path)
    end
  end
end
