class FilesController < ApplicationController
  require 'net/http'
  require 'json'

  include Util
  include Loggers
  include Models
  include DeviceFilters
  include Files

  # GET /files
  # GET /files.json
  def fw_file_check ()
    retMsg = "OK"
	  filepath = params[:file].path
    if File.exists?(filepath)
      task = {}
      task['action'] = "fwfilecheck"
      task['data'] = filepath
      http = create_api_conn()
      res = http.post("/files/acsapps",ActiveSupport::JSON.encode(task),'Content-type' => 'application/json')
      if res.code != '200'
        retMsg = res.body
      end
    end
    return retMsg
  end

  def index
    can?(:read, 'files') do
      @modelsAll = get_models()
      skip = params.include?(:page) ? (Integer(params[:page]) - 1) * Rails.configuration.page_size : nil

      if params.include?(:sort)
        sort_param = params[:sort].start_with?('-') ? params[:sort][1..-1] : params[:sort]
        sort_dir = params[:sort].start_with?('-') ? -1 : 1
        sort = {sort_param => sort_dir}
      else
        sort = nil
      end
      @query = {}
      
      if params.has_key?('query')
        @query = ActiveSupport::JSON.decode(URI.unescape(params['query']))
      end
      q = case_insensitive_transform(@query.dup)
      res = query_resource(create_api_conn(), 'files', q, nil, skip, Rails.configuration.page_size, sort)
      @files = res[:result]
      @total = res[:total]

      respond_to do |format|
        format.html # index.html.erb
        format.json { render json: @files }
      end
    end
  end

  # GET /files/new
  # GET /files/new.json
  def new
    can?(:create, 'files') do
      @modelsAll = get_models()
      @file = {}

      respond_to do |format|
        format.html # new.html.erb
        format.json { render json: @file }
      end
    end
  end

  # POST /files
  def upload
    object = "[Files Upload]"
    retMsg = "OK"
    can?(:create, 'files') do
      logger.debug("MODEL_TYPE: #{params[:model_type]}")
      filename = params[:id] = params[:file].original_filename

      if is_used_by_profiles(filename)
        flash[:error] = "Sorry, the #{filename} is already used in the profile, please change another filename."
      else	
        if retMsg == 'OK' || retMsg == "404 Not Found"
          http = create_api_conn()
          req = Net::HTTP::Put.new("/files/#{URI.escape(params[:file].original_filename)}", 'Content-type' => 'application/octet-stream')
          req.body = params[:file].read
          req['fileType'] = params[:file_type].strip
          req['model'] = params[:model].strip
          req['version'] = params[:version].strip
          res = http.request(req)
          if res.code == '201'
            flash[:success] = 'File saved'
            agileACSLog(6, object, "#{URI.escape(params[:file].original_filename)} upload succeed.")
          else
            flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
            agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
          end
        else
          flash[:error] = "#{retMsg}"
          agileACSLog(3, object, "#{retMsg}")
        end
      end
	  end
    redirect_to :action => :index
  end

  # DELETE /files/1
  # DELETE /files/1.json
  def destroy
    object = "[Files Destroy]"
    can?(:delete, 'files') do
      filename = params[:id]
      if is_used_by_profiles(filename)
        flash[:error] = "Sorry, the #{filename} is already used in the profile, please remove the related profiles in advance."
      else
        http = create_api_conn()
        res = http.delete("/files/#{URI.escape(filename)}", nil)
        if res.code == '200'
          flash[:success] = 'File deleted'
                agileACSLog(6, object, "#{filename} delete succeed.")
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
                agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
        end
      end

      redirect_to :action => :index
    end
  end
end
