# ForgetPasswordController
class ForgetPasswordController < ApplicationController
  include Util
  include Loggers

  def index
    @ttl = get_ttl('admin')
    return unless request.url.match(/.text$/)

    index_text('admin')
    nil
  end

  def update
    object = '[Reset Password]'
    vcode = params[:verification_code]
    http = create_api_conn()
    verification = { 'username' => 'admin', 'vcode' => vcode } # now username fixed to admin
    res = http.post('/forget_password/verify', ActiveSupport::JSON.encode(verification), 'Content-Type' => 'application/json')
    if res.code == '200'
      user = User.find_by username: verification['username']
      unless user.update(password: verification['username'])
        flash[:error] = user.errors.full_messages.join('; ')
        agileACSLog(3, object, "Reset password failed. #{user.errors.full_messages.join('; ')}")
        redirect_to controller: 'forget_password', action: 'index'
        return
      end
    end
    update_response(object, res)
  end

  def update_response(object, res)
    if res.code == '200'
      flash[:success] = 'Reset Password Success'
      agileACSLog(6, object, 'Reset Password Success.')
      redirect_to log_in_path
    elsif res.code == '202'
      flash[:warning] = res.body
      agileACSLog(4, object, res.body.to_s)
      redirect_to controller: 'forget_password', action: 'index'
    else
      flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
      redirect_to controller: 'forget_password', action: 'index'
    end
  end

  def index_text(username)
    object = '[Export Verification Info]'
    verification = { 'action' => 'generate', 'data' => username }
    http = create_api_conn()
    res = http.post('/forget_password/verification', ActiveSupport::JSON.encode(verification), 'Content-Type' => 'application/json')
    if res.code == '200'
      agileACSLog(6, object, 'Export verification info succeed.')
      path = '/tmp/verification_info'
      send_file path unless path.blank?
    elsif res.code == '202'
      flash[:warning] = res.body
      agileACSLog(4, object, res.body.to_s)
      redirect_to controller: 'forget_password', action: 'index'
    else
      flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
      redirect_to controller: 'forget_password', action: 'index'
    end
  end

  def get_ttl(username)
    http = create_api_conn()
    verification = { 'username' => username }
    res = http.post('/forget_password/ttl', ActiveSupport::JSON.encode(verification), 'Content-Type' => 'application/json')
    return -1 unless res.code == '200'

    ttl = JSON.parse(res.body)['ttl']
    ttl
  end
end
