# ---------------------------------------------------------------------------------------------------------
# class HomeController
# History:
#     20200706 bill.chen  move from module/basic/genieacs-gui/app/controllers/home_controller.rb,
#                         and revise to rubocop document format
# ---------------------------------------------------------------------------------------------------------
class HomeController < ApplicationController
  require 'json'
  require 'net/http'
  include Util
  include Models
  include DeviceFilters

  def index
    can?(:read, 'home') do
      user_filter = 0
      @query = {}
      @models_all = get_models()
      @filters = get_filters(1, Rails.configuration.home_index_parameters)

      if params.key?('query')
        user_filter = 1
        @query = ActiveSupport::JSON.decode(URI.decode_www_form_component(params['query']))
      end
      @graphs = fetch_cache_graphs(user_filter)
    end
  end

  def cpu
      http = create_api_conn()
      res = http.post('/home/usage/cpu', nil)
      if res.body == '404 Not Found'
        data = { 'usage': '0%', 'used': 0, 'available': 0 }
      else
        data = JSON.parse(res.body)
        data['usage'] = data['usage'].gsub(/[^0-9\.]/, '')
      end
      render json: data
  end

  def memory
      http = create_api_conn()
      res = http.post('/home/usage/memory', nil)
      if res.body == '404 Not Found'
        data = { 'usage': '0%', 'used': 0, 'available': 0 }
      else
        data = JSON.parse(res.body)
        data['usage'] = data['usage'].gsub(/[^0-9\.]/, '')
      end
      render json: data
  end

  private

  def fetch_cache_graphs(user_filter)
    Rails.cache.fetch('graphs', expires_in: 60.seconds) do
      threads = []
      graphs = ActiveSupport::JSON.decode(ERB.new(File.read(Rails.root.join('config/graphs.json.erb'))).result)
      graphs.each do |_group_name, group_graphs|
        group_graphs.each do |_graph_name, graph|
          graph.each do |slice|
            threads << Thread.new(slice) do |t_slice|
              update_graph_slice(user_filter, t_slice)
            end
          end
        end
      end
      threads.each(&:join)
      next graphs
    end
  end

  def update_graph_slice(user_filter, slice)
    http = create_api_conn()
    http.start()
    slice['href'] = if user_filter == 1
                      url_for(controller: 'devices', action: 'index', params: { query: ActiveSupport::JSON.encode(@query.merge(slice['query'])) }, only_path: true)
                    else
                      url_for(controller: 'devices', action: 'index', params: { query: ActiveSupport::JSON.encode(slice['query']) }, only_path: true)
                    end

    mq = { 'query' => @query.merge(slice['query']).to_json }
    aggregate_pipeline = generate_aggregate_pipeline(mq, true, true, false, Rails.configuration.home_index_parameters)
    res = aggregate(http, 'devices', aggregate_pipeline, 'HEAD')
    # puts "[#{slice['label']}]: aggregate_pipeline=#{aggregate_pipeline}"
    # puts "[#{slice['label']}]: res=#{res[:total]}"
    slice['data'] = res[:total].to_i
    slice.delete('query')
    http.finish()
  end
end
