class LicenseController < ApplicationController
  require 'csv'
  include Util
  include ActionController::Live
  include Loggers

  # GET /license
  # GET /license.json

    def index
      can?(:read, 'license') do
        if request.url.match(/.text$/)
          can?(:create, 'license') do
            index_text()
            return
          end
        end
        skip = params.include?(:page) ? (Integer(params[:page]) - 1) * Rails.configuration.page_size : nil
        res = query_resource(create_api_conn(), 'license', nil, nil, skip, Rails.configuration.page_size)
        @licenses = res[:result]
        @total = res[:total]
            respond_to do |format|
                format.html # index.html.erb
                format.json { render json: @licenses }
            end
      end
    end

    def export_csv
      can?(:update, 'license') do
          response.headers['Content-Type'] = 'text/csv'
          response.headers['Content-Disposition'] = "attachment; filename=\"total_license.csv\""
          res = query_resource(create_api_conn(), 'license', nil, nil, nil, nil)
          total_license = res[:result]
          
          total_license_text = ["Last Activated Date", "Feature Set", "Expired Date", "Expired in Days", "Status"]
          response.stream.write(total_license_text.to_csv())
          total_license.each do |f|    
            if f['_id'].include? "feature set license"
              total_license_text[0]=f['activated_date'].to_time(:utc).localtime.strftime("%Y-%m-%d %H:%M:%S")
              total_license_text[1]=TpConfig::get_vers_rename(f['feature_set'])
              if f['expired_timestamp'] == -1
                total_license_text[2]="infinite"
              else
                total_license_text[2]=Time.at(f['expired_timestamp']).localtime.strftime("%Y-%m-%d %H:%M:%S")
              end
              if f['expireddays'] == -1
                total_license_text[3]="infinite"
              else
                total_license_text[3]=f['expireddays']
              end
              total_license_text[4]=f['status']
              response.stream.write(total_license_text.to_csv)  
            end
          end  

          total_license_text = []
          response.stream.write(total_license_text.to_csv)

          total_license_text = ["Last Activated Date", "Control Number", "Avaliable Control Number","Expired Date", "Expired in Days", "Status"]
          response.stream.write(total_license_text.to_csv())
          total_license.each do |f|    
            if (f['_id'].include? "quantity license") || (f['_id'].include? "trial")
              total_license_text[0]=f['activated_date'].to_time(:utc).localtime.strftime("%Y-%m-%d %H:%M:%S")
              total_license_text[1]=f['controlnum']
              total_license_text[2]=f['available_controlnum']
              if f['expired_timestamp'] == -1
                if f['_id'] == "trial"
                  total_license_text[3]="infinite (Free-Trial)"
                else
                  total_license_text[3]="infinite"
                end
              else
                total_license_text[3]=Time.at(f['expired_timestamp']).localtime.strftime("%Y-%m-%d %H:%M:%S")
              end
              if f['expireddays'] == -1
                if f['_id'] == "trial"
                  total_license_text[4]="infinite (Free-Trial)"
                else
                  total_license_text[4]="infinite"
                end
              else
                total_license_text[4]=f['expireddays']
              end
              total_license_text[5]=f['status']
              response.stream.write(total_license_text.to_csv)  
            end
          end 

          total_license_text = []
          response.stream.write(total_license_text.to_csv)

          total_license_text = ["Last Activated Date", "Expired Date", "Expired in Days", "Status"]
          response.stream.write(total_license_text.to_csv())
          total_license.each do |f|    
            if f['_id'].include? "maintenance license"
              total_license_text[0]=f['activated_date'].to_time(:utc).localtime.strftime("%Y-%m-%d %H:%M:%S")
              total_license_text[1]=Time.at(f['expired_timestamp']).localtime.strftime("%Y-%m-%d %H:%M:%S")
              total_license_text[2]=f['expireddays']
              total_license_text[3]=f['status']
              response.stream.write(total_license_text.to_csv)  
            end
          end 
      end
      ensure
      response.stream.close

      #response.headers['Content-Type'] = 'text/csv'
      #response.headers['Content-Disposition'] = "attachment; filename=\"license.csv\""
      #res = query_resource(create_api_conn(), 'license', nil, nil, nil, nil)
      #license = res[:result]
      #license_text = ["ID","Name","Create Date","System Info", "System Info", "System Info","System Info", "System Info", "Activated date", "Invalid"]
      #response.stream.write(license_text.to_csv())
      #
      #license.each do |f|    
      #    license_text[0]=f['_id']
      #    license_text[1]=f['name']
      #    license_text[2]=f['create_date']
      #    license_text[3]=f['hdmodelid']
      #    license_text[4]=f['hdserial']
      #    license_text[5]=f['cpuid']
      #    license_text[6]=f['mac_str']
      #    license_text[7]=f['initkey']
      #    license_text[8]=f['activated_date']
      #    license_text[9]=f['disable']
      #    response.stream.write(license_text.to_csv)  
      #end 
      #
      #ensure
      #response.stream.close
  end
 
    def index_text
      can?(:update, 'license') do
          object = "[Export Server Info]"
          http = create_api_conn() 
          res = http.put("/license/acsapps", nil)
          if res.code == '200'
            agileACSLog(6, object, "Export server info succeed.")
            path = res.body
            unless path.blank?
              send_file path
            end
          elsif res.code == '202'
            flash[:warning] = res.body
            agileACSLog(4, object, "#{res.body}")
            redirect_to license_path
          else
            flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
            agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
            redirect_to license_path
          end
      end
    end
    # POST /license
    # POST /license.json
    def update    
      can?(:update, 'license') do
        object = "[Import License File]"
        if !params[ :license ].nil?
          filepath = "/tmp/agileacs_license"
          File.open("#{filepath}", "wb") do |f|
            f.write(params[:license].read)
          end
          if File.exists?(filepath)
            license = {}
            license['action'] = "update"
            license['data'] = filepath
            http = create_api_conn()
            res = http.post("/license/acsapps",ActiveSupport::JSON.encode(license),'Content-type' => 'application/json')
            if res.code == '200'
              result = {"msg" => "success","code" => res.code}
              agileACSLog(6, object, "Import license file succeed.")
            elsif res.code == '202'
              result = {"msg" => res.body,"code" => res.code}
              agileACSLog(4, object, "#{res.body}")
            else
              result = {"msg" => "Unexpected error (#{res.code}): #{res.body}","code" => res.code}
              agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
            end
          else
            result = {"msg" => "Unexpected error"}
            flash[:error] = "Unexpected error"
          end
          render :json => result
        end
      end
    end
	
	def destroy
		#can?(:delete, 'license') do //not used currently
      object = "[Destroy License]"
			if !params[ :id ].nil?
			  license = {}
			  license['action'] = "delete"
			  license['data'] = params[ :id ]
			  http = create_api_conn()
			  res = http.post("/license/acsapps",ActiveSupport::JSON.encode(license),'Content-type' => 'application/json')
			  if res.code == '200'
				flash[:success] = 'License deleted'
                agileACSLog(6, object, "Destroy license #{license['data']} succeed.")
			  elsif res.code == '202'
				flash[:warning] = res.body
                agileACSLog(4, object, "#{res.body}")
			  else
				flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
                agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
			  end
			end
		#end
  end

  def get_upgrade_status
    http = create_api_conn()
    data = { 'action' => 'upgrade_lock', 'data' => 'status'}
    res = http.post("/license/acsapps", ActiveSupport::JSON.encode(data), 'Content-type' => 'application/json')
    if res.code == '202'
      'none'
    elsif res.code == '200'
      res.body
    end
  end


end
