# Profiles Controller
class ProfilesController < ApplicationController
  require 'net/http'
  require 'json'
  PPPOE_DIR = '/opt/agileacs/pppoesetting'.freeze
  include Util
  include Models
  include Profiles
  include DeviceFilters
  include Match
  include Loggers

  APPS_LITE = InstanceProfileApps::APPS_LITE
  APPS_PRO = InstanceProfileApps::APPS_PRO
  FWSCHEDULE_APPENDIX = InstanceProfileApps::APPENDIX[0]

  helper_method :update_profile_name

  def pppoe_format_checking(file_name)
    aFile = File.new(file_name, 'r')
    isError = 0
    retMsg = 'OK'

    if aFile
      File.foreach(file_name).with_index do |line, line_num|
        # MAC Address,Username,Password,WirelessKey,PPPoE Username,PPPoE Password
        # AABBCC000000,,,,name000000,pwd000000
        if line_num >= 1
          if line !~ /^[0-9A-F]{12},\w*,\w*,\w*,.{1,64},.{1,64}(\r\n|\n)$/ || (line.split(/,/).length() != 6)
            retMsg = "Error: format is incorrect on line: #{line_num + 1} of uploaded file!"
            isError = 1
          end
        end

        break if isError == 1
      end
    else
      retMsg = "Error: Unable to open file [#{file_name}]!"
    end

    retMsg
  end

  def get_progress_filter_time(filter)
    filter_time = ''
    case filter
    when '24 hours'
      filter_time = (Time.now - 1.day.to_i).to_time.localtime.strftime('%F %T')
    when '3 days'
      filter_time = (Time.now - 3.days.to_i).to_time.localtime.strftime('%F %T')
    when '1 week'
      filter_time = (Time.now - 1.week.to_i).to_time.localtime.strftime('%F %T')
    when '1 month'
      filter_time = (Time.now - 1.month.to_i).to_time.localtime.strftime('%F %T')
    end
    filter_time
  end

  def index
    can?(:read, 'profiles') do
      @apps = if !TpConfig.get_config('features-enable')
                APPS_LITE
              else
                APPS_LITE + APPS_PRO
              end
      @apps = @apps.sort()

      @FWSCHEDULE_APPENDIX = FWSCHEDULE_APPENDIX
      @modelsAll = get_models()
      skip = params.include?(:page) ? (Integer(params[:page]) - 1) * Rails.configuration.page_size : nil

      if params.include?(:sort)
        sort_param = params[:sort].start_with?('-') ? params[:sort][1..-1] : params[:sort]
        sort_dir = params[:sort].start_with?('-') ? -1 : 1
        sort = { sort_param => sort_dir }
      else
        sort = nil
      end

      @query = {}
      queryCombined = {}
      if params.key?('query')
        @query = ActiveSupport::JSON.decode(URI.unescape(params['query']))
        q = case_insensitive_transform(@query.dup)
        if TpConfig.get_config('features-enable')
          queryFWSchedule = genFWScheduleFilters(q)
          if queryFWSchedule != {}
            t = queryFWSchedule.to_s
            queryCombined = if t.include?('$ne')
                              { '$and' => [q, queryFWSchedule] }
                            else
                              { '$or' => [q, queryFWSchedule] }
                            end
          else
            queryCombined = q
          end
        else
          queryCombined = q
        end
      end

      if !TpConfig.get_config('features-enable')
        query_lite = []
        APPS_LITE.each do |app|
          query_lite.push({ "app": app })
        end
        query_lite = { "$or": query_lite }
        res = query_resource(create_api_conn(), 'profiles',  queryCombined.merge(query_lite), nil, skip, Rails.configuration.page_size, sort)
      else
        res = query_resource(create_api_conn(), 'profiles',  queryCombined, nil, skip, Rails.configuration.page_size, sort)
      end

      # Make sure variable @query is  same as user input if params has key 'query'.
      @query = ActiveSupport::JSON.decode(URI.unescape(params['query'])) if params.key?('query')

      @profiles = res[:result]
      @total = res[:total]
      calculate_profiles_progresses(@profiles)
    end
    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: profiles }
    end
  end

  # GET /profiles/new
  # GET /profiles/new.json
  def new
    can?(:create, 'profiles') do
      @apps = if !TpConfig.get_config('features-enable')
                APPS_LITE
              else
                APPS_LITE + APPS_PRO
              end
      @apps = @apps.sort()

      @modelsAll = get_models()
      @query = if params.key?('query')
                 ActiveSupport::JSON.decode(URI.unescape(params['query']))
               else
                 {}
               end
      @profile = {}
      @profile['action_type'] = 'new'
      @query_type = 1
      @match = get_match @query_type

      res = query_resource(create_api_conn(), 'profiles/files', nil, nil, nil, nil)
      @filename = []
      @configname = []
      @pppoe_file = []

      res[:result].each do |item|
        tmp = { 'model' => item['metadata']['model'],
                'filename' => item['filename'],
                'version' => item['metadata']['version'] }
        if item['metadata']['fileType'].match('Configuration')
          @configname.push(tmp)
        elsif item['metadata']['fileType'].match('PPPoE')
          @pppoe_file.push(tmp)
        else
          @filename.push(tmp)
        end
      end
      respond_to do |format|
        format.html # new.html.erb
        format.json { render json: @profile }
      end
    end
  end

  # GET /profiles/1/edit
  def edit
    can?(:update, 'profiles') do
      @FWSCHEDULE_APPENDIX = FWSCHEDULE_APPENDIX
      @modelsAll = get_models()
      id = params[:id]
      res = query_resource(create_api_conn(), 'profiles', { '_id' => id })
      @profile = res[:result][0] || {}
      @profile['action_type'] = 'edit'
      res = query_resource(create_api_conn(), 'profiles/files', nil, nil, nil, nil)
      @filename = []
      @configname = []

      @query_type = @profile.key?('query_1') ? 1 : 0
      @match = get_match(@query_type)

      res[:result].each do |item|
        tmp = { 'model' => item['metadata']['model'],
                'filename' => item['filename'],
                'version' => item['metadata']['version'] }
        if item['metadata']['fileType'].match('Configuration')
          @configname.push(tmp)
        else
          @filename.push(tmp)
        end
      end
    end
  end

  # POST /profiles
  def update
    object = '[Profile Save]'
    can?(:update, 'profiles') do
      retMsg = 'OK'
      logger.debug("MODEL_TYPE: #{params[:model_type]}")
      if params[:app].match('PPPoE')
        if params[:file]
          tmpname = params[:file].path
          filename = params[:file].original_filename
          profilename = params[:name]
          params[:pppoe_file] = params[:file].original_filename

          FileUtils.mkdir_p(PPPOE_DIR) unless File.directory?(PPPOE_DIR)
          File.rename(tmpname, "#{PPPOE_DIR}/#{profilename}_#{filename}")

          retMsg = pppoe_format_checking "#{PPPOE_DIR}/#{profilename}_#{filename}"
        end
    end

      params['name'] = update_profile_name(params['name'], params['app'], true)

      if params[:app].match('Firmware Upgrade') || params[:app].match('Firmware Upgrade with Schedule') || params[:app].match('Configure File Download')
        if existed_model_profiles(params[:name], params[:app], params[:model])
          retMsg = "#{params[:app]} Profile for #{params[:model]} already existed. Please edit existed profile or destory existed profile then be able to add a new profile."
          agileACSLog(3, object, "#{params[:app]} Profile for #{params[:model]} already existed. Please edit existed profile or destory existed profile then be able to add a new profile.")
        end
      end

      if retMsg == 'OK' && checkActionType(params)
        profile = construct_profile(params)
        http = create_api_conn()
        res = http.put("/profiles/#{URI.escape(params['name'].strip)}", ActiveSupport::JSON.encode(profile), 'Content-type' => 'application/json')
        if res.code == '200'
          res = update_presets(params)
          if res.code != '200'
            flash[:error] = "Unexpected error when update presets (#{res.code}): #{res.body}"
            agileACSLog(3, object, "Unexpected error when update presets (#{res.code}): #{res.body}")
          end
          flash[:success] = 'Profiles saved'
          case params['app']
          when 'Firmware Upgrade with Schedule'
            week_arr = ['Sun.', 'Mon.', 'Tue.', 'Wed.', 'Thu.', 'Fri.', 'Sat.']
            repeat_weeks = []
            case params['action_type']
            when 'new'
              case params['triggering_conditions_repeat']
              when 'No repeat'
                agileACSLog(6, object, "Schedule #{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile save succeed, from #{params['no_repeat_start_date']} to #{params['no_repeat_end_date']}.")
              when 'Everyday'
                agileACSLog(6, object, "Schedule #{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile save succeed, from #{params['repeat_start_days']} to #{params['repeat_until_days']}, everyday #{params['repeat_timerange']}.")
              when 'Weekly'
                params['repeat_weeks'].each do |week|
                  repeat_weeks.push(week_arr[week.to_i])
                end
                agileACSLog(6, object, "Schedule #{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile save succeed, from #{params['repeat_start_days']} to #{params['repeat_until_days']}, every #{repeat_weeks.join(' ')}, #{params['repeat_timerange']}.")
              else
                agileACSLog(6, object, "#{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile save succeed.")
              end
            when 'edit'
              case params['triggering_conditions_repeat']
              when 'No repeat'
                agileACSLog(6, object, "Schedule #{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile edit succeed, from #{params['no_repeat_start_date']} to #{params['no_repeat_end_date']}.")
              when 'Everyday'
                agileACSLog(6, object, "Schedule #{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile edit succeed, from #{params['repeat_start_days']} to #{params['repeat_until_days']}, everyday #{params['repeat_timerange']}.")
              when 'Weekly'
                params['repeat_weeks'].each do |week|
                  repeat_weeks.push(week_arr[week.to_i])
                end
                agileACSLog(6, object, "Schedule #{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile edit succeed, from #{params['repeat_start_days']} to #{params['repeat_until_days']}, every #{repeat_weeks.join(' ')}, #{params['repeat_timerange']}.")
              else
                agileACSLog(6, object, "#{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile save succeed.")
            end
            else
              agileACSLog(6, object, "#{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile save succeed.")
            end
          else
            agileACSLog(6, object, "#{URI.escape(update_profile_name(params['name'], params['app'], false).strip)} profile save succeed.")
          end
        elsif res.code == '404'
          # If response is 404 not found, redirect to index directly.
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
          agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
        end
      else
        retMsg = "Profile name #{update_profile_name(params['name'], params['app'], false)} already exists." unless checkActionType(params)
        flash[:error] = retMsg
      end

      redirect_to action: :index
    end
  end

  # Progress /profiles/1
  # Progress /profiles/1.json
  def progress
    unless TpConfig.get_config('features-enable')
      redirect_to action: 'index'
      return
    end
    object = '[Profile Progress]'
    can?(:read, 'profiles') do
      id = params[:id]
      res = query_resource(create_api_conn(), 'profiles', { '_id' => id })
      @profile = res[:result][0] || {}
      userFilter = 0
      filter_time = 0
      @filter = params[:filter]
      if params.key?('filter') && params[:filter] != 'All'
        filter_time = get_progress_filter_time(params['filter'])
        userFilter = 1
      end
      calculate_profile_progress(@profile)
      @graphs = Rails.cache.fetch('graphs', expires_in: 60.seconds) do
        require 'net/http'
        threads = []
        graphs = ActiveSupport::JSON.decode(ERB.new(File.read(Rails.root.join('config/progress.json.erb'))).result)
        graphs.each do |_group_name, group_graphs|
          group_graphs.each do |_graph_name, graph|
            graph.each do |slice|
              threads << Thread.new(slice) do |slice|
                http = create_api_conn()
                http.start()
                progress_params = {}
                progress_params['progress_type'] =  slice['type']
                progress_params['userFilter'] =  userFilter
                progress_params['profile_filter'] =  @filter
                progress_params['filter_time'] =  filter_time
                if slice['type'] == 'done'
                  slice['query'] = progress_params
                  slice['href'] = url_for( action: 'dev_index', params: { query: ActiveSupport::JSON.encode(slice['query']) }, only_path: true)
                  slice['data'] = get_profile_done_num(http, @profile, userFilter, filter_time)
                elsif slice['type'] == 'todo'
                  slice['query'] = progress_params
                  slice['href'] = url_for( action: 'dev_index', params: { query: ActiveSupport::JSON.encode(slice['query']) }, only_path: true)
                  slice['data'] = get_profile_todo_num(http, @profile, userFilter, filter_time)
                elsif slice['type'] == 'fail'
                  slice['query'] = get_profile_faults_query(@profile, userFilter, filter_time)
                  slice['href'] = if userFilter == 1
                                    url_for(controller: 'faults', action: 'index', params: { query: ActiveSupport::JSON.encode(slice['query']), filter: @filter }, only_path: true)
                                  else
                                    url_for(controller: 'faults', action: 'index', params: { query: ActiveSupport::JSON.encode(slice['query']) }, only_path: true)
                                  end
                  slice['data'] = get_profil_failed_num(http, @profile, userFilter, filter_time)
                end
                slice.delete('query')
                http.finish()
              end
            end
          end
        end
        threads.each(&:join)
        next graphs
      end
    end
  end

  # DELETE /profiles/1
  # DELETE /profiles/1.json
  def destroy
    object = '[Profile Destroy]'
    can?(:delete, 'profiles') do
      http = create_api_conn()
      res = query_resource(create_api_conn(), 'profiles', { '_id' => params[:id] })
      profile = res[:result][0] || {}
      res = http.delete("/profiles/#{URI.escape(params[:id])}", nil)

      filename = profile['pppoe_file']
      profilename = profile['_id']
      file_path = "#{PPPOE_DIR}/#{profilename}_#{filename}"
      File.delete(file_path) if File.exist?(file_path)

      if res.code == '200'
        res = delete_presets(profile)
        if res.code != '200'
          flash[:error] = "Unexpected error when delete presets (#{res.code}): #{res.body}"
          agileACSLog(3, object, "Unexpected error when delete presets (#{res.code}): #{res.body}")
        end
        flash[:success] = 'Profiles deleted'
        agileACSLog(6, object, "#{URI.escape(update_profile_name(params['id'], params['app'], false))} profile destroy succeed.")
      else
        flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
      end

      redirect_to action: :index
    end
  end

  def checkActionType(profile)
    if profile['action_type'] == 'edit'
      true
    elsif profile['action_type'] == 'new'
      res = query_resource(create_api_conn(), 'profiles', { "_id": profile['name'] })
      res[:total] == 0
    end
  end

  def genFWScheduleFilters(query)
    ret = {}
    query.each do |key, val|
      if key.match('_id')
        if !val.is_a?(Hash)
          ret[key] = val + FWSCHEDULE_APPENDIX
        else
          ret[key.dup] = val.dup
          val.each do |k, v|
            case k
            when '$gt'
              query[key][k] = v + FWSCHEDULE_APPENDIX
              ret[key][k] = v + FWSCHEDULE_APPENDIX
            when '$lt'
              ret[key][k] = v
            when :$regex
              ret[key][k] = v.chop + FWSCHEDULE_APPENDIX + '$'
            when :$options
              ret[key][k] = v
            else
              ret[key][k] = v + FWSCHEDULE_APPENDIX
            end
          end
        end
      else
        if !val.is_a?(Hash)
          ret[key] = val
        else
          ret[key.dup] = val.dup
          val.each do |k, v|
            ret[key][k] = v
          end
        end
      end
    end
    ret
  end

  def dev_index
    can?(:read, 'devices') do
      http = create_api_conn()
      res = query_resource(create_api_conn(), 'profiles', { '_id' => params[:id] })
      @profile = res[:result][0] || {}
      @query = {}
      @userFilter = 0
      filter_time = 0
      @progress_type = 'done'
      @profile_filter = ''
      params_progress = {}
      if params.has_key?('query')
        @query = ActiveSupport::JSON.decode(URI.unescape(params['query']))
        if  @query.has_key?('progress_type')
          @progress_type = @query['progress_type']
        end
        if  @query.has_key?('userFilter')
          @userFilter = @query['userFilter']
        end
        if  @query.has_key?('filter_time')
          filter_time = @query['filter_time']
        end
        if  @query.has_key?('profile_filter')
          @profile_filter = @query['profile_filter']
        end
        progress_query = {}
        if @progress_type == 'done'
          progress_query = get_profile_done_query(@profile, @userFilter, filter_time, true)
        elsif @progress_type == 'todo'
          progress_query = get_profile_todo_query(@profile, @userFilter, filter_time, true)
        end
        if params.include?(:sort)
          params_progress[:sort] = params[:sort]
        end
        if params.has_key?(:page)
          params_progress[:page] = params[:page]
        end
        params_progress['query'] = ActiveSupport::JSON.encode(progress_query)
      end
      @total = 0
      @now = nil
      @devices = []
      aggregate_pipeline = []
      begin
        http = create_api_conn()
        aggregate_pipeline = generate_aggregate_pipeline(params_progress, true, true, true, Rails.configuration.profiles_index_parameters, Rails.configuration.profiles_summary_parameters)
        res = http.post("/profile_progress/devnum", ActiveSupport::JSON.encode(aggregate_pipeline), 'Content-type' => 'application/json')
         @total = res['total'].to_i
        page_num = (@total/10.0).ceil
        if page_num < params[:page].to_i
          params[:page] = (@total/10.0).ceil
        end
        aggregate_pipeline = generate_aggregate_pipeline(params_progress, false, true, true, Rails.configuration.profiles_index_parameters, Rails.configuration.profiles_summary_parameters)
        res = aggregate(http, 'devices', aggregate_pipeline)
        @now = res[:timestamp]
        @devices = res[:result]
        @message = res[:message]
      rescue => exception
        puts "exception occurs:\n #{exception}"
        puts exception.backtrace
        @total = 0
        @now = nil
        @devices = []
      end

      @appendix = InstanceProfileApps::APPENDIX

      respond_to do |format|
        format.html # dev_index.html.erb
        format.json { render json: @devices }
      end
    end
  end

end
