# Class ReportsController
class ReportsController < ApplicationController
  require 'net/http'
  require 'json'
  require 'csv'
  require 'fileutils'
  require 'date'
  include ActionController::Live
  include Util
  include Loggers
  include ReportsModule

  def index
    unless TpConfig.get_config('features-enable')
      redirect_to controller: 'home', action: 'index'
      return
    end

    can?(:read, 'reports') do
      @month_report_list = ['Current Month']
      @month_day_list = %w[1st 5th 10th 15th 20th 25th end]
      @month_day_db_list = %w[01 05 10 15 20 25 end]
      reportday = db_query_resource_report_setting()
      @month_report_list = process_month_report_list(@month_report_list, reportday)

      params['filter'] = 'Current Month' unless params.key?('filter')
      if params.key?('filterTop')
        month = month_transform(params['filterTop'], reportday)
        get_all_reports(month)
      elsif params.key?('filter')
        month = month_transform(params['filter'], reportday)
      end
      show_tab_content(params['tab'], month)
    end
  end

  def download_file
    raise NotAuthorized unless TpConfig.get_config('features-enable')

    file_path = '/tmp/' + params[:download_reports]
    file = File.open(file_path, 'rb')
    contents = file.read
    file.close

    File.delete(file_path) if File.exist?(file_path)

    send_data(contents, filename: params[:download_reports])
  end

  def process_data
    raise NotAuthorized unless TpConfig.get_config('features-enable')

    reportday = db_query_resource_report_setting()
    file_prefix = month_transform(params['month'], reportday)
    dir_name = 'Reports_' + file_prefix + '_' + Time.now.strftime('%Y%m%d')
    dir_path = dir_name
    system('mkdir', '-p', dir_path)
    tar_path = generate_tar_file(params, dir_name, dir_path, file_prefix)

    path = tar_path
    respond_to do |format|
      @path = path
      format.json { render json: { success: true, html: { url: @path } } }
      format.html {}
    end
    rm_dir_cmd = 'rm -r ' + dir_path
    system(rm_dir_cmd)
  end
end
