class RolesController < ApplicationController
  require 'json'
  def index
    can?(:read, 'roles') do
      off = params.include?(:page) ? (Integer(params[:page]) - 1) * Rails.configuration.page_size : 0
      lim = Rails.configuration.page_size
      @roles = Role.limit(lim).offset(off)
      @total = Role.count
    end
  end

  def new
    can?(:create, 'roles') do
      @role = Role.new
    end
  end

  def edit
    can?(:update, 'roles') do
      @role = Role.find(params['id'])
      @exist = Privilege.where('action = ? AND role_id = ?', 'exist', params['id'])
      @total = @role.privileges.count
      @owner_privileges = {}
      @module_name = {}
      menu_file = Dir.glob("#{Rails.root}/config/tplink/menu/*.json").sort
      menu_file.each do |path|
        file = File.read(path)
        menu = JSON.parse(file)
        next if menu['name'] == 'home'

        @module_name[menu['name']] = menu['link_to']
      end
      privileges_file = Dir.glob("#{Rails.root}/config/privileges/*.yml")
      privileges_file.each do |path|
        privileges = YAML.load_file(path)
        privileges.each do |resource, roles|
          @owner_privileges[resource] = roles['Owner']
        end
      end
    end
  end

  def create
    can?(:create, 'roles') do
      @role = Role.new(role_params)
      unless Role.find_by(name: params[:role][:name]).blank?
        flash[:error] = 'This name has been used'
        render('new') && return
      end

      if @role.save
        create_default_privilege('/home', 'read', @role.id)
        create_default_privilege('/preface', 'read', @role.id)
        create_default_privilege('/preface', 'create', @role.id)
        create_default_privilege('/preface', 'update', @role.id)
        create_default_privilege('/preface', 'delete', @role.id)

        flash[:success] = 'Role created'
        redirect_to roles_path
      else
        render 'new'
      end
    end
  end

  def update
    can?(:update, 'roles') do
      @role = Role.find(params['id'])
      if @role.update(role_params)
        redirect_to roles_path
      else
        render 'edit'
      end
    end
  end

  def destroy
    can?(:delete, 'roles') do
      @role = Role.find(params['id'])
      @role.destroy
      $db_change = true
      redirect_to roles_path
    end
  end

  private

  def role_params
    params.require(:role).permit(:name)
  end

  def create_default_privilege(resource, action, role_id)
    @role = Role.find(role_id)
    privilege = DefaultPrivilege.new
    privilege.resource = resource
    privilege.action = action
    privilege.weight = 1
    @role.default_privileges << privilege
    @role.save
  end
end
