# ---------------------------------------------------------------------------------------------------------
# class SystemController
# History:
#     20200710 bill.chen  move from module/basic/genieacs-gui/app/controllers/system_controller.rb,
#                         and revise to rubocop document format
# ---------------------------------------------------------------------------------------------------------
class SystemController < ApplicationController
  # GET /system
  # GET /system.json
  require 'json'
  include Util
  include Models
  include SystemModule
  before_action :check_upgrade_status, only: %i[update reboot poweroff cwmpsetting]

  # GET /system
  def index
    can?(:read, 'system') do
      http = create_api_conn()
      params['tab'] = 'syssetting' unless params.include? 'tab'
      case params['tab']
      when 'syssetting'   then get_system_setting(http)
      when 'cpesetting'   then get_cpe_setting()
      when 'upgrade'      then get_system_upgrade()
      when 'sysinfo'      then get_system_info(http)
      when 'cwmpsetting'  then get_cwmp_setting(http)
      when 'cpemodellist' then get_cpe_model_list(http)
      when 'ceipsetting'  then get_ceip_setting(http)
      when 'xmpp'         then get_xmpp(http)
      else @models_all = model_list(http, 1000, nil, nil, nil)[:models]
      end
    end
  end

  # POST /system/acsapps, update {}
  def update
    can?(:update, 'system') do
      http = create_api_conn()
      case params['tab']
      when 'xmpp' then
        object = '[XMPP]'
        task = sce_req_xmpp()
        post_setting(object, SYS_NBI_URL_XMPP, task, 'System', false)
        agileACSLog(6, object, 'Changed xmpp setting.')
      when 'xmppdebug' then
        # debugaction = params['system']['action']
        # debugcmd = params['system']['cmd']
        # queryall = 'var result=db.xmpp.find().toArray();result.forEach(function(item){print(item["_id"])})'
        # query = "var result=db.xmpp.findOne({_id:\"#{debugcmd}\"});printjson(result)"
        # queryusername = "var result=db.xmpp.findOne({_id:\"#{debugcmd}\"});print(result.username)"
        # remove = "var result=db.xmpp.remove({_id:\"#{debugcmd}\"});printjson(result)"
        # case debugaction
        # when 'xmpptool' then result = `/opt/agileacs/bin/xmpp-tool.sh #{debugcmd}`
        # when 'xmpplistclients' then result = `mongo localhost:27017/tplink --quiet --eval '#{queryall}'`
        # when 'xmppqueryclient' then result = `mongo localhost:27017/tplink --quiet --eval '#{query}'`
        # when 'xmppcleanlock' then
        #   `rm /tmp/xmppservice`
        #   result = 'lock cleaned'
        # when 'xmppremoveclient' then result = `mongo localhost:27017/tplink --quiet --eval '#{remove}'`
        # when 'xmppunregclient' then result = `/opt/ejabberd-19.05/bin/ejabberdctl unregister #{debugcmd} agileacs.com`
        # when 'xmpplistregclients' then result = `/opt/ejabberd-19.05/bin/ejabberdctl registered-users agileacs.com`
        # when 'xmpperaseclient' then
        #   uname = `mongo localhost:27017/tplink --quiet --eval '#{queryusername}'`.strip
        #   rmuser = `mongo localhost:27017/tplink --quiet --eval '#{remove}'`
        #   result = `/opt/ejabberd-19.05/bin/ejabberdctl unregister #{uname} agileacs.com`
        # end
        result = 'not allowed'
        render json: { 'result' => result }
      else
        object = '[System Setting]'
        system = get_system_setting(http)
        timezone_old = ActiveSupport::TimeZone::MAPPING.select { |_k, v| v == system['timezone'] }.values.first
        task = sce_req_sys_setting()
        data = { 'action' => 'update_system', 'data' => task}
        post_setting(object, SYS_NBI_URL_ACSAPPS, data, 'System', false)
        system = get_system_setting(http)
        timezone_new = ActiveSupport::TimeZone::MAPPING.select { |_k, v| v == system['timezone'] }.values.first
        agileACSLog(6, object, "Changed time zone to #{timezone_new}.") if timezone_old != timezone_new
      end
    end
  end

  # DELETE /system/upgrade
  def destroy
    can?(:update, 'system') do
      object = '[System Upgrade]'
      data = {'action' => 'upgrade'}
      data['data'] = {
        'action' => 'lock',
        'data' => 'cancel'
      }
      post_setting(object, SYS_NBI_URL_ACSAPPS, data, 'System', false)
      render json: { 'msg' => 'canceled' }
    end
  end

  # GET /system/time
  def time
    can?(:read, 'system') do
      sce_render_time()
    end
  end

  # POST /system/cpe
  def cpesetting
    can?(:update, 'system') do
      # informinterval = params[:informinterval]
      # intervaloffset = params[:intervaloffset]
      # task = { 'informinterval' => params[:informinterval], 'intervaloffset' => params[:intervaloffset] }
      data = { 'informinterval' => params[:informinterval], 'intervaloffset' => params[:intervaloffset] }
      post_setting('[CPE Setting]', SYS_NBI_URL_CPE, data, 'CPE')
    end
  end

  # POST /system/upgrade, check & upgrade
  def upgrade
    object = '[System Upgrade]'
    can?(:update, 'system') do
      upgrade_status = get_upgrade_status
      # POST /system/upgrade  data: { "get_status":"true" }
      render(json: { msg: upgrade_status }) && return if params.key?('get_status') && params['get_status'] == 'true'

      # POST /system/upgrade  data: { "file":xxx.img }
      if params.key?('file')
        render(json: { version: 'none', msg: upgrade_status }) && return if upgrade_status != 'none'

        res = check()
        res.body = 'error' if res.code != '200'
        render json: { version: res.body }
      end

      # POST /system/upgrade  data: { "filename":xxx.img }
      if params.key?('filename')
        render(json: { msg: upgrade_status }) && return if upgrade_status == 'timeout'

        filepath = '/tmp/agileacs_image.img'
        res = upgradefile(filepath)
        if res.code == '200'
          agileACSLog(6, object, "System upgrade succeed. Current #{res.body}")
        elsif res.code == '202'
          flash[:warning] = res.message
          agileACSLog(4, object, res.message.to_s)
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
          agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
        end
        render(json: { msg: 'upgrading' }) && return
      end
    end
  end

  # POST /system/acsapps, restart
  def reboot
    can?(:update, 'system') do
      http = create_api_conn()
      data = { 'action' => 'restart' }
      http.put(SYS_NBI_URL_ACSAPPS, ActiveSupport::JSON.encode(data), 'Content-type' => 'application/json')
    end
  end

  # POST /system/acsapps, shutdown
  def poweroff
    can?(:update, 'system') do
      http = create_api_conn()
      data = { 'action' => 'shutdown' }
      http.put(SYS_NBI_URL_ACSAPPS, ActiveSupport::JSON.encode(data), 'Content-type' => 'application/json')
    end
  end

  # POST /system/acsapps, cwmpupdate
  def cwmpsetting
    can?(:update, 'system') do
      # acs_username = params[:acs_username]
      # acs_password = params[:acs_password]
      # cpe_username = params[:cpe_username]
      # cpe_password = params[:cpe_password]
      # task = { 'acs_username' => params[:acs_username], 'acs_password' => params[:acs_password], 'cpe_username' => params[:cpe_username], 'cpe_password' => params[:cpe_password] }
      data = { 'action' => 'cwmpupdate' }
      data['data'] = { 'acs_username' => params[:acs_username], 'acs_password' => params[:acs_password], 'cpe_username' => params[:cpe_username], 'cpe_password' => params[:cpe_password] }
      post_setting('[CWMP Setting]', SYS_NBI_URL_ACSAPPS, data, 'CWMP', false)
    end
  end

  # POST /system/acsapps, config
  # cpe_model_update obsoletes ConfigController
  def cpe_model_update
    can?(:update, 'system') do
      if File.exist?(filepath = cpe_model_file())
        params[:tab] = 'cpemodellist'
        post_setting('[Import Config File]', SYS_NBI_URL_ACSAPPS, { 'action' => 'config', 'data' => filepath }, 'Import Config File')
      end
    end
  end

  # POST /system/ceip
  def set_ceip_setting
    can?(:update, 'system') do
      data = { 'wisp_region' => params[:region], 'wisp_country' => params[:country], 'wisp_name' => params[:name], 'wisp_email' => params[:email], 'wisp_cnpj' => params[:cnpj] }
      data['ceip_enabled'] = params[:ceip_enabled].to_i if params.key?('ceip_enabled')
      post_setting('[CEIP Setting]', SYS_NBI_URL_CEIP, { 'action' => 'save_info', 'data' => data }, 'CEIP')
    end
  end

  def test_ceip
    can?(:update, 'system') do
      msg = "Test Failed"
      http = create_api_conn()
      res = http.put(SYS_NBI_URL_CEIP, nil)
      if res.code == '200'
        filepath = res.body
        if filepath.blank?
          msg = "Test Failed(No result file)"
        else
          msg = IO.readlines(filepath)
        end
      elsif res.code == '202'
        msg = res.body
      end
      render(json: { msg: msg }) && return
    end
  end

  private

  def get_system_setting(http)
    query = {}
    query['_id'] = 'service'
    res = query_resource(http, SYS_NBI_URL_SYSSETTING[1..-1], query)
    @system = res[:result].size == 1 ? res[:result][0] : {}
    return @system if params[:tab] == 'syssetting'

    @memory = sce_sys_resource_usage(http, SYS_NBI_URL_USAGE_MEMORY)
    @memory['usage'] = @memory['usage'].gsub(/[^0-9\.]/, '')
  end

  def get_xmpp(http)
    res = http.get(SYS_NBI_URL_XMPP)
    @system = res.code == '404' ? sce_sys_default() : JSON.parse(res.body)
    @memory = sce_sys_resource_usage(http, SYS_NBI_URL_USAGE_MEMORY)
    @memory['usage'] = @memory['usage'].gsub(/[^0-9\.]/, '')
  end

  def get_cpe_setting
    res = query_resource(create_api_conn(), SYS_NBI_URL_CPE[1..-1], {})
    @cpesetting = res[:result][0].nil? ? { 'informinterval' => 86_400, 'intervaloffset' => 3600 } : res[:result][0]
  end

  def get_system_upgrade
    can_upgrade = false
    @upgrade_status = get_upgrade_status
    ceip = get_ceip_setting(create_api_conn())
    if ceip['status'] == 'activated'
      res = query_resource(create_api_conn(), 'system/license', nil, nil, nil, nil)
      licenses = res[:result]
      licenses.each do |license|
        can_upgrade = true if license['_id'] == 'maintenance license' && license['status'] == 'activated'
      end
    else
      can_upgrade = true
    end
    @canUpgrade = can_upgrade
  end

  def get_system_info(http)
    @storage = sce_sys_resource_usage(http, SYS_NBI_URL_USAGE_STORAGE, true)
    @cpu = sce_sys_resource_usage(http, SYS_NBI_URL_USAGE_CPU)
    @memory = sce_sys_resource_usage(http, SYS_NBI_URL_USAGE_MEMORY)

    @storage['usage'] = @storage['usage'].gsub(/[^0-9]/, '')
    @cpu['usage'] = @cpu['usage'].gsub(/[^0-9\.]/, '')
    @memory['usage'] = @memory['usage'].gsub(/[^0-9\.]/, '')
  end

  def get_cwmp_setting(http)
    data = { 'action' => 'cwmpget' }
    res = http.put(SYS_NBI_URL_ACSAPPS, ActiveSupport::JSON.encode(data), 'Content-type' => 'application/json')
    @cwmp = res.code == '404' ? { 'acs_password' => 'admin', 'acs_username' => 'admin', 'cpe_password' => 'admin', 'cpe_username' => 'admin' } : JSON.parse(res.body)
  end

  def get_cpe_model_list(http)
    # @models_all = []
    page = params.include?(:page) ? Integer(params[:page]) : nil
    skip = params.include?(:page) ? (page - 1) * Rails.configuration.page_size : nil
    @query = {}
    res = model_list(http, Rails.configuration.page_size, page, skip, @query)
    @total = res[:total]
    @models_all = res[:models]
    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @models_all }
    end
  end

  def get_ceip_setting(http)
    res = query_resource(http, SYS_NBI_URL_CEIP[1..-1], nil)
    @country = Rails.configuration.country
    @ceip = res[:result].size == 1 ? res[:result][0] : {}
  end

  def bytes_to_megabytes(bytes)
    sce_bytes_to_megabytes(bytes)
  end

  def get_upgrade_status
    http = create_api_conn()
    data = { 'action' => 'upgrade_lock', 'data' => 'status'}
    res = http.post(SYS_NBI_URL_ACSAPPS, ActiveSupport::JSON.encode(data), 'Content-type' => 'application/json')
    if res.code == '202'
      'none'
    elsif res.code == '200'
      res.body
    end
  end

  def check()
    tmpname = params[:file].path
    filepath = '/tmp/agileacs_image.img'
    File.rename(tmpname, filepath)
    task = {
      'action' => 'check',
      'data' => filepath
    }
    http = create_api_conn()
    http.post(SYS_NBI_URL_ACSAPPS, ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
  end

  def upgradefile(filepath)
    task = {
      'action' => 'upgrade',
      'data' => filepath
    }
    http = create_api_conn()
    http.post(SYS_NBI_URL_ACSAPPS, ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
  end
end
