# controller for /remote_assistance
class TroubleshootingController < ApplicationController
  include Loggers
  def index
    can?(:read, 'troubleshooting') do
      res = query_resource(create_api_conn(), 'troubleshooting', {})
      return unless res[:result][0]

      @result = res[:result][0]
      @status = get_pptp_status(@result['enable'])
    end
  end

  def status
    render json: { 'status' => get_pptp_status(params[:enable]) }
  end

  def update
    can?(:update, 'troubleshooting') do
      gdpr = params[:gdpr] == 'true' ? 'true' : 'false'
      data = { enable: params[:enable] ? 'on' : 'off', ipAddr: params[:ipAddr] == '' ? '0.0.0.0' : params[:ipAddr] }

      if data[:enable] == 'on'
        return redirect_to(remote_assistance_path) unless gdpr == 'true'

        agileACSLog(6, '[Remote Assistance]', 'Agree GDPR.')
      end

      http = create_api_conn()
      res = http.post('/troubleshooting/remote_assistance', ActiveSupport::JSON.encode(data), 'Content-Type' => 'application/json')

      if data[:enable] == 'on'
        return render(json: { 'message' => 'successfully' }) if res.code == '200'

        render(json: { 'message' => "Unexpected error (#{res.code}): #{res.body}" })
      else
        if res.code == '200'
          flash[:success] = 'successfully'
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        end
        redirect_to(remote_assistance_path)
      end
    end
  end
end

def get_pptp_status(enable = '')
  res = system("grep -oE '([0-9]{1,3}\.){3}[0-9]{1,3}' /tmp/ppp_status")
  status = res ? 'Conneted' : "Disconneted. Please check the network connection and click 'Save' again."
  status = 'Disconneted' if enable == 'off'
  status = 'Please Wait...' if enable == 'on'
  status
end
