class UsersController < ApplicationController
  
  def index
    can?(:read, 'users') do
      off = params.include?(:page) ? (Integer(params[:page]) - 1) * Rails.configuration.page_size : 0
      lim = Rails.configuration.page_size
      @users = User.limit(lim).offset(off)
      roles = Role.all
      @roles = {}
      roles.each do |role|
        @roles[role.id] = role.name
      end
      @total = User.count
    end
  end
  
  def new
    can?(:create, 'users') do
      if User.all.count > 19
        flash[:error] = "Only allow 20 users"
        redirect_to users_path
        return
      end
      @user = User.new
    end
  end
  
  def edit
    can?(:update, 'users') do
      @user = User.find(params['id'])
    end
  end
  
  def create
    can?(:create, 'users') do
      @user = User.new(user_params)
      
      if !User.find_by(:username => params[:user][:username]).blank? 
        flash[:error] = "This username has been used"
        render 'new' and return
      end

      if @user.save
        @user.user_roles.create(:role_id => params[:role_id])
        flash[:success] = "User created"
        $db_change = true
        redirect_to users_path
      else
        render 'new'
      end
    end
  end
  
  def update
    can?(:update, 'users') do
      @user = User.find(params['id'])
      if !user_params['password'].blank?
        $db_change = true if @user.update(password: params['user']['password'])
      end

      if !params['role_id'].blank?
        @user.user_roles.update(role_id: params['role_id'])
        $db_change = true
      end

      flash[:success] = "User saved"
      redirect_to users_path
    end
  end
  
  def destroy
    can?(:delete, 'users') do
      return if params['id'] == 1
      @user = User.find(params['id'])
      @user.destroy
      $db_change = true
      flash[:success] = "User destoryed"
      redirect_to users_path
    end
  end

  private
    def user_params
      params.require(:user).permit(:username, :password)
    end
end
