class DeviceTabList
  include Util
  include Parser

  # tab list: which tabs display on show page
  # | CPE Type        |lan|wan|wlan|diagnostics|mesh|tr tree|actions
  # | ----------------------------------------------------------------
  # | Pure Router     |W  |W  |W   |W          |N/A |W      |W
  # | Pure AP         |R  |N/A|W   |N/A        |N/A |R      |W
  # | Mesh Controller1|W  |W  |W   |W          |R   |W      |W
  # | Mesh Controller2|R  |N/A|W   |N/A        |R   |R      |W
  # | Mesh Controller3|W  |W  |W   |W          |R   |W      |W
  # | Mesh Satellite1 |R  |N/A|R   |N/A        |R   |R      |W (without configuration download)
  # | Mesh Satellite2 |R  |N/A|R   |N/A        |R   |R      |W (without configuration download)
  # | Mesh Satellite3 |R  |N/A|R   |N/A        |R   |R      |W (without configuration download)
  # ------------------------------------------------------------------
  # N/A:hide
  def initialize(http, device_id, tr181 = true)
    @list = { 'info' => 'w', 'actions' => 'w', 'log' => 'w' }
    @list_1 = { 'lan' => 'w', 'wan' => 'w', 'wlan' => 'w', 'mesh' => nil, 'diagnostics' => 'w', 'tr_tree' => 'w', 'cpe_type' => 'Pure Router' }
    @list_2 = { 'lan' => 'r', 'wan' => nil, 'wlan' => 'w', 'mesh' => nil, 'diagnostics' => nil, 'tr_tree' => 'r', 'cpe_type' => 'Pure AP' }
    @list_3 = { 'lan' => 'w', 'wan' => 'w', 'wlan' => 'w', 'mesh' => 'r', 'diagnostics' => 'w', 'tr_tree' => 'w', 'cpe_type' => 'Mesh Controller' }
    @list_4 = { 'lan' => 'r', 'wan' => nil, 'wlan' => 'w', 'mesh' => 'r', 'diagnostics' => nil, 'tr_tree' => 'r', 'cpe_type' => 'Mesh Controller' }
    @list_5 = { 'lan' => 'r', 'wan' => nil, 'wlan' => 'r', 'mesh' => 'r', 'diagnostics' => nil, 'tr_tree' => 'r', 'cpe_type' => 'Mesh Satellite' }

    @device_params = {}
    dump_params(get_data(http, device_id, tr181), @device_params)
  end

  def get_data(http, device_id, tr181)
    projection = get_projection(tr181)
    res = query_resource(http, 'devices', { '_id' => device_id }, projection)
    raise ActiveRecord::RecordNotFound if res[:result].size != 1
    res[:result][0]
  end

  def get_projection(tr181)
    root = tr181 ? 'Device' : 'InternetGatewayDevice'
    Rails.configuration.device_mode[root]
  end

  def get_list
    tab_list = @list_1 # default is pure router
    return tab_list.merge(@list) if @device_params.blank?

    root = 'Device'
    nodes = Rails.configuration.device_mode[root]
    muti_mode = ''
    mesh_work_mode = ''

    mesh_work_mode = @device_params[nodes[0]]['value'] if @device_params.key?(nodes[0]) && !@device_params[nodes[0]]['value'].nil?

    muti_mode = @device_params[nodes[1]]['value'] if @device_params.key?(nodes[1]) && !@device_params[nodes[1]]['value'].nil?

    if muti_mode == 'Router' && mesh_work_mode == '' # Pure Router
      tab_list = @list_1
    elsif muti_mode == 'AP' && mesh_work_mode == ''  # Pure AP
      tab_list = @list_2
    elsif muti_mode == 'Router' && mesh_work_mode == 'CAP' # Mesh Controller 1
      tab_list = @list_3
    elsif muti_mode == 'AP' && mesh_work_mode == 'CAP'     # Mesh Controller 2
      tab_list = @list_4
    elsif muti_mode == '' && mesh_work_mode == 'CAP' # Mesh Controller 3
      tab_list = @list_3
    elsif (muti_mode == 'RE' || muti_mode == 'Repeater') && mesh_work_mode == 'RE' # Mesh Satellite 1
      tab_list = @list_5
    elsif muti_mode == 'Satellite' && (mesh_work_mode == 'RE' || mesh_work_mode == '') # Mesh Satellite 2
      tab_list = @list_5
    elsif muti_mode == '' && mesh_work_mode == 'RE' # Mesh Satellite 3
      tab_list = @list_5
    end

    tab_list.merge(@list)
  end
end
