class LanEntry
    include Parser

    # instance variables definitions
    @dhcp_server_prefix = nil
    @dhcp_relay_prefix = nil
    @dhcp_server_enable = nil
    @dhcp_relay_enable = nil
    def initialize(root, device_params, device_params_groups)
        @params = {}
        config = Rails.configuration.subpage_parameters[root]
        config['lan'].each do |k, v|
            @params[k] = {}
        end
        @device_params = device_params
        @device_params_groups = device_params_groups 
        
        initValues()
    end

    def getParameters() 
        ret = {}
        @params.each do |field, attributes|
            if attributes.size == 0
                ret[field] = {}
            else 
                ret[field] = {'value'=>attributes['value'], 'writable'=>attributes['writable']}
            end
        end 
        return ret
    end

    def getFields()
        return @params.keys
    end

    def getValue(field)
        return @params[field]
    end
    
    def setValue(field, value, writable, path=nil)
        return false if field.nil? || value.nil? ||!@params.key?(field)
        @params[field]['value'] = value
        @params[field]['writable'] = writable if !writable.nil?
        @params[field]['path'] = path if !path.nil?
        return true
    end


    def initValues()
        function_names = []
        
        function_names << { "name" => "init_primary_IpAddress"}
        function_names << { "name" => "init_primary_Mask"}
        function_names << { "name" => "init_secondary_Enable"}
        function_names << { "name" => "init_secondary_IpAddress"}
        function_names << { "name" => "init_secondary_Mask"}
        function_names << { "name" => "init_dhcp"}

        call_functions(function_names)
    end
    
    # private
    def init_primary_IpAddress()
        #res = get_device_param(@device_params, "Device.IP.Interface.1.IPv4Address.1.IPAddress")
        res  = get_device_param_by_group(@device_params_groups, "Device.IP.Interface.1.IPv4Address.1", "IPAddress")
        res = res[0]
        setValue('primary_IpAddress', res['value'], res['writable'], res['path']) if !res.nil?
    end
    
    def init_primary_Mask()
        #res = get_device_param(@device_params, "Device.IP.Interface.1.IPv4Address.1.SubnetMask")
        res  = get_device_param_by_group(@device_params_groups, "Device.IP.Interface.1.IPv4Address.1", "SubnetMask")
        res = res[0]
        setValue('primary_Mask', res['value'], res['writable'], res['path']) if !res.nil?
    end

    def init_secondary_Enable()
        res = get_device_param_by_specific(@device_params, "Device.IP.Interface.1.IPv4Address.2.Enable")
        if !res.nil? 
            setValue('secondary_Enable', res['value'], res['writable'], res['path']) 
        else 
            setValue('secondary_Enable', 'false', 'false') 
        end
    end
    
    # private
    def init_secondary_IpAddress()
        #res  = get_device_param(@device_params, "Device.IP.Interface.1.IPv4Address.2.IPAddress")
        res = get_device_param_by_group(@device_params_groups, "Device.IP.Interface.1.IPv4Address.2", "IPAddress")
        res = res[0]
        setValue('secondary_IpAddress', res['value'], res['writable'], res['path']) if !res.nil?
    end
    
    # private
    def init_secondary_Mask()
        #res  = get_device_param(@device_params, "Device.IP.Interface.1.IPv4Address.2.SubnetMask")
        res = get_device_param_by_group(@device_params_groups, "Device.IP.Interface.1.IPv4Address.2", "SubnetMask")
        res = res[0]
        setValue('secondary_Mask', res['value'], res['writable'], res['path']) if !res.nil?
    end

    # private 
    def init_dhcp() 
        init_dhcp_server()
        init_dhcp_relay()
        return if @dhcp_server_enable.nil? && @dhcp_relay_enable.nil?
        
        dhcp_server_enable = @dhcp_server_enable.key?("value") ? @dhcp_server_enable['value'] : nil
        dhcp_relay_enable = @dhcp_relay_enable.key?("value") ? @dhcp_relay_enable['value'] : nil
        
        # process 'dhcp_enable' and 'dhcp_mode'
        if dhcp_server_enable.nil? 
            if dhcp_relay_enable.nil?
                return
            else
                setValue('dhcp_mode', 'relay', @dhcp_relay_enable['writable'])
                setValue('dhcp_enable', dhcp_relay_enable, @dhcp_relay_enable['writable'])
            end
        else 
            if dhcp_relay_enable.nil?
                setValue('dhcp_mode', 'server', @dhcp_server_enable['writable'])
                setValue('dhcp_enable', dhcp_server_enable, @dhcp_server_enable['writable'])
            else 
                setValue('dhcp_mode', "server", @dhcp_relay_enable['writable'] || @dhcp_server_enable['writable'])
                setValue('dhcp_mode', "relay", @dhcp_relay_enable['writable'] || @dhcp_server_enable['writable'] ) if dhcp_relay_enable == "true"
                setValue('dhcp_enable', (dhcp_server_enable == "true" || dhcp_relay_enable == "true" ) ? "true" : "false", @dhcp_relay_enable['writable'] || @dhcp_server_enable['writable'])
            end
        end
    end
    
    
    def init_dhcp_server() 
        
        res = match_rkey_val( @device_params, "Device.DHCPv4.Server.Pool.\\d+.Interface", "Device.IP.Interface.1.", true )
        return if res.nil? or res.size == 0
        path = res[0]['path']
        @dhcp_server_prefix, match, suffix = path.rpartition('.')
        
        #res = get_device_param(@device_params, "#{@dhcp_server_prefix}.Enable")
        res = get_device_param_by_group(@device_params_groups, "#{@dhcp_server_prefix}", "Enable")
        res = res[0]
        if !res.nil? 
            @dhcp_server_enable = res
            res = get_device_param(@device_params, "#{@dhcp_server_prefix}.MinAddress")
            setValue('dhcp_server_MinAddress', res['value'], res['writable'], res['path']) if !res.nil?
            res = get_device_param(@device_params, "#{@dhcp_server_prefix}.MaxAddress")
            setValue('dhcp_server_MaxAddress', res['value'], res['writable'], res['path']) if !res.nil?
            res = get_device_param(@device_params, "#{@dhcp_server_prefix}.LeaseTime")
            setValue('dhcp_server_LeaseTime', res['value'], res['writable'], res['path']) if !res.nil?
        end
    end
    
    def init_dhcp_relay()
        res = match_rkey_val( @device_params, "Device.DHCPv4.Relay.Forwarding.\\d+.Interface", "Device.IP.Interface.1." , true)
        return if res.nil? or res.size == 0
        path = res[0]['path']
        @dhcp_relay_prefix, match, suffix = path.rpartition('.')
        
        #res = get_device_param(@device_params, "#{@dhcp_relay_prefix}.Enable")
        res = get_device_param_by_group(@device_params_groups, "#{@dhcp_relay_prefix}", "Enable")
        res = res[0]
        if !res.nil? 
            @dhcp_relay_enable = res
            res = get_device_param(@device_params, "#{@dhcp_relay_prefix}.DHCPServerIPAddress")
            setValue('dhcp_relay_RemoteIp', res['value'], res['writable'], res['path']) if !res.nil?
        end
    end

    def generate_set_parameters(params) 
        ret = []
        params.each do | k, v | 
            res = generate_set_parameter(k, v)
            ret << res if !res.nil?
        end
        reconstruct_set_parameters_for_secondary_Enable(params, ret)
        reconstruct_set_parameters_for_dhcp(params, ret)
        return ret
    end


    def generate_set_parameter(field, new_val) 
        ret = nil
        p = nil
        
        case field
        when 'primary_IpAddress', 'primary_Mask', 'secondary_IpAddress', 'secondary_Mask', 'dhcp_server_MinAddress', 'dhcp_server_MaxAddress', 'dhcp_server_LeaseTime', 'dhcp_relay_RemoteIp'
            p = generate_set_parameters_by_field(field, new_val) if @params[field].key?("path") && !@params[field]['path'].nil?
        end
        
        ret = [field, p] if !p.nil? && p.size > 0
        return ret
    end

    def generate_set_parameters_by_field(field, new_val)
        ret = {}
        begin
            path = @params[field]['path']
            ret[path] = new_val 
        rescue => exception
            ret = nil
        end
        return ret;
    end
    
    
    def add_key(key, value, arr)
        found = false
        arr.each do | e |
            if key == e[0]
                found = true
                e[1] = value
            end
        end
        if !found
            arr << [key, value]
        end
    end 

    def delete_key(key, arr)
        arr.each do | e |
            if key == e[0]
                arr.delete e
            end
        end
    end 

    def has_key(key, arr)
        found = false
        arr.each do | e |
            if key == e[0]
                found = true
                break
            end
        end
        return found
    end 

    def reconstruct_set_parameters_for_secondary_Enable(params, ret)
        if !params.key?("secondary_Enable")
            delete_key("secondary_IpAddress", ret)
            delete_key("secondary_Mask", ret)
            return
        end

        return if @params['secondary_Enable']['path'].nil?

        prefix, match, suffix = @params['secondary_IpAddress']['path'].rpartition('.')
        res = get_device_param_by_group(@device_params_groups, prefix, "Enable")
        val = res[0]['value']
       
        case params["secondary_Enable"]
        when 'false'
            delete_key("secondary_IpAddress", ret)
            delete_key("secondary_Mask", ret)
            add_key('secondary_Enable', {"#{prefix}.Enable" => "false"}, ret) if val == "true"
        when 'true'
            add_key('secondary_Enable', {"#{prefix}.Enable" => "true"}, ret)  if val == "false"
        end
    end

    def reconstruct_set_parameters_for_dhcp(params, ret)
        if params.key?("dhcp_enable")
            dhcp_enable = params["dhcp_enable"]
        else
            dhcp_enable = @params["dhcp_enable"]["value"] if @params.has_key?("dhcp_enable") && !@params["dhcp_enable"].nil?
            # delete_key('dhcp_server_MinAddress', ret)
            # delete_key('dhcp_server_MaxAddress', ret)
            # delete_key('dhcp_server_LeaseTime', ret)
            # delete_key('dhcp_relay_RemoteIp', ret)
            
        end

        if params.key?("dhcp_mode")
            dhcp_mode = params['dhcp_mode']
        else
            dhcp_mode = @params["dhcp_mode"]["value"] if @params.has_key?("dhcp_mode") && !@params["dhcp_mode"].nil?
        end

        case dhcp_enable
        when "false"
            delete_key('dhcp_server_MinAddress', ret)
            delete_key('dhcp_server_MaxAddress', ret)
            delete_key('dhcp_server_LeaseTime', ret )
            delete_key('dhcp_relay_RemoteIp', ret )
            if params.key?("dhcp_enable")
                add_key('dhcp_server_Enable', {"#{@dhcp_server_prefix}.Enable" => "false"}, ret) #if server_enable == "true"
                add_key('dhcp_relay_Enable', {"#{@dhcp_relay_prefix}.Enable" => "false"}, ret) #if relay_enable == "true"
            end
        when "true"
            case dhcp_mode
            when 'server'
                # delete dhcp_relay param, disable dhcp_relay, enable dhcp_server
                delete_key('dhcp_relay_RemoteIp', ret)
                if params.key?("dhcp_mode")
                    add_key('dhcp_relay_Enable', {"#{@dhcp_relay_prefix}.Enable" => "false"}, ret) #if relay_enable == "true"
                    add_key('dhcp_server_Enable', {"#{@dhcp_server_prefix}.Enable" => "true"}, ret) #if server_enable == "false"
                end
            when 'relay'
                # delete dhcp_server param, disable dhcp_server, enable dhcp_relay
                delete_key('dhcp_server_MinAddress', ret)
                delete_key('dhcp_server_MaxAddress', ret)
                delete_key('dhcp_server_LeaseTime', ret)
                if params.key?("dhcp_mode")
                    add_key('dhcp_server_Enable', {"#{@dhcp_server_prefix}.Enable" => "false"}, ret) #if server_enable == "true"
                    add_key('dhcp_relay_Enable', {"#{@dhcp_relay_prefix}.Enable" => "true"}, ret) #if relay_enable == "false"
                end
            end
        end
    end
end
