
module MeshType 
    NONE = "None"
    CONTROLLER = "Controller"
    SATELLITE = "Satellite"
end

class MeshTopology
    #attr_reader :ap_device_path
    include Parser 
    def initialize(devices, devices_selected, controller = nil)
        @devices = devices
        @devices_selected = devices_selected
        @controller = controller
        @device_params = {}
        dump_params(devices, @device_params)
        @device_params_groups = group_params(@device_params)
        @params = {}
        initValues()

        # reset id, mesh_type, mac for satellite
        if ( !@devices_selected['_mesh'].blank? && @devices_selected['_mesh']['type'] == MeshType::SATELLITE)
            
            setValue('id', @devices_selected["_id"])
            setValue('mesh_type', @devices_selected['_mesh']['type'])
            
            begin
                setValue("mac", @devices_selected["Device"]["Ethernet"]["Link"]["1"]["MACAddress"]["_value"])
            rescue => ex
                puts "ex=#{ex}"
            end
        end

        #puts @params
    end

    def setValue(field, value)
        @params[field] = value if !value.nil?
    end

    def getValue(field)
        return @params[field]
    end

    def setValue_by_specific_path(field, specific_path, default_value) 
        value = get_device_param_by_specific(@device_params, specific_path)
        value = default_value if value.nil?

        if value.include?("value")
            value = value["value"]
        end
        setValue(field, value)
    end

    def getResults() 
        return @params.clone
    end

    def initValues
        init_basic_info

        case @params['mesh_type']
        when 'Controller'
            call_functions([{ "name" => "init_mesh" }])
        end
    end

    def init_basic_info
        call_functions([{ "name" => "init_id" }])
        call_functions([{ "name" => "init_mesh_type" }])
        call_functions([{ "name" => "init_mesh_cap" }])
        call_functions([{ "name" => "init_mac" }])
        call_functions([{ "name" => "init_last_updated" }])
    end
    
    def init_id
        setValue('id', @devices["_id"])
    end

    def init_mesh_type
        setValue('mesh_type', "None")
        setValue('mesh_type', @devices["_mesh"]["type"]) if !@devices["_mesh"].nil? && !@devices["_mesh"]["type"].nil?
    end

    def init_mesh_cap
        setValue('mesh_cap', "None")
        setValue('mesh_cap', @devices["_mesh"]["cap"])if !@devices["_mesh"].nil? && !@devices["_mesh"]["cap"].nil?
    end

    def init_mac
        setValue_by_specific_path('mac', "Device.Ethernet.Link.1.MACAddress", "00:00:00:00:00:00")
    end

    def init_last_updated
        # Device.WiFi.MultiAP.APDevice._timestamp
        value = ""
        begin
            value = @devices['Device']['WiFi']['MultiAP']['APDevice']['_timestamp']
            value = Time.parse(value).to_time.localtime.strftime('%FT%T.%3NZ')
        rescue => exception
            value =  "unknown" 
            if not @devices['_lastInform'].blank?
                begin 
                    value = Time.parse(@devices['_lastInform']).to_time.localtime.strftime('%FT%T.%3NZ')
                rescue 
                    value =  "unknown" 
                end
            end
        end
        setValue('last_updated', value);
    end


    def init_mesh
        res = get_device_params( @device_params, "Device.WiFi.MultiAP.APDevice.\\d+.MACAddress")
        return if res.nil? or res.size == 0
        
        mesh = []
        res.each do | obj | 
            e = get_mesh_entry(obj)
            mesh << e.getParameters if !e.nil?
        end
        setValue('mesh', mesh)
    end

    def find_device_name(mac, force_controller = false)
        ret = []
        if force_controller
            o = get_device_param_by_specific(@device_params, "Device.DeviceInfo.X_TP_DevName")
            ret << ((!o.nil? and o.include?('value') and !o['value'].blank? ) ? o['value'] : "")
            return ret
        end

        ret[0] = mac
        if mac.blank? or mac.eql?("00:00:00:00:00:00")
            ret[0] = ""
        elsif mac == getValue('mac')
            o = get_device_param_by_specific(@device_params, "Device.DeviceInfo.X_TP_DevName")
            ret[0] = o['value'] if !o.nil? and o.include?('value') and !o['value'].blank?
        else
            a = match_rkey_val( @device_params, "Device.X_TP_RE_MAPPING.\\d+.X_TP_ReMac", mac, true )
            if !a.blank? and !a[0]['value'].blank?
                prefix, match, subfix = a[0]['path'].rpartition('.')
                o = get_device_param_by_specific(@device_params, "#{prefix}.X_TP_DevName")
                ret[0] = o['value'] if !o.nil? and o.include?('value') and !o['value'].blank?
                ret[1] = prefix
            end
        end
        return ret
    end
    

    # input:
    #   mac_1 = {"MACAddress"=>{"path"=>"Device.WiFi.MultiAP.APDevice.1.MACAddress", "value"=>"AA:BB:CC:DD:EE:01", "writable"=>"false"}
    # output:
    #
    # limit: 
    #   For each entry of Device.WiFi.MultiAP.APDevice, those parameters are required. 
    #     Device.WiFi.MultiAP.APDevice.{i}.MACAddress
    #     Device.WiFi.MultiAP.APDevice.{i}.BackhaulMACAddress
    #     Device.WiFi.MultiAP.APDevice.{i}.BackhaulLinkType
    def get_mesh_entry(mac_1)
        return nil if mac_1.nil? or !mac_1.include?('path')

        prefix, match, subfix = mac_1['path'].rpartition('.')
        ap_device_path = prefix

        mac_2 = get_device_param_by_specific(@device_params, "#{prefix}.BackhaulMACAddress")
        link_type = get_device_param_by_specific(@device_params, "#{prefix}.BackhaulLinkType")
        
        return nil if mac_2.blank? or link_type.blank? or !mac_2.include?('value') or !link_type.include?('value')
        mac_1['device_name'] = mac_1['value']
        mac_2['device_name'] = mac_2['value']
        
        if mac_2['value'].blank? or mac_2['value'].eql?("00:00:00:00:00:00") or link_type['value'].eql?("None")
            mac_1['mesh_type'] = MeshType::CONTROLLER
            mac_1['device_name'] = find_device_name(mac_1['value'], true)[0]
            mac_2['device_name'] = ""
        else 
            mac_1['mesh_type'] = MeshType::SATELLITE
            o = find_device_name(mac_1['value'])
            mac_1['device_name'] = o[0]
            mac_1['re_mapping_path'] = o[1] if !o[1].blank?
            mac_2['device_name'] = find_device_name(mac_2['value'])[0]
        end
        return MeshEntry.new( @device_params, @device_params_groups, mac_1, mac_2, link_type, @controller)
    end
end
