class SsidEntry
    include Parser

    attr_reader :access_point_path, :ssid_path, :radio_path
    def initialize(root, device_params, device_params_groups, access_point_path, ssid_path, radio_path)
        @access_point_path = access_point_path
        @ssid_path = ssid_path
        @radio_path = radio_path
        
        @params = {}
        config = Rails.configuration.subpage_parameters[root]
        config['wlan'].each do |k, v|
            @params[k] = {}
        end
        
        @params['sup_transmit_power'] = {}
        @params['auto_channel_enable'] = {'hidden'=>true}
        @params['WEPKey1'] = {'hidden'=>true}
        @params['WEPKey2'] = {'hidden'=>true}
        @params['WEPKey3'] = {'hidden'=>true}
        @params['WEPKey4'] = {'hidden'=>true}
        @params['SAEPassphrase'] = {'hidden'=>true}
        
        @device_params = device_params
        @device_params_groups = device_params_groups 
        
        #puts "SsidEntry.initialize: @access_point_path = #{@access_point_path}"
        #puts "SsidEntry.initialize: @ssid_path = #{@ssid_path}"
        #puts "SsidEntry.initialize: @radio_path = #{@radio_path}"
        
        initValues()
    end

    def getParameters(params = @params.clone) 
        ret = {}
        @params.each do |field, attributes|
            if attributes.key?('hidden') && attributes['hidden']
                next
            end
            if attributes.size == 0 
                ret[field] = {}
            else 
                v = conv_to_display_string(field, attributes['value'])
                w = attributes['writable']
                ret[field] = {'value'=>v, 'writable'=>w}
            end
        end 
        
        return ret
    end
    
    def conv_to_display_string(field, val) 
        return val if val.nil?
        
        ret = val
        case field
        when 'transmit_power'
            sup_values = @params['sup_transmit_power']['value']
            return ret if sup_values.nil?
            
            arr = sup_values.split(",").inject([]) {|s, x| s << x.to_i; }
            arr = arr.sort.uniq
            if arr.size == 3
                ret = 'Low'    if val.to_i == arr[arr.size-3]
                ret = 'Middle' if val.to_i == arr[arr.size-2]
                ret = 'High'   if val.to_i == arr[arr.size-1]
            end
        when 'sec_encryption'
            case val #TKIP;AES;TKIPandAES;
            when 'TKIPandAES'
                ret = 'Auto'
            end
        end
        return ret
    end
    
    def conv_from_display_string(field, val) 
        return val if val.nil?
        
        ret = val
        case field
        when 'transmit_power'
            sup_values = @params['sup_transmit_power']['value']
            return val if sup_values.nil?
            arr = sup_values.split(",").inject([]) {|s, x| s << x.to_i; }
            arr = arr.sort.uniq
            if arr.size == 3
                ret = arr[arr.size-3].to_s if val == 'Low'   
                ret = arr[arr.size-2].to_s if val == 'Middle'
                ret = arr[arr.size-1].to_s if val == 'High'  
            end
        when 'sec_encryption'
            ret = 'TKIPandAES' if val == 'Auto'
        end
        return ret
    end
    
    
    def is_proprietary_field?(field)
        return false if @params.key? field or @params[field].nil? or @params[field]['path']
        
        ret = false
        path = @params[field]['path']
        return false if path.nil?
        
        prefix, match, subfix = path.rpartition('.')
        if subfix.start_with? "X_TP_"
            ret = true
        end
        return ret
    end

    def getFields()
        return getParameters().keys
    end

    def getValue(field)
        return @params[field].clone
    end
    
    def setValue(field, value, writable, path=nil)
        #puts "setValue: field?#{@params.key?(field)}, field=#{field}, value=#{value}, writable=#{writable}, path=#{path}"
        return false if field.nil? || !@params.key?(field)
        @params[field]['value'] = "#{value}"
        @params[field]['writable'] = "#{writable}"
        @params[field]['path'] = path if !path.nil?
        return true
    end

    def initValues()
        function_names = []
        
        # inner used
        function_names << { "name" => "init_WEPKey1" }
        function_names << { "name" => "init_WEPKey2" }
        function_names << { "name" => "init_WEPKey3" }
        function_names << { "name" => "init_WEPKey4" }
        function_names << { "name" => "init_SAEPassphrase" }
        function_names << { "name" => "init_sup_transmit_power" }
        function_names << { "name" => "init_auto_channel_enable" }
        
        # the parameters are defined in config/tplink/device_subpage_parameters.yml
        function_names << { "name" => "init_index" }
        function_names << { "name" => "init_ap_type" }
        function_names << { "name" => "init_band" }
        function_names << { "name" => "init_radio_enable" }
        function_names << { "name" => "init_enable" }
        function_names << { "name" => "init_ssid" }
        function_names << { "name" => "init_sec_encryption" }
        function_names << { "name" => "init_sec_radius_ip" }
        function_names << { "name" => "init_sec_radius_port" }
        function_names << { "name" => "init_sec_radius_password" }
        function_names << { "name" => "init_sec_KeyPassphrase" }
        function_names << { "name" => "init_BasicAuthenticationMode" }
        function_names << { "name" => "init_Standard" }
        function_names << { "name" => "init_channel" }
        function_names << { "name" => "init_channel_bandwidth" }
        function_names << { "name" => "init_current_channel_bandwidth" }
        function_names << { "name" => "init_transmit_power" }
        function_names << { "name" => "init_sup_channel" }
        function_names << { "name" => "init_sup_security" }
        function_names << { "name" => "init_sup_channel_bandwidth" }
        function_names << { "name" => "init_security_mode" }
        function_names << { "name" => "init_WEPKeyIndex_" }
        function_names << { "name" => "init_WEPKey" }
        function_names << { "name" => "init_band_steering" }
        function_names << { "name" => "init_max_supported_ssids" }

        call_functions(function_names)
    end

    def setValue_by_specific_path(field, specific_path, skip_path = false, new_writable = nil) 
        obj = get_device_param_by_specific(@device_params, specific_path)
        setValue(field, obj['value'], ((new_writable.nil?) ? obj['writable']: new_writable), ((skip_path) ? nil : obj['path']) ) if !obj.nil?
    end

    def init_WEPKey1()
        # Device.WiFi.AccessPoint.1.Security.X_TP_WEPKey1  
        setValue_by_specific_path('WEPKey1', "#{@access_point_path}.Security.X_TP_WEPKey1") if !@access_point_path.nil?
    end

    def init_WEPKey2()
        # Device.WiFi.AccessPoint.1.Security.X_TP_WEPKey2  
        setValue_by_specific_path('WEPKey2', "#{@access_point_path}.Security.X_TP_WEPKey2") if !@access_point_path.nil?
    end

    def init_WEPKey3()
        # Device.WiFi.AccessPoint.1.Security.X_TP_WEPKey3  
        setValue_by_specific_path('WEPKey3', "#{@access_point_path}.Security.X_TP_WEPKey3") if !@access_point_path.nil?
    end

    def init_WEPKey4()
        # Device.WiFi.AccessPoint.1.Security.X_TP_WEPKey4  
        setValue_by_specific_path('WEPKey4', "#{@access_point_path}.Security.X_TP_WEPKey4") if !@access_point_path.nil?
    end 

    def init_SAEPassphrase() 
        # Device.WiFi.AccessPoint.1.Security.SAEPassphrase  
        setValue_by_specific_path('SAEPassphrase', "#{@access_point_path}.Security.SAEPassphrase") if !@access_point_path.nil?
    end

    def init_sup_transmit_power()
        # Device.WiFi.Radio.1.TransmitPowerSupported 
        setValue_by_specific_path('sup_transmit_power', "#{@radio_path}.TransmitPowerSupported", false) if !@radio_path.nil?
    end

    def init_auto_channel_enable() 
        # Device.WiFi.Radio.1.AutoChannelEnable 
        setValue_by_specific_path('auto_channel_enable', "#{@radio_path}.AutoChannelEnable") if !@radio_path.nil?
    end


    def init_index() 
        return if @ssid_path.nil?
        a, b, c = @ssid_path.rpartition('.')
        setValue('index', c, "false")
    end
    
    def init_ap_type()
        # Device.WiFi.AccessPoint.1.X_TP_APType 
        setValue_by_specific_path('ap_type', "#{@access_point_path}.X_TP_APType", false, "false") if !@access_point_path.nil?
    end

    def init_band()
        # Device.WiFi.Radio.1.OperatingFrequencyBand 
        setValue_by_specific_path('band', "#{@radio_path}.OperatingFrequencyBand", false, "false") if !@radio_path.nil?
    end

    def init_radio_enable()
        # Device.WiFi.AccessPoint.1.Enable 
        setValue_by_specific_path('radio_enable', "#{@access_point_path}.Enable") if !@access_point_path.nil?
    end

    def init_enable()
        # Device.WiFi.AccessPoint.1.Enable
        setValue_by_specific_path('enable', "#{@access_point_path}.Enable") if !@access_point_path.nil?
        if @params['ap_type'].nil? or @params['ap_type']['value'] == 'Primary'
            @params['enable']['writable'] = 'false'
            @params['enable'].delete 'path'
        end
    end
    
    def init_ssid()
        # Device.WiFi.SSID.1.SSID
        setValue_by_specific_path('ssid', "#{@ssid_path}.SSID") if !@ssid_path.nil?
    end

    def init_sec_encryption()
        # if EncryptionMode don't exist, use X_TP_WPAWPA2EncryptionMode 
        # Device.WiFi.AccessPoint.1.Security.EncryptionMode = TKIP;AES;TKIP,AES;AES,TKIP;
        # Device.WiFi.AccessPoint.1.Security.X_TP_WPAWPA2EncryptionMode = TKIP;AES;TKIPandAES;

        setValue_by_specific_path('sec_encryption', "#{@access_point_path}.Security.EncryptionMode") if !@access_point_path.nil?

        if @params['sec_encryption'].key? 'value'
            case @params['sec_encryption']['value']
            when 'TKIP,AES', 'AES,TKIP'
              @params['sec_encryption']['value'] = 'TKIPandAES'
              return
            when 'TKIP', 'AES'
              return
            end
        end

        setValue_by_specific_path('sec_encryption', "#{@access_point_path}.Security.X_TP_WPAWPA2EncryptionMode") if !@access_point_path.nil?
        
        # If value = "", return "TKIPandAES" as default valueE
        if @params['sec_encryption'].key? 'value'
            case @params['sec_encryption']['value']
            when ""
                @params['sec_encryption']['value'] = "TKIPandAES"
            end
        end
    end

    def init_sec_radius_ip()
        # Device.WiFi.AccessPoint.1.Security.RadiusServerIPAddr
        setValue_by_specific_path('sec_radius_ip', "#{@access_point_path}.Security.RadiusServerIPAddr") if !@access_point_path.nil?
    end

    def init_sec_radius_port()
        # Device.WiFi.AccessPoint.1.Security.RadiusServerPort
        setValue_by_specific_path('sec_radius_port', "#{@access_point_path}.Security.RadiusServerPort") if !@access_point_path.nil?
    end

    def init_sec_radius_password()
        # Device.WiFi.AccessPoint.1.Security.RadiusSecret
        setValue_by_specific_path('sec_radius_password', "#{@access_point_path}.Security.RadiusSecret") if !@access_point_path.nil?
    end
    
    def init_sec_KeyPassphrase()
        # Device.WiFi.AccessPoint.1.Security.KeyPassphrase
        setValue_by_specific_path('sec_KeyPassphrase', "#{@access_point_path}.Security.KeyPassphrase") if !@access_point_path.nil?
    end
    

    def init_BasicAuthenticationMode()
        # Device.WiFi.AccessPoint.1.Security.X_TP_WEPAuthenticationMode:  None;Shared;Auto;
        setValue_by_specific_path('BasicAuthenticationMode', "#{@access_point_path}.Security.X_TP_WEPAuthenticationMode") if !@access_point_path.nil?
        
        # If value = "", return "Auto" as default value
        if @params['BasicAuthenticationMode'].key? 'value'
            case @params['BasicAuthenticationMode']['value']
            when ""
                @params['BasicAuthenticationMode']['value'] = "Auto"
            end
        end
    end

    def init_Standard()
        # Device.WiFi.Radio.1.OperatingStandards 
        setValue_by_specific_path('Standard', "#{@radio_path}.OperatingStandards") if !@radio_path.nil?
    end

    def init_channel()
        # Device.WiFi.Radio.1.Channel 
        setValue_by_specific_path('channel', "#{@radio_path}.Channel") if !@radio_path.nil?
        setValue('current_channel', @params['channel']['value'], false) if !@params['channel']['value'].nil?
        @params['channel']['value'] = 'Auto' if @params['auto_channel_enable']['value'] == "true"
    end

    def init_channel_bandwidth()
        # Device.WiFi.Radio.1.OperatingChannelBandwidth
        setValue_by_specific_path('channel_bandwidth', "#{@radio_path}.OperatingChannelBandwidth") if !@radio_path.nil?
    end

    def init_current_channel_bandwidth()
        # Device.WiFi.Radio.1.CurrentOperatingChannelBandwidth
        setValue_by_specific_path('current_channel_bandwidth', "#{@radio_path}.CurrentOperatingChannelBandwidth") if !@radio_path.nil?
    end

    def init_transmit_power()
        # Device.WiFi.Radio.1.TransmitPower
        setValue_by_specific_path('transmit_power', "#{@radio_path}.TransmitPower") if !@radio_path.nil?
    end

    def init_sup_channel()
        return if @radio_path.nil?
        # Device.WiFi.Radio.1.PossibleChannels
        # Device.WiFi.Radio.1.AutoChannelSupported
        channel_5g = [36, 40, 44, 48, 52, 56, 60, 64, 100, 104, 108, 112, 116, 120, 124, 128, 132, 136, 140, 144, 149, 153, 157, 161, 165].freeze
        sup_values = []
        possible_channels = get_device_param_by_specific(@device_params, "#{@radio_path}.PossibleChannels")
        auto_channel_supported = get_device_param_by_specific(@device_params, "#{@radio_path}.AutoChannelSupported")
        
        if possible_channels.nil? # if no node, return mex set
            band = nil
            obj = get_device_param_by_specific(@device_params, "#{@radio_path}.OperatingFrequencyBand")
            
            band = obj['value']  if !obj.nil? && !obj['value'].nil? 
            case band
            when "2.4GHz"
                1.upto(13) { |x| sup_values << x; }
            when "5GHz"
                arr = [36,40,44,48,149,153,157,161]
                arr.each { |x| sup_values << x; }
            end
        else 
            value, writable, path =  possible_channels['value'], possible_channels['writable'], possible_channels['path']
            if !value.nil?
                arr = value.split(',')
                arr.each do | v |
                    vv = v.split('-')
                    if vv.size == 1
                        sup_values << v.to_i
                    elsif vv.size == 2
                        range_min = vv[0].to_i > vv[1].to_i ? vv[1].to_i : vv[0].to_i
                        range_max = vv[0].to_i < vv[1].to_i ? vv[1].to_i : vv[0].to_i
                        if channel_5g.include? range_min
                            min_index = channel_5g.index(range_min)
                            max_index = channel_5g.index(range_max)
                            min_index.upto(max_index) { |x| sup_values << channel_5g[x]; }
                        else
                            range_max.downto(range_min) { |x| sup_values << x; } if range_min <= range_max
                        end
                    end
                end
            end
        end

        
        sup_values = sup_values.sort()
        if auto_channel_supported.nil? # if no node, return mex set
            sup_values.unshift("Auto")  
        else
            if auto_channel_supported['value'] == 'true'
                sup_values.unshift("Auto")  
            end
        end
        
       #puts "init_sup_channel: possible_channels = #{possible_channels}"
       #puts "init_sup_channel: auto_channel_supported = #{auto_channel_supported}"
       #puts "init_sup_channel: sup_channel = #{sup_values.join(",")}"
        
        setValue("sup_channel", sup_values.join(","), false)
    end

    def init_sup_security()
        # Device.WiFi.AccessPoint.1.Security.ModesSupported 
        setValue_by_specific_path('sup_security', "#{@access_point_path}.Security.ModesSupported", false, "false") if !@access_point_path.nil?
    end

    def init_sup_channel_bandwidth()
        return if @radio_path.nil?
        # Device.WiFi.Radio.1.SupportedOperatingChannelBandwidths
        sup_values = []
        
        supported_obj = get_device_param_by_specific(@device_params, "#{@radio_path}.SupportedOperatingChannelBandwidths")
        
        if supported_obj.nil? # if no node, return mex set
            band = nil
            obj = get_device_param_by_specific(@device_params, "#{@radio_path}.OperatingFrequencyBand")
            band = obj['value'] if !obj.nil? && !obj['value'].nil? 
            case band
            when "2.4GHz"
                arr = ['Auto', '20MHz', '40MHz']
                arr.each { |x| sup_values << x; }
            when "5GHz"
                arr = ['Auto', '20MHz', '40MHz', '80MHz']
                arr.each { |x| sup_values << x; }
            end
        else 
            value, writable, path = supported_obj['value'], supported_obj['writable'], supported_obj['path']
            if !value.nil?
                arr = value.split(',')
                arr.each do | v |
                    sup_values << v
                end
            end
        end
        
       #puts "init_sup_channel_bandwidth: supported_obj = #{supported_obj}"
       #puts "init_sup_channel_bandwidth: sup_values = #{sup_values.join(",")}"
        
        setValue("sup_channel_bandwidth", sup_values.join(","), false)
    end

    def init_security_mode()
        # Device.WiFi.AccessPoint.1.Security.ModeEnabled 
        setValue_by_specific_path('security_mode', "#{@access_point_path}.Security.ModeEnabled") if !@access_point_path.nil?
    end

    def init_WEPKeyIndex_()
        # Device.WiFi.AccessPoint.1.Security.X_TP_WEPKeyIndex: 1-4
        setValue_by_specific_path('WEPKeyIndex_', "#{@access_point_path}.Security.X_TP_WEPKeyIndex") if !@access_point_path.nil?
    end

    def init_WEPKey()
        return if @access_point_path.nil?
        arr = ['WEPKeyIndex_', 'WEPKey1', 'WEPKey2', 'WEPKey3', 'WEPKey4']
        arr.each do | field |
            return if @params[field]['value'].nil?
        end
        setValue('WEPKey', "", true) 
    end


    def init_band_steering()
        # if
        # Device.X_TP_MESH_WIFI.MeshEnable == true
        # or
        # Device.WiFi.X_TP_BandSteering.Enable  == true
        # band_steering is 'on'

        obj = get_device_param_by_specific(@device_params, 'Device.X_TP_MESH_WIFI.MeshEnable')
        if !obj.nil? && obj['value'] && obj['value'] == 'true'
            setValue('band_steering', 'on', false, 'Device.X_TP_MESH_WIFI.MeshEnable')
            return
        end

        obj = get_device_param_by_specific(@device_params, 'Device.WiFi.X_TP_BandSteering.Enable')
        if !obj.nil?
            setValue('band_steering', ((obj['value'] == 'true') ? 'on': 'off'), false, 'Device.WiFi.X_TP_BandSteering.Enable' )
        else
            setValue('band_steering', 'false', false, nil)
        end
    end

    def init_max_supported_ssids()
        # Device.WiFi.Radio.1.MaxSupportedSSIDs 
        setValue_by_specific_path('max_supported_ssids', "#{@radio_path}.MaxSupportedSSIDs", false) if !@radio_path.nil?
    end

    def add_key(key, value, arr)
        found = false
        arr.each do | e |
            if key == e[0]
                found = true
                e[1] = value
            end
        end
        if !found
            arr << [key, value]
        end
    end 

    def delete_key(key, arr)
        arr.each do | e |
            if key == e[0]
                arr.delete e
            end
        end
    end 

    def has_key(key, arr)
        found = false
        arr.each do | e |
            if key == e[0]
                found = true
                break
            end
        end
        return found
    end 
    
    def generate_set_parameters(params) 
        ret = []
        
        params.each do | k, v |
            params[k] = conv_from_display_string(k, v)
        end
        
        
        params.each do | k, v | 
            case k
            when "sec_KeyPassphrase"
                case params['security_mode'] 
                when "WPA3-Personal"
                    if @params["SAEPassphrase"].key?("path") && !@params["SAEPassphrase"]['path'].nil?
                        res = [k, { @params["SAEPassphrase"]["path"] => v }]
                    else
                        res = generate_set_parameter(k, v) 
                    end
                when "WPA3-Personal-Transition"
                    if @params["SAEPassphrase"].key?("path") && !@params["SAEPassphrase"]['path'].nil?
                        ret << ["SAEPassphrase", { @params["SAEPassphrase"]["path"] => v }]
                    end 
                    res = generate_set_parameter(k, v) 
                else
                    res = generate_set_parameter(k, v)
                end
            else 
                res = generate_set_parameter(k, v)
            end
            ret << res if !res.nil?
        end
        
        reconstruct_set_parameters_for_security(params, ret)
        reconstruct_set_parameters_for_channel(params, ret)


        #ret.each do |f,o|
        #    o.each do |path, val|
        #        obj = get_device_param_by_specific(@device_params, path)
        #        if !obj.nil? && obj['writable'] == 'false'
        #            puts "#{path} is writable? #{obj['writable']}"
        #        end
        #    end
        #end
        #puts "generate_set_parameters: params=#{@params}"
        #puts "generate_set_parameters: ret=#{ret}"
        return ret
    end

    # support single parameter only
    def generate_set_parameter(field, new_val) 
        return nil if @params[field].nil? || (!@params[field]['writable'].nil? && @params[field]['writable'] == "false")
        
        ret = nil
        p = nil
        case field
        when 'ap_type', 'band', 'sup_channel', 'sup_security', 'sup_channel_bandwidth'
            # for read-only field
        when 'radio_enable', 'enable', 'ssid', 'sec_radius_ip', 'sec_radius_port', 'sec_radius_password', 'sec_KeyPassphrase', 'BasicAuthenticationMode', 'Standard', 'channel_bandwidth', 'transmit_power', 'WEPKeyIndex_', 'WEPKey'
            p = generate_set_parameters_by_field(field, new_val) if @params[field].key?("path") && !@params[field]['path'].nil?
        when 'security_mode'
            p = generate_set_parameter_for_security_mode(new_val)
        when 'sec_encryption'
            p = generate_set_parameter_for_sec_encryption(field,new_val) if @params[field].key?("path") && !@params[field]['path'].nil?
        end
        
        ret = [field, p] if !p.nil? && p.size > 0
        return ret
    end

    def generate_set_parameter_for_sec_encryption(field,new_val)
        if new_val == "TKIPandAES"
            ret = {}
            path = @params['sup_security']['path']
            if !@params['sup_security']['path'].match('X_TP_WPAWPA2EncryptionMode')
                new_val = 'TKIP,AES'
            end
        end

        return generate_set_parameters_by_field(field,new_val)
    end

    def generate_set_parameters_by_field(field, new_val)
        ret = {}
        begin
            path = @params[field]['path']
            ret[path] = new_val 
        rescue => exception
            ret = nil
        end
        return ret
    end
    
    def generate_set_parameter_for_security_mode(new_val)
        ret = {}
        
        field = nil
        begin
            sup_values = @params['sup_security']['value']
            return nil if sup_values.nil?
            
            arr = sup_values.split(",")
            return nil if ! arr.include?(new_val)
            
            path = @params['security_mode']['path']
            ret[path] = new_val 
        rescue => exception
            ret = nil
        end
        
        return ret
    end
    
    def reconstruct_set_parameters_for_security(params, ret) 
        security_mode = params['security_mode']
        if security_mode.nil?
            security_mode = @params['security_mode']['value']
        end
        
        fields_none = []
        fields_wep = ['WEPKeyIndex_', 'WEPKey']
        fields_personal = ['sec_encryption', 'sec_KeyPassphrase', 'SAEPassphrase']
        fields_enterprise = ['sec_encryption', 'sec_radius_ip', 'sec_radius_port', 'sec_radius_password']
        
        fields_superset = []
        fields_superset = fields_superset.concat(fields_wep).concat(fields_personal).concat(fields_enterprise).uniq
        
        fields_target = []
        case security_mode
        when 'None'
            fields_target = fields_none
        when 'WEP-64', 'WEP-128'
            fields_target = fields_wep
            target_name = "WEPKey#{params['WEPKeyIndex_']}"
            path = @params[target_name]['path']
            value = params['WEPKey']
            add_key(target_name, {path => value}, ret ) if !path.nil? && !value.nil? 
        when 'WPA-Personal', 'WPA2-Personal', 'WPA-WPA2-Personal', 'WPA3-Personal', 'WPA3-Personal-Transition'
            fields_target = fields_personal
            if params.key?('sec_KeyPassphrase') && !params['sec_KeyPassphrase'].nil?
                psktype = nil
                if has_key('sec_KeyPassphrase', ret) 
                    ret.each do | e |
                        if e[0] == 'sec_KeyPassphrase'
                            k = e[1].keys[0]
                            v = e[1][k]
                           if v.length == 64
                                e[1].delete k 
                                kk = "#{k}".gsub!("KeyPassphrase", "PreSharedKey")
                                e[1][kk] = v
                                psktype = 'Preshared'
                            else
                                psktype = 'KeyPassphrase'
                           end
                        end
                    end
                end
                add_key("X_TP_PSKType", { "#{@access_point_path}.Security.X_TP_PSKType" => psktype }, ret) if not psktype.nil?
            end
        when 'WPA-Enterprise', 'WPA2-Enterprise', 'WPA-WPA2-Enterprise'
            fields_target = fields_enterprise
        end
            
        fields_superset.each do |a| 
            delete_key(a, ret) if !fields_target.include?(a)
        end
    end
    
    
    def reconstruct_set_parameters_for_channel(params, ret)
        
        # 'auto_channel_enable': 
        #    Device.WiFi.Radio.1.AutoChannelEnable
        # 'channel':
        #    Device.WiFi.Radio.1.Channel
        
        begin
            
            sup_values = @params['sup_channel']['value']
            return nil if sup_values.nil?
            
            new_val = params['channel']
            arr = sup_values.split(",")
            return nil if ! arr.include?(new_val)
            
            if new_val == "Auto" 
                path = @params['auto_channel_enable']['path']
                add_key('auto_channel_enable', {"#{path}" => "true"}, ret ) if !path.nil?
            else 
                path = @params['channel']['path']
                add_key('channel', {"#{path}" => new_val}, ret ) if !path.nil?
                
                path = @params['auto_channel_enable']['path']
                add_key('auto_channel_enable', {"#{path}" => "false"}, ret ) if !path.nil?
            end
        rescue => exception
            ret = nil
        end
        
        return ret
    end
    
end
