class WanDynamicEntry < WanEntry
    def initialize(root, device_params, device_params_groups, ip_interface, wan_conn_type)
        super(root, device_params, device_params_groups, ip_interface, wan_conn_type)
        initValues()
    end


    def initValues()
        
        function_names = []
        
        function_names << { "name" => "init_wan_ifc_name" }
        function_names << { "name" => "init_wan_conn_type" }
        function_names << { "name" => "init_vlan_setting" }
        function_names << { "name" => "init_dhcp_unicast" }
        function_names << { "name" => "init_dns_setting" }
        function_names << { "name" => "init_max_mtu" }
        function_names << { "name" => "init_igmp_proxy" }
        function_names << { "name" => "init_nat_setting" }
        function_names << { "name" => "init_default_gw" }
        function_names << { "name" => "init_wan_conn_status" }
        function_names << { "name" => "init_wan_ip_info" }

        call_functions(function_names)
    end

    def init_wan_ip_info()
        res = match_rkey_rval(@device_params, "#{@ip_interface}IPv4Address.\\d+.IPAddress", ".*", true)
        res.each do | obj |
            setValue('ExternalIPAddress', obj['value'], obj['writable'], obj['path'])
        end

        res = match_rkey_rval(@device_params, "#{@ip_interface}IPv4Address.\\d+.SubnetMask", ".*", true)
        res.each do | obj |
            setValue('SubnetMask', obj['value'], obj['writable'], obj['path'])
        end
        
        ###DefaultGateway for BBA 2.7 :Device.IP.Interface.{i}.IPv4Address.1.X_TP_Gateway
        ###DefaultGateway for BBA 2.5 #### Device.Routing.Router.1.IPv4Forwarding.{i}.GatewayIPAddress
        res_gw = @device_params["#{@ip_interface}IPv4Address.1.X_TP_Gateway"]
        if not res_gw.nil?
            setValue('DefaultGateway', res_gw['value'], res_gw['writable'], res_gw['path'])
        else    
            res_gw = match_rkey_val(@device_params, "Device.Routing.Router.\\d+.IPv4Forwarding.\\d+.Interface", "#{@ip_interface}", true)
            res_gw.each do | obj |
                prefix, match, subfix = obj['path'].rpartition('.')
                res = match_rkey_rval(@device_params, "#{prefix}.GatewayIPAddress", ".*", true)
                res.each do | obj1 |
                    setValue('DefaultGateway', obj1['value'], obj1['writable'], obj1['path'])
                end
            end
        end

        value = []
        res_dns = match_rkey_val(@device_params, "Device.DNS.Relay.Forwarding.\\d+.Interface", "#{@ip_interface}")
        res_dns.each do | obj |
            prefix, match, subfix = obj['path'].rpartition('.')
            res = match_rkey_rval(@device_params, "#{prefix}.DNSServer", ".*", true)
            res.each do | obj1 |
                if obj1['value'] != "0.0.0.0,0.0.0.0" and obj1['value'] != ""
                    value.push(obj1['value']) 
                end
            end
        end
        setValue('DNSServers', value.join(","), false)
    end
    def init_wan_conn_status()
        status = "Disconnected" 
        res = match_rkey_rval(@device_params, "#{@ip_interface}Status", ".*", true)
        return if res.nil?

        res.each do | obj |
            res_dhcp = match_rkey_val(@device_params, "Device.DHCPv4.Client.\\d+.Interface", "#{@ip_interface}", true)
            return if res_dhcp.nil?

            res_dhcp.each do | obj1 |
                prefix, match, subfix = obj1['path'].rpartition('.')
                res_dhcp_status = match_rkey_rval(@device_params, "#{prefix}.DHCPStatus", ".*", true)
                return if res_dhcp_status.nil?

                res_dhcp_status.each do | obj2 |
                    if obj['value'].match(/Up/i) 
                        if obj2['value'].match(/Bound/i)
                            status = "Connected"
                        elsif obj2['value'].match(/Init/i)
                            status = "Disconnected"
                        else
                            status = "Connecting"
                        end
                    end 

                    setValue('wan_conn_status', status, obj2['writable'], obj2['path'])
                end
            end
        end   
    end
    
    def init_dhcp_unicast()
        # Device.DHCPv4.Client.{i}.X_TP_Unicast
        res = match_rkey_val(@device_params, "Device.DHCPv4.Client.\\d+.Interface", "#{@ip_interface}")
        return if res.nil?
        
        res.each do | obj |
            prefix, match, subfix = obj['path'].rpartition('.')
            
            ref_table = @device_params_groups[prefix]
            next if ref_table.nil?
            
            unicast = ref_table['X_TP_Unicast']
            next if unicast.nil?
            setValue('dhcp_unicast', unicast['value'], unicast['writable'], unicast['path'])
        end
        
        #if @params['dhcp_unicast']['value'].nil?
        #    setValue('dhcp_unicast', "false", "false")
        #end
    end
    

    
    
    def generate_set_parameters(params) 
        ret = []
        
        # check the given params is valid for nat_enable, full_cone_nat
        if params.key?('full_cone_nat') 
            o, v = nil, nil
            if params.key?('nat_enable')
                v = params['nat_enable'].to_s
            else 
                v = @params['nat_enable']['value'].to_s
            end
            
            if v == 'false' # if nat_enable == false,  full_cone_nat = false
                params['full_cone_nat'] = false
            end
        end
        
        params.each do | k, v | 
            res = generate_set_parameter(k, v)
            ret << res if !res.nil?
        end
        generate_set_parameters_by_defaultGw(params, ret)
        
        reconstruct_set_parameters_for_dns_specific_assign(params, ret)
        
        return ret
    end


    def generate_set_parameter(field, new_val) 
        return nil if @params[field].nil? || (!@params[field]['writable'].nil? && @params[field]['writable'] == "false")
        
        ret = nil
        p = nil
        
        case field
        when 'dhcp_unicast', 'max_mtu', 'igmp_proxy', 'nat_enable', 'full_cone_nat'
            p = generate_set_parameters_by_field(field, new_val) if @params[field].key?("path") && !@params[field]['path'].nil?
        else 
            # can't be modified for an existing interface
        end
        
        #puts "---"
        #puts "generate_set_parameter: field=#{field}, new_val=#{new_val}, obj=#{@params[field]}"
        #puts "generate_set_parameter: @params[field]=#{@params[field]}"
        #puts "generate_set_parameter: p=#{p}"
        ret = [field, p] if !p.nil? && p.size > 0
        return ret
    end
    
    
end
