class WanEntry
    
    attr_reader :ip_interface, :wan_conn_type, :ip_interface_xxtp
    def initialize(root, device_params, device_params_groups, ip_interface, connection_type, ip_interface_xxtp = nil)
        @ip_interface = ip_interface
        @wan_conn_type = connection_type
        
        @params = {}
        config = Rails.configuration.subpage_parameters[root]
        config['wan'][@wan_conn_type].each do |k, v|
            @params[k] = {}
        end
        
        @device_params = device_params
        @device_params_groups = device_params_groups 
        
        # find another ip interface if connection_type = 'l2tp' or 'pptp'
        # combined with another ip interface for X_TP_L2TP and X_TP_PPTP
        if ip_interface_xxtp.nil?
            @ip_interface_xxtp = find_xxtp_ip_interface(ip_interface, connection_type) 
        else
            @ip_interface_xxtp = ip_interface_xxtp
        end
        
        #puts "WanEntry.initialize: @ip_interface=#{@ip_interface}"
        #puts "WanEntry.initialize: @ip_interface_xxtp=#{@ip_interface_xxtp}"  
        #puts "WanEntry.initialize: @wan_conn_type=#{@wan_conn_type}"
        
    end
    
    def find_xxtp_ip_interface(ip_interface, ct) 
        ret = nil
        begin 
            case ct
            when 'l2tp', 'pptp'
                fake_intfs = match_rkey_val(@device_params, "Device.X_TP_FakeIntf.\\d+.Interface", ip_interface, true)
               #puts "find_xxtp_ip_interface: fake_intfs = #{fake_intfs}"
                if fake_intfs.size == 0
                    return nil
                else
                    fake_intfs.each do | fake_intf | 
                        prefix, match, suffix = fake_intf['path'].rpartition('.')
                        
                        # xxtp = l2tp or pptp
                        xxtp_path = (ct == 'l2tp') ? "Device.X_TP_L2TP.Interface.\\d+.LowerLayers" : "Device.X_TP_PPTP.Interface.\\d+.LowerLayers"
                        xxtps = match_rkey_val(@device_params, xxtp_path, "#{prefix}#{match}", true) 
                        if xxtps.size > 0
                            prefix, match, suffix = xxtps[0]['path'].rpartition('.')
                            ifcs = match_rkey_val(@device_params, "Device.IP.Interface.\\d+.LowerLayers", "#{prefix}#{match}") 
                            ret = ifcs[0]['path'].sub("LowerLayers", "") if ifcs.size > 0
                        end
                    end
                    
                end
            else
                ret = nil
            end
        rescue => exception
            ret = nil
        end
        return ret
    end

    def getParameterPaths(_fields)
        ret = []
        _fields.each do |f|
            if @params.key? f 
                ret <<  @params[f]['path']
            end
        end 
        return ret
    end
    
    def getParameters()
        #puts "getParameters: @ip_interface=#{@ip_interface}"
        #puts "getParameters: @ip_interface_xxtp=#{@ip_interface_xxtp}" 
        #puts "getParameters: @wan_conn_type=#{@wan_conn_type}"
       
        ret = {}
        @params.each do |field, attributes|
            if attributes.size == 0
                ret[field] = {}
            else
                v = conv_to_display_string(field, attributes['value'])
                w = attributes['writable']
                ret[field] = {'value'=>v, 'writable'=>w}            
            end
        end 
        return ret
    end
    
    def conv_to_display_string(field, val) 
        return val if val.nil?
        ret = val
        case field
        when 'addressing_type'
            ret = 'dhcp' if val == 'DHCP'
            ret = 'static' if val == 'Static'
        when 'connection_trigger'
            ret = 'Auto' if val == 'AlwaysOn'
            ret = 'OnDemand' if val == 'OnDemand'
            ret = 'Manually' if val == 'Manual'
        when 'wan_conn_status'   
            if val == 'Disconnect'
                ret = 'Disconnected'    
            end
        end
        return ret
    end
    
    def conv_from_display_string(field, val) 
        return val if val.nil?
        ret = val
        case field
        when 'addressing_type'
            ret = 'DHCP' if val == 'dhcp'
            ret = 'Static' if val == 'static'
        when 'connection_trigger'
            ret = 'AlwaysOn' if val == 'Auto'
            ret = 'OnDemand' if val == 'OnDemand'
            ret = 'Manual' if val == 'Manually'
        end
        return ret
    end
    
    def is_proprietary_field?(field)
        return false if @params.key? field or @params[field].nil? or @params[field]['path']
        
        ret = false
        path = @params[field]['path']
        return false if path.nil?
        
        prefix, match, subfix = path.rpartition('.')
        if subfix.start_with? "X_TP_"
            ret = true
        end
        return ret
    end

    def getFields()
        return getParameters().keys
    end

    def getValue(field)
        return @params[field].clone
    end
    
    def setValue(field, value, writable, path=nil)
       #puts "setValue: field?=#{@params.key?(field)}, field=#{field}, value=#{value}, writable=#{writable}, path=#{path}"
        return false if field.nil? || !@params.key?(field)
        @params[field]['value'] = "#{value}"
        @params[field]['writable'] = "#{writable}"
        @params[field]['path'] = path if !path.nil?
        return true
    end

    def init_wan_ifc_name()
        ip_interface_path = "#{@ip_interface}Name"
        
        if @ip_interface.nil? && !@ip_interface_xxtp.nil?
            ip_interface_path = "#{@ip_interface_xxtp}Name"
        end
        
        res = @device_params[ip_interface_path]
        setValue('wan_ifc_name', res['value'], res['writable'], res['path']) if !res.nil?
    end

    def init_wan_conn_type()
        setValue('wan_conn_type', @wan_conn_type, 'false') if !@wan_conn_type.nil?
    end

    def init_vlan_setting(ref_path = "#{@ip_interface}LowerLayers") 
        ref_table = getReferenceTable(ref_path)
        vid_object = (ref_table.nil?) ? nil : ref_table['VLANID']
        return if ref_table.nil? 
        
        vid = 0
        begin
            vid = vid_object['value'].to_i
        rescue => exception
            vid = 0
           #puts "exception happends:\n#{exception}"
           #puts "vid_object :#{vid_object}"
           #puts "#{exception}"
        end
        
        setValue('vlan_id', vid_object['value'], false, vid_object['path']) 
        setValue('vlan_enable', ((vid > 0) ? true: false), false) # need to check for priority-tag setting
        
        res = (ref_table.nil?) ? nil : ref_table['X_TP_VLANPr']
        if !res.nil? 
            v = res['value'].to_i
            setValue('vlan_priority', res['value'], res['writable'], res['path'])  if v >= 0 && v <= 7
        end
        
    end
    
    def init_max_mtu(path = "#{@ip_interface}MaxMTUSize", path2=nil) 
        # Device.IP.Interface.\\d+.MaxMTUSize
        res = @device_params[path]
        setValue('max_mtu', res['value'], res['writable'], res['path']) if !res.nil?
        if !path2.nil?
            res2 = @device_params[path2]
            if !res2.nil?
                writable = (res['writable'] == 'true' and res2['writable'] == 'true') ? "true" : "false"
                @params['max_mtu']['writable'] = writable
            end
        end
    end
    
    def init_igmp_proxy() 
        # Device.IP.Interface.\\d+.X_TP_IGMPProxyEnable
        res = @device_params["#{@ip_interface}X_TP_IGMPProxyEnable"]
        setValue('igmp_proxy', res['value'], res['writable'], res['path']) if !res.nil?
    end
    
    def init_nat_setting()
        # Device.NAT.InterfaceSetting.1.Interface = Device.IP.Interface.\\d+.
        res = match_rkey_val(@device_params, "Device.NAT.InterfaceSetting.\\d+.Interface", "#{@ip_interface}", true)
        
        return if res.nil? or res.size == 0
        
        prefix, match, subfix = res[0]['path'].rpartition('.')
        
        v1, v2 = @device_params["#{prefix}.Enable"], @device_params["#{prefix}.X_TP_FullconeNATEnabled"]
        
        setValue('nat_enable', v1['value'], v1['writable'], v1['path'])  if !v1.nil? 
        setValue('full_cone_nat', v2['value'], v2['writable'], v2['path']) if !v2.nil? 

    end
    
    def init_dns_setting()
        res = match_rkey_val(@device_params, "Device.DNS.Relay.Forwarding.\\d+.Interface", "#{@ip_interface}")
        if res.nil?
            setValue('dns_specific_assign', "false", "false")
            return
        end
        
        res.each do | obj |
            prefix, match, subfix = obj['path'].rpartition('.')
            
            ref_table = @device_params_groups[prefix]
            next if ref_table.nil?
            
            v1, v2, v3 = ref_table['Enable'], ref_table['Type'], ref_table['DNSServer']
            next if v1.nil? or v2.nil? or v3.nil?
            
            if v2['value'] == 'Static'
                setValue('specific_dns_servers', v3['value'], v3['writable'], v3['path'])
                setValue('dns_specific_assign', (v3['value']=="") ? "false": "true", v3['writable'])
                return
            end
        end
        
        if @params['dns_specific_assign']['value'].nil?
            setValue('dns_specific_assign', "false", "false")
        end
    end

    def init_default_gw
        gw_name = []

        res = get_device_params(@device_params, 'Device.IP.Interface.\\d+.X_TP_DefaultGatewayUsed')
        if !res.blank?
          res.each do |inf|
            prefix, match, subfix = inf['path'].rpartition('.')
            type = @device_params["#{prefix}.X_TP_ConnType"]
            next if type.nil?

            case type['value']
            when 'DHCP', 'StaticIP', 'PPPoE'
              name = @device_params["#{prefix}.Name"]
              gw_name << name['value'] unless name.nil?
              setValue('defaultGw', name['value'], true) if inf['value'] == 'true'
            when 'L2TP-PPP', 'PPTP-PPP'
              # Internet connection : 'L2TP-PPP','PPTP-PPP'
              # use secondary inf name
              xxtp_inf = @device_params["#{prefix}.LowerLayers"]
              next if xxtp_inf.nil?

              fake_intf = @device_params["#{xxtp_inf['value']}LowerLayers"]
              next if fake_intf.nil?

              second_connection_inf = @device_params["#{fake_intf['value']}Interface"]
              next if second_connection_inf.nil?

              name = @device_params["#{second_connection_inf['value']}Name"]

              gw_name << name['value'] unless name.nil?
              setValue('defaultGw', name['value'], true) if inf['value'] == 'true'
            when 'L2TP-DHCP', 'PPTP-DHCP', 'L2TP-StaticIP', 'PPTP-StaticIP'
              # Secondary connection :'L2TP-DHCP','PPTP-DHCP','L2TP-StaticIP','PPTP-StaticIP'
              name = @device_params["#{prefix}.Name"]
              next if name.nil?

              setValue('defaultGw', name['value'], true) if inf['value'] == 'true'
            end
          end
          gw_name.uniq! if gw_name.uniq

          setValue('sup_defaultGw', gw_name.join(','), false)
        else
            #If both of DestIPAddress, DestSubnetMask are empty or 0.0.0.0, the interface is default gateway.
            #Device.Routing.Router.\\d+.IPv4Forwarding.\\d+.Interface = Device.IP.Interface.\\d+.
            #
            
            arr_sup_defaultGw = []
            res = get_device_params(@device_params, "Device.Routing.Router.\\d+.IPv4Forwarding.\\d+.Enable" )
            res.each do |enable|
                prefix, match, subfix = enable['path'].rpartition('.')
                group = @device_params_groups["#{prefix}"]
                
                dest_ip, dest_mask, interface, origin = group['DestIPAddress']['value'], group['DestSubnetMask']['value'], group['Interface']['value'], group['Origin']['value']
                
                if (dest_ip == "" && dest_mask == "") or (dest_ip == "0.0.0.0" && dest_mask == "0.0.0.0")
                    ifc_name = get_device_param_by_specific(@device_params, "#{interface}Name")
                    type = get_default_gw_interface_type(interface)
                    
                    obj = {}
                    obj['ifc_name'] = ifc_name['value']
                    obj['enable'] = enable
                    obj['origin'] = origin
                    obj['interface'] = interface
                    obj['name_display'] = type[:display_name]
                    obj['name'] = type[:display_name]
                    obj['wan_conn_type'] = type[:wan_conn_type]
                    
                    arr_sup_defaultGw << obj
                end
            end
           
            arr_sup_defaultGw.each do | element |
                gw_name << element['name']
            end
                
            gw_name.uniq! if gw_name.uniq      

            @sup_defaultGw = []
            sup_defaultGw_names = []
        
            arr_sup_defaultGw.each do | element |
                # Example: re-format the selected display name
                # element['name'] = "#{element['name']}##{element['ifc_name']}" if element['name'] != element['ifc_name']
                
                @sup_defaultGw << element
                enable = element['enable']
                if enable['value'] == 'true'
                    setValue('defaultGw', element['name'], true) 
                end
            end
            
            setValue('sup_defaultGw', gw_name.join(',') , false)
            
            #puts "@sup_defaultGw.size = #{@sup_defaultGw.size} = #{@params['sup_defaultGw']}"
            #puts "init_default_gw: @params['defaultGw'] = #{@params['defaultGw']} "
            #puts "init_default_gw: @params['sup_defaultGw'] = #{@params['sup_defaultGw']} "
            #puts "init_default_gw: @sup_defaultGw = #{@sup_defaultGw} "
        end
    end
    
    def init_superfluous
        return if @wan_conn_type != 'l2tp' && @wan_conn_type != 'pptp'
        if @ip_interface.nil?
            setValue('superfluous', true, false)
        else 
            setValue('superfluous', false, false)
        end
    end
    
    def get_default_gw_interface_type(ip_interface)
        ret = {:display_name => nil, :wan_conn_type=>nil}
        
        l2tp= get_l2tp_ip_interface(ip_interface)
        pptp= get_pptp_ip_interface(ip_interface)

        begin
            if l2tp.size + pptp.size == 1 
                ret[:display_name] = get_device_param_by_specific(@device_params, "#{ip_interface}Name")['value']
                ret[:wan_conn_type] = "l2tp_pptp"
                xxtp_interface_path = (l2tp.size > 0) ? l2tp[0]['value'] : pptp[0]['value']
                fake_interfaces = match_rkey_rval(@device_params, "#{xxtp_interface_path}LowerLayers", "Device.X_TP_FakeIntf.\\d+.")
                return ret if fake_interfaces.size == 0
                
                fake_interfaces.each do | fake_interface | 
                    fake_interface_path = fake_interface['value']
                    _ip_interface = get_device_param_by_specific(@device_params, "#{fake_interface_path}Interface")
                    if !_ip_interface.nil?
                        ret[:display_name] = get_device_param_by_specific(@device_params, "#{_ip_interface['value']}Name")['value']
                    end
                end
            else 
                ret[:display_name] = get_device_param_by_specific(@device_params, "#{ip_interface}Name")['value']
                fake_interfaces = match_rkey_val(@device_params, "Device.X_TP_FakeIntf.\\d+.Interface", ip_interface)
                case fake_interfaces.size 
                when 0
                    ret[:wan_conn_type] = "static_dhcp_ppp"
                else
                    ret[:wan_conn_type] = "l2tp_pptp"
                end
            end
        rescue => exception
            puts exception
        end
        return ret
    end
    
    def get_l2tp_ip_interface(interface_path)
        return match_rkey_rval(@device_params, "#{interface_path}LowerLayers", "Device.X_TP_L2TP.Interface.\\d+.")
    end
    
    def get_pptp_ip_interface(interface_path)
        return match_rkey_rval(@device_params, "#{interface_path}LowerLayers", "Device.X_TP_PPTP.Interface.\\d+.")
    end
    
    def add_key(key, value, arr)
        found = false
        arr.each do | e |
            if key == e[0]
                found = true
                e[1] = value
            end
        end
        if !found
            arr << [key, value]
        end
    end 

    def delete_key(key, arr)
        arr.each do | e |
            if key == e[0]
                arr.delete e
            end
        end
    end 

    def has_key(key, arr)
        found = false
        arr.each do | e |
            if key == e[0]
                found = true
                break
            end
        end
        return found
    end 

    def generate_set_parameters_by_field(field, new_val)
        ret = {}
        begin
            path = @params[field]['path']
            ret[path] = new_val 
        rescue => exception
            ret = nil
        end
        return ret
    end
    
    def generate_set_parameters_by_defaultGw(params, ret)
        #Device.IP.Interface.{i}.X_TP_DefaultGatewayUsed

        begin
          # ##for BBA2.6###(C5 Pro)
          res = get_device_params(@device_params, 'Device.IP.Interface.\\d+.X_TP_DefaultGatewayUsed')
          unless res.blank?
            res_new_gw = match_rkey_val(@device_params, 'Device.IP.Interface.\\d+.Name', params['defaultGw'])

            unless res_new_gw.blank?
              prefix, match, subfix = res_new_gw[0]['path'].rpartition('.')
              conn_type = @device_params["#{prefix}.X_TP_ConnType"]
              return if conn_type.nil?

              new_gw_used = @device_params["#{prefix}.X_TP_DefaultGatewayUsed"]

              case conn_type['value']
              when 'L2TP-DHCP', 'PPTP-DHCP', 'L2TP-StaticIP', 'PPTP-StaticIP'
                # Secondary connection :'L2TP-DHCP','PPTP-DHCP','L2TP-StaticIP','PPTP-StaticIP'
                # need to set Internet connection inf to gw inf
                fake_inf = match_rkey_val(@device_params, 'Device.X_TP_FakeIntf.\\d+.Interface', "#{prefix}.")
                return if res_new_gw.blank?

                prefix2, match2, subfix2 = fake_inf[0]['path'].rpartition('.')
                xxtp = if conn_type['value'].match('L2TP').nil?
                         'X_TP_PPTP'
                       else
                         'X_TP_L2TP'
                       end
                xxtp_inf = match_rkey_val(@device_params, "Device.#{xxtp}.Interface.\\d+.LowerLayers", "#{prefix2}.")
                return if xxtp_inf.blank?

                prefix3, match3, subfix3 = xxtp_inf[0]['path'].rpartition('.')
                ip_inf = match_rkey_val(@device_params, 'Device.IP.Interface.\\d+.LowerLayers', "#{prefix3}.")
                return if ip_inf.blank?

                prefix4, match4, subfix4 = ip_inf[0]['path'].rpartition('.')
                new_gw_used = @device_params["#{prefix4}.X_TP_DefaultGatewayUsed"]
              end

              old_gw_used_list = match_rkey_val(@device_params, 'Device.IP.Interface.\\d+.X_TP_DefaultGatewayUsed', 'true')

              if !new_gw_used.blank? && !old_gw_used_list.blank? # ##The given defaultGw '#{defaultGw_new}' found
                old_gw_used = old_gw_used_list[0]

                if new_gw_used['path'] != old_gw_used['path'] # ##The defaultGw is changed
                  add_key('defaultGw_new', { new_gw_used['path'] => 'true' }, ret) if new_gw_used['value'] == 'false'
                  add_key('defaultGw_old', { old_gw_used['path'] => 'false' }, ret) if old_gw_used['value'] == 'true'
                end
              end
            end
            return
          end
            # ##for BBA2.6 END####

            ###for BBA2.5####
            defaultGw_new = params['defaultGw']
            #puts "defaultGw_new= #{defaultGw_new}"
            found = false
            #puts "@sup_defaultGw"
            element_new=nil
            element_old=nil
            
            @sup_defaultGw.each do | element |
                if defaultGw_new == element['name']
                    interface = element['interface']
                    origin = element['origin']
                    enable = element['enable']
                    wan_conn_type = element['wan_conn_type']
                    
                    case wan_conn_type
                    when 'l2tp_pptp'
                        addressing_type_new = params['addressing_type']
                        case addressing_type_new
                        when "static"
                            found = true
                            element_new = element if origin == "Static"
                        when "dhcp"
                            dhcp_status = get_dhcp_status_by_inf(interface)
                            case dhcp_status
                            when "Disconnected"
                                found = true
                                element_new = element if origin == "IPCP"
                            else
                                found = true
                                element_new = element if origin == "DHCPv4"
                            end
                        end
                    when 'static_dhcp_ppp'
                        found = true
                        element_new = element
                    end
                end
                
                if element['enable']['value'] == 'true'
                    element_old = element
                end
            end
            #puts "element_old=#{element_old}"
            #puts "element_new=#{element_new}"
            if !element_old.nil? && !element_new.nil? 
                if element_old['enable']['path'] == element_new['enable']['path']
                    puts "The defaultGw is not changed. Do nothing. "
                    return
                end
            end

            if !element_new.nil?
                add_key('defaultGw_old', {element_old['enable']['path'] => "false"} , ret) if !element_old.nil?
                add_key('defaultGw_new', {element_new['enable']['path'] => "true" } , ret)
            else
                puts "The given defaultGw '#{defaultGw_new}' does not found. Do nothing. "
            end
            ###for BBA2.5 END###
        rescue => exception
            puts exception.message
            puts exception.backtrace.join("\n")
        end
        #puts "generate_set_parameters_by_defaultGw: ret = #{ret}"
        
    end
    
    
    def get_dhcp_status_by_inf(inf_path)
        status = "Disconnected" 
        res = @device_params["#{inf_path}Status"]
        if res['value'].match(/Up/i)
            res_type = match_rkey_val(@device_params, "Device.DHCPv4.Client.\\d+.Interface", "#{inf_path}", true)
            prefix, match, subfix = res_type[0]['path'].rpartition('.')
            res_dhcp_status = @device_params["#{prefix}.DHCPStatus"]

            if res_dhcp_status['value'].match(/Bound/i)
                status = "Connected"
            elsif res_dhcp_status['value'].match(/Init/i)
                status = "Disconnected"
            else
                status = "Connecting"
            end
        end    
        return status
    end
    
    def reconstruct_set_parameters_for_dns_specific_assign(params, ret) 
        if !params.key?("dns_specific_assign") or (@params['dns_specific_assign'].key?('writable') && @params['dns_specific_assign']['writable'] == "false")
            delete_key("specific_dns_servers", ret)
            return
        end
        
        delete_key("specific_dns_servers", ret) if params["dns_specific_assign"] == "false"
    end
    
    # return the reference group (only next level) by the given reference_full_path
    # 
    # ref_full_path: the reference full path
    # return: a hash. The key is the subfix and the value contains 'path', 'value', 'writable'
    # 
    # For example, if "Device.IP.Interface.5.LowerLayers" = "Device.Ethernet.VLANTermination.2."
    #   ref_full_path: "Device.IP.Interface.5.LowerLayers"
    #   return: {
    #      "VLANID"=>{"path"=>"Device.Ethernet.VLANTermination.1.VLANID", "value"=>"0", "writable"=>"true"}, 
    #      "LowerLayers"=>{"path"=>"Device.Ethernet.VLANTermination.1.LowerLayers", "value"=>"Device.Ethernet.Link.4.", "writable"=>"true"}, 
    #      "LastChange"=>{"path"=>"Device.Ethernet.VLANTermination.1.LastChange", "value"=>"0", "writable"=>"false"}, 
    #      "Name"=>{"path"=>"Device.Ethernet.VLANTermination.1.Name", "value"=>"eth0-p1.0", "writable"=>"false"}, 
    #      "Alias"=>{"path"=>"Device.Ethernet.VLANTermination.1.Alias", "value"=>"", "writable"=>"true"}, 
    #      "Status"=>{"path"=>"Device.Ethernet.VLANTermination.1.Status", "value"=>"Up", "writable"=>"false"}, 
    #      "Enable"=>{"path"=>"Device.Ethernet.VLANTermination.1.Enable", "value"=>"true", "writable"=>"true"}
    #   }
    def getReferenceTable(ref_full_path) 
        res = @device_params["#{ref_full_path}"]
        a, b, c= res['value'].rpartition('.')
        return @device_params_groups[a]
    end
end