class WanL2tpEntry < WanEntry
    def initialize(root, device_params, device_params_groups, ip_interface, wan_conn_type, ip_interface_xxtp = nil)
        super(root, device_params, device_params_groups, ip_interface, wan_conn_type, ip_interface_xxtp)
        
        if !@ip_interface.nil? && !@ip_interface_xxtp.nil?
            initValues()
        end
        
        if @ip_interface.nil? && !@ip_interface_xxtp.nil?
            function_names = []
            function_names << { "name" => "init_wan_ifc_name" }
            function_names << { "name" => "init_wan_conn_type" }
            function_names << { "name" => "init_superfluous" }
            call_functions(function_names)
        end
        
    end


    def initValues()
        
        function_names = []

        function_names << { "name" => "init_wan_ifc_name" }
        function_names << { "name" => "init_wan_conn_type" }
        function_names << { "name" => "init_vlan_setting" }
        function_names << { "name" => "init_l2tp_setting" } if !@ip_interface_xxtp.nil?
        function_names << { "name" => "init_l2tp_ip_setting" }
        function_names << { "name" => "init_l2tp_gw_setting" }
        function_names << { "name" => "init_l2tp_dns_setting" }
        function_names << { "name" => "init_max_mtu", "argvs" => ["#{@ip_interface_xxtp}MaxMTUSize", "#{@xxtp_path}MaxMRUSize"] }if !@ip_interface_xxtp.nil?
        function_names << { "name" => "init_default_gw" }
        function_names << { "name" => "init_superfluous" }

        call_functions(function_names)
    end

    def init_l2tp_setting()
        return if @ip_interface_xxtp.nil?
        
        res = get_device_param_by_specific(@device_params, "#{@ip_interface_xxtp}LowerLayers")
        return if res.nil?
        
        @xxtp_path = res['value']
        
        enable = get_device_param_by_specific(@device_params, "#{@xxtp_path}Enable")
        return if enable.nil?
        
        get_l2tp_mapping.each do |k, v|
            obj  = get_device_param_by_specific(@device_params, "#{@xxtp_path}#{v}")
            setValue(k, obj['value'], obj['writable'], obj['path']) if !obj.nil?
        end
        
    end
    
    def get_l2tp_mapping() 
        ret = {}
        ret['username'] = 'Username'
        ret['password'] = 'Password'
        ret['wan_conn_status'] = 'ConnectionStatus'
        ret['serverIpOrName'] = 'ServerIpOrName'
        ret['connection_trigger'] = 'ConnectionTrigger'
        ret['x_max_idle_time'] = 'IdleDisconnectTime'
        ret['x_ip_vpn'] = 'LocalIPAddress'
        ret['x_dns_servers_vpn'] = 'DNSServers'
        return ret
    end

    def init_l2tp_ip_setting()
        # Device.IP.Interface.4.IPv4Address.1.AddressingType
        res = get_device_params(@device_params, "#{@ip_interface}IPv4Address.\\d+.AddressingType")
        return if res.nil?
        
        @sup_addressing_types = [] 
        sup_addressing_type_names = []
        res.each do | addressing_type |
            @sup_addressing_types << addressing_type
            sup_addressing_type_names << addressing_type['value'] if !addressing_type['value'].nil?
            
            prefix, match, subfix = addressing_type['path'].rpartition('.')
            ref_table = @device_params_groups[prefix]
            next if ref_table.nil?
            
            enable, ip, mask = ref_table['Enable'], ref_table['IPAddress'], ref_table['SubnetMask']
            next if enable.nil? or ip.nil? or mask.nil?
            
            if enable['value'] == 'true'
                setValue('addressing_type', addressing_type['value'], addressing_type['writable'], addressing_type['path'])
            end
            
            if addressing_type['value'] == 'Static'
                setValue('x_ip', ip['value'], ip['writable'], ip['path'])
                setValue('x_mask', mask['value'], mask['writable'], mask['path'])
            elsif addressing_type['value'] == 'DHCP'  
                setValue('x_ip_dynamic', ip['value'], ip['writable'], ip['path'])
                setValue('x_mask_dynamic', mask['value'], mask['writable'], mask['path'])   
            end
        end
        
        if sup_addressing_type_names.length != 2
            @params["addressing_type"]['writable'] = "false"
        end
        
        setValue('sup_addressing_type', sup_addressing_type_names.sort.join(',').downcase, "false")
        
    end
    
    def init_l2tp_gw_setting() 
        ###DefaultGateway for BBA 2.7 :Device.IP.Interface.{i}.IPv4Address.1.X_TP_Gateway
        ###DefaultGateway for BBA 2.5 #### Device.Routing.Router.1.IPv4Forwarding.{i}.GatewayIPAddress
        res = @device_params["#{@ip_interface}IPv4Address.1.X_TP_Gateway"]
        if not res.nil?
            type = @device_params["#{@ip_interface}IPv4Address.1.AddressingType"]
            if type['value'] == "DHCP"
                setValue('x_gw_dynamic', res['value'], res['writable'], res['path'])
            else
                setValue('x_gw', res['value'], res['writable'], res['path'])
            end
        else    
            res = match_rkey_val(@device_params, "Device.Routing.Router.\\d+.IPv4Forwarding.\\d+.Interface", "#{@ip_interface}")
            return if res.nil? or res.size == 0
            
            res.each do | interface |
                prefix, match, subfix = interface['path'].rpartition('.')
                
                enable, gwIpAddr, origin = @device_params["#{prefix}.Enable"], @device_params["#{prefix}.GatewayIPAddress"], @device_params["#{prefix}.Origin"]
                return if gwIpAddr.nil?
            
                if origin['value'] == 'Static'
                    setValue('x_gw', gwIpAddr['value'], gwIpAddr['writable'], gwIpAddr['path'])
                elsif origin['value'] == 'DHCPv4'
                    setValue('x_gw_dynamic', gwIpAddr['value'], gwIpAddr['writable'], gwIpAddr['path'])
                end
            end
        end
    end
    
    def init_l2tp_dns_setting()
        res = match_rkey_val(@device_params, "Device.DNS.Relay.Forwarding.\\d+.Interface", "#{@ip_interface}")
        return if res.nil? or res.size == 0
        
        res.each do | obj |
            prefix, match, subfix = obj['path'].rpartition('.')
            
            ref_table = @device_params_groups[prefix]
            next if ref_table.nil?
            
            enable, type, dns_servers = ref_table['Enable'], ref_table['Type'], ref_table['DNSServer']
            next if enable.nil? or type.nil? or dns_servers.nil?
            
            addressing_type = @params['addressing_type']
            if addressing_type.nil? 
                init_l2tp_ip_setting()
                addressing_type = @params['addressing_type']
            end

            case type['value']
            when 'Static'
                # the DNS Servers for Device.IP.Interface.{i}.IPv4Address.{i}.AddressingType = Static
                setValue('x_dns_servers_static', dns_servers['value'], dns_servers['writable'], dns_servers['path']) # active if addressing_type == 'Static'
            when 'DHCPv4'
                 # the DNS Servers for Device.IP.Interface.{i}.IPv4Address.{i}.AddressingType = DHCP
                setValue('x_dns_servers_dynamic', dns_servers['value'], dns_servers['writable'], dns_servers['path']) # active if addressing_type == 'dhcp'
            # when 'DHCPv6'
            #     # the DNS Servers for Device.IP.Interface.{i}.IPv6Address.{i}
            #     setValue('x_dns_servers_dhcpv6', dns_servers['value'], dns_servers['writable'], dns_servers['path']) # active if addressing_type == 'dhcp'     
            end
        end
    end
    
    
    def generate_set_parameters(params) 
        ret = []
        # puts "wan-l2tp_entry.rb generate_set_parameters: @params=#{@params}"
        
        params.each do | k, v |
            params[k] = conv_from_display_string(k, v)
        end
        
        params.each do | k, v | 
            res = generate_set_parameter(k, v)
            ret << res if !res.nil?
        end
        generate_set_parameters_by_defaultGw(params, ret)
        
        reconstruct_set_parameters_for_addressing_type(params, ret)
        reconstruct_set_parameters_for_connection_trigger(params, ret)
        reconstruct_set_parameters_for_xxtp_max_mru_size(params, ret)

        return ret
    end


    def generate_set_parameter(field, new_val) 
        return nil if @params[field].nil? || (!@params[field]['writable'].nil? && @params[field]['writable'] == "false")
        
        ret = nil
        p = nil
        
        case field
        when 'username', 'password', 'addressing_type', 'serverIpOrName', 'x_ip', 'x_mask', 'x_gw', 'x_dns_servers_static', 'connection_trigger', 'max_mtu', 'x_max_idle_time'
            p = generate_set_parameters_by_field(field, new_val) if @params[field].key?("path") && !@params[field]['path'].nil?
        else 
            # can't be modified for an existing interface
        end
        #puts "---"
        #puts "generate_set_parameter: field=#{field}, new_val=#{new_val}, obj=#{@params[field]}"
        #puts "generate_set_parameter: @params[field]=#{@params[field]}"
        #puts "generate_set_parameter: p=#{p}"
        
        ret = [field, p] if !p.nil? && p.size > 0
        return ret
    end
    
    def reconstruct_set_parameters_for_addressing_type(params, ret)
    
        addressing_type_old = @params['addressing_type']['value']
        addressing_type_new = params['addressing_type']
        
        if addressing_type_old.nil? or addressing_type_new.nil? 
            delete_key("x_mask", ret)
            delete_key("x_ip", ret)
            delete_key("x_gw", ret)
            delete_key("x_dns_servers", ret)
            return
        end
        
        if addressing_type_old == addressing_type_new
            case addressing_type_old
            when 'DHCP'
                delete_key("x_mask", ret)
                delete_key("x_ip", ret)
                delete_key("x_gw", ret)
                delete_key("x_dns_servers", ret)
            when 'Static'
                
            end
        else
            obj_dis, obj_en = nil, nil
            @sup_addressing_types.each do | addressing_type |
                prefix, b, c = addressing_type['path'].rpartition('.')
                path = "#{prefix}.Enable"
                obj_dis = {"#{path}" => "false" } if addressing_type['value'] == addressing_type_old
                obj_en  = {"#{path}" => "true" } if addressing_type['value'] == addressing_type_new
            end
            
            add_key('addressing_type_old_disable', obj_dis, ret) if !obj_dis.nil?
            add_key('addressing_type_new_enable', obj_en, ret) if !obj_en.nil?
            
            if addressing_type_new['value'] == 'DHCP'
                delete_key("x_mask", ret)
                delete_key("x_ip", ret)
                delete_key("x_gw", ret)
                delete_key("x_dns_servers", ret)
            end
        end
    end
    
    def reconstruct_set_parameters_for_connection_trigger(params, ret)
        connection_trigger_old = @params['connection_trigger']['value']
        connection_trigger_new = params['connection_trigger']
        
        if connection_trigger_new.nil? or connection_trigger_old.nil? or connection_trigger_old.size == 0 
            delete_key("x_max_idle_time", ret)
            return
        end
        
        
        if connection_trigger_old == connection_trigger_new
            case connection_trigger_old
            when 'OnDemand', 'Manual'
            when 'AlwaysOn'
                delete_key("x_max_idle_time", ret)
            else 
                delete_key("x_max_idle_time", ret)
            end
        else
            case connection_trigger_new
            when 'OnDemand', 'Manual'
            when 'AlwaysOn'
                delete_key("x_max_idle_time", ret)
            else 
                delete_key("x_max_idle_time", ret)
            end
        end
    end

    
    # also set X_TP_L2TP/X_TP_PPTP interface's max mru size = ip.interface.{i}.MaxMTUSize
    def reconstruct_set_parameters_for_xxtp_max_mru_size(params, ret)
        return if !params.key?('max_mtu')

        if @params['max_mtu']['writable'] == 'false' 
            delete_key("max_mtu", ret)
            return
        end
        
        obj  = get_device_param_by_specific(@device_params, "#{@xxtp_path}MaxMRUSize")
        if !obj.nil? && obj['writable'] == 'true'
            add_key('xxtp_max_mru_size', { "#{@xxtp_path}MaxMRUSize" => params['max_mtu'] }, ret)
        end
    end
    
end
