class WanPppoeEntry < WanEntry
    def initialize(root, device_params, device_params_groups, ip_interface, wan_conn_type)
        super(root, device_params, device_params_groups, ip_interface, wan_conn_type)
        initValues()
    end


    def initValues()
        function_names = []
        
        function_names << { "name" => "init_wan_ifc_name" }
        function_names << { "name" => "init_wan_conn_type" }
        function_names << { "name" => "init_vlan_setting" }
        
        function_names << { "name" => "init_ppp_setting" }
        function_names << { "name" => "init_ppp_static_ip" }
        
        function_names << { "name" => "init_dns_setting" }
        function_names << { "name" => "init_max_mtu", "argvs" => ["#{@ip_interface}MaxMTUSize", "#{@ppp_ifc_path}MaxMRUSize"] }
        function_names << { "name" => "init_igmp_proxy" }
        function_names << { "name" => "init_nat_setting" }
        function_names << { "name" => "init_default_gw" }
        function_names << { "name" => "init_ppp_dynamic_ip" }
        
        call_functions(function_names)
    end
    
    def init_vlan_setting()
        res = get_device_param_by_specific(@device_params, "#{@ip_interface}LowerLayers")
        return if res.nil?
        
        super("#{res['value']}LowerLayers")
    end
    
    def init_ppp_setting()
        res = get_device_param_by_specific(@device_params, "#{@ip_interface}LowerLayers")
        return if res.nil?
        
        @ppp_ifc_path = res['value']
        enable = get_device_param_by_specific(@device_params, "#{@ppp_ifc_path}Enable")
        return if enable.nil?
        
        if enable['value'] == 'true'
            get_ppp_mapping.each do |k, v|
                obj  = get_device_param_by_specific(@device_params, "#{@ppp_ifc_path}#{v}")
                setValue(k, obj['value'], obj['writable'], obj['path']) if !obj.nil?
            end
        end
    end
    
    def get_ppp_mapping() 
        ret = {}
        ret['username'] = 'Username'
        ret['password'] = 'Password'
        ret['connection_trigger'] = 'ConnectionTrigger'
        ret['wan_conn_status'] = 'ConnectionStatus'
        ret['x_max_idle_time'] = 'IdleDisconnectTime'
        ret['auth_type'] = 'AuthenticationProtocol'
        ret['pppoe_service_name'] = 'PPPoE.ServiceName'
        ret['pppoe_ac_name'] = 'PPPoE.ACName'
        ret['pppoe_echo_interval'] = 'LCPEcho'
        return ret
    end
    def init_ppp_dynamic_ip()
        res = match_rkey_val(@device_params, "#{@ip_interface}IPv4Address.\\d+.AddressingType", "IPCP")
        if res.nil?
            return
        end
            
        res = @device_params["#{@ip_interface}IPv4Address.1.IPAddress"]
        setValue('ExternalIPAddress', res['value'], res['writable'], res['path']) if not res.nil?

        res = @device_params["#{@ip_interface}IPv4Address.1.SubnetMask"]
        setValue('SubnetMask', res['value'], res['writable'], res['path']) if not res.nil?

        ###DefaultGateway for BBA 2.7 :Device.IP.Interface.{i}.IPv4Address.1.X_TP_Gateway
        ###DefaultGateway for BBA 2.5 #### Device.Routing.Router.1.IPv4Forwarding.{i}.GatewayIPAddress
        res_gw = @device_params["#{@ip_interface}IPv4Address.1.X_TP_Gateway"]
        if not res_gw.nil?
            setValue('DefaultGateway', res_gw['value'], res_gw['writable'], res_gw['path'])
        else
          res_gw = match_rkey_val(@device_params, "Device.Routing.Router.1.IPv4Forwarding.\\d+.Interface", "#{@ip_interface}", true)
          res_gw.each do |obj|
            prefix, match, subfix = obj['path'].rpartition('.')
              res = match_rkey_rval(@device_params, "#{prefix}.GatewayIPAddress", '.*', true)
              res.each do | obj1 |
                setValue('DefaultGateway', obj1['value'], obj1['writable'], obj1['path'])
              end
          end
        end

        value = []
        res_dns = match_rkey_val(@device_params, "Device.DNS.Relay.Forwarding.\\d+.Interface", "#{@ip_interface}")
        res_dns.each do | obj |
          prefix, match, subfix = obj['path'].rpartition('.')
          res = match_rkey_rval(@device_params, "#{prefix}.DNSServer", '.*', true)

          res.each do | obj1 |
            value.push(obj1['value']) if obj1['value'] != '0.0.0.0,0.0.0.0' && obj1['value'] != ''
          end
        end
        setValue('DNSServers', value.join(","), false)
    end
    def init_ppp_static_ip()
        # Device.IP.Interface.4.IPv4Address.1.AddressingType
        res = match_rkey_val(@device_params, "#{@ip_interface}IPv4Address.\\d+.AddressingType", "Static")
        if res.nil?
            setValue('ppp_use_static_ip', "false", "false")
            return
        end
        
        res.each do | obj |
            prefix, match, subfix = obj['path'].rpartition('.')
            
            ref_table = @device_params_groups[prefix]
            next if ref_table.nil?
            
            v1, v2, v3 = ref_table['Enable'], ref_table['Status'], ref_table['IPAddress']
            return if v1.nil? or v2.nil? or v3.nil?
            
            setValue('ppp_use_static_ip', v1['value'], v1['writable'], v1['path'])
            setValue('ppp_external_ip', v3['value'], v3['writable'], v3['path'])
        end
        
        if @params['ppp_use_static_ip']['value'].nil?
            setValue('ppp_use_static_ip', "false", "false")
        end
    end
    
    def generate_set_parameters(params) 
        ret = []
        
        # check the given params is valid for nat_enable, full_cone_nat
        if params.key?('full_cone_nat') 
            o, v = nil, nil
            if params.key?('nat_enable')
                v = params['nat_enable'].to_s
            else 
                v = @params['nat_enable']['value'].to_s
            end
            
            if v == 'false' # if nat_enable == false,  full_cone_nat = false
                params['full_cone_nat'] = false
            end
        end
        
        params.each do | k, v |
            params[k] = conv_from_display_string(k, v)
        end
        
        #puts "generate_set_parameters: @params = #{@params}"
        params.each do | k, v | 
            res = generate_set_parameter(k, v )
            ret << res if !res.nil?
        end
        generate_set_parameters_by_defaultGw(params, ret)
        
        reconstruct_set_parameters_for_dns_specific_assign(params, ret)
        reconstruct_set_parameters_for_ppp_max_mru_size(params, ret)
        reconstruct_set_parameters_for_ppp_use_static_ip(params, ret)
        
        return ret
    end


    def generate_set_parameter(field, new_val) 
        return nil if @params[field].nil? || (!@params[field]['writable'].nil? && @params[field]['writable'] == "false")
        
        ret = nil
        p = nil
        
        case field
        when 'username', 'password', 'connection_trigger', 'x_max_idle_time', 'auth_type', 'pppoe_service_name', 'pppoe_ac_name', 'max_mtu', 'igmp_proxy', 'nat_enable', 'full_cone_nat', 'pppoe_echo_interval', 'ppp_external_ip', 'specific_dns_servers'
            p = generate_set_parameters_by_field(field, new_val) if @params[field].key?("path") && !@params[field]['path'].nil?
        else 
            # can't be modified for an existing interface
        end
        #puts "---"
        #puts "generate_set_parameter: field=#{field}, new_val=#{new_val}, obj=#{@params[field]}"
        #puts "generate_set_parameter: @params[field]=#{@params[field]}"
        #puts "generate_set_parameter: p=#{p}"
        
        ret = [field, p] if !p.nil? && p.size > 0
        return ret
    end
    
    def reconstruct_set_parameters_for_ppp_use_static_ip(params, ret) 
        if !params.key?("ppp_use_static_ip") or (@params['ppp_use_static_ip'].key?('writable') && @params['ppp_use_static_ip']['writable'] == "false")
            delete_key("ppp_external_ip", ret)
            return
        end

        prefix, match, suffix = @params['ppp_external_ip']['path'].rpartition('.')
        enable  = get_device_param_by_specific(@device_params, "#{prefix}.Enable")
        return if enable.nil?
        
        case params['ppp_use_static_ip']
        when 'false'
            delete_key('ppp_external_ip', ret)
            add_key('ppp_use_static_ip', {"#{prefix}.Enable" => "false"}, ret) if enable['value'] == "true"
        when 'true'
            add_key('ppp_use_static_ip', {"#{prefix}.Enable" => "true"}, ret)  if enable['value'] == "false"
        end
    end

    # also set ppp interface's max mru size = ip.interface.{i}.MaxMTUSize
    def reconstruct_set_parameters_for_ppp_max_mru_size(params, ret)
        return if !params.key?('max_mtu')
        
        if @params['max_mtu']['writable'] == 'false' 
            delete_key("max_mtu", ret)
            return
        end
        
        obj  = get_device_param_by_specific(@device_params, "#{@ppp_ifc_path}MaxMRUSize")
        if !obj.nil? && obj['writable'] == 'true'
            add_key('ppp_max_mru_size', { "#{@ppp_ifc_path}MaxMRUSize" => params['max_mtu'] }, ret)
        end
    end
end
