class WanStaticEntry < WanEntry
    def initialize(root, device_params, device_params_groups, ip_interface, wan_conn_type)
        super(root, device_params, device_params_groups, ip_interface, wan_conn_type)
        initValues()
    end

    def initValues()
        function_names = []
        
        function_names << { "name" => "init_wan_ifc_name" }
        function_names << { "name" => "init_wan_conn_type" }
        function_names << { "name" => "init_vlan_setting" }
        function_names << { "name" => "init_static_ip_setting" }
        function_names << { "name" => "init_static_gw" }
        function_names << { "name" => "init_dns_servers" }
        function_names << { "name" => "init_max_mtu" }
        function_names << { "name" => "init_igmp_proxy" }
        function_names << { "name" => "init_nat_setting" }
        function_names << { "name" => "init_default_gw" }
        function_names << { "name" => "init_wan_conn_status" }
        
        call_functions(function_names)

    end

    def init_wan_conn_status()
        status = "Disconnected" 
        res = @device_params["#{@ip_interface}Status"]
        if res['value'].match(/Up/i)
            status = "Connected"
    
        end 

        setValue('wan_conn_status', status, res['writable'], res['path']) if not res.nil?
    end
    def init_static_ip_setting()
        ref_table =@device_params_groups["#{@ip_interface}IPv4Address.1"]
        
        v1, v2, v3 = ref_table['Enable'], ref_table['IPAddress'], ref_table['SubnetMask']
        
        return if v1.nil? or v2.nil? or v3.nil?
        
        if v1['value'] == 'true'
            setValue('static_ip', v2['value'], v2['writable'], v2['path'])
            setValue('static_mask', v3['value'], v3['writable'], v3['path'])
        end 
    end

    def init_static_gw() 
        ###DefaultGateway for BBA 2.7 :Device.IP.Interface.{i}.IPv4Address.1.X_TP_Gateway
        ###DefaultGateway for BBA 2.5 :Device.Routing.Router.{i}.IPv4Forwarding.{i}.GatewayIPAddress
        res = @device_params["#{@ip_interface}IPv4Address.1.X_TP_Gateway"]
        if not res.nil?
            setValue('static_gw', res['value'], res['writable'], res['path'])
        else
            res = match_rkey_val(@device_params, "Device.Routing.Router.\\d+.IPv4Forwarding.\\d+.Interface", "#{@ip_interface}", true)
            return if res.nil? or res.size == 0
            
            res.each do | obj |
                prefix, match, subfix = obj['path'].rpartition('.')
                
                enable, gw = @device_params["#{prefix}.Enable"], @device_params["#{prefix}.GatewayIPAddress"]
            
                return if gw.nil? 
                
                setValue('static_gw', gw['value'], gw['writable'], gw['path'])
            end
        end
    end

    def init_dns_servers()
        res = match_rkey_val(@device_params, "Device.DNS.Relay.Forwarding.\\d+.Interface", "#{@ip_interface}")
        return if res.nil? or res.size == 0
        
        res.each do | obj |
            prefix, match, subfix = obj['path'].rpartition('.')
            
            ref_table = @device_params_groups[prefix]
            next if ref_table.nil?
            
            v1, v2, v3 = ref_table['Enable'], ref_table['Type'], ref_table['DNSServer']
            next if v1.nil? or v2.nil? or v3.nil?
            
            
            if v2['value'] == 'Static' # for dns setting of static interface
                setValue('dns_servers', v3['value'], v3['writable'], v3['path'])
            end
            
        end
    end

    def generate_set_parameters(params) 
        ret = []
        
        # check the given params is valid for nat_enable, full_cone_nat
        if params.key?('full_cone_nat') 
            o, v = nil, nil
            if params.key?('nat_enable')
                v = params['nat_enable'].to_s
            else 
                v = @params['nat_enable']['value'].to_s
            end
            
            if v == 'false' # if nat_enable == false,  full_cone_nat = false
                params['full_cone_nat'] = false
            end
        end
        
        params.each do | k, v | 
            res = generate_set_parameter(k, v)
            ret << res if !res.nil?
        end
        generate_set_parameters_by_defaultGw(params, ret)
        
        return ret
    end

    def generate_set_parameter(field, new_val) 
        return nil if @params[field].nil? || (!@params[field]['writable'].nil? && @params[field]['writable'] == "false")
        
        ret = nil
        p = nil
        
        case field
        when 'static_ip', 'static_mask', 'static_gw', 'dns_servers', 'max_mtu', 'igmp_proxy', 'nat_enable', 'full_cone_nat'
            p = generate_set_parameters_by_field(field, new_val) if @params[field].key?("path") && !@params[field]['path'].nil?
        else 
            # can't be modified for an existing interface
        end
        #puts "---"
        #puts "generate_set_parameter: field=#{field}, new_val=#{new_val}, obj=#{@params[field]}"
        #puts "generate_set_parameter: @params[field]=#{@params[field]}"
        #puts "generate_set_parameter: p=#{p}"
        
        ret = [field, p] if !p.nil? && p.size > 0
        return ret
    end

end
