class WifiScanEntry
  require 'net/http'
  require 'json'
  require 'csv'

  include Util
  include DiagLock
  include Loggers
  include DevicesShowLib2

  @@DIAG_TASK_NAME = "WifiScan"
  
  @@DIAG_STATE = "Device.WiFi.NeighboringWiFiDiagnostic.DiagnosticsState"
  @@DIAG_RESULT = "Device.WiFi.NeighboringWiFiDiagnostic.Result"
 
  # 'diag_lock': is the common lock for wifiscan and tr143
  @@wifi_basic_scan = {:status => "<Empty>", :task_id => "", :total => "", :progress => "start", :rawdata => nil, :diag_lock => "" }
  @@scan_count = 0
  @@last_scan_time = ""
  @@wifi_scan_task = {:get_parametervalue => "", :set_parametervalue => "", :get_bssdescentry => ""}
  
  @results = nil
  @message = {}
  def initialize()
    @message = {} if @message.nil?
    @@wifi_basic_scan[:diag_lock] = ""
    @results = {}
    @results['@@wifi_basic_scan'] = @@wifi_basic_scan
    @results['@@scan_count'] = @@scan_count
    @results['@@last_scan_time'] = @@last_scan_time
    @results['@@wifi_scan_task'] = @@wifi_scan_task
    @results['render_result'] = false
  end
  
  def getParameters()
    @results['@@wifi_basic_scan'] = @@wifi_basic_scan
    @results['@@scan_count'] = @@scan_count
    @results['@@last_scan_time'] = @@last_scan_time
    @results['@@wifi_scan_task'] = @@wifi_scan_task
    @results['@scan_wifi_result'] = @scan_wifi_result
    @results['@wifi_scan_supported'] = @wifi_scan_supported
    @results['@message'] = @message 
    
    #puts "getParameters(): ret = #{@results}"
    
    case @@wifi_basic_scan[:status] 
    when 'TimeOut', 'Completed Successfully'
        if get_diagnostic_lock(@device_id)['result'] == @@DIAG_TASK_NAME
          destory_diagnostic_lock(@device_id) 
        end
    end
    
    return @results
  end

  def process_wifi_scan(http, id, params, is_tr181) 
    @device_id = id
    #puts "process_wifi_scan: id=#{id}"
    #puts "process_wifi_scan: params=#{params.to_json}"
    if params['progress'] == 'scan_start' || params['progress'] == 'scan_refresh' || params['progress'] == 'scan_result'
      @results['render_result'] = true
      task_id = params['task_id']
      progress = params['progress']
      http = create_api_conn()
      
      
      if params['progress'] == 'scan_start' && task_id == ""
          #diag_lock = get_diagnostic_lock(@device_id)['result']
          #if !diag_lock.nil? && diag_lock != @@DIAG_TASK_NAME
          #    @@wifi_basic_scan[:diag_lock] = diag_lock
          #    @@wifi_basic_scan[:status] = "<Empty>"
          #    @@wifi_basic_scan[:task_id] = ""
          #    @@wifi_basic_scan[:total] = ""
          #    @@wifi_basic_scan[:progress] = "scan_start"
          #    @@wifi_basic_scan[:rawdata] = nil
          #    @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
          #    return
          #else 
              @@wifi_basic_scan[:diag_lock] = ""
              wifiscan_timeout = Rails.configuration.device_config['WiFiScanTimeout']  ? Rails.configuration.device_config['WiFiScanTimeout'].to_i  : 65
              save_diagnostic_lock(@device_id, @@DIAG_TASK_NAME, wifiscan_timeout)
          #end
         
          wifiscanbasic(id,task_id,progress)
          return
      end
      wifiscanbasic(id,task_id,progress)
      return
    else
      @wifi_scan_supported = wifi_scan_supported?(http, id)
      #default run no wi-fi scan progress
      @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
      @@scan_count = 0;
      if params['progress'] == nil
        @scan_wifi_result['status'] = "<Empty>"
        @scan_wifi_result['total'] = ""
        @@wifi_scan_task = {:get_parametervalue => "", :set_parametervalue => "", :get_bssdescentry => ""}
      end
      #puts "@scan_wifi_result=#{@scan_wifi_result}"
    end
  end

  def wifiscanbasic(dev_id,task_id,progress)
    http = create_api_conn()
    http.start()
    #puts "wifiscanbasic: dev_id = #{dev_id}"
    #puts "wifiscanbasic: task_id = #{task_id}"
    #puts "wifiscanbasic: progress = #{progress}"
    # wifi scan init ,default scan_count = 0
    if task_id == "" && progress == "scan_start"
      @@scan_count = 0
      @@wifi_scan_task = {:get_parametervalue => "", :set_parametervalue => "", :get_bssdescentry => ""}
    end
    if progress == 'scan_start' || progress == 'scan_result'
      @@scan_count += 1
    else
      @@scan_count += 5
    end
    # status : TimeOut
    if @@scan_count >= 60
      http.finish()
      agileACSLog(4, "[Device Detail][Wi-Fi Scan]#{dev_id}", "Wi-Fi scan timeout.")
      destorywifiscan("TimeOut","timeout")
      return
    end

    if task_id != "" && !task_id.start_with?("update_") # have task_id , query task is done?
      info = wifiscan_in_task_queue(http,task_id,dev_id)
      #puts "task_id = #{task_id}"
      #puts "info = #{info}"
      
      if info == task_id
        @@wifi_basic_scan[:status] = "In Progress"
        @@wifi_basic_scan[:task_id] = task_id
        @@wifi_basic_scan[:total] = ""
        @@wifi_basic_scan[:progress] = progress
        @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
        #render json: @scan_wifi_result
        http.finish();
        return
      else
        @@wifi_scan_task.each do |k, v|
          if task_id == v
            @@wifi_scan_task[k] = ""
          end
        end
        if progress == "scan_start"
          if !isFinished(dev_id)
            task_id = @@wifi_scan_task[:get_parametervalue]
            @@wifi_basic_scan[:status] = "In Progress"
            @@wifi_basic_scan[:task_id] = task_id
            @@wifi_basic_scan[:total] = ""
            @@wifi_basic_scan[:progress] = progress
            @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
            #render json: @scan_wifi_result
            http.finish();
            return
          else 
            # if refresh DiagnosticsState and find completed, goto the next step to refresh results
            progress = "scan_refresh"
          end
        elsif progress == "scan_refresh"
          progress = "scan_result"
        end
        task_id = ""
        
        #puts "  ** progress = #{progress}"
        #puts "  ** task_id = #{task_id}"
      end
    end

    if (task_id == "" || task_id.start_with?("update_")) && progress == "scan_start"
      #puts "wifiscanbasic: device id =#{dev_id}"
      #puts "wifiscanbasic: progress =#{progress}"
      #puts "wifiscanbasic: task_id =#{task_id}"
      if task_id == ""
        task_result = refreshDiagnosticsObject(http, dev_id)
        #puts "wifiscanbasic: task_result=#{task_result}"
        diagnosticsState = nil
        case  task_result[:task_status] 
        when "done"
            diagnosticsState = task_result[:value]
            if diagnosticsState.nil?
                @wifi_scan_supported = false
                return
            end
        when "processing"
          #puts "task_result=#{task_result}"
          @@wifi_basic_scan[:status] = "In Progress"
          @@wifi_basic_scan[:task_id] = "update_"+task_result[:task_id]
          @@wifi_basic_scan[:total] = ""
          @@wifi_basic_scan[:progress] = "scan_start"
          @@wifi_basic_scan[:rawdata] = nil
          @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
          return
        end
      else 
        task_id = task_id.split("_")[1]
        tasks = query_resource(http, 'tasks', {'device' => dev_id}, nil, nil, nil, {'timestamp' => 1})[:result]
        task_status = nil
        if !tasks.to_s.include?(task_id)
          task_status = "done"
          diagnosticsState = query_path(http, dev_id, @@DIAG_STATE)
        else 
          task_status = "processing"
          @@wifi_basic_scan[:status] = "In Progress"
          @@wifi_basic_scan[:task_id] = "update_"+task_id
          @@wifi_basic_scan[:total] = ""
          @@wifi_basic_scan[:progress] = "scan_start"
          @@wifi_basic_scan[:rawdata] = nil
          @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
          return
        end
      end
      
      # ----------------------------------------------
      # setParameterValues  Device.WiFi.NeighboringWiFiDiagnostic.DiagnosticsState = Requested , start 
      # ----------------------------------------------
      #puts "diagnosticsState = #{diagnosticsState}"
      # initial value = "", otherwise: None|Requested|Completed|Error
      case diagnosticsState
      when "", "Completed", "Error"
          #puts "----------- setWlanScanParameterEnable "
          setWlanScanParameterEnable = {:name => "setParameterValues", :parameterValues => [[@@DIAG_STATE, "Requested"]]}
          res_scan = http.post("/devices/#{CGI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(setWlanScanParameterEnable), 'Content-type' => 'application/json')
          #puts setWlanScanParameterEnable
          res_scan_task = JSON.parse(res_scan.body)
          @@wifi_scan_task[:set_parametervalue] = res_scan_task['_id']
          if res_scan.code == "200" || res_scan.code == "202"
            if res_scan.message == "Device is unreachable"
              @message[:warning] = res_scan.message
              #status : <Empty> , progress : off-line
              http.finish()
              destorywifiscan("<Empty>", "off-line")
              return
            end
            agileACSLog(6, "[Device Detail][Wi-Fi Scan]#{dev_id}", "Start Wi-Fi scanning.")
          else
            @message[:error] = "Unexpected error (#{res_scan.code}): #{res_scan.body}"
            http.finish()
            destorywifiscan("<Empty>", "off-line")
            return
          end  
          @@wifi_basic_scan[:status] = "In Progress"
          @@wifi_basic_scan[:task_id] = res_scan_task['_id']
          @@wifi_basic_scan[:total] = ""
          @@wifi_basic_scan[:progress] = "scan_start"
          @@wifi_basic_scan[:rawdata] = nil
          #puts " @@wifi_basic_scan = #{ @@wifi_basic_scan}"
          #puts " @@wifi_scan_task = #{ @@wifi_scan_task}"
          @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
          #render json: @scan_wifi_result
      when "Requested", "None"
          if !isFinished(dev_id)
            task_id = @@wifi_scan_task[:get_parametervalue]
            @@wifi_basic_scan[:status] = "In Progress"
            @@wifi_basic_scan[:task_id] = task_id
            @@wifi_basic_scan[:total] = ""
            @@wifi_basic_scan[:progress] = "scan_start"
            @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
            #render json: @scan_wifi_result
          end
      end
      http.finish();
      return
    elsif task_id == "" && progress == "scan_refresh"
      #puts "progress = #{progress}"
      #puts "task_id = #{task_id}"
      #puts "refresh object for #{@@DIAG_RESULT}"
      info = wifiscanresult(dev_id) # get db last update time
      @@last_scan_time = info['_timestamp']
      refreshWlanScanResult = {:name => "refreshObject", :objectName => @@DIAG_RESULT }
      
      res = http.post("/devices/#{CGI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(refreshWlanScanResult), 'Content-type' => 'application/json')
      res_task = JSON.parse(res.body)
      @@wifi_scan_task[:get_bssdescentry] = res_task['_id']
      #puts "res_task['_id'] =  #{res_task['_id']}"
      #puts "res.code =  #{res.code}"
      if res.code == '200' || res.code == '202'
        if res.message == "Device is unreachable"
          @message[:warning] = res.message
          #status : <Empty> , progress : off-line
          http.finish()
          destorywifiscan("<Empty>", "off-line")
          return
        end
        agileACSLog(6, "[Device Detail][Wi-Fi Scan]#{dev_id}", "Get Wi-Fi scan results.")
      else
        @message[:error] = "Unexpected error (#{res.code}): #{res.body}"
        http.finish()
        destorywifiscan("<Empty>", "off-line")
        return
      end
      @@wifi_basic_scan[:status] = "In Progress"
      @@wifi_basic_scan[:task_id] = res_task['_id']
      @@wifi_basic_scan[:total] = ""
      @@wifi_basic_scan[:progress] = "scan_refresh"
      @@wifi_basic_scan[:rawdata] = nil
      @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
      #render json: @scan_wifi_result
    elsif task_id == "" && progress == "scan_result"
      # is DB redy and get result , if not send "scan_result"
      info = wifiscanresult(dev_id)
      getlastscantime = info['_timestamp']
      if @@last_scan_time != getlastscantime && info['_scanresult'][0][:bssid] != ""
        @@wifi_basic_scan[:status] = "Completed Successfully"
        @@wifi_basic_scan[:task_id] = ""
        @@wifi_basic_scan[:total] = info['_wificount'].to_s
        @@wifi_basic_scan[:progress] = "scan_final"
        @@wifi_basic_scan[:rawdata] = info['_scanresult']
        @@wifi_basic_scan[:pchannels] = @possible_channels
        agileACSLog(6, "[Device Detail][Wi-Fi Scan]#{dev_id}", "Wi-Fi scan completed.")
        @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
        #render json: @scan_wifi_result
      else
        @@wifi_basic_scan[:status] = "In Progress"
        @@wifi_basic_scan[:task_id] = @@wifi_scan_task[:get_bssdescentry]
        @@wifi_basic_scan[:total] = ""
        @@wifi_basic_scan[:progress] = "scan_result"
        @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
        #render json: @scan_wifi_result
      end
    end
    http.finish()
  end

  def wifiscan_in_task_queue(http,task_id,dev_id)
    info = ""
    tasks = query_resource(http, 'tasks', {'device' => dev_id}, nil, nil, nil, {'timestamp' => 1})[:result]
    #puts "tasks = #{tasks}"
    if tasks.to_s.include?(task_id)
      info = task_id
    end
    return info
  end

  def wifiscanresult(dev_id)
      create_api_conn() do |http|
        project = [@@DIAG_RESULT, 'Device.WiFi.Radio']
        res = query_resource(http, 'devices', {'_id' => dev_id}, project)
        if res[:result].size != 1
            raise ActiveRecord::RecordNotFound
        end
        scanentrycheck = res[:result][0]
        device_scan_params = flatten_params(scanentrycheck)
        bands = get_bands(device_scan_params)
        
        #puts "wifiscanresult: bands=#{bands}"
        
        includscanentry = "NeighboringWiFiDiagnostic.Result"  
        possible_channels_entry = 'PossibleChannels'
        tmp ={}
        tmp['_timestamp'] = ""
        wificounts = 0
        scanresult_json = []
        wifi_entry = {:id => "", :band=> "", :ssid => "", :bssid => "", :rssi => "", :channel => "", :security => "", :encryption => "", :bandwidth => "" }
        reset_possible_channels()
        #regex_1 = Regexp.new("Device.WiFi.NeighboringWiFiDiagnostic.Result.\\d.")
        #regex_2 = Regexp.new("Device.WiFi.NeighboringWiFiDiagnostic.Result.\\d.\\w+")

        for v in device_scan_params
            if v['_path'].include?(includscanentry)
                path = v['_path']
                str_arr = v['_path'].split(".")
                if str_arr.last().gsub(/[^0-9]/, '') != "" && wificounts != 0
                    wifi_entry[:id] = wificounts
                    scanresult_json.push(wifi_entry) if wifi_entry[:channel].to_s != "0"
                    wifi_entry = {:id => "", :band=> "", :ssid => "", :bssid => "", :rssi => "", :channel => "", :security => "", :encryption => "", :X_TP_ExtensionChannel => '' }
                end
                case str_arr.last()  
                when 'Result'
                    tmp['_timestamp'] = v['_timestamp']
                when 'Radio'
                    wifi_entry[:band] = bands[v['_value']]
                when 'SSID'
                    wificounts += 1
                    wifi_entry[:ssid] = v['_value'].to_s
                when 'BSSID'
                    wifi_entry[:bssid] = v['_value'].to_s
                when 'SignalStrength'
                    wifi_entry[:rssi] = v['_value'].to_s
                when 'Channel'
                    wifi_entry[:channel] = v['_value'].to_s
                when 'SecurityModeEnabled'
                    wifi_entry[:security] = v['_value'].to_s
                when 'EncryptionMode'
                    wifi_entry[:encryption] = v['_value'].to_s
                when 'OperatingChannelBandwidth'
                  wifi_entry[:bandwidth] = v['_value'].to_s
                when 'X_TP_ExtensionChannel'
                  wifi_entry[:X_TP_ExtensionChannel] = v['_value'].to_s
                end
            end   

            if v['_path'].include?(possible_channels_entry)
              set_possible_channels(v['_value'])
            end
        end
        wifi_entry[:id] = wificounts
        scanresult_json.push(wifi_entry) if wifi_entry[:channel].to_s != "0"
        tmp['_wificount'] = scanresult_json.size
        tmp['_scanresult'] = scanresult_json
        return tmp
      end
  end

  def destorywifiscan(status,progress)
    http = create_api_conn()
    http.start()
    @@wifi_scan_task.each do |key, value|
      if value != ""
        res = http.delete("/tasks/#{value}")
      end
    end
    @@wifi_basic_scan[:status] = status
    @@wifi_basic_scan[:task_id] = ""
    @@wifi_basic_scan[:total] = ""
    @@wifi_basic_scan[:progress] = progress
    @@wifi_basic_scan[:rawdata] = nil
    @@wifi_basic_scan[:diag_lock] = ""
    http.finish()
    @@scan_count = 0
    @scan_wifi_result = ActiveSupport::JSON.decode(@@wifi_basic_scan.to_json)
    #render json: @scan_wifi_result
  end
  
  
    def get_bands(device_scan_params)
        ret = {}
        device_scan_params.each do |param|
            if param['_path'].include?("WiFi.Radio")
                prefix, match, suffix = param['_path'].rpartition('.')
                case suffix 
                when 'OperatingFrequencyBand'
                    ret["#{prefix}."] =  param['_value']
                end
            end
        end
        return ret
    end
    
    def wifi_scan_supported?(http, dev_id)
        res = query_resource(http, 'devices', {'_id' => dev_id, "Device.WiFi.NeighboringWiFiDiagnostic.DiagnosticsState" => {'$exists': true}}, ['Device.WiFi.NeighboringWiFiDiagnostic.DiagnosticsDState'])
        return (res[:result].size == 0) ? false : true
    end
    
    def isFinished(dev_id)
      ret = false
      http = create_api_conn()
      http.start()
      project = [@@DIAG_STATE]
      res = query_resource(http, 'devices', {'_id' => dev_id}, project)
      if res[:result].size != 1
          raise ActiveRecord::RecordNotFound
      end
      res = res[:result][0]
      
      device_scan_params = flatten_params(res)
      device_scan_params.each do | p |
        if p.include?"DiagnosticsState"
            #puts "isFinished: >> the refreshObject task is finished, open db to read DiagnosticsState = #{p['DiagnosticsState']['_value']}"
            case p['DiagnosticsState']['_value']
            when "Requested"
                ret = false
            else
                ret = true
                return ret
            end
        end
      end
      
      refreshWlanScanParameter = {:name => "refreshObject", :objectName => @@DIAG_STATE}
      #puts "isFinished: >> DiagnosticsState is not finished. insert an task:#{refreshWlanScanParameter} "
      
      res_refresh = http.post("/devices/#{CGI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(refreshWlanScanParameter), 'Content-type' => 'application/json')
      res_refresh_task = JSON.parse(res_refresh.body)
      @@wifi_scan_task[:get_parametervalue] = res_refresh_task['_id']
      
      #puts "isFinished: >> inserted task id is #{res_refresh_task['_id']}"
      
      http.finish()
      return ret
    end
    
    def refreshDiagnosticsObject(http, dev_id)
        return refresh_and_load_value(http, dev_id, @@DIAG_STATE, @@DIAG_STATE)
    end
    
    
    # return {:task_id=> x1, :task_status=> x2, :value => x3}
    #    task_id: "" if the refresh task is done. Otherwise, return the task_id of refresh task
    #    task_status:  "processing" or "done"
    #    value:  database value of the given path if refresh task is done. Otherwise, it is nil
    def refresh_and_load_value(http, dev_id, refresh_path, query_path)
        diagnosticsState = nil
        refreshWlanScanParameter = {:name => "refreshObject", :objectName => refresh_path }
        #refreshWlanScanParameter = {'name' => "getParameterValues", 'parameterNames' => [refresh_path] }
        #puts "refresh_path: refreshWlanScanParameter = #{refreshWlanScanParameter}"
        task_result = addTask(http, dev_id, refreshWlanScanParameter)
        diagnosticsState = query_path(http, dev_id, query_path) if task_result[:task_status] == "done"
        task_result[:value] = diagnosticsState
        return task_result
    end
    
    def query_path(http, dev_id, path)
        ret = nil
        project = [path]
        res = query_resource(http, 'devices', {'_id' => dev_id}, project)
        if res[:result].size != 1
            raise ActiveRecord::RecordNotFound
        end
        res = res[:result][0]
        
        suffix = path.split(".").last()
        device_scan_params = flatten_params(res)
        device_scan_params.each do | p |
            if p.include?(suffix)
                ret = p[suffix]['_value']
                return ret
            end
        end
        return ret
    end

    def addTask(http, dev_id, task)
        #puts "addTask: http = #{http}"
        #puts "addTask: dev_id = #{dev_id}"
        #puts "addTask: task = #{task}"
        ret = {:task_id => nil, :task_status => "start", :code => nil, :message => nil}
        res = http.post("/devices/#{CGI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
        task_id = JSON.parse(res.body)['_id']
        ret[:task_id] = task_id
        ret[:task_status] = "processing"
        ret[:code] = res.code
        ret[:message] = (res.code == "200" || res.code == "202" ) ? res.message: res.body
        #puts "addTask: ret 11= #{ret}"
        
        tasks = query_resource(http, 'tasks', {'device' => dev_id}, nil, nil, nil, {'timestamp' => 1})[:result]
        #puts "addTask: tasks = #{tasks}"
        if !tasks.to_s.include?(task_id)
            ret[:task_status] = "done"
            ret[:task_id] = ""
#        else 
#          puts "-----------#{tasks}"
        end
        #puts "addTask: ret 22= #{ret}"
        return ret
    end

  def flatten_params(params, prefix = '')
    output = []
    for n, v in params
      next if n.start_with?('_') or v.is_a?(String)
      v['_path'] = "#{prefix}#{n}"
      output << v

      if not v.include?('_value')
        output += flatten_params(v, prefix ? "#{prefix}#{n}." : "#{n}.")
      end
    end
    return output
  end

end
