# lib/tplink/filters.rb

module DeviceFilters
  # return the javascript filter parameters, defined in config/tplink/index_parameter.yml with filters = 1  
  def get_filters(filters=1, index_parameters_list = nil)
    ret = {}
    if index_parameters_list.nil?
      index_parameters_list = Rails.configuration.index_parameters_tplink
    end

    index_parameters_list.each do | label, definition |
      if filters.nil? || definition['filters'].to_i == filters
        ret[definition['field']] = label
      end
    end
    return ret
  end



  # transform hash to case insensitive based on parameter case_insensitive.
  def case_insensitive_transform(hash, isProfiles = false)
    case_insensitive = []
    if isProfiles == true

    else
      if hash.key?("_case_insensitive")
        case_insensitive = hash["_case_insensitive"]
        hash.delete("_case_insensitive")
      end
      hash.each do |t|
        if t[1].is_a?(String) and case_insensitive.include?(t[0])
          t[1] = "^" + Regexp.escape(t[1]) + "$"
          hash[t[0]] = { "$regex": t[1], "$options": "i"}
        end
      end
    end
    return hash
  end

  # return the parameters mapping [<field>][<root_node>] = <path> defined in 
  # config/tplink/index_parameters.yml and
  # config/tplink/summary_parameters.yml
  private
  def get_parameters_mapping(index_parameters_list = nil, summary_parameters_list = nil)
    mapping = {}
    if index_parameters_list.nil?
      index_parameters_list = Rails.configuration.index_parameters_tplink
    end
    index_parameters_list.each do | label, definition |
      mapping[definition['field']] = {}
      definition['paths'].each do |root_node, path|
        mapping[definition['field']][root_node] = path
      end
    end

    if summary_parameters_list.nil?
      summary_parameters_list = Rails.configuration.summary_parameters_tplink
    end
    summary_parameters_list.each do | label, definition |
      next if mapping.key? definition['field']
      mapping[definition['field']] = {}
      definition['paths'].each do |root_node, path|
        mapping[definition['field']][root_node] = path
      end
    end
    return mapping
  end

DB_FILTER_TYPE_FIELD = 0
DB_FILTER_TYPE_098 =1
#  DB_FILTER_TYPE_181=2  #not support yet

# type: the type for the given hash. Ex: DB_FILTER_TYPE_FIELD  or DB_FILTER_TYPE_098
 def get_db_filter_query(hash, type=DB_FILTER_TYPE_FIELD, return_field_only=false , index_parameters_list = nil, summary_parameters_list = nil)
    ret = {}
    return ret if hash.nil? or hash.length == 0

    mapping = get_parameters_mapping(index_parameters_list, summary_parameters_list)
    tmp181, tmp098 , tmpfield= {}, {}, {}  # stored the result hash for tr-098, tr-181, field

    test = [mapping, hash, type, tmp181, tmp098, tmpfield]
    test = conv_db_query test
    tmp181= test[3]
    tmp098= test[4]
    tmpfield= test[5]

    if return_field_only
      return tmpfield
    end

    tmp181["Device._object"] = {"$exists"=> "true"}  if !tmp181.empty?
    tmp098["InternetGatewayDevice._object"] = {"$exists"=> "true"} if !tmp098.empty?

    if !tmp181.empty? && !tmp098.empty?
      ret = { "$or"=> [ tmp181, tmp098] }
    end
    return ret
  end

  def conv_db_query(input)
    mapping = input[0]
    hash = input[1]
    type = input[2]
    tmp181 = input[3]
    tmp098 = input[4]
    tmpfield = input[5]
    hash.each do |key, val|
      _key_181 = key
      _key_098 = key
      _key_field = key
      case type
      when DB_FILTER_TYPE_FIELD
        if mapping.key? key
          _key_181 = "#{mapping[key]['Device']}"
          _key_098 = "#{mapping[key]['InternetGatewayDevice']}"
          _key_field = key
        end
      when DB_FILTER_TYPE_098
        mapping.each do |_field, _element|
            if _element['InternetGatewayDevice'] == key
              _key_181 = "#{mapping[_field]['Device']}"
              _key_098 = "#{mapping[_field]['InternetGatewayDevice']}"
              _key_field = _field
              next
            end
        end
      end
      if val.is_a? (Hash)
        a = conv_db_query([mapping, val, type, {}, {}, {}])
        tmp181[_key_181] = a[3]
        tmp098[_key_098] = a[4]
        tmpfield[_key_field] = a[5]

      elsif val.is_a? (Array)
        tmp181[_key_181] = []
        tmp098[_key_098] = []
        tmpfield[_key_field] = []
        val.each do |element|
          a = conv_db_query([mapping, element, type, {}, {}, {}])
          tmp181[_key_181] << a[3]
          tmp098[_key_098] << a[4]
          tmpfield[_key_field] << a[5]
        end
      elsif val.is_a? (String)
        tmp181[_key_181] = val.clone
        tmp098[_key_098] = val.clone
        tmpfield[_key_field] = val.clone
      end
    end
    return [mapping, hash, type, tmp181, tmp098, tmpfield]
  end

  def get_projects(index_parameters_list = nil)
    ret = {}
    if index_parameters_list.nil?
      index_parameters_list = Rails.configuration.index_parameters_tplink
    end

    index_parameters_list.each do | label, definition |
      a = []
      ex_fields = ["_lastInform", "IP", "Mesh", "UserName"]
      next if ex_fields.include?(definition['field']) 
      definition['paths'].each do |root_node, path|
        a << "$#{path}._value" 
      end
      ret[definition['field']] = { "$ifNull" => a } if a.length > 0
    end

    ret['IP'] = "$_ipaddress"
    ret['UserName'] = "$_username"
    ret['HomeId'] = { "$ifNull" => ["$_homeId", ""] }
    ret['ZoneId'] = { "$ifNull" => ["$_zoneId", ""] }
    ret['CellPhone'] = { "$ifNull" => ["$_cellPhone", ""] }
    ret['CPF'] = { "$ifNull" => ["$_cpf", ""] }

    ret['_lastInform'] = 1 

    ret['Mesh'] =  get_mesh_project

    ret['isTr181'] = { "$ifNull" => ["$Device._object", false]}

    return ret
  end

  private def get_mesh_project()
    #ret = {"$ifNull" =>[ { "$cond" => { "if" => "$Device.X_TP_RE_MAPPING.1._object", "then" => "Controller", "else" => "-" } } ,"-"]}
    ret = { "$ifNull" => ["$_mesh.type", "N/A"] }
    ret
  end

  private 
  def get_sort_parm(param)
    sort_name = param.start_with?('-') ? param[1..-1] : param
    sort_dir = param.start_with?('-') ? -1 : 1
    return { sort_name => sort_dir}
  end 

  def generate_aggregate_pipeline(params, calculate_total_number=false, match_first=true, skip_unescape=false, index_parameters_list = nil, summary_parameters_list = nil)
    ret = []

    query, sort, skip = nil, nil, nil
    
    if skip_unescape
      query = ActiveSupport::JSON.decode(params['query']) if params.has_key?('query')
    else
      query = ActiveSupport::JSON.decode(URI.unescape(params['query'])) if params.has_key?('query')
    end

    query = case_insensitive_transform(query) if not query.nil? and query.length > 0
    
    sort = get_sort_parm(params[:sort]) if params.include?(:sort)
    
    skip = (Integer(params[:page]) - 1) * Rails.configuration.page_size if params.include?(:page)

    limit = Rails.configuration.page_size

    if calculate_total_number
      skip, limit = nil, nil
    end
    
    if match_first
      ret << {'$match' => get_db_filter_query(query, DB_FILTER_TYPE_FIELD, false, index_parameters_list, summary_parameters_list) } if not query.nil? and query.length > 0
      ret << {'$project' => get_projects(index_parameters_list) }
    else
      ret << {'$project' => get_projects(index_parameters_list) }
      ret << {'$match' => query } if not query.nil? and query.length > 0
    end
    ret << {'$sort' => sort } if not sort.nil?
    ret << {'$skip' => skip} if !skip.nil?
    ret << {'$limit' => limit } if !limit.nil?
    return ret
  end

  def aggregate(http, resource, aggregate_pipeline, method="POST")
    ret = nil
    begin
      require 'util'
      ret = aggregate_resource(http, resource, aggregate_pipeline, method)
      
      if ret[:message] == "404 Not Found" && resource == "devices"
        ret = query_resource_if_devices_not_found(http, resource, aggregate_pipeline, method, index_parameters_list)
      end
    rescue => exception
      # puts 'I am rescued.'
    ensure
      # puts 'Always gets executed.'
    end
    return ret
  end


  def query_resource_if_devices_not_found(http, resource, aggregate_pipeline , method="POST", index_parameters_list = nil)
    ret = (method == "HEAD") ? {:total => 0} : {:total => 0, :result => [], :message => "Unknown"}
    query = {}
    sort = nil
    skip = nil
    limit = nil
    begin
      aggregate_pipeline.each do |element|
        if !element['$match'].blank? && !element['$match']['$or'].blank? &&  element['$match']['$or'].length >=2
          query = element['$match']['$or'][1]
        end
        sort = element['$sort'] if element.key?('$sort') 
        skip = element['$skip'] if element.key?('$skip') 
        limit = element['$limit'] if element.key?('$limit') 
      end
      query["InternetGatewayDevice._object"] = {"$exists"=> "true"}
      
      if !sort.blank?
        mapping = {}
        if index_parameters_list.nil?
          index_parameters_list = ails.configuration.index_parameters_tplink
        end
        index_parameters_list.each do | label, definition |
          mapping[definition['field']] = definition['paths']['InternetGatewayDevice']
        end
        sort = sort.transform_keys(&mapping.method(:[]))
      end
      
      case method
      when "POST"
        #sort = get_sort_parm(params[:sort]) if params.include?(:sort)
        #skip = (Integer(params[:page]) - 1) * Rails.configuration.page_size if params.include?(:page)
        #limit = Rails.configuration.page_size
        res = query_resource(http, 'devices', query, nil, skip, limit, sort)
      when "HEAD"
        args = {}
        args['query'] = ActiveSupport::JSON.encode(query) if query
        args['skip'] = skip if skip
        args['limit'] = limit if limit
        args['sort'] = ActiveSupport::JSON.encode(sort) if sort
        res = http.head("/#{resource}?#{args.to_query}") 
      end


      if method == "HEAD"
        ret[:total] = res["total"]
        return ret
      end

      if !res.blank?
        ret[:total] = res[:total]
        if method == "POST"
          ret[:timestamp] = res[:timestamp]
          ret[:message] = res[:message]
          devices = res[:result]
          devices.each do | device | 
            hash = {}
            hash['_id'] = device['_id']
            if index_parameters_list.nil?
              index_parameters_list = ails.configuration.index_parameters_tplink
            end
            index_parameters_list.each do | label, definition |
              hash[definition['field']] = ""
            end

            hash['SerialNumber'] = helpers.get_param("InternetGatewayDevice.DeviceInfo.SerialNumber._value", device)
            hash['ProductClass'] = helpers.get_param("InternetGatewayDevice.DeviceInfo.ProductClass._value", device)
            hash['ModelName'] = helpers.get_param("InternetGatewayDevice.DeviceInfo.ModelName._value", device)
            hash['SwVers'] = helpers.get_param("InternetGatewayDevice.DeviceInfo.SoftwareVersion._value", device)
            hash['HwVers'] = helpers.get_param("InternetGatewayDevice.DeviceInfo.HardwareVersion._value", device)
            hash['MAC'] = helpers.get_param("InternetGatewayDevice.LANDevice.1.LANEthernetInterfaceConfig.1.MACAddress._value", device)
            hash['IP'] = helpers.get_param("_ipaddress", device)
            hash['Mesh'] = "N/A"
            hash['_lastInform'] = device['_lastInform']
            
            ret[:result] << hash
          end
        end
      end
    rescue => exception
        puts "exception occurs:\n #{exception}"
        puts exception.backtrace 
    end
    
    #puts "query_resource_by_device_not_found: ret = #{ret}"
    return ret
  end
end
