# module for devices
module DevicesShowLib
  include Util

  def connected_result(flash_type, flash_message, object, log_level, log_message)
    flash.now[flash_type] = flash_message
    agileACSLog(log_level, object, log_message)
  end

  def insert_preload_task(escaped_device_id, http, task, object, connection_request = false)
    timeout = 3000
    task['preload'] = '1' unless task.nil?
    url = "/devices/#{escaped_device_id}/tasks?timeout=#{timeout}"
    url += '&connection_request' if connection_request
    res = http.post(url, ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
    if res.code == '200'
      agileACSLog(6, object, 'Device is refreshing.')
    elsif res.code == '202'
      agileACSLog(4, object, res.message.to_s)
    else
      agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
    end
  end

  def reload_tree(is_tr181, escaped_device_id, http, object, connect_status = false)
    cpe_connected = false
    if connect_status == false
      timeout = 3000
      res = http.post("/devices/#{escaped_device_id}/tasks?timeout=#{timeout}&connection_request", nil)
      cpe_connected = true if res.code == '200'
    elsif connect_status == true
      cpe_connected = true
    end
    # if connected
    return unless cpe_connected == true

    arr = is_tr181 ? %w[
      Device.Ethernet Device.PPP Device.X_TP_FakeIntf Device.X_TP_L2TP Device.X_TP_PPTP
      Device.IP Device.InterfaceStack Device.DHCPv4 Device.NAT Device.DNS Device.Routing
      Device.WiFi.Radio Device.WiFi.SSID Device.WiFi.AccessPoint Device.WiFi.X_TP_BandSteering.Enable
      Device.WiFi.NeighboringWiFiDiagnostic.DiagnosticsState Device.X_TP_MESH_WIFI.MeshEnable Device.XMPP.Connection.1.Status
    ] : %w[InternetGatewayDevice.WANDevice InternetGatewayDevice.LANDevice InternetGatewayDevice.XMPP.Connection.1.Status]
    task = { 'name' => 'getParameterValues', 'parameterNames' => arr }
    insert_preload_task(escaped_device_id, http, task, object, true)
  end

  def device_connected?(is_tr181, escaped_device_id, http, log, object, preload = 0)
    ret = 'reachable'
    begin
      cpe_status, task_status = check_cpe_status(escaped_device_id, http)
      case cpe_status
      when 0
        case task_status
        when 0, 1
          reload_tree(is_tr181, escaped_device_id, http, object, true) if preload == 1
          connected_result(:success, I18n.t(:device_is_reacahble, 'scope': :devices), object, 6, 'Device is reachable.') if log == 1
          return ret, task_status
        else
          ret = 'busy'
          connected_result(:warning, I18n.t(:device_is_busy, 'scope': :devices), object, 4, 'Device is busy.') if log == 1
          return ret, 2
        end
      when 1
        ret = 'unreachable'
        reload_tree(is_tr181, escaped_device_id, http, object, true) if preload == 1
        connected_result(:warning, I18n.t(:device_is_unreachable, 'scope': :devices), object, 4, 'Device is unreachable.') if log == 1
      else
        ret = 'error'
        connected_result(:error, I18n.t(:unexpecte_error, 'scope': :devices), object, 3, 'Unexpected error.') if log == 1
      end
    rescue StandardError => e
      puts "exception occurs:\n #{e}"
      puts e.backtrace
      ret = 'exception'
    end
    return ret
  end

  def system_xmpp_enable?(http)
    ret = 'disable'
    res = http.get('/devices/xmpp')
    if res.code == '200'
      system_setting = JSON.parse(res.body)
      ret = system_setting['xmpp']
    end
    ret
  end

  def check_tab_valid(tab, id, http, write = false, params)
    list = DeviceTabList.new(http, id).get_list()
    @tab_list = list.except('cpe_type')
    @is_mesh_satellite = list['cpe_type'] == 'Mesh Satellite'
    if write && @tab_list[tab] != 'w'
      case tab
      when 'lan', 'wan', 'wlan'
        xxan_ignore_keys = %w[delete_items reset_list refresh_summary]
        xxan_ignore_keys.each { |key| return true if params.key?(key) }
      when 'mesh'
        mesh_ignore_keys = %w[refresh_summary commit]
        mesh_ignore_keys.each { |key| return true if params.key?(key) }
      when 'tr_tree'
        return true if params.key?('refresh_summary')

        tr_ignore_commit_keys = %w[getParameterValues deleteObject refreshObject refresh_summary]
        commit = ActiveSupport::JSON.decode(params['commit'])
        tr_ignore_commit_keys.each { |key| return true if commit[0]['name'] == key }
      end
      return false
    end
    return false if @tab_list[tab].nil?

    true
  end

  def valid_tab?(is_tr181)
    valid = false
    case params['tab']
    when 'info', 'lan', 'wan', 'wlan', 'tr_tree', 'diagnostics', 'actions'
      valid = true
    when 'log'
      valid = TpConfig.get_config('features-enable')
    when 'mesh'
      # mesh is valid on 181 devices and pro feature
      valid = is_tr181 && TpConfig.get_config('features-enable')
    else
      valid = false
    end
    # check permissions for 181
    valid = check_tab_valid(params[:tab], params[:id], create_api_conn(), false, params) if valid && is_tr181
    valid
  end

  def refresh(device_id, http, params, object, success_msg = 'refresh successfully', set_flash_msg = true)
    return if http.nil? || params.nil? || params.empty?

    timeout = 3000
    task = { 'name' => 'getParameterValues', 'parameterNames' => params }
    res = http.post("/devices/#{device_id}/tasks?timeout=#{timeout}&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
    log_http_res(object, res, success_msg, set_flash_msg)
  end

  def log_http_res(object, res, success_msg = 'ok', set_flash_msg = true)
    return if res.nil?

    case res.code
    when '200'
      flash[:success] = success_msg if set_flash_msg
      agileACSLog(6, object, success_msg)
    when '202'
      flash[:warning] = res.message if set_flash_msg
      agileACSLog(4, object, res.message.to_s)
    else
      flash[:error] = "Unexpected error (#{res.code}): #{res.body}" if set_flash_msg
      agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
    end
  end

  def get_data(http, device_id, tab)
    projection = get_projection(tab)
    res = query_resource(http, 'devices', { '_id' => device_id }, projection)
    raise ActiveRecord::RecordNotFound if res[:result].size != 1

    res[:result][0]
  end

  def get_projection(tab, input_root = 'Device')
    return Rails.configuration.device_info_parameters[input_root] if input_root.eql?('InternetGatewayDevice')

    ret = []
    Rails.configuration.device_info_parameters[input_root].each do |element|
      tabs = element['tabs'].split(/,/)
      tabs.each do |t|
        ret << element['object'] if t.squish == tab
      end
    end

    ret << (input_root == 'Device' ? 'Device.WiFi.X_TP_BandSteering.Enable' : 'InternetGatewayDevice.WiFi.X_TP_BandSteering.Enable') if tab == 'wlan'

    ret.size.positive? ? ret : nil
  end

  def getParameter(http, id, _commit, inf)
    task = ActiveSupport::JSON.decode(params['commit'])[0]
    unless inf.nil?
      d = DeviceEntry.new(http, id, 'wan')
      task['parameterNames'] = d.get_wan_entry_paths(inf, task['parameterNames'])
    end
    res = http.post("/devices/#{CGI.escape(id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
    http.finish()
    return true if res.code == '200'

    false
  end

  def update_task(is_tr181)
    escaped_device_id = CGI.escape(params[:id])
    http = create_api_conn()
    object = "[Device Detail][Refresh]#{escaped_device_id}"
    cpe_status, task_status = device_connected?(is_tr181, escaped_device_id.to_s, http, 1, object)
    if params.include? 'refresh_summary'
      can?(:read, 'devices') do
        if task_status != 2
          to_refresh = ActiveSupport::JSON.decode(params['refresh_summary'])
          to_refresh['objects'].each do |o|
            task = { 'name' => 'refreshObject', 'objectName' => o }
            http.post("/devices/#{escaped_device_id}/tasks", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
          end

          to_refresh['custom_commands'].each do |o|
            task = { 'name' => 'customCommand', 'command' => "#{o[16..-1]} get" }
            http.post("/devices/#{escaped_device_id}/tasks", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
          end

          device_id = escaped_device_id
          success_msg = nil
          if is_tr181
            case params[:tab]
            when 'mesh'
              unless params[:controllerId].blank?
                device_id = CGI.escape(params[:controllerId])
                save_diagnostic_lock(device_id, 'Mesh', 60)
                success_msg = 'Mesh topology is refreshing'
              end
            end
          end

          task = { 'name' => 'getParameterValues', 'parameterNames' => to_refresh['parameters'] }
          res = http.post("/devices/#{device_id}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
          if res.code == '200'
            flash[:success] = success_msg.blank? ? 'Device is refreshing' : success_msg
            agileACSLog(6, object, 'Device is refreshing.')
          elsif res.code == '202'
            message = res.message
            message = message + ', task queued.' if message == 'Device is unreachable'
            flash[:warning] = message
            agileACSLog(4, object, res.message.to_s)
          else
            flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
            agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
          end
        end
      end
    end

    if params.include? 'add_tag'
      can?(:create, 'devices') do
        tag = ActiveSupport::JSON.decode(params['add_tag']).strip
        res = http.post("/devices/#{escaped_device_id}/tags/#{CGI.escape(tag)}", nil)

        if res.code == '200'
          flash[:success] = 'Tag added'
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        end
      end
    end

    if params.include? 'remove_tag'
      can?(:delete, 'devices') do
        tag = ActiveSupport::JSON.decode(params['remove_tag']).strip
        res = http.delete("/devices/#{escaped_device_id}/tags/#{CGI.escape(tag)}", nil)

        if res.code == '200'
          flash[:success] = 'Tag removed'
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        end
      end
    end

    if params.include? 'delete_items'
      can?(:delete, 'devices') do
        tag = ActiveSupport::JSON.decode(params['delete_items'])
        type = tag[0]['type']
        tag[0].delete('type')

        res = http.put("/csmenus/#{CGI.escape(type.downcase)}", tag[0].to_json(), 'Content-type' => 'application/json')

        if res.code == '200'
          flash[:success] = 'Items removed'
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        end
      end
    end

    if params.include? 'reset_list'
      can?(:delete, 'devices') do
        tag = ActiveSupport::JSON.decode(params['reset_list'])
        type = tag[0]['type']

        res = http.delete("/csmenus/#{CGI.escape(type.downcase)}", nil)

        if res.code == '200'
          flash[:success] = 'List reset'
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        end
      end
    end

    if params.include? 'commit'
      tasks = ActiveSupport::JSON.decode(params['commit'])
      tasks.each_with_index do |task, index|
        case task['name']
        when 'getParameterValues'
          action = :read
          if is_tr181
            if params.include?('tab') && (params[:tab] == 'wan')
              d = DeviceEntry.new(http, params[:id], params[:tab])
              inf_name = params['inf_name']
              task['parameterNames'] = d.get_wan_entry_paths(inf_name, task['parameterNames'])
            end
          end

        when 'setParameterValues'
          action = :update

          if is_tr181
            # for lan, wan, wlan-primary ssid
            if task['parameterValues'].is_a?(Hash)
              d = DeviceEntry.new(http, params[:id], params[:tab])
              task['parameterValues'] = d.generate_set_parameters(task['parameterValues'])
            end

            # for wlan-mssid
            if task['parameterValues'][0].is_a?(Hash)
              d = DeviceEntry.new(http, params[:id], params[:tab])
              tmp_array = []

              task['parameterValues'].each do |n|
                tmp = d.generate_set_parameters(n)
                tmp_array += tmp
              end
              task['parameterValues'] = tmp_array
            end
          end
          if task['diagnostic']
            @diag_flag = true
            if task['diagnostic'] != 'UDPEchoConfig'
              save_diagnostic_lock(params[:id], task['diagnostic'], 60)
            else
              handle_diagnostics_task(params, task, is_tr181)
            end
            task = get_diagnostic(params[:id], task, is_tr181)
          end
        when 'addObject'
          action = :create
        when 'deleteObject'
          action = :delete
        when 'reboot'
          action = :update
        when 'factoryReset'
          action = :update
        when 'download'
          action = :update
        else
          action = :update
        end
        resource = 'devices'
        object = "[Device Detail][Setting]#{escaped_device_id}"
        if task_status != 2 or (task_status == 2 and @diag_flag)
          if index < tasks.count - 1
            res = http.post("/devices/#{escaped_device_id}/tasks", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
            get_parameters_after_set_parameters_failed(task, res, @diag_flag)
          else
            res = if !@diag_flag
                    http.post("/devices/#{escaped_device_id}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
                  else
                    http.post("/devices/#{escaped_device_id}/tasks?timeout=3000&connection_request&device_online_threshold", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
                  end
            get_parameters_after_set_parameters_failed(task, res, @diag_flag)
            if res.code == '200'
              flash[:success] = 'Tasks committed'
              agileACSLog(6, object, "Tasks committed #{action} #{resource}")
              if task['diagnostic']
                handle_diagnostics_task(params, task, is_tr181) if task['diagnostic'] == 'UDPEchoConfig'
              end
            elsif res.code == '202'
              message = res.message
              message = message + ', task queued.' if message == 'Device is unreachable'
              flash[:warning] = message
              agileACSLog(4, object, "#{message} #{action} #{resource}")
            else
              flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
              agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
            end
          end
        end
      end
    end
    redirect_to action: :show, tab: params[:tab], isTr181: is_tr181
  end

  def get_parameters_after_set_parameters_failed(task, res, diag_flag)
    # if res.code != 200, res.message = "Task faulted" or "Task timeout"
    # refresh mondoDB if CPE failed to set Parameters
    if (res.code != '200') && TpConfig.get_config('AUTO_GET_AFTER_SET_TIMEOUT_OR_FAULT') && (task['name'] == 'setParameterValues') && !diag_flag
      unless task['parameterValues'].empty?
        task['name'] = 'getParameterValues'
        path_list = []
        task['parameterValues'].each do |element|
          path = element[0]
          prefix, _match, _subfix = path.rpartition('.')

          x1 = nil
          path_list.each_with_index do |o, i|
            if o.start_with?(prefix) || prefix.start_with?(o)
              x1 = o.length < prefix.length ? o : prefix
              path_list[i] = x1
            end
            break unless x1.nil?
          end
          path_list.push prefix if x1.nil?
        end
        unless path_list.empty?
          task.delete('parameterValues')
          task['parameterValues'] = path_list
          http.post("/devices/#{CGI.escape(params[:id])}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
        end
      end
    end
  end

  def handle_diagnostics_task(params, task, is_tr181)
    if task['Enable']
      if task['timeout']
        save_diagnostic_lock(params[:id], task['diagnostic'], 60 * task['timeout'].to_i)
      else
        save_diagnostic_lock(params[:id], task['diagnostic'], 60 * 5)
      end
    else
      if is_tr181
        send_diagnostic_refresh(params[:id], 'Device.IP.Diagnostics.UDPEchoConfig')
      else
        send_diagnostic_refresh(params[:id], 'InternetGatewayDevice.UDPEchoConfig')
      end
      destory_diagnostic_lock(params[:id])
    end
  end
end
