# module for devices
module DevicesShowLib2
  include DevicesShowLib
  include Util

  def refresh(device_id, http, params, object, success_msg = 'refresh successfully', set_flash_msg = true)
    return if http.nil? || params.nil? || params.empty?

    timeout = 3000
    task = { 'name' => 'getParameterValues', 'parameterNames' => params }
    res = http.post("/devices/#{CGI.escape(device_id)}/tasks?timeout=#{timeout}&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
    log_http_res(object, res, success_msg, set_flash_msg)
  end

  def reset_possible_channels
    @possible_channels = {:chans2g => [], :chans5g => []}
  end

  def set_possible_channels(possible_channels_value)
    chans = possible_channels_value.split(',')
    chans.each do |item|
      converted = item.split('-')
      case converted.length
      when 1
        if converted[0].to_i > 30
          @possible_channels[:chans5g].push(converted[0].to_i) unless @possible_channels[:chans5g].include?(converted[0].to_i)
        else
          @possible_channels[:chans2g].push(converted[0].to_i) unless @possible_channels[:chans2g].include?(converted[0].to_i)
        end
      when 2
        start_channel = converted[0].to_i
        end_channel = converted[1].to_i + 1 # we need to include end channel
        step_width = start_channel > 30 ? 4 : 1
        (start_channel...end_channel).step(step_width).to_a.each do |ci|
          if ci > 30
            @possible_channels[:chans5g].push(ci) unless @possible_channels[:chans5g].include?(ci)
          else
            @possible_channels[:chans2g].push(ci) unless @possible_channels[:chans2g].include?(ci)
          end
        end
      else
        puts 'wrong input'
      end
    end
    @possible_channels[:chans5g].sort_by! { |p| p }
    @possible_channels[:chans2g].sort_by! { |p| p }
  end

  def update_task(is_tr181)
    escaped_device_id = CGI.escape(params[:id])
    http = create_api_conn()
    object = "[Device Detail][Refresh]#{escaped_device_id}"
    cpe_status, task_status = device_connected?(is_tr181, escaped_device_id.to_s, http, 1, object)
    if params.include? 'refresh_summary'
      can?(:read, 'devices') do
        if task_status != 2
          to_refresh = ActiveSupport::JSON.decode(params['refresh_summary'])
          to_refresh['objects'].each do |o|
            task = { 'name' => 'refreshObject', 'objectName' => o }
            http.post("/devices/#{escaped_device_id}/tasks", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
          end

          to_refresh['custom_commands'].each do |o|
            task = { 'name' => 'customCommand', 'command' => "#{o[16..-1]} get" }
            http.post("/devices/#{escaped_device_id}/tasks", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
          end

          device_id = escaped_device_id
          success_msg = nil
          if is_tr181
            case params[:tab]
            when 'mesh'
              unless params[:controllerId].blank?
                device_id = CGI.escape(params[:controllerId])
                save_diagnostic_lock(device_id, 'Mesh', 60)
                success_msg = 'Mesh topology is refreshing'
              end
            end
          end

          task = { 'name' => 'getParameterValues', 'parameterNames' => to_refresh['parameters'] }
          res = http.post("/devices/#{device_id}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
          if res.code == '200'
            flash[:success] = success_msg.blank? ? 'Device is refreshing' : success_msg
            agileACSLog(6, object, 'Device is refreshing.')
          elsif res.code == '202'
            message = res.message
            message = message + ', task queued.' if message == 'Device is unreachable'
            flash[:warning] = message
            agileACSLog(4, object, res.message.to_s)
          else
            flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
            agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
          end
        end
      end
    end

    if params.include? 'add_tag'
      can?(:create, 'devices') do
        tag = ActiveSupport::JSON.decode(params['add_tag']).strip
        res = http.post("/devices/#{escaped_device_id}/tags/#{CGI.escape(tag)}", nil)

        if res.code == '200'
          flash[:success] = 'Tag added'
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        end
      end
    end

    if params.include? 'remove_tag'
      can?(:delete, 'devices') do
        tag = ActiveSupport::JSON.decode(params['remove_tag']).strip
        res = http.delete("/devices/#{escaped_device_id}/tags/#{CGI.escape(tag)}", nil)

        if res.code == '200'
          flash[:success] = 'Tag removed'
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        end
      end
    end

    if params.include? 'delete_items'
      can?(:delete, 'devices') do
        tag = ActiveSupport::JSON.decode(params['delete_items'])
        type = tag[0]['type']
        tag[0].delete('type')

        res = http.put("/csmenus/#{CGI.escape(type.downcase)}", tag[0].to_json(), 'Content-type' => 'application/json')

        if res.code == '200'
          flash[:success] = 'Items removed'
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        end
      end
    end

    if params.include? 'reset_list'
      can?(:delete, 'devices') do
        tag = ActiveSupport::JSON.decode(params['reset_list'])
        type = tag[0]['type']

        res = http.delete("/csmenus/#{CGI.escape(type.downcase)}", nil)

        if res.code == '200'
          flash[:success] = 'List reset'
        else
          flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
        end
      end
    end

    if params.include? 'commit'
      tasks = ActiveSupport::JSON.decode(params['commit'])
      tasks.each_with_index do |task, index|
        case task['name']
        when 'getParameterValues'
          action = :read
          if is_tr181
            if params.include?('tab') && (params[:tab] == 'wan')
              d = DeviceEntry.new(http, params[:id], params[:tab])
              inf_name = params['inf_name']
              task['parameterNames'] = d.get_wan_entry_paths(inf_name, task['parameterNames'])
            end
          end

        when 'setParameterValues'
          action = :update

          if is_tr181
            # for lan, wan, wlan-primary ssid
            if task['parameterValues'].is_a?(Hash)
              d = DeviceEntry.new(http, params[:id], params[:tab])
              task['parameterValues'] = d.generate_set_parameters(task['parameterValues'])
            end

            # for wlan-mssid
            if task['parameterValues'][0].is_a?(Hash)
              d = DeviceEntry.new(http, params[:id], params[:tab])
              tmp_array = []

              task['parameterValues'].each do |n|
                tmp = d.generate_set_parameters(n)
                tmp_array += tmp
              end
              task['parameterValues'] = tmp_array
            end
          end
          if task['diagnostic']
            @diag_flag = true
            if task['diagnostic'] != 'UDPEchoConfig'
              save_diagnostic_lock(params[:id], task['diagnostic'], 60)
            else
              if task['Enable']
                if task['timeout']
                  save_diagnostic_lock(params[:id], task['diagnostic'], 60 * task['timeout'].to_i)
                else
                  save_diagnostic_lock(params[:id], task['diagnostic'], 60 * 5)
                end
              end
            end
            task = get_diagnostic(params[:id], task, is_tr181)
          end
        when 'addObject'
          action = :create
        when 'deleteObject'
          action = :delete
        when 'reboot'
          action = :update
        when 'factoryReset'
          action = :update
        when 'download'
          action = :update
        else
          action = :update
        end
        resource = 'devices'
        can?(action, resource) do
          object = "[Device Detail][Setting]#{escaped_device_id}"
          if task_status != 2 or (task_status == 2 and @diag_flag)
            if index < tasks.count - 1
              res = http.post("/devices/#{escaped_device_id}/tasks", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
              get_parameters_after_set_parameters_failed(task, res, @diag_flag)
            else
              res = if !@diag_flag
                      http.post("/devices/#{escaped_device_id}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
                    else
                      http.post("/devices/#{escaped_device_id}/tasks?timeout=3000&connection_request&device_online_threshold", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
                    end
              get_parameters_after_set_parameters_failed(task, res, @diag_flag)
              if res.code == '200'
                flash[:success] = 'Tasks committed'
                agileACSLog(6, object, "Tasks committed #{action} #{resource}")
                if task['diagnostic'] && task['diagnostic'] == 'UDPEchoConfig'
                  if !task['Enable']
                    if is_tr181
                      send_diagnostic_refresh(params[:id], 'Device.IP.Diagnostics.UDPEchoConfig')
                    else
                      send_diagnostic_refresh(params[:id], 'InternetGatewayDevice.UDPEchoConfig')
                    end
                    destory_diagnostic_lock(params[:id])
                  end
                end
              elsif res.code == '202'
                message = res.message
                message = message + ', task queued.' if message == 'Device is unreachable'
                flash[:warning] = message
                agileACSLog(4, object, "#{message} #{action} #{resource}")
              else
                flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
                agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
              end
            end
          end
        end
      end
    end
    redirect_to action: :show, tab: params[:tab], isTr181: is_tr181
  end

end
