include Util

module DiagLock
  def refresh_diagnostic_object(dev_id, obj_name)
    task_obj = {'name' => 'refreshObject', 'objectName' => obj_name}
    http = create_api_conn()
    http.start()
    http.post("/devices/#{URI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task_obj), 'Content-type' => 'application/json')
    http.finish()
    sleep(3)
  end

  def get_device_node_value(dev_id, node_name)
    http = create_api_conn()
    http.start()
    project = [node_name]
    res = query_resource(http, 'devices', {'_id' => dev_id}, project)
    dev_params = flatten_params(res[:result][0]);
    ret_val = nil
    dev_params.each do |param|
      if param['_path'] == node_name
        ret_val = param['_value']
      end
    end
    http.finish()
    return ret_val
  end


  #
  # Return nil if the device is no diagnostic task. Otherwise, return the diagnostic name
  #
  def get_diagnostic_lock(dev_id, controller_id = '')
    ret = nil
    if controller_id != ""
      task_obj = {"action": "isLocked", "controller_id": controller_id}
    else
      task_obj = {"action": "isLocked"}
    end

    http = create_api_conn()
    http.start()
    res = http.post("/diaglock/#{URI.escape(dev_id)}", ActiveSupport::JSON.encode(task_obj), 'Content-type' => 'application/json')
    http.finish()

    res_json = JSON.parse(res.body)
    return res_json
  end


  #
  # Save diagnostic lock
  # - device_id
  # - task_name: WifiScan|IPPingDiagnostics|TraceRouteDiagnostics|UploadDiagnostics|DownloadDiagnostics|UDPEchoConfig|NSLookupDiagnostics
  # - timeout: unit = seconds.
  #
  def save_diagnostic_lock(dev_id, task_name, timeout)
    ret = nil

    task_obj = {"action": "lockDiag", "diagName": task_name, "timeout": timeout}

    http = create_api_conn()
    http.start()
    res = http.post("/diaglock/#{URI.escape(dev_id)}", ActiveSupport::JSON.encode(task_obj), 'Content-type' => 'application/json')
    http.finish()

    res_json = JSON.parse(res.body)
    ret = res_json['result']
    return ret
  end



  #
  # Destory diagnostic lock
  #
  def destory_diagnostic_lock(dev_id)
    ret = nil

    task_obj = {"action": "unlockDiag"}

    http = create_api_conn()
    http.start()
    res = http.post("/diaglock/#{URI.escape(dev_id)}", ActiveSupport::JSON.encode(task_obj), 'Content-type' => 'application/json')
    http.finish()

    res_json = JSON.parse(res.body)
    ret = res_json['result']
    return ret
  end
end
