# diag_util.rb

#
# Set diagnostic parameter 'XXXXStatus" to None
#
module DiagUtil
  DIAG_PREFIX_TR181 = 'Device.IP.Diagnostics.'.freeze
  DIAG_PREFIX_DNS_TR181 = 'Device.DNS.Diagnostics.'.freeze
  DIAG_PREFIX_TR098 = 'InternetGatewayDevice.'.freeze
  DIAG_NODES_TR181 = { 'IPPingDiagnostics' => 'IPPing',
                       'TraceRouteDiagnostics' => 'TraceRoute', 'UploadDiagnostics' => 'UploadDiagnostics',
                       'DownloadDiagnostics' => 'DownloadDiagnostics',
                       'UDPEchoConfig' => 'UDPEchoConfig', 'NSLookupDiagnostics' => 'NSLookupDiagnostics' }.freeze
  DIAG_NODES_TR098 = { 'IPPingDiagnostics' => 'IPPingDiagnostics',
                       'TraceRouteDiagnostics' => 'TraceRouteDiagnostics', 'UploadDiagnostics' => 'UploadDiagnostics',
                       'DownloadDiagnostics' => 'DownloadDiagnostics',
                       'UDPEchoConfig' => 'UDPEchoConfig', 'NSLookupDiagnostics' => 'NSLookupDiagnostics' }.freeze
  def send_diagnostic_reset(dev_id, diag_state)
    task_obj = { 'name' => 'setParameterValues', 'parameterValues' => [[diag_state, 'None']] }
    http = create_api_conn()
    http.start()
    http.post("/devices/#{CGI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task_obj), 'Content-type' => 'application/json')
    http.finish()
  end

  #
  # Refresh specific DataModel node
  #
  def send_diagnostic_refresh(dev_id, obj_name)
    sleep(4)
    task_obj = { 'name' => 'refreshObject', 'objectName' => obj_name }
    http = create_api_conn()
    http.start()
    http.post("/devices/#{CGI.escape(dev_id)}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task_obj), 'Content-type' => 'application/json')
    http.finish()
  end

  #
  # Reorder diagnostic test
  #
  def reorder_diagnostic_list(diag_list)
    display_order = ['IP Ping Test', 'Trace Route Test', 'Upload Test', 'Download Test', 'UDP Echo Test', 'NS Lookup Test']
    ret_arr = []
    display_order.each do |v|
      ret_arr.push(v) if diag_list.include?(',' + v + ',')
    end
    ret_arr
  end

  def get_diagnostics_prefix(is_tr181)
    path_prefixes = {}
    path_prefixes['tr181'] = { 'IPPingDiagnostics' => DIAG_PREFIX_TR181 + DIAG_NODES_TR181['IPPingDiagnostics'],
                               'TraceRouteDiagnostics' => DIAG_PREFIX_TR181 + DIAG_NODES_TR181['TraceRouteDiagnostics'],
                               'UploadDiagnostics' => DIAG_PREFIX_TR181 + DIAG_NODES_TR181['UploadDiagnostics'],
                               'DownloadDiagnostics' => DIAG_PREFIX_TR181 + DIAG_NODES_TR181['DownloadDiagnostics'],
                               'UDPEchoConfig' => DIAG_PREFIX_TR181 + DIAG_NODES_TR181['UDPEchoConfig'],
                               'NSLookupDiagnostics' => DIAG_PREFIX_DNS_TR181 + DIAG_NODES_TR181['NSLookupDiagnostics'] }
    path_prefixes['tr098'] = { 'IPPingDiagnostics' => DIAG_PREFIX_TR098 + DIAG_NODES_TR098['IPPingDiagnostics'],
                               'TraceRouteDiagnostics' => DIAG_PREFIX_TR098 + DIAG_NODES_TR098['TraceRouteDiagnostics'],
                               'UploadDiagnostics' => DIAG_PREFIX_TR098 + DIAG_NODES_TR098['UploadDiagnostics'],
                               'DownloadDiagnostics' => DIAG_PREFIX_TR098 + DIAG_NODES_TR098['DownloadDiagnostics'],
                               'UDPEchoConfig' => DIAG_PREFIX_TR098 + DIAG_NODES_TR098['UDPEchoConfig'],
                               'NSLookupDiagnostics' => DIAG_PREFIX_TR098 + DIAG_NODES_TR098['NSLookupDiagnostics'] }
    if is_tr181
      path_prefixes['tr181']
    else
      path_prefixes['tr098']
    end
  end

  def get_diagnostic(dev_id, params, is_tr181)
    ret_params = params
    path_prefixes = get_diagnostics_prefix(is_tr181)

    case params['diagnostic']
    when 'IPPingDiagnostics'
      # FIXMD if need to convert to TR181 supporting
      path_prefix = path_prefixes['IPPingDiagnostics']

      # Refresh diag obj
      refresh_diagnostic_object(dev_id, path_prefix)

      if params['cancel']
        node_value = get_device_node_value(dev_id, path_prefix + '.DSCP')
        params['DSCP'] = ((node_value - 0) + 1).modulo(64)

        ret_params['parameterValues'] = [[path_prefix + '.DSCP', params['DSCP']]]

        # params.delete("cancel");
      else
        # Reset diagnostic
        # send_diagnostic_reset(dev_id, path_prefix + '.DiagnosticsState')

        ret_params['parameterValues'] = [[path_prefix + '.Interface', params['Interface']]]
        ret_params['parameterValues'] << [path_prefix + '.Host', params['Host']]
        ret_params['parameterValues'] << [path_prefix + '.NumberOfRepetitions', params['NumberOfRepetitions']]
        ret_params['parameterValues'] << [path_prefix + '.Timeout', params['Timeout']]
        ret_params['parameterValues'] << [path_prefix + '.DataBlockSize', params['DataBlockSize']]
        ret_params['parameterValues'] << [path_prefix + '.DSCP', params['DSCP']]
        ret_params['parameterValues'] << [path_prefix + '.DiagnosticsState', 'Requested']

        params.delete('Interface')
        params.delete('Host')
        params.delete('NumberOfRepetitions')
        params.delete('Timeout')
        params.delete('DataBlockSize')
      end
      params.delete('DSCP')

      ret_params
    when 'TraceRouteDiagnostics'
      # FIXMD if need to convert to TR181 supporting
      path_prefix = path_prefixes['TraceRouteDiagnostics']

      # Refresh diag obj
      refresh_diagnostic_object(dev_id, path_prefix)

      if params['cancel']
        node_value = get_device_node_value(dev_id, path_prefix + '.DSCP')
        params['DSCP'] = ((node_value - 0) + 1).modulo(64)

        ret_params['parameterValues'] = [[path_prefix + '.DSCP', params['DSCP']]]

        # params.delete("cancel");
      else
        # Reset diagnostic
        # send_diagnostic_reset(dev_id, path_prefix + '.DiagnosticsState')

        ret_params['parameterValues'] = [[path_prefix + '.Interface', params['Interface']]]
        ret_params['parameterValues'] << [path_prefix + '.MaxHopCount', params['MaxHopCount']]
        ret_params['parameterValues'] << [path_prefix + '.Host', params['Host']]
        ret_params['parameterValues'] << [path_prefix + '.NumberOfTries', params['NumberOfTries']]
        ret_params['parameterValues'] << [path_prefix + '.Timeout', params['Timeout']]
        ret_params['parameterValues'] << [path_prefix + '.DataBlockSize', params['DataBlockSize']]
        ret_params['parameterValues'] << [path_prefix + '.DSCP', params['DSCP']]
        ret_params['parameterValues'] << [path_prefix + '.DiagnosticsState', 'Requested']

        params.delete('Interface')
        params.delete('MaxHopCount')
        params.delete('Host')
        params.delete('NumberOfTries')
        params.delete('Timeout')
        params.delete('DataBlockSize')
      end
      params.delete('DSCP')

      ret_params
    when 'UploadDiagnostics'
      # FIXMD if need to convert to TR181 supporting
      path_prefix = path_prefixes['UploadDiagnostics']

      # Refresh diag obj
      refresh_diagnostic_object(dev_id, path_prefix)

      if params['cancel']
        node_value = get_device_node_value(dev_id, path_prefix + '.DSCP')
        params['DSCP'] = ((node_value - 0) + 1).modulo(64)

        ret_params['parameterValues'] = [[path_prefix + '.DSCP', params['DSCP']]]

        # params.delete("cancel");
      else
        # Reset diagnostic
        # send_diagnostic_reset(dev_id, path_prefix + '.DiagnosticsState')

        ret_params['parameterValues'] = [[path_prefix + '.Interface', params['Interface']]]
        ret_params['parameterValues'] << [path_prefix + '.UploadURL', params['UploadURL']]
        ret_params['parameterValues'] << [path_prefix + '.EthernetPriority', params['EthernetPriority']]
        ret_params['parameterValues'] << [path_prefix + '.TestFileLength', params['TestFileLength']]
        ret_params['parameterValues'] << [path_prefix + '.DSCP', params['DSCP']]
        ret_params['parameterValues'] << [path_prefix + '.DiagnosticsState', 'Requested']

        # Clean file on server
        str_arr = params['UploadURL'].split('/')

        file_path = if params['UploadURL'].include?('ftp')
                      '/var/ftp/upload/' + str_arr.last()
                    else
                      '/usr/share/nginx/html/upload/' + str_arr.last()
                    end
        File.delete(file_path) if File.exist?(file_path)

        params.delete('Interface')
        params.delete('UploadURL')
        params.delete('EthernetPriority')
        params.delete('TestFileLength')
      end
      params.delete('DSCP')
      ret_params
    when 'DownloadDiagnostics'
      # FIXMD if need to convert to TR181 supporting
      path_prefix = path_prefixes['DownloadDiagnostics']

      # Refresh diag obj
      refresh_diagnostic_object(dev_id, path_prefix)

      if params['cancel']
        node_value = get_device_node_value(dev_id, path_prefix + '.DSCP')
        params['DSCP'] = ((node_value - 0) + 1).modulo(64)

        # ret_params["parameterValues"] = [[path_prefix + ".DSCP", params['DSCP']]]
        ret_params['parameterValues'] = [[path_prefix + '.DownloadURL ', '']]

        # params.delete("cancel");
      else
        # Reset diagnostic
        # send_diagnostic_reset(dev_id, path_prefix + '.DiagnosticsState')

        ret_params['parameterValues'] = [[path_prefix + '.Interface', params['Interface']]]
        ret_params['parameterValues'] << [path_prefix + '.DownloadURL', params['DownloadURL']]
        ret_params['parameterValues'] << [path_prefix + '.EthernetPriority', params['EthernetPriority']]
        ret_params['parameterValues'] << [path_prefix + '.DSCP', params['DSCP']]
        ret_params['parameterValues'] << [path_prefix + '.DiagnosticsState', 'Requested']

        params.delete('Interface')
        params.delete('DownloadURL')
        params.delete('EthernetPriority')
      end
      params.delete('DSCP')
      ret_params
    when 'UDPEchoConfig'
      # FIXMD if need to convert to TR181 supporting
      path_prefix = path_prefixes['UDPEchoConfig']

      # Refresh diag obj
      refresh_diagnostic_object(dev_id, path_prefix)

      if params['Enable'].to_s == 'true'
        ret_params['parameterValues'] = [[path_prefix + '.Interface', params['Interface']]]
        ret_params['parameterValues'] << [path_prefix + '.UDPPort', params['UDPPort']]
        ret_params['parameterValues'] << [path_prefix + '.SourceIPAddress', params['SourceIPAddress']]
      end
      ret_params['parameterValues'] << [path_prefix + '.Enable', params['Enable']]

      params.delete('Interface')
      params.delete('UDPPort')
      params.delete('SourceIPAddress')
      # params.delete("Enable");

      ret_params
    when 'NSLookupDiagnostics'
      # FIXMD if need to convert to TR181 supporting
      path_prefix = path_prefixes['NSLookupDiagnostics']

      # Refresh diag obj
      refresh_diagnostic_object(dev_id, path_prefix)

      if params['cancel']
        node_value = get_device_node_value(dev_id, path_prefix + '.Timeout')
        params['Timeout'] = ((node_value - 0) + 1).modulo(1000) + 5000

        ret_params['parameterValues'] = [[path_prefix + '.Timeout', params['Timeout']]]

        # params.delete("cancel");
        params.delete('Timeout')
      else
        # Reset diagnostic
        # send_diagnostic_reset(dev_id, path_prefix + '.DiagnosticsState')

        ret_params['parameterValues'] = [[path_prefix + '.Interface', params['Interface']]]
        ret_params['parameterValues'] << [path_prefix + '.HostName', params['HostName']]
        ret_params['parameterValues'] << [path_prefix + '.DNSServer', params['DNSServer']]
        ret_params['parameterValues'] << [path_prefix + '.Timeout', params['Timeout']]
        ret_params['parameterValues'] << [path_prefix + '.NumberOfRepetitions', params['NumberOfRepetitions']]
        ret_params['parameterValues'] << [path_prefix + '.DiagnosticsState', 'Requested']

        params.delete('Interface')
        params.delete('HostName')
        params.delete('DNSServer')
        params.delete('Timeout')
        params.delete('NumberOfRepetitions')
      end

      ret_params
    else
      params
    end
  end

  #
  # Get WAN interface list for diagnostic test
  #
  def get_diatnostic_wan_list(device_params)
    wan_if_list = {}
    device_params.each do |v|
      next unless v['_path'].include?('WANIPConnection') || v['_path'].include?('WANPPPConnection') || v['_path'].include?('TP_Connection')

      wan_if_list[v['_path'].to_s.from(0).to(-6)] = v['_value'] if v['_path'].include?('.Name')
    end
    wan_if_list
  end

  #
  # Get diagnostic test result
  #
  def get_diagnostic_result(device_params, is_tr181)
    diag_result = {}
    diag_nodes = if is_tr181
                   DIAG_NODES_TR181
                 else
                   DIAG_NODES_TR098
                 end
    device_params.each do |v|
      str_arr = v['_path'].split('.')

      if v['_path'].include?(diag_nodes['IPPingDiagnostics']) && str_arr.last() != diag_nodes['IPPingDiagnostics']
        diag_result['IPPingDiagnostics.' + str_arr.last()] = (v['_value'].nil? ? '' : v['_value'])
      end
      if v['_path'].include?(diag_nodes['TraceRouteDiagnostics']) && str_arr.last() != diag_nodes['TraceRouteDiagnostics']
        if (is_tr181 && str_arr.length == 5) || (!is_tr181 && str_arr.length == 3)
          diag_result['TraceRouteDiagnostics.' + str_arr.last()] = (v['_value'].nil? ? '' : v['_value'])
        elsif v['_path'].include?('RouteHops')
          if is_tr181 && str_arr.length == 7
            diag_result['TraceRouteDiagnostics.' + str_arr[4] + '.' + str_arr[5] + '.' + str_arr[6]] = (v['_value'].nil? ? '' : v['_value'])
          elsif !is_tr181 && str_arr.length == 5
            diag_result['TraceRouteDiagnostics.' + str_arr[2] + '.' + str_arr[3] + '.' + str_arr[4]] = (v['_value'].nil? ? '' : v['_value'])
          end
        end
      end
      if v['_path'].include?(diag_nodes['UploadDiagnostics']) && str_arr.last() != diag_nodes['UploadDiagnostics']
        diag_result['UploadDiagnostics.' + str_arr.last()] = (v['_value'].nil? ? '' : v['_value'])
      end
      if v['_path'].include?(diag_nodes['DownloadDiagnostics']) && str_arr.last() != diag_nodes['DownloadDiagnostics']
        diag_result['DownloadDiagnostics.' + str_arr.last()] = (v['_value'].nil? ? '' : v['_value'])
      end
      if v['_path'].include?(diag_nodes['NSLookupDiagnostics']) && str_arr.last() != diag_nodes['NSLookupDiagnostics']
        diag_result['NSLookupDiagnostics.' + str_arr.last()] = (v['_value'].nil? ? '' : v['_value'])
        if (is_tr181 && str_arr.length == 5) || (!is_tr181 && str_arr.length == 3)
          diag_result['NSLookupDiagnostics.' + str_arr.last()] = (v['_value'].nil? ? '' : v['_value'])
        elsif v['_path'].include?('Result')
          if is_tr181 && str_arr.length == 7
            diag_result['NSLookupDiagnostics.' + str_arr[4] + '.' + str_arr[5] + '.' + str_arr[6]] = (v['_value'].nil? ? '' : v['_value'])
          elsif !is_tr181 && str_arr.length == 5
            diag_result['NSLookupDiagnostics.' + str_arr[2] + '.' + str_arr[3] + '.' + str_arr[4]] = (v['_value'].nil? ? '' : v['_value'])
          end
        end
      end
      next unless v['_path'].include?(diag_nodes['UDPEchoConfig']) && str_arr.last() != diag_nodes['UDPEchoConfig']

      diag_result['UDPEchoConfig.' + str_arr.last()] = (v['_value'].nil? ? '' : v['_value'])
    end
    diag_result
  end
end
