# lib/tplink/match.rb

module Match
  include DeviceFilters

  # return the javascript filter parameters, defined in config/tplink/profiles_index_parameters.yml with match = 1  
  def get_match(query_type, match=1 ) 
    ret = {}
    Rails.configuration.profiles_index_parameters.each do | label, definition |
      if match.nil? || definition['match'].to_i == match
        if query_type == 1
          ret[definition['field']] = label 
        else
          ret[definition['paths']['InternetGatewayDevice']] = label
        end
      end
    end
    return ret
  end

  # Convert the given hash to result.
  # The mapping rule is defined in config/tplink/profiles_index_parameters.yml and config/tplink/profiles_summary_parameters.yml
  #
  # hash: 
  #    the given query. Only support pure 'field' or 'InternetGatewayDevice' type.
  # hash_type : 
  #    'field' or 'InternetGatewayDevice' to indicated the type of the given hash
  # append: 
  #     if true, specific usage if params['model_type'] == "1" of profile.rb
  # 
  # return: {'Device':<TR-181>, 'InternetGatewayDevice':<TR-181>, 'field': <Mapping_Field>}
  def get_queries(hash, hash_type, append = false)
    ret = {}
    query_98 = nil
    query_181 = nil
    query_field = nil

    case hash_type
    when "field"
      results = get_preconditions(hash)
      query_98 = results['InternetGatewayDevice']
      query_181 = results['Device']
      query_field = hash.clone
    when "InternetGatewayDevice"
      results = transfer_precondition_098(hash)
      query_98 = hash.clone
      query_181 = results[0]
      query_field = results[1]
    end

    if append
      query_98['InternetGatewayDevice.DeviceInfo.ProductClass'] = "IGD"
      query_98['InternetGatewayDevice.DeviceInfo.Manufacturer'] = "TP-LINK"
      query_181['Device.DeviceInfo.ProductClass'] = "Device"
      query_181['Device.DeviceInfo.Manufacturer'] = "TP-LINK"
      query_field['Manufacturer'] = "TP-LINK"
    end

    ret['InternetGatewayDevice'] = query_98
    ret['Device'] = query_181
    ret['field'] = query_field
    return ret
  end


  # hash: the GUI given query with field defined in config/tplink/profiles_index_parameters.yml
  # return the preconditions including 'Device' and 'InternetGatewayDevice'
  def get_preconditions(hash)
    ret = {'Device'=>{}, 'InternetGatewayDevice'=>{}}
    return ret if hash.nil?
    
    mapping = get_parameters_mapping
    tmp181, tmp098, tmpfield = {}, {}, {}
    test = [mapping, hash, DB_FILTER_TYPE_FIELD, tmp181, tmp098, tmpfield]
    test = conv_db_query test
    tmp181= test[3]
    tmp098= test[4]

    ret['Device'] = tmp181 
    ret['InternetGatewayDevice'] = tmp098 
    return ret
  end

  # return the parameters mapping [<tr-098_path>] = [<tr-181_path>, <field>], defined in
  # config/tplink/profiles_index_parameters.yml and
  # config/tplink/profiles_summary_parameters.yml
  private
  def get_match_params_mapping() 
    mapping = {}
    Rails.configuration.profiles_index_parameters.each do | label, definition |
      mapping[definition['paths']['InternetGatewayDevice']] = [ definition['paths']['Device'], definition['field']]
    end
    
    Rails.configuration.profiles_summary_parameters.each do | label, definition |
      next if mapping.key? definition['paths']['InternetGatewayDevice']
      mapping[definition['paths']['InternetGatewayDevice']] = [ definition['paths']['Device'], definition['field']]
    end
    return mapping
  end

  # transfer tr-098 hash to [<tr-181 hash>, <filter hash>]
  def transfer_precondition_098(hash)
    return [{}, {}] if hash.nil?
    mapping = get_match_params_mapping

    tmp181, filter = {}, {}
    result = transfer_query([mapping, hash, tmp181, filter])
    tmp181= result[2]
    filter = result[3]
    
    return [tmp181, filter]
  end

  private 
  def transfer_query(input) 
    mapping = input[0]
    hash = input[1]
    tmp181 = input[2]
    filter = input[3]
    
    hash.each do |key, val|
      _key_181 = key
      _key_match = key
      if mapping.key? key
        _key_181 = mapping[key][0]
        _key_match = mapping[key][1]
      end

      if val.is_a? (Hash)
        a = transfer_query([mapping, val, {}, {}])
        tmp181[_key_181] = a[2]
        filter[_key_match] = a[3]
      elsif val.is_a? (Array)
        tmp181[_key_181] = []
        filter[_key_match] = []
        val.each do |element| 
          a = transfer_query([mapping, element, {}, {}])
          tmp181[_key_181] << a[2]
          filter[_key_match] << a[3]
        end
      elsif val.is_a? (String)
        tmp181[_key_181] = val.clone
        filter[_key_match] = val.clone
      end
    end
    return [mapping, hash, tmp181, filter]
  end
end
