# module for mesh
module Mesh
  include MeshDataType

  # get the database project for mesh tab
  def get_mesh_data_project(data_type = MeshDataType::DT_ALL)
    return nil if data_type.nil?

    ret = []
    ret << '_mesh'
    if data_type == MeshDataType::DT_ALL
      ret << 'Device.Ethernet.Link.1.MACAddress'
      ret << 'Device.DeviceInfo.X_TP_DevName'
      ret << 'Device.WiFi.MultiAP.APDevice'
      ret << 'Device.Hosts'
      ret << 'Device.X_TP_RE_MAPPING'
    end

    # Those parameters ared appended for the other views/_xxx.html.erb
    ret << 'Device.DeviceInfo.ModelName'
    ret << '_lastInform'
    ret << '_ipaddress'

    ret
  end

  def has_mesh_data(res)
    return false if res.nil? || res['_mesh'].nil? || res['_mesh']['type'].nil?

    true
  end

  def find_mesh_controller(device)
    return nil unless has_mesh_data(device)
    return device if device['_mesh']['_type'] == 'Controller'
    return nil if device.nil? || device['_mesh'].nil? || device['_mesh']['cap'].blank?

    controllerId = device['_mesh']['cap']
    return device if controllerId == device['_id']

    res = query_resource(create_api_conn(), 'devices', { '_id' => controllerId }, get_mesh_data_project(MeshDataType::DT_ALL))
    return nil if res[:result].size != 1

    res[:result][0]
  end

  def generate_mesh_topology(device)
    return nil if (!class_exists? 'MeshTopology') || device.blank? || !has_mesh_data(device)

    mesh_controller = find_mesh_controller(@device)
    return nil if mesh_controller.nil?

    ret = nil
    begin
      ret = MeshTopology.new(mesh_controller, @device).getResults()
    rescue StandardError => e
      ret = nil
    end

    ret
  end

  def class_exists?(name)
    true if Kernel.const_get(name)
  rescue NameError
    false
  end
end
