NOTIFICATIONS_MODULE_NAME = '[Notifications]'.freeze()

# module for notifications_controller
module NotificationsModule
  include Util
  def get_info_by(tab = 'mail')
    res = query_resource(create_api_conn(), 'notification', { '_id' => 'mail' })
    result = {}
    if tab == 'mail'
      result = res[:result][0]['settings'] if res[:result][0]
    elsif tab == 'task'
      if res[:result][0]
        event_list = res[:result][0]['event_list']

        # remove test event
        event_list.except!('000')

        # remove subject&context
        event_list.each do |_k, v|
          v.except!('subject')
          v.except!('context')
        end

        event_list.each do |_k, v|
          result[v['event']] = v
          result[v['event']].except!('event')
        end
      end
    end
    result
  end

  def generate_task_data(task)
    data = {}
    res = query_resource(create_api_conn(), 'notification', '_id' => 'mail')
    if res[:result][0]
      event_list = res[:result][0]['event_list']

      event_list.map do |id, v|
        next unless id != '000'

        item = v.except('event').except('subject').except('context')
        item.map do |kk, _vv|
          value = false
          if task.include?(v['event'])
            value = task[v['event']][kk] if task[v['event']].include?(kk)
          end
          data["event_list.#{id}.#{kk}"] = value
        end
      end
    end
    data
  end

  def handle_notifications_succeed(res)
    if params.include?('test')
      res.body.match('failed') ? agileACSLog(3, NOTIFICATIONS_MODULE_NAME, res.body.to_s) : agileACSLog(6, NOTIFICATIONS_MODULE_NAME, res.body.to_s)
      render(json: { 'message' => res.body })
      true # task is done
    else
      task_name = params['tab'] == 'mail' ? 'Mail' : 'Task'
      flash[:success] = "#{task_name} Settings saved"
      agileACSLog(6, NOTIFICATIONS_MODULE_NAME, "#{task_name} Settings save succeed.")
      false # task is not done
    end
  end

  def send_notifications_request(url, data)
    res = create_api_conn().post(url, ActiveSupport::JSON.encode(data), 'Content-Type' => 'application/json')
    if res.code == '200'
      return if handle_notifications_succeed(res)
    else
      flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      agileACSLog(3, NOTIFICATIONS_MODULE_NAME, "Unexpected error (#{res.code}): #{res.body}")
    end
    redirect_to action: :index, tab: params['tab']
  end

  def optional_mail_settings(mail_settings)
    mail_settings['enable_alert'] = params.include?('enableAlert') ? params['enableAlert'] : false
    mail_settings['enable_auth'] = params.include?('enableAuth') ? params['enableAuth'] : false
    if mail_settings['enable_auth'] == 'true'
      mail_settings['username'] = params['username']
      mail_settings['password'] = params['password']
    end
    return unless mail_settings['service'] == 'custom'

    mail_settings['smtp_server'] = params['smtp_server']
    mail_settings['port'] = params['port']
  end
end
