module Parser

    def dump_params(params, result, prefix = nil)
        return if params == nil? || result == nil?
        params.each do |key, value|
            next if key.start_with?('_') || value.nil? || !value.is_a?(Hash) 
            if value.key? "_value"
                #result[(prefix.nil?) ? "#{key}": "#{prefix}.#{key}"] = value.clone.except('_object').except('_type').except('_timestamp')
                path = (prefix.nil?) ? "#{key}": "#{prefix}.#{key}"
                val  = value['_value']
                writable  = value.key?("_writable") ? value['_writable'] : false
                result[path] = {'path'=> path, 'value'=> "#{val}", 'writable' => "#{writable}" }
            else
                dump_params(value, result, (prefix.nil?) ? "#{key}": "#{prefix}.#{key}")
            end
        end
    end

    def group_params(params) 
        ret = {}
        params.each do | path, attributes |
            name, match, suffix = path.rpartition('.')
            begin
                ret[name] = {}  if !ret.key?(name)
                ret[name][suffix] = attributes
            rescue => exception
                puts "ex:\n#{exception}"
            end
            
        end
        return ret
    end
  
    # type = 1: for dump parames
    # type = 2: for group params
    def match_rkey(hash, search_path, type, first)
        ret = []
        search = Regexp.new(search_path.to_s) unless search_path.is_a?(Regexp)
        if type == 1 
            hash.each do | key, value |
                if key =~ search
                    ret << {"path"=>key, "value"=>value['value'], "writable"=> ( value.key?("writable") ? value['writable'] : "false" ) }
                    return ret if first
                end
            end  
        elsif type == 2
            hash.each do | group_name, elements |
                if group_name =~ search
                    elements.each do | field, object| 
                        a = object
                        ret << { field=> a  } 
                    end
                    return ret if first
                end
            end  
        end
        
        return ret
    end

    def match_rkey_val(hash, search_path, search_val, first=false)
        ret = []
        search = Regexp.new(search_path.to_s) unless search_path.is_a?(Regexp)
        hash.each do | key, value |
            if key =~ search
                if value['value'].to_s.eql? search_val.to_s
                    ret << {"path"=>key, "value"=>value['value'].to_s, "writable"=> ( value.key?("writable") ? value['writable'].to_s : "false" ) }
                    return ret if first
                end
            end
        end
        return ret
    end
    
    def match_rkey_rval(hash, search_path, search_val, first=false)
        ret = []
        regex_path = Regexp.new(search_path.to_s) unless search_path.is_a?(Regexp)
        regex_val = Regexp.new(search_val.to_s) unless search_val.is_a?(Regexp)
        hash.each do | key, value |
            if key =~ regex_path
                if value['value'].to_s =~ regex_val
                    ret << {"path"=>key, "value"=>value['value'].to_s, "writable"=> ( value.key?("writable") ? value['writable'].to_s : "false" ) }
                    return ret if first
                end
            end
        end
        return ret
    end

	def get_device_param_by_specific(hash, specific_path)
		return nil if hash.nil? or specific_path.nil?
		ret = nil
		ret = hash[specific_path] if hash.key? specific_path
		return ret
	end

    def get_device_params(hash, search_path)
        return match_rkey(hash, search_path, 1, false)
    end

    def get_device_param(hash, search_path)
        res = match_rkey(hash, search_path, 1, true)
        return nil if res.nil? || res.size == 0
        return res[0]
    end
#
#    def get_device_params_by_group(hash, search_path)
#        ret = []
#        res = match_rkey(hash, search_path, 2, false)
#        return ret
#    end
    
    def get_device_param_by_group(hash, search_path, field, search_val = nil)
        ret = []
        res = match_rkey(hash, search_path, 2, false)
        res.each do | element |
            if element.key? field
				if search_val.nil?
					ret << element[field]
				else
					regex_val = Regexp.new(search_val.to_s) unless search_val.is_a?(Regexp)
					if element[field]['value'].to_s =~ regex_val
						ret << element[field]
					end
				end
            end
        end
        return ret
    end
    
    
    # return true if different; return false if the same
    def diff?(v1, v2)
        return ( v1.to_s != v2.to_s )
    end
	
	def rmatch?(val, match_rval) 
		ret = false;
		regex_val = Regexp.new(match_rval.to_s) unless match_rval.is_a?(Regexp)
		if val.to_s =~ regex_val
			ret = true
		end
		return ret
	end
 
    def call_functions(function_names)
        begin
            function_names.each do | o |
                if o.include? "name"
                    # if the function name doesn't exist, continue
                    next if not respond_to?(o["name"].to_sym)
                    
                    if o.include? "argvs"
                        send( o["name"], *o["argvs"] )
                    else 
                        send( o["name"] )
                    end
                end
            end
        rescue => exception
            puts exception
            puts exception.backtrace 
        end
    end
end
