# app type: 'Firmware Upgrade', 'Configure File Download', 'ESSID', 'PPPoE', 'Periodic Inform Interval', 'Firmware Upgrade with Schedule'
# model : 'C5', 'C20', '840N', '850N'
FWUP_CH = 1
CONFIG_CH = 2
ESSID_CH = 3
PPPOE_CH = 4
INFORM_CH = 5
FWUP_SCHEDULE_CH = 6
MAX_RETRY = 2

module Profiles
  include DeviceFilters
  include Match
  include InstanceProfileApps
  def profile_application_name
    application_list = {}
    application_list[FWUP_CH] = 'Firmware Upgrade'
    application_list[CONFIG_CH] = 'Configure File Download'
    application_list[ESSID_CH] = 'ESSID'
    application_list[PPPOE_CH] = 'PPPoE'
    application_list[INFORM_CH] = 'Periodic Inform Interval'
    application_list[FWUP_SCHEDULE_CH] = 'Firmware Upgrade with Schedule'
    application_list
  end

  def update_fault_channel(name, app)
    InstanceProfileApps::APPENDIX.each_with_index do |item, index|
      name = name.sub! item, '' if app.match(InstanceProfileApps::APPS_PRO[index])
    end
    name
  end

  # Update that profile name need to add or remove appendix.
  def update_profile_name(name, app, add = true)
    if add
      InstanceProfileApps::APPENDIX.each_with_index do |item, index|
        name += item if app.match(InstanceProfileApps::APPS_PRO[index])
      end
    else
      InstanceProfileApps::APPENDIX.each_with_index do |item, index|
        name = name.chomp(item) if app.match(InstanceProfileApps::APPS_PRO[index])
      end
    end
    name
  end

  def existed_model_profiles(name, app, model)
    existed = FALSE
    res = query_resource(create_api_conn(), 'profiles', nil)
    total = res[:total]
    if total > 0
      profiles = res[:result]
      (0..(total - 1)).each do |i|
        if (profiles[i]['model'] == model) && (profiles[i]['app'] == app) && (profiles[i]['_id'] != name)
          existed = TRUE
          break
        end
      end
    end
    existed
  end

  def remove_tags_from_devices(tag)
    http = create_api_conn()
    p "remove device tag  all/tags/#{CGI.escape(tag)}"
    res = http.delete("/profiles/all/tags/#{CGI.escape(tag)}", nil)
    res
  end

  def remove_faults_from_profile(channel)
    http = create_api_conn()
    p "remove device tag  all/faults/#{CGI.escape(channel)}"
    res = http.delete("/profiles/all/faults/#{CGI.escape(channel)}", nil)
    res
  end

  def update_fwup_preset_remove_softver_query(query)
    if query['$and'].count > 0
      (0..(query['$and'].count -1)).each do |i|
        if query['$and'][i]['InternetGatewayDevice.DeviceInfo.SoftwareVersion']
          query['$and'].delete_at(i)
          update_fwup_preset_remove_softver_query(query)
        end
      end
    end
  end

  def update_fwup_presets(params, addition)
    query = {}
    query = ActiveSupport::JSON.decode(CGI.unescape(params['profile.query'])) if params.key?('profile.query')
    query['InternetGatewayDevice.DeviceInfo.ModelName'] = params['model']
    #  query['InternetGatewayDevice.DeviceInfo.HardwareVersion'] = profile['hwver'] if profile.has_key?('hwver') # not verify yet
    if addition == 3
      if !query.nil? && query.key?('$and')
        update_fwup_preset_remove_softver_query(query)
        query.delete('$and') if query['$and'].count == 0
      end
      query['InternetGatewayDevice.DeviceInfo.SoftwareVersion'] = params['fw_version']
    end
    append_tplink = false
    # append_tplink = true if params['model_type'] == "1"
    queries = get_queries(query, 'InternetGatewayDevice', append_tplink)

    res = nil
    queryList = [{ 'query' => queries['InternetGatewayDevice'], 'append_str' => '' }, { 'query' => queries['Device'], 'append_str' => '_181' }]
    queryList.each do |element|
      res = update_fwup_preset(params, addition, element['append_str'], element['query'])
      return res if res.code != '200'
    end
    res
  end

  def update_fwup_preset(params, addition, append_str, query)
    presetname = ''
    preset = {}
    preset['channel'] = if params['app'] == 'Firmware Upgrade with Schedule'
                          "#{params['name']}_#{FWUP_SCHEDULE_CH}"
                        else
                          "#{params['name']}_#{FWUP_CH}"
                        end
    preset['weight'] = addition
    params['default_inform_interval'] = '3600'
    case addition
    when 1
      presetname = "#{params['name']}_FWUpgradeTag#{append_str}"
      tags = "{\"$ne\":\"#{params['name']}_NeedUpgradeDone\"}"
      query['_tags'] = ActiveSupport::JSON.decode(tags)
      preset['precondition'] = query.to_json
      if params['app'] == 'Firmware Upgrade with Schedule'
        case params['triggering_conditions_repeat']
        when 'No repeat'
          configuration = "[{\"type\":\"provision\",\"name\":\"FWUpgradeTagSchedule#{append_str}\",\"args\": [\"#{params['name']}\", \"#{params['name']}_NeedUpgradeSchedule\", \"#{params['name']}_NeedUpgradeScheduleToDo\", \"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", \"#{params['triggering_conditions_repeat']}\", \"#{params['fw_version']}\", \"#{params['no_repeat_start_date']}\", \"#{params['no_repeat_end_date']}\", \"#{params['fw_file']}\" ] } ]"
        when 'Everyday'
          start_date = Date.parse(params['repeat_start_days'])
          end_date = Date.parse(params['repeat_until_days'])
          date_arr = (start_date..end_date).map { |date| date.strftime('%Y/%m/%d') }
          date_arr = date_arr.join(', ')
          configuration = "[{\"type\":\"provision\",\"name\":\"FWUpgradeTagSchedule#{append_str}\",\"args\": [\"#{params['name']}\", \"#{params['name']}_NeedUpgradeSchedule\", \"#{params['name']}_NeedUpgradeScheduleToDo\", \"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", \"#{params['triggering_conditions_repeat']}\", \"#{params['fw_version']}\", \"#{date_arr}\", \"#{params['repeat_timerange']}\", \"#{params['fw_file']}\" ] } ]"
        when 'Weekly'
          start_date = Date.parse(params['repeat_start_days'])
          end_date = Date.parse(params['repeat_until_days'])
          date_arr = ((start_date..end_date).to_a.select { |k| params['repeat_weeks'].map(&:to_i).include?(k.wday) }).map { |date| date.strftime('%Y/%m/%d') }
          date_arr = date_arr.join(', ')
          configuration = "[{\"type\":\"provision\",\"name\":\"FWUpgradeTagSchedule#{append_str}\",\"args\": [\"#{params['name']}\", \"#{params['name']}_NeedUpgradeSchedule\", \"#{params['name']}_NeedUpgradeScheduleToDo\", \"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", \"#{params['triggering_conditions_repeat']}\", \"#{params['fw_version']}\", \"#{date_arr}\", \"#{params['repeat_timerange']}\", \"#{params['fw_file']}\" ] } ]"
        else
          puts 'Unexcepted error when preset update fwup.'
        end
      else
        configuration = "[{\"type\":\"provision\",\"name\":\"FWUpgradeTag2#{append_str}\",\"args\": [\"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", #{params['trigger_interval']}, \"#{params['fw_version']}\", \"#{params['name']}\", \"#{params['fw_file']}\" ] } ]"
      end

      preset['configurations'] = ActiveSupport::JSON.decode(configuration)
      preset['schedule'] = ''
      preset['events'] = ''
    when 2
      presetname = "#{params['name']}_FWUpgradeDo#{append_str}"
      tags = "{\"$ne\":\"#{params['name']}_NeedUpgradeDone\"}"
      query['_tags'] = ActiveSupport::JSON.decode(tags)
      preset['precondition'] = query.to_json
      if params['app'] == 'Firmware Upgrade with Schedule'
        case params['triggering_conditions_repeat']
        when 'No repeat'
          configuration = "[ { \"type\" : \"provision\", \"name\" : \"FWUpgradeDoSchedule#{append_str}\", \"args\" : [ \"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", \"#{params['fw_file']}\", \"#{params['fw_version']}\", \"#{params['name']}_#{FWUP_SCHEDULE_CH}\", #{MAX_RETRY}, #{params['default_inform_interval']}, \"#{params['name']}\", \"#{params['triggering_conditions_repeat']}\", \"#{params['no_repeat_start_date']}\", \"#{params['no_repeat_end_date']}\" ] } ]"
        when 'Everyday'
          start_date = Date.parse(params['repeat_start_days'])
          end_date = Date.parse(params['repeat_until_days'])
          date_arr = (start_date..end_date).map { |date| date.strftime('%Y/%m/%d') }
          date_arr = date_arr.join(', ')
          configuration = "[ { \"type\" : \"provision\", \"name\" : \"FWUpgradeDoSchedule#{append_str}\", \"args\" : [ \"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", \"#{params['fw_file']}\", \"#{params['fw_version']}\", \"#{params['name']}_#{FWUP_SCHEDULE_CH}\", #{MAX_RETRY}, #{params['default_inform_interval']}, \"#{params['name']}\", \"#{params['triggering_conditions_repeat']}\", \"#{date_arr}\", \"#{params['repeat_timerange']}\" ] } ]"
        when 'Weekly'
          start_date = Date.parse(params['repeat_start_days'])
          end_date = Date.parse(params['repeat_until_days'])
          date_arr = ((start_date..end_date).to_a.select { |k| params['repeat_weeks'].map(&:to_i).include?(k.wday) }).map { |date| date.strftime('%Y/%m/%d') }
          date_arr = date_arr.join(', ')
          configuration = "[ { \"type\" : \"provision\", \"name\" : \"FWUpgradeDoSchedule#{append_str}\", \"args\" : [ \"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", \"#{params['fw_file']}\", \"#{params['fw_version']}\", \"#{params['name']}_#{FWUP_SCHEDULE_CH}\", #{MAX_RETRY}, #{params['default_inform_interval']}, \"#{params['name']}\", \"#{params['triggering_conditions_repeat']}\", \"#{date_arr}\", \"#{params['repeat_timerange']}\" ] } ]"
        else
          puts 'Unexcepted error when preset update fwup.'
        end
      else
        configuration = "[ { \"type\" : \"provision\", \"name\" : \"FWUpgradeDo2#{append_str}\", \"args\" : [ \"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", \"#{params['fw_file']}\", \"#{params['fw_version']}\", \"#{params['name']}_#{FWUP_CH}\", #{MAX_RETRY}, #{params['default_inform_interval']}, \"#{params['name']}\" ] } ]"
      end
      preset['configurations'] = ActiveSupport::JSON.decode(configuration)
      preset['schedule'] = ''
      event = '{ "2 PERIODIC" : true }'
      preset['events'] = JSON.parse(event)
    when 3
      presetname = "#{params['name']}_FWUpgradeDone#{append_str}"
      query['_tags'] = "#{params['name']}_NeedUpgrade"
      preset['precondition'] = query.to_json
      configuration = if params['app'] == 'Firmware Upgrade with Schedule'
                        "[ { \"type\" : \"provision\", \"name\" : \"FWUpgradeDoneSchedule#{append_str}\", \"args\" : [ \"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", #{params['default_inform_interval']}, \"#{params['name']}\" ] } ]"
                      else
                        "[ { \"type\" : \"provision\", \"name\" : \"FWUpgradeDone2#{append_str}\", \"args\" : [ \"#{params['name']}_NeedUpgrade\", \"#{params['name']}_NeedUpgradeDone\", #{params['default_inform_interval']}, \"#{params['name']}\" ] } ]"
                      end
      preset['configurations'] = ActiveSupport::JSON.decode(configuration)
      preset['schedule'] = ''
      event = '{ "7 TRANSFER COMPLETE" : true, "M Download" : true }'
      preset['events'] = JSON.parse(event)
    when 4
      preset['weight'] = 0
      presetname = "#{params['name']}_SetProfileMatch#{append_str}"
      tags = "{\"$ne\":\"#{params['name']}_ProfileMatched\"}"
      query['_tags'] = ActiveSupport::JSON.decode(tags)
      preset['precondition'] = query.to_json
      configuration = "[ { \"type\" : \"provision\", \"name\" : \"ProfileMatch#{append_str}\", \"args\" : [ \"#{params['name']}_ProfileMatched\" ] } ]"
      preset['configurations'] = ActiveSupport::JSON.decode(configuration)
      preset['schedule'] = ''
      event = '{}'
      preset['events'] = JSON.parse(event)
    end
    http = create_api_conn()
    res = http.put("/profiles/presets/#{CGI.escape(presetname)}", ActiveSupport::JSON.encode(preset))
    res
  end

  def update_config_presets(params, addition)
    query = {}
    query = ActiveSupport::JSON.decode(CGI.unescape(params['profile.query'])) if params.key?('profile.query')
    query['InternetGatewayDevice.DeviceInfo.ModelName'] = params['model']
    append_tplink = false
    # append_tplink = true if params['model_type'] == "1"
    queries = get_queries(query, 'InternetGatewayDevice', append_tplink)

    res = nil
    queryList = [{ 'query' => queries['InternetGatewayDevice'], 'append_str' => '' }, { 'query' => queries['Device'], 'append_str' => '_181' }]
    queryList.each do |element|
      res = update_config_preset(params, addition, element['append_str'], element['query'])
      return res if res.code != '200'
    end
    res
  end

  def update_config_preset(params, addition, append_str, query)
    presetname = ''
    preset = {}
    preset['channel'] = "#{params['name']}_#{CONFIG_CH}" # configuration download : 2
    preset['weight'] = addition
    case addition
    when 1
      presetname = "#{params['name']}_VendorConfigDo#{append_str}"
      tags = "{\"$ne\":\"#{params['name']}_VendorConfigDone\"}"
      query['_tags'] = ActiveSupport::JSON.decode(tags)
      preset['precondition'] = query.to_json
      configuration = "[ { \"type\" : \"provision\", \"name\" : \"VendorConfigDo2#{append_str}\", \"args\" : [ \"#{params['name']}_VendorConfigDo\", \"#{params['name']}_VendorConfigDone\", \"#{params['config']}\", \"#{params['name']}_#{CONFIG_CH}\", #{MAX_RETRY}, \"#{params['name']}\"] } ]"
      preset['configurations'] = ActiveSupport::JSON.decode(configuration)
      preset['schedule'] = ''
      event = '{ "2 PERIODIC" : true }'
      preset['events'] = JSON.parse(event)
    when 2
      presetname = "#{params['name']}_VendorConfigDone#{append_str}"
      query['_tags'] = "#{params['name']}_VendorConfigDo"
      preset['precondition'] = query.to_json
      configuration = "[ { \"type\" : \"provision\", \"name\" : \"VendorConfigDone#{append_str}\", \"args\" : [  \"#{params['name']}_VendorConfigDo\", \"#{params['name']}_VendorConfigDone\", \"#{params['name']}\"] } ]"
      preset['configurations'] = ActiveSupport::JSON.decode(configuration)
      preset['schedule'] = ''
      event = '{ "7 TRANSFER COMPLETE" : true, "M Download" : true }'
      preset['events'] = JSON.parse(event)
    end
    http = create_api_conn()
    res =  http.put("/profiles/presets/#{CGI.escape(presetname)}", ActiveSupport::JSON.encode(preset))
    res
  end

  def update_essid_preset(params); end

  def update_pppoe_presets(params)
    query = {}
    query = ActiveSupport::JSON.decode(CGI.unescape(params['profile.query'])) if params.key?('profile.query')
    query['InternetGatewayDevice.DeviceInfo.ModelName'] = params['model']
    append_tplink = false
    # append_tplink = true if params['model_type'] == "1"
    queries = get_queries(query, 'InternetGatewayDevice', append_tplink)

    res = nil
    queryList = [{ 'query' => queries['InternetGatewayDevice'], 'append_str' => '' }, { 'query' => queries['Device'], 'append_str' => '_181' }]
    queryList.each do |element|
      res = update_pppoe_preset(params, element['append_str'], element['query'])
      return res if res.code != '200'
    end
    res
  end

  def update_pppoe_preset(params, append_str, query)
    presetname = "#{params['name']}_PPPoESetTag#{append_str}"

    preset = {}
    preset['channel'] = "#{params['name']}_#{PPPOE_CH}"
    preset['weight'] = 1

    # * Tag checking
    tags = "{\"$ne\":\"#{params['name']}_set_pppoe\"}"
    query['_tags'] = ActiveSupport::JSON.decode(tags)

    # * provisions setting
    # * args: tag_name, profile_name, model_name
    configuration = "[ { \"type\" : \"provision\", \"name\" : \"PPPoE_Setting#{append_str}\", \"args\" : [ \"#{params['name']}_set_pppoe\",\"#{params['name']}\",\"#{params['model']}\", \"#{params['name']}_#{params['pppoe_file']}\" ] } ]"
    preset['configurations'] = ActiveSupport::JSON.decode(configuration)

    preset['precondition'] = query.to_json
    preset['schedule'] = ''
    # * Accept All Event from CPE
    event = '{ }'
    preset['events'] = JSON.parse(event)

    http = create_api_conn()
    res =  http.put("/profiles/presets/#{CGI.escape(presetname)}", ActiveSupport::JSON.encode(preset))
    res
  end

  def update_inform_presets(params, addition)
    query = {}
    query = ActiveSupport::JSON.decode(CGI.unescape(params['profile.query'])) if params.key?('profile.query')
    query['InternetGatewayDevice.DeviceInfo.ModelName'] = params['model']
    append_tplink = false
    # append_tplink = true if params['model_type'] == "1"
    queries = get_queries(query, 'InternetGatewayDevice', append_tplink)

    res = nil
    queryList = [{ 'query' => queries['InternetGatewayDevice'], 'append_str' => '' }, { 'query' => queries['Device'], 'append_str' => '_181' }]
    queryList.each do |element|
      res = update_inform_preset(params, addition, element['append_str'], element['query'])
      return res if res.code != '200'
    end
    res
  end

  def update_inform_preset(params, addition, append_str, query)
    preset = {}
    preset['channel'] = "#{params['name']}_#{INFORM_CH}"
    case addition
    when 1
      preset['weight'] = 1
      presetname = "#{params['name']}_SetPeriodicInterval#{append_str}"
      tags = "{\"$ne\":\"#{params['name']}_SetPeriodicInterval\"}"
      query['_tags'] = ActiveSupport::JSON.decode(tags)
      preset['precondition'] = query.to_json
      configuration = "[ { \"type\" : \"provision\", \"name\" : \"InformInterval#{append_str}\", \"args\" : [ \"#{params['name']}_SetPeriodicInterval\", #{params['inform_interval'].to_i}, \"#{params['name']}\" ] } ]"
      preset['configurations'] = ActiveSupport::JSON.decode(configuration)
      preset['schedule'] = ''
      event = '{ "2 PERIODIC" : true }'
      preset['events'] = JSON.parse(event)
    when 2
      preset['weight'] = 0
      presetname = "#{params['name']}_SetProfileMatch#{append_str}"
      tags = "{\"$ne\":\"#{params['name']}_ProfileMatched\"}"
      query['_tags'] = ActiveSupport::JSON.decode(tags)
      preset['precondition'] = query.to_json
      configuration = "[ { \"type\" : \"provision\", \"name\" : \"ProfileMatch#{append_str}\", \"args\" : [ \"#{params['name']}_ProfileMatched\" ] } ]"
      preset['configurations'] = ActiveSupport::JSON.decode(configuration)
      preset['schedule'] = ''
      event = '{}'
      preset['events'] = JSON.parse(event)
    end
    http = create_api_conn()
    res =  http.put("/profiles/presets/#{CGI.escape(presetname)}", ActiveSupport::JSON.encode(preset))
    res
  end

  def delete_fwup_presets(profile, addition)
    res = nil
    presetnameList = []

    presetnameList = ["#{profile['_id']}_FWUpgradeTag", "#{profile['_id']}_FWUpgradeTag_181"] if addition == 1
    presetnameList = ["#{profile['_id']}_FWUpgradeDo", "#{profile['_id']}_FWUpgradeDo_181"] if addition == 2
    presetnameList = ["#{profile['_id']}_FWUpgradeDone", "#{profile['_id']}_FWUpgradeDone_181"] if addition == 3
    presetnameList = ["#{profile['_id']}_SetProfileMatch", "#{profile['_id']}_SetProfileMatch_181"] if addition == 4

    presetnameList.each do |presetname|
      http = create_api_conn()
      res = http.delete("/profiles/presets/#{CGI.escape(presetname)}", nil)
    end
    res
  end

  def delete_config_presets(profile, addition)
    res = nil
    presetnameList = []

    presetnameList = ["#{profile['_id']}_VendorConfigDo", "#{profile['_id']}_VendorConfigDo_181"] if addition == 1
    presetnameList = ["#{profile['_id']}_VendorConfigDone", "#{profile['_id']}_VendorConfigDone_181"] if addition == 2

    presetnameList.each do |presetname|
      http = create_api_conn()
      res = http.delete("/profiles/presets/#{CGI.escape(presetname)}", nil)
    end
    res
  end

  def delete_essid_preset(profile); end

  def delete_pppoe_presets(profile)
    res = nil
    presetnameList = ["#{profile['_id']}_PPPoESetTag", "#{profile['_id']}_PPPoESetTag_181"]
    presetnameList.each do |presetname|
      # TODO: Add tag cleaning mechanism
      http = create_api_conn()
      res = http.delete("/profiles/presets/#{CGI.escape(presetname)}", nil)
    end
    res
  end

  def delete_inform_presets(profile, addition)
    res = nil
    presetnameList = ["#{profile['_id']}_SetPeriodicInterval", "#{profile['_id']}_SetPeriodicInterval_181"]
    presetnameList = ["#{profile['_id']}_SetProfileMatch", "#{profile['_id']}_SetProfileMatch_181"] if addition == 2
    presetnameList.each do |presetname|
      http = create_api_conn()
      res = http.delete("/profiles/presets/#{CGI.escape(presetname)}", nil)
    end
    res
  end

  def update_presets(params)
    app = params['app']
    case app
    when 'Firmware Upgrade'
      # For firmware upgrade need to update three presets "FWUpgradeTag/FWUpgradeDo/FWUpgradeDone"
      res = update_fwup_presets(params, 1)
      res = update_fwup_presets(params, 2)
      res = update_fwup_presets(params, 3)
      res = update_fwup_presets(params, 4)
      remove_tags_from_devices("#{params['name']}_NeedUpgrade")
      remove_tags_from_devices("#{params['name']}_NeedUpgradeDone")
      remove_tags_from_devices("#{params['name']}_ProfileMatched")
      remove_faults_from_profile("#{params['name']}_#{FWUP_CH}")
    when 'Configure File Download'
      res = update_config_presets(params, 1)
      res = update_config_presets(params, 2)
      remove_tags_from_devices("#{params['name']}_VendorConfigDo")
      remove_tags_from_devices("#{params['name']}_VendorConfigDone")
      remove_faults_from_profile("#{params['name']}_#{CONFIG_CH}")
    when 'ESSID'
      res = update_essid_preset(params)
    when 'PPPoE'
      res = update_pppoe_presets(params)
      remove_tags_from_devices("#{params['name']}_set_pppoe")
      remove_faults_from_profile("#{params['name']}_#{PPPOE_CH}")
    when 'Periodic Inform Interval'
      res = update_inform_presets(params, 1)
      res = update_inform_presets(params, 2)
      remove_tags_from_devices("#{params['name']}_SetPeriodicInterval")
      remove_tags_from_devices("#{params['name']}_ProfileMatched")
      remove_tags_from_devices('ProfileMatched')
      remove_faults_from_profile("#{params['name']}_#{INFORM_CH}")
    when 'Firmware Upgrade with Schedule'
      # For firmware upgrade with schedule need to update three presets "FWUpgradeTag/FWUpgradeDo/FWUpgradeDone"
      res = update_fwup_presets(params, 1)
      res = update_fwup_presets(params, 2)
      res = update_fwup_presets(params, 3)
      res = update_fwup_presets(params, 4)
      remove_tags_from_devices("#{params['name']}_NeedUpgradeSchedule")
      remove_tags_from_devices("#{params['name']}_NeedUpgradeScheduleToDo")
      remove_tags_from_devices("#{params['name']}_NeedUpgrade")
      remove_tags_from_devices("#{params['name']}_NeedUpgradeDone")
      remove_tags_from_devices("#{params['name']}_ProfileMatched")
      remove_faults_from_profile("#{params['name']}_#{FWUP_SCHEDULE_CH}")
    end
    res
  end

  def delete_presets(profile)
    app = profile['app']
    case app
    when 'Firmware Upgrade'
      # For firmware upgrade need to delete three presets "FWUpgradeTag/FWUpgradeDo/FWUpgradeDone"
      res = delete_fwup_presets(profile, 1)
      res = delete_fwup_presets(profile, 2)
      res = delete_fwup_presets(profile, 3)
      res = delete_fwup_presets(profile, 4)
      remove_tags_from_devices("#{profile['_id']}_NeedUpgrade")
      remove_tags_from_devices("#{profile['_id']}_NeedUpgradeDone")
      remove_tags_from_devices("#{profile['_id']}_ProfileMatched")
      remove_tags_from_devices('ProfileMatched')
      remove_faults_from_profile("#{profile['_id']}_#{FWUP_CH}")
    when 'Configure File Download'
      res = delete_config_presets(profile, 1)
      res = delete_config_presets(profile, 2)
      remove_tags_from_devices("#{profile['_id']}_VendorConfigDo")
      remove_tags_from_devices("#{profile['_id']}_VendorConfigDone")
      remove_faults_from_profile("#{profile['_id']}_#{CONFIG_CH}")
    when 'ESSID'
      res = delete_essid_preset(profile)
    when 'PPPoE'
      res = delete_pppoe_presets(profile)
      remove_tags_from_devices("#{profile['_id']}_set_pppoe")
      remove_faults_from_profile("#{profile['_id']}_#{PPPOE_CH}")
    when 'Periodic Inform Interval'
      res = delete_inform_presets(profile, 1)
      res = delete_inform_presets(profile, 2)
      remove_tags_from_devices("#{profile['_id']}_SetPeriodicInterval")
      remove_tags_from_devices("#{profile['_id']}_ProfileMatched")
      remove_tags_from_devices('ProfileMatched')
      remove_faults_from_profile("#{profile['_id']}_#{INFORM_CH}")
    when 'Firmware Upgrade with Schedule'
      # For firmware upgrade with schedule need to delete three presets "FWUpgradeTag/FWUpgradeDo/FWUpgradeDone"
      res = delete_fwup_presets(profile, 1)
      res = delete_fwup_presets(profile, 2)
      res = delete_fwup_presets(profile, 3)
      res = delete_fwup_presets(profile, 4)
      remove_tags_from_devices("#{profile['_id']}_NeedUpgradeSchedule")
      remove_tags_from_devices("#{profile['_id']}_NeedUpgradeScheduleToDo")
      remove_tags_from_devices("#{profile['_id']}_NeedUpgrade")
      remove_tags_from_devices("#{profile['_id']}_NeedUpgradeDone")
      remove_tags_from_devices("#{profile['_id']}_ProfileMatched")
      remove_tags_from_devices('ProfileMatched')
      remove_faults_from_profile("#{profile['_id']}_#{FWUP_SCHEDULE_CH}")
    end
    res
  end

  # return field_query if field_only = true
  # or aggregate_pipeline if field_only = false
  def get_aggregate_query_by_098(query98, field_only = false)
    profile_params = {}
    field_query = get_db_filter_query(query98, DeviceFilters::DB_FILTER_TYPE_098, true, Rails.configuration.profiles_index_parameters, Rails.configuration.profiles_summary_parameters)
    if field_only
      field_query
    else
      profile_params['query'] = ActiveSupport::JSON.encode(field_query)
      aggregate_pipeline = generate_aggregate_pipeline(profile_params, true, true, true, Rails.configuration.profiles_index_parameters, Rails.configuration.profiles_summary_parameters)
      aggregate_pipeline
    end
  end

  def get_profile_total_query(profile, userFilter, filter_time, field_only = false)
    query = ActiveSupport::JSON.decode(profile['query'])
    query['InternetGatewayDevice.DeviceInfo.ModelName'] = profile['model'] if profile.key?('model')
    #  query['InternetGatewayDevice.DeviceInfo.HardwareVersion'] = profile['hwver'] if profile.has_key?('hwver')  # not verify yet
    if userFilter == 1
      query_time = "{\"$gt\":\"#{filter_time}\"}"
      query['_lastInform'] = ActiveSupport::JSON.decode(query_time)
    end
    aggregate_query = get_aggregate_query_by_098(query, field_only)
    aggregate_query
  end

  def get_profile_faults_query(profile, userFilter, filter_time)
    query = {}
    app = profile['app']
    case app
    when 'Firmware Upgrade'
      channel = FWUP_CH
    when 'Configure File Download'
      channel = CONFIG_CH
    when 'PPPoE'
      channel = PPPOE_CH
    when 'Periodic Inform Interval'
      channel = INFORM_CH
    when 'Firmware Upgrade with Schedule'
      channel = FWUP_SCHEDULE_CH
    end
    query['channel'] = "#{profile['_id']}_#{channel}"

    if userFilter == 1
      query_time = "{\"$gt\":\"#{filter_time.to_i}\"}"
      query['timestamp'] = ActiveSupport::JSON.decode(query_time)
    end
    query
  end

  def get_profile_exclude_faults_query(profile, query, userFilter, filter_time)
    total = 0
    exclude = []
    query_faults = get_profile_faults_query(profile, userFilter, filter_time)
    res = query_resource(create_api_conn(), 'faults', query_faults)
    total = res[:total]
    if total > 0
      faults = res[:result]
      if total > 1
        (0..(total - 1)).each do |i|
          exclude_dev = { '$ne' => faults[i]['device'] }
          device = { '_id' => exclude_dev }
          exclude.push(device)
        end
        query['$and'] = exclude
      else
        faults = res[:result]
        exclude_dev =  { '$ne' => faults[0]['device'] }
        query['_id'] = exclude_dev
      end
    end
  end

  def get_profile_done_query(profile, userFilter, filter_time, field_only = false)
    query = ActiveSupport::JSON.decode(profile['query'])
    query['InternetGatewayDevice.DeviceInfo.ModelName'] = profile['model'] if profile.key?('model')
    #  query['InternetGatewayDevice.DeviceInfo.HardwareVersion'] = profile['hwver'] if profile.has_key?('hwver')  # not verify yet
    app = profile['app']
    case app
    when 'Firmware Upgrade'
      donetag = "#{profile['_id']}_NeedUpgradeDone"
      if !query.nil? && query.key?('$and')
        update_fwup_preset_remove_softver_query(query)
        query.delete('$and') if query['$and'].count == 0
      end
      query.delete('InternetGatewayDevice.DeviceInfo.SoftwareVersion') if query.key?('InternetGatewayDevice.DeviceInfo.SoftwareVersion')
    when 'Configure File Download'
      donetag = "#{profile['_id']}_VendorConfigDone"
    when 'PPPoE'
      donetag = "#{profile['_id']}_set_pppoe"
    when 'Periodic Inform Interval'
      donetag = "#{profile['_id']}_SetPeriodicInterval"
    when 'Firmware Upgrade with Schedule'
      donetag = "#{profile['_id']}_NeedUpgradeDone"
      if !query.nil? && query.key?('$and')
        update_fwup_preset_remove_softver_query(query)
        query.delete('$and') if query['$and'].count == 0
      end
      query.delete('InternetGatewayDevice.DeviceInfo.SoftwareVersion') if query.key?('InternetGatewayDevice.DeviceInfo.SoftwareVersion')
    end
    query['_tags'] = donetag
    if userFilter == 1
      query_time = "{\"$gt\":\"#{filter_time}\"}"
      query['_lastInform'] = ActiveSupport::JSON.decode(query_time)
    end
    aggregate_query = get_aggregate_query_by_098(query, field_only)
    aggregate_query
  end

  def get_profile_todo_query(profile, userFilter, filter_time, field_only = false)
    query = ActiveSupport::JSON.decode(profile['query'])
    query['InternetGatewayDevice.DeviceInfo.ModelName'] = profile['model'] if profile.key?('model')
    #  query['InternetGatewayDevice.DeviceInfo.HardwareVersion'] = profile['hwver'] if profile.has_key?('hwver')  # not verify yet
    app = profile['app']
    case app
    when 'Firmware Upgrade'
      donetag = "#{profile['_id']}_NeedUpgradeDone"
    when 'Configure File Download'
      donetag = "#{profile['_id']}_VendorConfigDone"
    when 'PPPoE'
      donetag = "#{profile['_id']}_set_pppoe"
    when 'Periodic Inform Interval'
      donetag = "#{profile['_id']}_SetPeriodicInterval"
    when 'Firmware Upgrade with Schedule'
      donetag = "#{profile['_id']}_NeedUpgradeDone"
    end
    tags = "{\"$ne\":\"#{donetag}\"}"
    query['_tags'] = ActiveSupport::JSON.decode(tags)

    if userFilter == 1
      query_time = "{\"$gt\":\"#{filter_time}\"}"
      query['_lastInform'] = ActiveSupport::JSON.decode(query_time)
    end
    get_profile_exclude_faults_query(profile, query, userFilter, filter_time)
    aggregate_query = get_aggregate_query_by_098(query, field_only)
    aggregate_query
  end

  def get_profile_todo_progress_query(profile, userFilter, filter_time, field_only = false)
    query = ActiveSupport::JSON.decode(profile['query'])
    query['InternetGatewayDevice.DeviceInfo.ModelName'] = profile['model'] if profile.key?('model')
    #  query['InternetGatewayDevice.DeviceInfo.HardwareVersion'] = profile['hwver'] if profile.has_key?('hwver')  # not verify yet
    app = profile['app']
    case app
    when 'Firmware Upgrade'
      donetag = "#{profile['_id']}_NeedUpgradeDone"
    when 'Configure File Download'
      donetag = "#{profile['_id']}_VendorConfigDone"
    when 'PPPoE'
      donetag = "#{profile['_id']}_set_pppoe"
    when 'Periodic Inform Interval'
      donetag = "#{profile['_id']}_SetPeriodicInterval"
    when 'Firmware Upgrade with Schedule'
      donetag = "#{profile['_id']}_NeedUpgradeDone"
    end
    tags = "{\"$ne\":\"#{donetag}\"}"
    query['_tags'] = ActiveSupport::JSON.decode(tags)

    if userFilter == 1
      query_time = "{\"$gt\":\"#{filter_time}\"}"
      query['_lastInform'] = ActiveSupport::JSON.decode(query_time)
    end
    aggregate_query = get_aggregate_query_by_098(query, field_only)
    aggregate_query
  end

  def get_profile_total_num(http, profile, userFilter, filter_time)
    device_query = get_profile_total_query(profile, userFilter, filter_time)
    ret = get_device_num(http, device_query)
    ret
  end

  def get_profile_done_num(http, profile, userFilter, filter_time)
    device_query = get_profile_done_query(profile, userFilter, filter_time)
    ret = get_device_num(http, device_query)
    ret
  end

  def get_profile_todo_num(http, profile, userFilter, filter_time)
    device_query = get_profile_todo_query(profile, userFilter, filter_time)
    ret = get_device_num(http, device_query)
    ret
  end

  def get_profil_failed_num(http, profile, _userFilter, _filter_time)
    query = get_profile_faults_query(profile, 0, 0)
    res = query_resource(http, 'faults', query)
    ret = res[:total]
    ret
  end

  def get_device_num(http, aggregate_pipeline)
    res = http.post("/profile_progress/devnum", ActiveSupport::JSON.encode(aggregate_pipeline), 'Content-type' => 'application/json')
    #res = aggregate(http, 'devices', aggregate_pipeline, 'HEAD')
    p "res['total'] = #{res['total']}"
    ret = res['total'].to_i
    ret
  end

  def calculate_profiles_progresses(profiles)
    profiles_query = []
    progresses = []
    profiles.each do |profile|
      single_query = {}
      single_query['id'] = profile['_id']
      single_query['done'] = get_profile_done_query(profile, 0, 0)
      single_query['todo'] = get_profile_todo_progress_query(profile, 0, 0)
      single_query['failed'] = get_profile_faults_query(profile, 0, 0)
      single_query['total'] = get_profile_total_query(profile, 0, 0)
      profiles_query.push(single_query)
    end
    http = create_api_conn()
    res = http.post('/profile_progress/all', ActiveSupport::JSON.encode(profiles_query), 'Content-type' => 'application/json')
    if res.code == '200'
      progresses = JSON.parse(res.body)
      profiles.each do |profile|
        progresses.each do |progress|
          if profile['_id'] == progress['id']
            profile['progress'] = progress
            profile['failed_query'] = get_profile_faults_query(profile, 0, 0)
          end
        end
      end
    else
      progress = {}
      progress['done'] = 0
      progress['todo'] = 0
      progress['failed'] = 0
      progress['total'] = 0
      profiles.each do |profile|
        profile['failed_query'] = {}
        profile['progress'] = progress
      end
    end
    profiles
  end

  def calculate_profile_progress(profile)
    calculate_profiles_progresses([profile])[0]
  end

  private

  def get_profile_query(params)
    query_98 = {}
    query_field = {}
    begin
      if params.key?('query')
        query = ActiveSupport::JSON.decode(CGI.unescape(params['query']))
      else
        return { 'query' => query_98, 'query_1' => query_field }
      end

      if params[:query_type] == '0'
        queries = get_queries(query, 'InternetGatewayDevice')
        query_field = queries['field']
        query_98 = queries['InternetGatewayDevice']
      else
        queries = get_queries(query, 'field')
        query_field = queries['field']
        query_98 = queries['InternetGatewayDevice']
      end
    rescue StandardError => e
      query_98 = {}
      query_field = {}
    end
    { 'query' => query_98, 'query_1' => query_field }
  end

  def construct_profile(params)
    app = params['app']
    profile = {}
    profile['app'] = params['app']
    profile['model'] = params['model']

    q = get_profile_query(params)
    profile['query_1'] = ActiveSupport::JSON.encode(q['query_1'])
    profile['query'] = ActiveSupport::JSON.encode(q['query'])
    params['profile.query'] = profile['query']
    case app
    when 'Firmware Upgrade'
      profile['fw_file'] = params['fw_file']
      profile['trigger_interval'] = params['trigger_interval'].to_i
      profile['default_inform_interval'] = 3600
    when 'Configure File Download'
      profile['config'] = params['config']
    when 'ESSID'
      # to do add ESSID required to profile
    when 'PPPoE'
      profile['pppoe_file'] = params['pppoe_file']
    when 'Periodic Inform Interval'
      profile['model'] = params['model']
      profile['inform_interval'] = params['inform_interval'].to_i
    when 'Firmware Upgrade with Schedule'
      profile['fw_file'] = params['fw_file']
      profile['triggering_conditions_repeat'] = params['triggering_conditions_repeat']
      profile['totalCpeCount'] = params['totalCpeCount'].to_i
      profile['countCpeEachDay'] = params['countCpeEachDay'].to_i
      profile['countExecuteDay'] = params['countExecuteDay'].to_i
      profile['default_inform_interval'] = 3600
      case profile['triggering_conditions_repeat']
      when 'No repeat'
        profile['no_repeat_start_date'] = params['no_repeat_start_date']
        profile['no_repeat_end_date'] = params['no_repeat_end_date']
      when 'Everyday'
        profile['repeat_start_days'] = params['repeat_start_days']
        profile['repeat_until_days'] = params['repeat_until_days']
        profile['repeat_timerange'] = params['repeat_timerange']
      when 'Weekly'
        profile['repeat_start_days'] = params['repeat_start_days']
        profile['repeat_until_days'] = params['repeat_until_days']
        profile['repeat_timerange'] = params['repeat_timerange']
        profile['repeat_weeks'] = params['repeat_weeks']
      else
        puts 'Unexcepted error when profiles construct.'
      end
    end
    profile
  end
end
