# module Reports for reports_controller
module ReportsModule
  FWSCHEDULE_APPENDIX = '-FWSCHEDULE'.freeze
  def show_tab_content(tab, month)
    case tab
    when 'report_setting'
      report_setting()
    when 'cpe_inventory'
      get_cpe_inventory(month)
      respond_to do |format|
        format.html # index.html.erb
        format.json { render json: @inventory_reports }
      end
    when 'user_operation'
      get_user_operation(month)
      respond_to do |format|
        format.html # index.html.erb
        format.json { render json: @operations }
      end
    when 'profile_stats'
      get_profile_stats(month)
      respond_to do |format|
        format.html # index.html.erb
        format.json { render json: @profile_stats }
      end
    when 'system_usage'
      get_system_usage(month)
    when 'invalid_device'
      get_invalid_device(month)
      respond_to do |format|
        format.html # index.html.erb
        format.json { render json: @invalid_reports }
      end
    else
      get_cpe_inventory(month)
      respond_to do |format|
        format.html # index.html.erb
        format.json { render json: @inventory_reports }
      end
    end
  end

  def process_month_report_list(month_report_list, reportday)
    tmp_month = []
    res = query_resource(create_api_conn, 'cpeInventory', {})
    res[:result].each { |x| month_report_list.push(x['_id']) }
    month_report_list = month_report_list.sort.reverse

    month_report_list.each do |x|
      if reportday != '01'
        reportday = Time.now.end_of_month.strftime('%d') if reportday == 'end'
        if reportday.to_i > Time.now.strftime('%d').to_i
          tmp_month.push(x) if x == Time.now.strftime('%Y%m')
          if x != 'Current Month'
            tmp_month.push(x) if x > Time.now.strftime('%Y%m')
          end
        else
          tmp_month.push(x) if x == (Time.now + 1.month).strftime('%Y%m')
          if x != 'Current Month'
            tmp_month.push(x) if x > (Time.now + 1.month).strftime('%Y%m')
          end
        end
      else
        tmp_month.push(x) if x == (Time.now + 1.month).strftime('%Y%m')
        if x != 'Current Month'
          tmp_month.push(x) if x > (Time.now + 1.month).strftime('%Y%m')
        end
      end
    end

    tmp_month.each { |x| month_report_list.delete(x) }
    month_report_list
  end

  def month_transform(param, reportday)
    if param == 'Current Month'
      if reportday != '01'
        reportday = Time.now.end_of_month.strftime('%d') if reportday == 'end'
        month = if reportday.to_i > Time.now.strftime('%d').to_i
                  Time.now.strftime('%Y%m')
                else
                  (Time.now + 1.month).strftime('%Y%m')
                end
      else
        month = (Time.now + 1.month).strftime('%Y%m')
      end
    else
      month = param
    end
    month
  end

  def generate_tar_file(params, dir_name, dir_path, file_prefix)
    params.each do |param|
      full_name = dir_path + '/' + file_prefix + param
      if param.include? 'png'
        data_url = params[param]
        png = Base64.decode64(data_url['data:image/png;base64,'.length..-1])
        File.open(full_name, 'wb') { |f| f.write(png) }
      elsif param.include? 'csv'
        data = JSON.parse(params[param])
        CSV.open(full_name, 'wb') do |csv|
          data.each do |rows|
            csv << rows
          end
        end
      end
    end
    tar_file_cmd = 'tar -zcPf ' + dir_name + '.tar.gz ' + dir_path
    system(tar_file_cmd)
    tar_path = dir_path + '.tar.gz'
    mv_tar_cmd = 'mv ' + tar_path + ' /tmp'
    system(mv_tar_cmd)
    tar_path
  end

  def db_query_resource_report_setting
    res_report_setting = query_resource(create_api_conn, 'reportSetting', { "_id": 'report_setting' })
    reportday = if res_report_setting[:message] == '404 Not Found'
                  '01'
                else
                  res_report_setting[:result][0]['reportday']
                end
    reportday
  end

  def calculate_month_range(month, arr_info, ref_time)
    ref_date_time = Time.at(ref_time).to_datetime

    control_until = 0
    until (ref_date_time + 1.month).strftime('%Y%m') == month
      ref_date_time -= 1.month
      control_until += 1

      break if control_until > 6
    end
    arr_info.push(ref_date_time)
    arr_info.push(((ref_date_time + 1.month) - 1.second))
    arr_info
  end

  def get_report_output(tab)
    output = ''
    case tab
    when 'cpe_inventory'
      output = `nodejs /opt/genieacs/bin/agileacs_report.js update cpeinventory`
    when 'user_operation'
      output = `nodejs /opt/genieacs/bin/agileacs_report.js update useroperation`
    when 'profile_stats'
      output = `nodejs /opt/genieacs/bin/agileacs_report.js update profilestats`
    when 'system_usage'
      output = `nodejs /opt/genieacs/bin/agileacs_report.js collect reportsetting`
    when 'invalid_device'
      output = `nodejs /opt/genieacs/bin/agileacs_report.js update invaliddevice`
    else
      system('nodejs /opt/genieacs/bin/agileacs_report.js collect all')
      output = `nodejs /opt/genieacs/bin/agileacs_report.js collect reportsetting`
    end
    output
  end

  def get_month_info(month, tab)
    arr_info = []
    reportday = db_query_resource_report_setting()
    current_month = month_transform('Current Month', reportday)
    if month == current_month
      arr_info.push('Current Month')
      output = get_report_output(tab)
      arr_output = output.split(',')
      start_time = arr_output[0].scan(/[0-9]/).join('')
      end_time = arr_output[1].scan(/[0-9]/).join('')
      arr_info.push(start_time.to_i / 1000)
      arr_info.push(end_time.to_i / 1000)
    else
      arr_info.push(month)
      output = `nodejs /opt/genieacs/bin/agileacs_report.js collect reportsetting`
      arr_output = output.split(',')
      ref_time = arr_output[0].scan(/[0-9]/).join('')
      ref_time = ref_time.to_i / 1000
      arr_info = calculate_month_range(month, arr_info, ref_time)
    end
    arr_info
  end

  def report_setting
    res = query_resource(create_api_conn, 'reportSetting', { "_id": 'report_setting' })

    if res[:message] == '404 Not Found'
      res_index = @month_day_db_list.index('01')
    else
      @report_setting = res[:result]
      res_index = @month_day_db_list.index(@report_setting[0]['reportday'])
    end
    reportday = @month_day_list[res_index]

    if params.key?('reportday')
      reportday = params['reportday']
      index = @month_day_list.index(reportday)
      object = '[Report Setting]'
      http = create_api_conn
      res = http.post("/reportSetting/#{CGI.escape(@month_day_db_list[index].strip)}", nil)
      if res.code == '200'
        flash.now[:success] = "Set Collect report data at every #{reportday} of month succeed"
        agileACSLog(6, object, "Set Collect report data at every #{reportday} of month succeed")
      elsif res.code == '404'
        # If response is 404 not found, redirect to index directly.
      else
        flash.now[:error] = "Unexpected error (#{res.code}): #{res.body}"
        agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
      end
    end
    @reportday = reportday
    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @report_setting }
    end
  end

  def cpe_inventory_query(month)
    query = {}
    query['_id'] = month
    query
  end

  def get_cpe_inventory(month)
    @inventory_reports = []
    arr_info = get_month_info(month, 'cpe_inventory')
    @filter = arr_info[0]
    @start_time = Time.at(arr_info[1].to_i).strftime('%Y/%m/%d')
    @end_time = Time.at(arr_info[2].to_i).strftime('%Y/%m/%d')
    @total = 9
    @model = []
    @registered = []
    @online = []
    @outage = []
    query = cpe_inventory_query(month)
    res = query_resource(create_api_conn, 'cpeInventory', query)
    @inventory_reports = res[:result]
    @inventory_reports.each do |inventory_report|
      inventory_report['report'].each do |report|
        @model.push(report['model'])
        @registered.push(report['registered'])
        @online.push(report['online'])
        @outage.push(report['outage'])
      end
    end
    @inventory_reports
  end

  def user_operation_query(month)
    query = {}
    query['_id'] = month
    query
  end

  def get_user_operation(month)
    @operations = []
    arr_info = get_month_info(month, 'user_operation')
    @filter = arr_info[0]
    @start_time = Time.at(arr_info[1].to_i).strftime('%Y/%m/%d')
    @end_time = Time.at(arr_info[2].to_i).strftime('%Y/%m/%d')
    @operation = []
    @operation_users = []
    @operation_user_roles = []
    @succeed = []
    @failed = []
    tmp_anonymous_data = {
      'login_success' => 0,
      'login_failed' => 0,
      'change_password_success' => 0,
      'change_password_failed' => 0
    }
    @query  = user_operation_query(month)
    res = query_resource(create_api_conn, 'userOperation', @query)
    @operations = res[:result]
    @total = 9
    @operations.each do |operation|
      operation['report'].each do |report|
        if Rails.configuration.users.has_key?(report['user']) and !Rails.configuration.users[report['user']]['roles'][0].nil?
          @operation.push(report['operation'])
          @operation_users.push(report['user'])
          @operation_user_roles.push(Rails.configuration.users[report['user']]['roles'][0])
          @succeed.push(report['succeed'])
          @failed.push(report['failed'])
        else
          if report['operation'] == 'login'
            tmp_anonymous_data['login_success'] += report['succeed']
            tmp_anonymous_data['login_failed'] += report['failed']
          elsif report['operation'] == 'change_password'
            tmp_anonymous_data['change_password_success'] += report['succeed']
            tmp_anonymous_data['change_password_failed'] += report['failed']
          end
        end
      end
    end

    if tmp_anonymous_data['login_success'] != 0 || tmp_anonymous_data['login_failed'] != 0
      @operation.push('login')
      @operation_users.push('Anonymous')
      @operation_user_roles.push('Anonymous')
      @succeed.push(tmp_anonymous_data['login_success'])
      @failed.push(tmp_anonymous_data['login_failed'])
    end

    if tmp_anonymous_data['change_password_success'] != 0 || tmp_anonymous_data['change_password_failed'] != 0
      @operation.push('change_password')
      @operation_users.push('Anonymous')
      @operation_user_roles.push('Anonymous')
      @succeed.push(tmp_anonymous_data['change_password_success'])
      @failed.push(tmp_anonymous_data['change_password_failed'])
    end


    @uniq_operation_user_roles = @operation_user_roles.uniq
    @operations
  end

  def profile_stats_query(month)
    query = {}
    query['_id'] = month
    query
  end

  def get_profile_stats(month)
    @profile_reports = []
    @profile_stats = []
    arr_info = get_month_info(month, 'profile_stats')
    @filter = arr_info[0]
    @start_time = Time.at(arr_info[1].to_i).strftime('%Y/%m/%d')
    @end_time = Time.at(arr_info[2].to_i).strftime('%Y/%m/%d')
    @profile = []
    @app = []
    @model = []
    @complete = []
    @error = []
    @query = profile_stats_query(month)
    res = query_resource(create_api_conn, 'profileStats', @query)
    @profile_stats = res[:result]
    @total = 9
    @profile_stats.each do |profile|
      profile['report'].each do |report|
        report['profile'] = report['profile'].chomp(FWSCHEDULE_APPENDIX) if report['profile'].include? FWSCHEDULE_APPENDIX
        @profile.push(report['profile'])
        @app.push(report['app'])
        @model.push(report['model'])
        @complete.push(report['complete'])
        @error.push(report['error'])
      end
    end
    @profile_stats
  end

  def get_system_usage(month)
    arr_info = get_month_info(month, 'system_usage')
    @filter = arr_info[0]
    @start_time = Time.at(arr_info[1].to_i).strftime('%Y/%m/%d')
    @end_time = Time.at(arr_info[2].to_i).strftime('%Y/%m/%d')
    @start_timestamp = arr_info[1].to_i
    @end_timestamp = arr_info[2].to_i

    cpu_output = `rrdtool fetch /opt/agileacs/data/rrd/agileacs_sys_cpu.rrd -s #{@start_timestamp} -e #{@end_timestamp} AVERAGE`
    @arr_cpu_timestamp = []
    @arr_cpu_datetime = []
    @arr_cpu_percentage = []
    arr_cpu_output = cpu_output.split("\n")
    (2...arr_cpu_output.length).each do |i|
      arr_tmp = arr_cpu_output[i].split(':')
      @arr_cpu_timestamp.push(arr_tmp[0])
      @arr_cpu_percentage.push(arr_tmp[1].to_f.to_i)
    end

    @arr_cpu_timestamp.each { |x| @arr_cpu_datetime.push(Time.at(x.to_i).strftime('%Y/%m/%d %T')) }

    memory_output = `rrdtool fetch /opt/agileacs/data/rrd/agileacs_sys_memory.rrd -s #{@start_timestamp} -e #{@end_timestamp} AVERAGE`
    @arr_memory_timestamp = []
    @arr_memory_datetime = []
    @arr_memory_percentage = []
    arr_memory_output = memory_output.split("\n")
    (2...arr_memory_output.length).each do |i|
      arr_tmp = arr_memory_output[i].split(':')
      @arr_memory_timestamp.push(arr_tmp[0])
      @arr_memory_percentage.push(arr_tmp[1].to_f.to_i)
    end

    @arr_memory_timestamp.each { |x| @arr_memory_datetime.push(Time.at(x.to_i).strftime('%Y/%m/%d %T')) }

    storage_output = `rrdtool fetch /opt/agileacs/data/rrd/agileacs_sys_storage.rrd -s #{@start_timestamp} -e #{@end_timestamp} AVERAGE`
    @arr_storage_timestamp = []
    @arr_storage_datetime = []
    @arr_storage_percentage = []
    arr_storage_output = storage_output.split("\n")
    (2...arr_storage_output.length).each do |i|
      arr_tmp = arr_storage_output[i].split(':')
      @arr_storage_timestamp.push(arr_tmp[0])
      @arr_storage_percentage.push(arr_tmp[1].to_f.to_i)
    end

    @arr_storage_timestamp.each { |x| @arr_storage_datetime.push(Time.at(x.to_i).strftime('%Y/%m/%d %T')) }
  end

  def generate_invalid_device_aggregate
    aggregate_pipeline = []
    aggregate_pipeline << { '$unwind' => '$report' }
    aggregate_pipeline << { '$project' => { 'report' => 1, 'timestamp' => 1, 'total' => { '$add' => ['$report.expired', '$report.invalid'] } } }
    aggregate_pipeline << { '$sort' => { 'total' => -1 } }
    aggregate_pipeline << { '$group' => { '_id' => '$_id', 'report' => { '$push' => { 'model' => '$report.model', 'expired' => '$report.expired', 'invalid' => '$report.invalid', 'total' => '$total' } }, 'timestamp' => { '$first' => '$timestamp' } } }
    aggregate_pipeline << { '$sort' => { '_id' => 1 } }
    aggregate_pipeline
  end

  def invalid_device_query(month)
    query = {}
    query['month'] = month
    query
  end

  def get_invalid_device(month)
    @invalid_reports = []
    arr_info = get_month_info(month, 'invalid_device')
    @filter = arr_info[0]
    @start_time = Time.at(arr_info[1].to_i).strftime('%Y/%m/%d')
    @end_time = Time.at(arr_info[2].to_i).strftime('%Y/%m/%d')

    @start_timestamp = arr_info[1].to_i
    @end_timestamp = arr_info[2].to_i
    @date = []
    @invalid_model = []
    @expired = []
    @invalid = []
    res = aggregate_resource(create_api_conn, 'invalidDevice', generate_invalid_device_aggregate, 'GET')
    @invalid_reports = res[:result]
    @total = 9
    @invalid_reports.each do |invalid_device|
      next unless invalid_device['timestamp'] / 1000 >= @start_timestamp && invalid_device['timestamp'] / 1000 <= @end_timestamp

      @date.push(invalid_device['_id'])
      invalid_model_tmp = []
      expired_tmp = []
      invalid_tmp = []
      invalid_device['report'].each_with_index do |report, index|
        next unless index < 10

        invalid_model_tmp.push(report['model'])
        if report['expired'].nil?
          expired_tmp.push(0)
        else
          expired_tmp.push(report['expired'])
        end

        if report['invalid'].nil?
          invalid_tmp.push(0)
        else
          invalid_tmp.push(report['invalid'])
        end
      end
      @invalid_model.push(invalid_model_tmp)
      @expired.push(expired_tmp)
      @invalid.push(invalid_tmp)
    end
    @invalid_reports
  end

  def get_all_reports(month)
    json_reports = { success: true }
    arr_info = get_month_info(month, 'all')
    @filter = arr_info[0]
    @start_time = Time.at(arr_info[1].to_i).strftime('%Y/%m/%d')
    @end_time = Time.at(arr_info[2].to_i).strftime('%Y/%m/%d')
    json_reports['start_time'] = @start_time
    json_reports['end_time'] = @end_time
    @start_timestamp = arr_info[1].to_i
    @end_timestamp = arr_info[2].to_i

    # Get CPE Inventory Data
    get_cpe_inventory(month)
    json_reports['cpe_inventory'] = { model: @model, registered: @registered, online: @online, outage: @outage }

    # Get User Operation Data
    get_user_operation(month)
    json_reports['user_operation'] = { operation_users: @operation_users, operation_user_roles: @operation_user_roles, 
                                      operation: @operation, succeed: @succeed, failed: @failed }

    # Get Profile Statistics Data
    get_profile_stats(month)
    json_reports['profile_stats'] = { profile: @profile, app: @app, model: @model, complete: @complete, error: @error }

    # Get System Usage Data
    get_system_usage(month)
    json_reports['system_usage'] = { datetime: @arr_cpu_datetime, cpu: @arr_cpu_percentage, memory: @arr_memory_percentage, storage: @arr_storage_percentage }

    # Get Invalid Device Data
    get_invalid_device(month)
    json_reports['invalid_device'] = { date: @date, invalid_model: @invalid_model, expired: @expired, invalid: @invalid }

    respond_to do |format|
      format.json { render json: json_reports.to_json }
      format.html {}
    end
    nil
  end
end
