# Router topology
module Router
  include Parser
  #     {
  #     "name": "A",
  #     "_type": "host",
  #     "children": [
  #       {
  #         "name": "good",
  #         "_type": "client",
  #         "_number": 1,
  #         "children": []
  #       },
  #       {
  #         "name": "normal",
  #         "_type": "client",
  #         "_number": 1,
  #         "children": []
  #       },
  #       {
  #         "name": "Wired",
  #         "_type": "client",
  #         "_number": 1,
  #         "children": []
  #       }
  #     ]
  #   }

  client_format = {
    '_type' => 'client',
    '_number' => 0,
    'children' => []
  }

  def generate_topology(device)
    return nil if device.blank?
   
    ip_address = get_device_param_by_specific(device, 'Device.Ethernet.Link.1.MACAddress')
    return if !ip_address.blank? && !ip_address['value'].blank?
   
    clients_info = find_clients_info(device)
    return nil if clients_info.nil?

    ret = []
    clients_info.each do |level, num|
      tmp = client_format.clone
      tmp[:name] = level
      tmp[:_number] = num
      ret.push(tmp)
    end
  end

  def find_clients_info(device)
    ret = {"wired"=>0,"good"=>0,"normal"=>0,"poor"=>0}

    res = match_rkey_val(device, 'Device.Hosts.Host.\\d+.Active', 'true')

    res.each do |o|
      prefix, match, subfix = o['path'].rpartition('.')

      ip_address = get_device_param_by_specific(@device_params, "#{prefix}.ip_address")
      next if !ip_address.blank? && !ip_address['value'].blank?

      lan_conn_type = get_device_param_by_specific(@device_params, "#{prefix}.X_TP_LanConnType")
      next if lan_conn_type.blank?

      case lan_conn_type['value']
      when '0' # wired(0)
        ret['wired'] += 1
      when '1' # wireless(1)
        rssi = get_device_param_by_specific(@device_params, "#{prefix}.X_TP_Rssi")
        next if rssi.blank? && !rssi['value'].blank?
        level = get_rssi_level(rssi['value'].to_i)
        ret[level] += 1
      end
    end
    ret
  end

  def get_rssi_level(rssi)
    if rssi >= -50
      'good'
    elsif rssi >= -70 && rssi < -50
      'normal'
    elsif rssi < -70
      'poor'
    end
  end
end
