# ---------------------------------------------------------------------------------------------------------
# module SystemModule
# History:
#     20200710 bill.chen  module extension functions, moved from SystemController or additional functions
#                         sce: SystemControllerExtension, is renamed to SystemModule
# ---------------------------------------------------------------------------------------------------------
module SystemModule
  include Loggers
  include SystemNbiUrl

  def sce_sys_default
    # system['hosturl'] = '169.254.0.199'
    # system['hostip'] = '169.254.0.199'
    # system['httpport'] = '7547'
    # system['httpsport'] = '7548'
    # system['submask'] = '255.255.255.0'
    # system['gateway'] = '169.254.0.199'
    # system['dnsserver'] = '8.8.8.8'
    # system['timezone'] = 'Etc/UTC'
    # system['region'] = 'UTC'
    # system['xmpp'] = 'disable'
    # system['xmpp_keepalive_interval'] = '3600'
    # system['log_severity'] = 6

    { 'hosturl' => '169.254.0.199', 'hostip' => '169.254.0.199',
      'httpport' => '7547', 'httpsport' => '7548',
      'submask' => '255.255.255.0', 'gateway' => '169.254.0.199', 'dnsserver' => '8.8.8.8',
      'timezone' => 'Etc/UTC', 'region' => 'UTC',
      'xmpp' => 'disable', 'xmpp_keepalive_interval' => '300', 'xmppstatus' => 'stopped', 'log_severity' => 6 }
  end

  def sce_req_sys_setting
    # system = {}
    # # system['use_ip'] = params[:ipordomain]
    # system['hosturl'] = params[:hosturl]
    # system['hostip'] = params[:host_ip]
    # system['httpport'] = params[:httpport]
    # system['httpsport'] = params[:httpsport]
    # system['submask'] = params[:submask]
    # system['gateway'] = params[:gateway]
    # system['dnsserver'] = params[:dns]
    # system['timezone'] = ActiveSupport::TimeZone::MAPPING.select { |k, _v| k == params[:timezone][0] }.values.first
    # system['region'] = params[:timezone][0]
    # system['xmpp'] = params[:xmpp].nil? ? 'disable' : params[:xmpp]
    # system['xmpp_keepalive_interval'] = params[:xmpp_keepalive_interval].nil? ? '3600' : params[:xmpp_keepalive_interval]
    # system['log_severity'] = 6
    # system

    { 'hosturl' => params[:hosturl], 'hostip' => params[:host_ip],
      'httpport' => params[:httpport], 'httpsport' => params[:httpsport],
      'submask' => params[:submask], 'gateway' => params[:gateway], 'dnsserver' => params[:dns],
      'timezone' => ActiveSupport::TimeZone::MAPPING.select { |k, _v| k == params[:timezone][0] }.values.first, 'region' => params[:timezone][0],
      'xmpp' => params[:xmpp].nil? ? 'disable' : params[:xmpp],
      'xmpp_keepalive_interval' => params[:xmpp_keepalive_interval].nil? ? 300 : params[:xmpp_keepalive_interval],
      'log_severity' => 6 }
  end

  def sce_req_xmpp
    { 'xmpp' => params[:xmpp].nil? ? 'disable' : params[:xmpp],
      'xmpp_keepalive_interval' => params[:xmpp_keepalive_interval].nil? ? 300 : params[:xmpp_keepalive_interval].to_i }
  end

  def sce_bytes_to_megabytes(bytes)
    (bytes / (1024.0 * 1024.0))
  end

  def sce_sys_resource_usage(http, url, used = false)
    target = {}
    res = http.post(url, nil)
    if res.code == '404'
      target['usage'] = '0%'
    else
      target = JSON.parse(res.body)
      if used
        target['used'] = target['used'].to_i
        target['usage'] = ((target['used'].to_f / target['total'].to_f) * 100).to_i.to_s + '%'
      end
    end
    target
  end

  def sce_render_resource_usage(url)
    http = create_api_conn()
    res = http.post(url, nil)
    if res.body == '404 Not Found'
      data = { 'usage': '0%', 'used': 0, 'available': 0 }
    else
      data = JSON.parse(res.body)
      data['usage'] = data['usage'].gsub(/[^0-9\.]/, '')
    end
    render json: data
  end

  def sce_render_time
    data = { 'time' => DateTime.now.strftime('%Y-%m-%d %H:%M:%S') }
    render json: data
  end

  def default_models(page_size, page)
    models = [
      'TL-WR840N', 'TL-WR850N', 'Archer C5', 'Archer C20', 'XN020-G3v', 'Archer XR500v', 'EN020-F5', 'EC120-F5', 'EC230-G1', 'EC330-G5u',
      'HC220-G1', 'HC221-G1R', 'HC221-G1W', 'Archer VR1200v', 'Archer VR1210v', 'VC420-G3v'
    ]
    page = 1 if page.nil? || page.zero?
    mlist = { total: 0, models: [] }
    mlist[:total] = models.length
    mlist[:models] = models[page_size * (page - 1), page_size]
    mlist
  end

  def model_list(http, page_size, page, skip, query)
    mlist = { total: 0, models: [] }
    res = query_resource(http, SYS_NBI_URL_MODELS_GET[1..-1], query, nil, skip, page_size, { '_id' => 1 }) # sort = { '_id' => 1 }
    if res[:message] == '404 Not Found'
      mlist = default_models(page_size, page)
    else
      mlist[:total] = res[:total]
      res[:result].each do |model|
        mlist[:models] << model['_id']
      end
    end
    mlist
  end

  def cpe_model_file
    filepath = '/tmp/_x0-9y_'
    if params.key?('config_file')
      tmpname = params[:config_file].path
      filepath = "/tmp/#{params[:config_file].original_filename}"
      File.rename(tmpname, filepath)
    end
    filepath
  end

  def post_setting(object, url, data, atom, redir = true)
    http = create_api_conn()
    res = http.post(url, ActiveSupport::JSON.encode(data), 'Content-type' => 'application/json')
    if res.code == '200'
      flash[:success] = 'successfully'
      agileACSLog(6, object, atom + ' setting succeed.')
    elsif res.code == '202'
      if object == '[Import Config File]'
        flash[:warning] = res.body
        agileACSLog(4, object, res.body.to_s)
      else
        flash[:warning] = res.message
        agileACSLog(4, object, res.message.to_s)
      end
    elsif res.code == '405'
      render json: { 'msg' => 'busy' }, status: 405
    else
      flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
    end
    redirect_to action: :index, tab: params[:tab] if redir
  end

  def post_upgrade(object, url, data)
    http = create_api_conn()
    res = http.post(url, ActiveSupport::JSON.encode(data), 'Content-type' => 'application/json')
    if res.code == '200'
      render json: { version: res.body }
      agileACSLog(6, object, "System upgrade succeed. Current #{res.body}")
    elsif res.code == '202'
      flash[:warning] = res.message
      agileACSLog(4, object, res.message.to_s)
    else
      flash[:error] = "Unexpected error (#{res.code}): #{res.body}"
      agileACSLog(3, object, "Unexpected error (#{res.code}): #{res.body}")
    end
    render :json => {:msg => "upgrading" } and return
  end

  def check_upgrade_status
    @upgrade_status = get_upgrade_status

    if @upgrade_status == 'timeout' && params.key?('file')
      msg = 'Upgrade is timeout. Would you like to try again?'
      flash[:error] = msg
      return
    end

    if @upgrade_status == 'block'
      msg = 'System is upgrading. Please wait for the upgrade to compelete.'
      flash[:error] = msg
      redirect_to :back
    end
  end

  def get_upgrade_status
    data = { 'action' => 'upgrade' }
    data['data'] = {
      'action' => 'lock',
      'data' => 'status'
    }
    http = create_api_conn()
    res = http.post(SYS_NBI_URL_ACSAPPS, ActiveSupport::JSON.encode(data), 'Content-type' => 'application/json')

    if res.code == '202'
      'none'
    elsif res.code == '200'
      res.body
    end
  end
end
