module Util
  # CPE Status
  REACHABLE = 0
  UNREACHABLE = 1 
  
  # CPE Task Status (Only check when CPE is REACHABLE)
  TASK_NONEXISTED = 0
  TASK_EXISTED = 1
  TASK_DIAGLOCKED = 2
# +--------------------------------------Return value for all situation when check CPE status----------------------------------------+
# +----------------------+------------------+-----------------------+-----------------------+----------------------------------------+
# |     CPE status       |  CPE Task Status | Relation between      |     Normal Task       |    Diaglock related Task               |
# |                      |                  | Diaglock and  Task    |                       |    (TR143, WifiScan, Mesh)             |
# +----------------------+------------------+----------+------------+-----------------------+----------------------------------------+
# +----------------------|------------------| Diaglock  | Task      |-----------------------+----------------------------------------|
# +----------------------+------------------+----------+------------+-----------------------+----------------------------------------+
# |      REACHABLE(0)    |TASK_NONEXISTED(0)| Not Exist | Not Exist | Insert new Task       | Insert new Task                        |
# |      REACHABLE(0)    | TASK_EXISTED(1)  | Not Exist | Exist     | Insert new Task       | Don’t Insert new Task                  |
# |      REACHABLE(0)    |TASK_DIAGLOCKED(2)| Exist     |   N/A     | Don’t Insert new Task | Don’t Insert new Task                  |
# |     UNREACHABLE(1)   |      N/A         |    N/A    |   N/A     | Insert new Task       | Don’t Insert new Task                  |
# +----------------------+------------------+----------+------------+-----------------------+----------------------------------------+

  HTTP_SETTING = {}

  def get_http_setting() 
    if HTTP_SETTING.blank?
      HTTP_SETTING['host'] = Rails.configuration.respond_to?(:agileacs_api_host) ? Rails.configuration.agileacs_api_host : Rails.configuration.genieacs_api_host
      HTTP_SETTING['port'] = Rails.configuration.respond_to?(:agileacs_api_port) ? Rails.configuration.agileacs_api_port : Rails.configuration.genieacs_api_port
      HTTP_SETTING['use_ssl'] = Rails.configuration.respond_to?(:agileacs_api_use_ssl) ? Rails.configuration.agileacs_api_use_ssl : Rails.configuration.genieacs_api_use_ssl
      #puts HTTP_SETTING
    end
    return HTTP_SETTING
  end

  def create_api_conn(&block)
    config = get_http_setting
    host = config["host"]
    port = config["port"]
    use_ssl = config["use_ssl"]

    if block_given?
      Net::HTTP.start(host, port, {use_ssl: use_ssl}, &Proc.new)
    else
      http = Net::HTTP.new(host, port)
      http.use_ssl = use_ssl
      return http
    end
  end

  def query_resource(http, resource, query, projection = nil, skip = nil, limit = nil, sort = nil)
    args = {}
    args['query'] = ActiveSupport::JSON.encode(query) if query
    args['projection'] = projection.join(',') if projection
    args['skip'] = skip if skip
    args['limit'] = limit if limit
    args['sort'] = ActiveSupport::JSON.encode(sort) if sort

    config = get_http_setting
    host = config["host"]
    port = config["port"]
    use_ssl = config["use_ssl"]

    request = Net::HTTP::Get.new("/#{resource}?#{args.to_query}")
    http.request request do |res|
      total = res['total'].to_i
      timestamp = res['Date'].to_time
      if not block_given?
        ret = {timestamp: timestamp, total: total, result: []}
      end
      counter = 0
      line = ''
      res.read_body() do |chunk|
        chunk.each_line() do |l|
          if l == "404 Not Found"
            ret[:message] = "404 Not Found"
            return ret
          end
          line += l
          next if not line.end_with?($/)

          if line.end_with?(",#{$/}")
            obj = JSON.parse(line[0...0-",#{$/}".length])
            if ret
              ret[:result] << obj
            else
              yield(obj, counter, total, timestamp)
            end
            counter += 1
            line = ''
          elsif line != $/ and line != "[#{$/}" and line != "]#{$/}"
            obj = JSON.parse(line[0...0-"#{$/}".length])
            if ret
              ret[:result] << obj
            else
              yield(obj, counter, total, timestamp)
            end
            counter += 1
          end
          line = ''
        end
      end
      if ret
        return ret
      else
        yield(nil, 0, total, timestamp) if counter == 0
      end
    end
  end

  def aggregate_resource(http, resource, aggregate_pipeline, method="POST")
    args = {}
    args['aggregate_pipeline'] = ActiveSupport::JSON.encode(aggregate_pipeline) if aggregate_pipeline

    timestamp = Time.now 
    total = 0
    result = []
    message = nil
    if method.eql?("POST") or method.eql?("GET") or method.eql?("HEAD")
      case method.upcase 
      when "GET"
        res = http.get("/aggregate/#{resource}?#{args.to_query}") 
      when "POST"
        res = http.post("/aggregate/#{resource}?", ActiveSupport::JSON.encode(args), 'Content-type' => 'application/json')
      when "HEAD"
        res = http.head("/aggregate/#{resource}?#{args.to_query}") 
      end

      if method.eql? "HEAD"
        ret ={:total => res['total'].to_i}
        if res['total'].nil?
          ret[:message] = "404 Not Found"
        end
        return ret 
      end

      timestamp = res['Date'].to_time
      case res.code
      when "200"
        total = res['total'].to_i
        result = JSON.parse(res.body)
        result.each do |device|
          a = device['_lastInform']
          if !a.nil? 
            device['_lastInform'] = Time.parse a
          end
        end
      else 
        message = res.body
      end
    else 
      message = "aggregate_resource: with invalid method"
    end

    ret ={timestamp: timestamp, total: total, result: result}
    ret[:message] = message if !message.nil?

    return ret
  end

  def online_check(escaped_device_id, http)
    task = { 'name' => 'isOnline', 'parameterNames' => [] }
    res = http.post("/devices/#{escaped_device_id}/tasks?timeout=3000&connection_request", ActiveSupport::JSON.encode(task), 'Content-type' => 'application/json')
    return false if res.code != '200'

    true
  end

  def check_cpe_status(escaped_device_id, http)
    unless online_check(escaped_device_id, http)
      return UNREACHABLE unless online_check(escaped_device_id, http)
    end

    # check Diaglock
    diaglock_obj = { 'action': 'isLocked' }
    res = http.post("/diaglock/#{escaped_device_id}", ActiveSupport::JSON.encode(diaglock_obj), 'Content-type' => 'application/json')
    diaglock = JSON.parse(res.body)['result']
    # check Tasks
    tasks = query_resource(http, 'tasks', { 'device' => escaped_device_id }, nil, nil, nil, { 'timestamp' => 1 })[:result]
    if !diaglock.empty?
      return REACHABLE, TASK_DIAGLOCKED
    elsif tasks.size.positive?
      return REACHABLE, TASK_EXISTED
    else
      return REACHABLE, TASK_NONEXISTED
    end
  end
end
