#!/usr/bin/env node
/*  Copyright(c) 2019-2029 TP-LINK Technologies Co.Ltd.
 *
 * file      agileacs_lastinfrom.js
 * brief     nodejs command to restore lastinform from redis to mongodb
 * details   /usr/bin/nodejs /opt/genieacs/bin/agileacs_lastinfrom.js

 * author    Sharon Lin
 * version   1.0.0
 * date      14Sep19
 *
 * history   \arg 1.0.0, 14Sep19, Sharon Lin, Create the file.
 */

/**************************************************************************************************/
/*                                           VARIABLES                                            */
/**************************************************************************************************/
var path = require('path');
var fs = require('fs');
var exec = require('child_process').exec;
var dir = path.resolve(path.dirname(fs.realpathSync(__filename)), '..');
process.chdir(dir);
var args = process.argv.slice(2);
var action = args[0];
const MongoClass = require('tplib/mongo-class')
const agileConfig = require('tplib/agile-config')
const stringTable = require('tplib/string-table.json')
const dbTmp = new MongoClass(agileConfig.get(stringTable.url.dbTmp))
const dbGenie = new MongoClass(agileConfig.get(stringTable.url.dbGenie))
const AGILEACS_DB_TIMEOUT=120000;
const enable_logfile=true;
const enable_debug=false;
const MONGOTMP_DEBUG_LOG_FILE="/tmp/mongodb_tmp.debug"
const ACS_SYSTEM_LOG_FILE="/var/log/genieacs/acs-system.log"
const MONGODB_TMP_DATA_STATUS="/tmp/mongodb_tmp_status"
var update_count;
var child_process = null
async function waitSystemReady( action) {
  try {
    await Promise.all([
      dbGenie.waitConnected(),
      dbTmp.waitConnected()
    ])
    log_time("["+ action+"] connnect to db")
  } catch(err) {
    log_time("["+ action+"] Cannot connnect to db", true)
    data_status("failed")
    endProcess(0)
  }
}

function endProcess(exitCode) {
  dbTmp.disconnectDb()
  dbGenie.disconnectDb()
  process.exit(exitCode)
}

Date.prototype.toIsoString = function() {
  function pad(number) {
    if (number < 10) return '0' + number
    return number
  }
  return this.getFullYear() +
    '-' + pad(this.getMonth() + 1) +
    '-' + pad(this.getDate()) +
    'T' + pad(this.getHours()) +
    ':' + pad(this.getMinutes()) +
    ':' + pad(this.getSeconds()) +
    '.' + (this.getMilliseconds() / 1000).toFixed(3).slice(2, 5) +
    'Z'
}

let data_status = function (status)
{
    if(status)
    {
        fs.writeFileSync(MONGODB_TMP_DATA_STATUS, status);
    }
    else
    {
        try
        {
            fs.unlinkSync(MONGODB_TMP_DATA_STATUS);
        }
        catch(e)
        {
            log_time("[data_status] fs.unlinkSync err = " + e);
        }
    }
}

let log_time = function (prefix, logfile = false)
{
    let level = '7';
    let level_str = '[DEBUG]';
    let host = '127.0.0.1';
    let objectname = '[Database Enhancement]:'
    let current_time = new Date().toIsoString();
    let log_str = current_time + ' ' + level_str+ ' ' + level + ' ' + host + ' ' + objectname + ' ' +  prefix;
    //console.log(log_str)
    if(enable_debug)
        fs.appendFileSync(MONGOTMP_DEBUG_LOG_FILE, log_str+"\n");
    if(enable_logfile && logfile)
        fs.appendFileSync(ACS_SYSTEM_LOG_FILE, log_str+"\n");
}

let usage = function ()
{
    console.log("[usage] action: update | restore");
}


let insert_all = function (devices, dbTmp)
{
    return new Promise((resolve, reject) => {
        return dbTmp.insertMany(stringTable.collection.devices, devices, function (err, res){
            if(err)
            {
                if(err.message){
                    log_time("insert_all  err: "+err.message);
                    reject(err.message);
                }
                else{
                    console.log("insert_all  err: "+err);
                    log_time("insert_device  err: "+err);
                    reject(err);
                }
            }
            resolve();
        });
    });
}


let testnow_lastinform = function (device, dbTmp)
{
    return new Promise((resolve, reject) => {
        var update = {
           '$set': {}
         };
        update['$set']['_lastInform'] = new Date();
        return dbTmp.update(stringTable.collection.devices, {'_id': device['_id']}, update, function (err, res){
            if(err){
                if(err.message){
                    log_time("testnow_lastinform  err: "+err.message);
                    reject(err.message);
                }
                else{
                    log_time("testnow_lastinform  err: "+err);
                    reject(err);
                }
            }
            update_count++;
            //console.log("[testnow_lastinform] device['_id']: "+device['_id']);
            resolve(device['_id']);
        });
    });
}

let update_all = function (devices, dbGenie)
{
    return new Promise((resolve, reject) => {
        child_process = exec("/opt/agileacs/bin/acsutils_mongotmp update", (err, stdout, stderr) => {
            child_process = null
	    if (err) reject(err)
            if (stderr) reject(stderr)
            if (stdout)
            {
                return resolve(stdout)
            }
            else
                return resolve("No stdout failed");
        });
    })
}

let restore_all = function (devices, dbGenie)
{
    return new Promise((resolve, reject) => {
        child_process = exec("/opt/agileacs/bin/acsutils_mongotmp restore", (err, stdout, stderr) => {
            child_process = null
            if (err) reject(err)          
            if (stderr)reject(stderr)
            if (stdout)
            {
                return resolve(stdout)
            }
            else
                return resolve("No stdout failed");
        });
    })
}

let testnow_all = function (devices, dbTmp)
{
    return Promise.all(devices.map(device => {
        return testnow_lastinform(device, dbTmp)
    }))
}

let get_current_cpuinfo = function ( action )
{
    return new Promise((resolve) => {
        return exec("/bin/cat /proc/cpuinfo |grep MHz| awk '{print $4}'", (err, stdout, stderr) => {
            if (stderr || err) log_time("[" + action +"] /bin/cat /proc/cpuinfo failed. ", true);
            if (stdout)
            {
                var strings = stdout.split(/(?:\r\n|\r|\n)/g);
                var logstr = "";
                for( let line=0;line < strings.length-1; line++)
                {
                    if( logstr.length )
                    logstr = logstr + '/' + strings[line];
                    else
                    logstr = strings[line];
                }
                log_time("[" + action +"] CPU Info (MHz): " +logstr , true);
            }
            resolve();
        });
    });
}

let get_current_meminfo = function ( action )
{
    return new Promise((resolve) => {
        return exec("/bin/cat /proc/meminfo |grep Mem| awk '{print $1, $2}'", (err, stdout, stderr) => {
            if (stderr || err) log_time("[" + action +"] /bin/cat /proc/meminfo failed. ", true);
            if (stdout)
            {
                var strings = stdout.split(/(?:\r\n|\r|\n)/g);
                var logstr = "";
                for( let line=0;line < strings.length-1; line++)
                {
                    if( logstr.length )
                    logstr = logstr + ', ' + strings[line];
                    else
                    logstr = strings[line];
                }
                log_time("[" + action +"] " + logstr , true);
            }
            resolve();
        });
    });
}


let abort = function ( action )
{
    log_time("[" + action +"] Cannot be finished in " + AGILEACS_DB_TIMEOUT + " ms, forcely abort.", true);
    if( action === 'update') data_status("failed");
    return Promise.all([
        get_current_cpuinfo( action ),
        get_current_meminfo( action )
    ]).then(() => {
        if(child_process){
            //child_process.kill()
            log_time("[" + action +"] kill "+ child_process.pid, true);
        }
        return endProcess(0)
    })

}

let update = async function () {
  await waitSystemReady("update")
  let proj = [{
    "$project": {
      "SerialNumber":{"$ifNull":["$Device.DeviceInfo.SerialNumber._value","$InternetGatewayDevice.DeviceInfo.SerialNumber._value"]},
      "ProductClass":{"$ifNull":["$Device.DeviceInfo.ProductClass._value","$InternetGatewayDevice.DeviceInfo.ProductClass._value"]},
      "ModelName":{"$ifNull":["$Device.DeviceInfo.ModelName._value","$InternetGatewayDevice.DeviceInfo.ModelName._value"]},
      "SwVers":{"$ifNull":["$Device.DeviceInfo.SoftwareVersion._value","$InternetGatewayDevice.DeviceInfo.SoftwareVersion._value"]},
      "HwVers":{"$ifNull":["$Device.DeviceInfo.HardwareVersion._value","$InternetGatewayDevice.DeviceInfo.HardwareVersion._value"]},
      "MAC":{"$ifNull":["$Device.Ethernet.Link.1.MACAddress._value","$InternetGatewayDevice.LANDevice.1.LANEthernetInterfaceConfig.1.MACAddress._value"]},
      "XmppStatus":{"$ifNull":["$Device.XMPP.Connection.1.Status._value","$InternetGatewayDevice.XMPP.Connection.1.Status._value"]},
      "IP":"$_ipaddress",
      "url": {"$ifNull": ["$Device.ManagementServer.ConnectionRequestURL._value", "$InternetGatewayDevice.ManagementServer.ConnectionRequestURL._value"]},
      "_ipaddress":1,
      "_lastInform":1,
      "_username":1,
      "Device._object":1,
      "Device.DeviceInfo.SerialNumber":1,
      "Device.DeviceInfo.ProductClass":1,
      "Device.DeviceInfo.ModelName":1,
      "Device.DeviceInfo.SoftwareVersion":1,
      "Device.DeviceInfo.HardwareVersion":1,
      "Device.Ethernet.Link.1.MACAddress":1,
      "Device.XMPP.Connection.1.Status":1,
      "Device.ManagementServer.ConnectionRequestURL":1,
      "Device.X_TP_MESH_WIFI.WorkMode":1,
      "InternetGatewayDevice._object":1,
      "InternetGatewayDevice.DeviceInfo.SerialNumber":1,
      "InternetGatewayDevice.DeviceInfo.ProductClass":1,
      "InternetGatewayDevice.DeviceInfo.ModelName":1,
      "InternetGatewayDevice.DeviceInfo.SoftwareVersion":1,
      "InternetGatewayDevice.DeviceInfo.HardwareVersion":1,
      "InternetGatewayDevice.LANDevice.1.LANEthernetInterfaceConfig.1.MACAddress":1,
      "InternetGatewayDevice.XMPP.Connection.1.Status":1,
      "InternetGatewayDevice.ManagementServer.ConnectionRequestURL":1,
      "_tags": 1,
      "_mesh": 1,
      "_homeId": { "$ifNull": ["$_homeId", ""] },
      "_zoneId": { "$ifNull": ["$_zoneId", ""] },
      "_cellPhone": { "$ifNull": ["$_cellPhone", ""] },
      "_cpf": { "$ifNull": ["$_cpf", ""] },
      "isTr181":{"$ifNull":["$Device._object",false]}
    }
  }]
  log_time("[update] Start", true);
  data_status();
  return dbGenie.aggregate(stringTable.collection.devices, proj, function(err, devices) {
    if(err) {
        if(err.message){
            log_time("dbGenie find err: " + err.message)
            console.log("dbGenie find err: " + err.message)
        }
        else{
            log_time("dbGenie find err: " + err)
            console.log("dbGenie find err: " + err)
        }
      data_status("failed")
      return endProcess(0)
    }
    if(devices && devices.length) {
      log_time("[update] need to restore devices data (" + devices.length +") from DB in disk", true)
      insert_all(devices, dbTmp)
      .then(async () => {
        log_time("[update] restored devices data (" + devices.length +") to DB in ram", true)
        try{
            let ret = await update_all()
            log_time("[update] update devices data to DB in ram ["+ ret + "]", true);
            if(ret === 'success')
                data_status("ready");
            else
                data_status("failed");
        }
        catch(e){
            log_time("[update] update devices data error ["+ e + "]", true);
        }
        endProcess(0)
      })
      .catch(err => {
        log_time("[update] restored devices data error ["+ err + "]", true);
        data_status("failed")
        endProcess(0)
      })
    } else {
      log_time("[update] No devices", true);
      data_status("ready")
      endProcess(0)
    }
  })
}

let restore = async function ()
{
    log_time("[restore] Start", true);
    try{
        let ret = await restore_all()
        log_time("[restore] restore devices data to DB in disk ["+ ret + "]", true);
    }
    catch(e){
        log_time("[restore] restore devices data error ["+ e + "]", true);
    }
    return endProcess(0)
}

let testnow = function ()
{
    let proj = [{
        "$project":{
            "_lastInform":1
        }
    }]
    Promise.all([
        dbTmp.waitConnected(),
        dbGenie.waitConnected()
      ])
      .then(() =>{
        log_time("[testnow] connnect to tmpmongoDb")
        update_count = 0;
        return dbTmp.aggregate(stringTable.collection.devices, proj,function(err,devices){
            if(err)
            {
                console.log("dbTmp find err: "+err);
            }
            if(devices && devices.length)
            {
                log_time("[testnow] update lastinform as now to DB in ram for (" + devices.length +") devices", true);
                testnow_all(devices, dbTmp)
                .then((result) => {
                    log_time("[testnow] updated lastinform as now to DB in ram for (" + devices.length +") devices", true);
                    process.exit(0);
                })
            }
            else
            {
                log_time("[testnow] No devices")
                process.exit(0);
            }
        })
      })
      .catch(() =>{
        log_time("Cannot connect to dbTmp & dbGenie");
        log_time("[testnow] Cannot connect to dbTmp & dbGenie");
          process.exit(0);
      })
}

//console.log("action = " + action);
if ( action === 'update') {
  setTimeout(abort, AGILEACS_DB_TIMEOUT, action)
  update()
} else if ( action === 'restore') {
  setTimeout(abort, AGILEACS_DB_TIMEOUT, action)
  restore()
} else if ( action === 'testnow') testnow()
else usage()
