#!/usr/bin/env node
/*  Copyright(c) 2009-2019 TP-LINK Technologies Co.Ltd.
 *
 * file      agileacs_report.js
 * brief     nodejs command to generate monthly report
 * details   /usr/bin/nodejs /opt/genieacs/bin/agileacs_report.js [action] [item]
 *           [action]: collect || update
 *                     collect => [item]: all || reportsetting
 *                                update => [item]: cpeinventory || useroperation || profilestats || invaliddevice
 * author    Hsinhao Yao
 * version   1.0.0
 * date      31July19
 *
 * history   \arg 1.0.0, 31July19, Hsinhao Yao, Create the file.
 */

/**************************************************************************************************/
/*                                           DEFINES                                              */
/**************************************************************************************************/

var config = { "storage_period" : 6};

/**************************************************************************************************/
/*                                           VARIABLES                                            */
/**************************************************************************************************/

var url = "mongodb://127.0.0.1:27017";

let MongoClient = require('mongodb').MongoClient;

let redisClient;

const redis = require("redis");
const REDIS_HOST = 'localhost';
const REDIS_PORT = 6379;
const REDIS_DB = 0;

var REGEX_REDIS_DEL_KEY = /.+_tasks_faults_operations/;

function reset_profiles() {
    return MongoClient.connect(url, function(err, db) {
        let genieacs = db.db("genieacs");
        return genieacs.collection('profiles').updateMany( { "_id" : /\w+\-FWSCHEDULE/ }, { "$set" : {'totalCpeCount': 0, 'countCpeEachDay': 0, 'countExecuteDay': 0} }, function( error, results) {
            if (error)  throw error;
            console.log("[profiles]:"+results.result.nModified+" documents are updated.");
            return genieacs.collection('devices').updateMany( {}, {"$pull": {"_tags": /\w+\-FWSCHEDULE_\w+/  } }, function( error, results) {
                if (error)  throw error;
                console.log("[devices]: "+results.result.nModified+" documents are updated.");
                return genieacs.collection('faults').deleteMany( {"channel": /\w+\-FWSCHEDULE_.+/}, function( error, results) {
                   if (error)  throw error;
                    console.log("[faults]: "+results.result.n+" documents are deleted.");
                    db.close();
                    return clear_faults_and_operations();
                }); // end of return genieacs.collection('faults').deleteMany(...
            }); // end of return genieacs.collection('devices').updateMany(...
        }); // end of return genieacs.collection('profiles').updateMany(...
    }); // end of MongoClient.connect(...
}

function clear_faults_and_operations() {
    return (redisClient = redis.createClient(REDIS_PORT, REDIS_HOST)).on('connect', function() {
        console.log("[redisClient] connected");
        return redisClient.select(REDIS_DB, function(err) {
            if (err) throw err;
            console.log("[redisClient] select "+REDIS_DB);
            
            return redisClient.keys('*', function (err, keys) {
                var del_keys = [];
                keys.forEach(function(key, pos){
                    if ( REGEX_REDIS_DEL_KEY.test(key) ) 
                        del_keys.push(key);
                });
                console.log("[redisClient] del_keys.length="+del_keys.length);
                console.log(del_keys);
                
                if ( del_keys.length > 0 ) {
                    return redisClient.del(keys, function(err, o) {
                        if (err) throw err;
                        console.log( "[redisClient] "+ o + " keys are deleted.");
                        redisClient.quit();
                    });
                } else {
                    return redisClient.quit();
                }
            });
        });
    });
}


reset_profiles();



