#!/usr/bin/env node

'use strict'
var path = require('path')
var fs = require('fs')
var dir = path.resolve(path.dirname(fs.realpathSync(__filename)), '..')
process.chdir(dir)

const AgileCryptor = require('tplib/agile-cryptor')
const MongoClass = require('tplib/mongo-class')
const agileConfig = require('tplib/agile-config')
const stringTable = require('tplib/string-table.json')

const LOG_FILE = '/var/log/genieacs/acsapps.log'
const ENCRYPT_ENABLE = agileConfig.get('ENCRYPT_ENABLE')
const agileCryptor = new AgileCryptor(ENCRYPT_ENABLE, agileConfig.getRedisConfig())
const dbTplink = new MongoClass(agileConfig.get(stringTable.url.dbTplink))
const enable_logfile = true
const enable_debug = true

function log(prefix) {
  var current_time = new Date()
  var log_str = '[decrypt_backup.js] '+ current_time + ': ' + prefix
  if(enable_debug) console.log(log_str)
  if(enable_logfile) fs.appendFile(LOG_FILE, log_str+"\n", err => {
    if(err) console.log(err)
  })
}

Date.prototype.toIsoString = function() {
  function pad(number) {
    if (number < 10) {
      return '0' + number
    }
    return number
  }
  return this.getFullYear() +
    '-' + pad(this.getMonth() + 1) +
    '-' + pad(this.getDate()) +
    'T' + pad(this.getHours()) +
    ':' + pad(this.getMinutes()) +
    ':' + pad(this.getSeconds()) +
    '.' + (this.getMilliseconds() / 1000).toFixed(3).slice(2, 5) +
    'Z'
}

let close = function(exitCode=1){
  dbTplink.disconnectDb()
  agileCryptor.disconnectRedis()
  return process.exit(exitCode)
}

async function decrypt_cwmp() {
  try {
    let cwmpSetting = await dbTplink.findOne(stringTable.collection.sysSetting, { '_id': 'cwmp', 'encrypt': 1 })
    if(cwmpSetting) { // found encrypted setting, decrypt it
      let decryptedResult = await Promise.all([
        agileCryptor.decrypt(cwmpSetting.acs_username),
        agileCryptor.decrypt(cwmpSetting.acs_password),
        agileCryptor.decrypt(cwmpSetting.cpe_username),
        agileCryptor.decrypt(cwmpSetting.cpe_password),
      ])

      await dbTplink.update(stringTable.collection.sysSetting, { '_id': 'cwmp' }, { $set: {
        'acs_username': decryptedResult[0],
        'acs_password': decryptedResult[1],
        'cpe_username': decryptedResult[2],
        'cpe_password': decryptedResult[3],
        'encrypt': null
      }})
    }
  } catch(err) {
    console.log(err)
    throw new Error('Decrypt syssetting failed')
  }
}

async function decrypt_notification (){
  let mail_username, mail_password
  try {
    let mail = await dbTplink.aggregate(stringTable.collection.notification, [
      { $match : { '_id' : 'mail' }}, {
        $project: {
          'username' : '$settings.username',
          'password' : '$settings.password',
          'encrypt' : '$settings.encrypt'
        }
      }
    ]).toArray()
    if(mail[0].encrypt == 1) {
      mail_username = mail[0].username ? await agileCryptor.decrypt(mail[0].username) : ''
      mail_password = mail[0].password ? await agileCryptor.decrypt(mail[0].password) : ''
      await dbTplink.update(stringTable.collection.notification, { '_id': 'mail' }, {
        $set: {
          'settings.username': mail_username,
          'settings.password': mail_password,
          'settings.encrypt': null
        }
      })
    }
  } catch(err) {
    console.log(err)
    throw new Error('Decrypt notification failed')
  }
}

async function startDecrypt() {
  if(ENCRYPT_ENABLE) {
    try {
      // wait dbs to be connected
      await dbTplink.waitConnected()
      // decrypt all necessary data
      await Promise.all([
        decrypt_cwmp(),
        decrypt_notification()
      ])
      close(0)
    } catch(err) {
      log('ERROR:', err)
      close()
    }
  } else {
    log('encrypt feature is disabled')
    close(0)
  }
}

startDecrypt()
