#!/usr/bin/env node
'use strict'

const { agileConfig, stringTable, MongoClass } = require('tplib')

const args = process.argv.slice(2)
const event_id = args[0]
const mailjs_path = "/opt/genieacs/bin/mail.js"

const tpDb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))

async function get_sys_enable(system) {
  try {
    await tpDb.waitConnected()
    let settings = await tpDb.find(stringTable.collection.notification, { "_id": system })
    return settings[0]["settings"]["enable_alert"]
  } catch(err) {
    console.log(err)
  }
}

async function get_event_enable(id) {
  try {
    await tpDb.waitConnected()
    let settings = await tpDb.find(stringTable.collection.notification, { "_id": "mail" }, { "event_list": 1 })
    return settings[0]["event_list"][id]
  } catch(err) {
    console.log(err)
  }
}

async function main() {
  const email_enabled = await get_sys_enable("mail")
  const event = await get_event_enable(event_id)

  if (email_enabled && event.active) {
    const cmd = "node " + mailjs_path + " " + event_id
    return require("child_process").exec(cmd, function(err, stdout) {
      if (err) console.log(err)
      else if (stdout.match(/Email sent.+OK/)) console.log("[SEND Successfully]" + event.event)
      else console.log("[SEND failed]" + event.event)
      process.exit()
    })
  }
  tpDb.disconnectDb()
  process.exit()
}

main()
.catch(() => {
  tpDb.disconnectDb()
  process.exit(1)
})
