'use strict'

const koa = require("koa")
const { agileConfig, stringTable, agileLogger } = require('tplib')

const app = new koa()

let getRouteList = {}
let server = null

// reserved for JWT
/*
const koajwt = require('koa-jwt')
const secret = 'AgileACS'
// optional: custom 401 response
// default 401 response body is ${err.message} (String)
app.use( (ctx, next) => {
  return next().catch( err => {
    if( 401 === err.status ) {
      Object.assign(ctx, { body: { status: 401, result: "auth failed" }})
    }
    else throw err
  })
})
app.use(koajwt({ secret }).unless({ path: [/^\/public/] }))
*/

// add signal handler
function endProcess() {
  if(server) server.close()
  agileLogger.disconnectDb()
  agileLogger.close()
  for(let key in getRouteList) {
    getRouteList[key](true)
  }
  setTimeout(() => {
    process.exit(0)
  }, 1000)
}
process.on('SIGINT', endProcess)
process.on('SIGTERM', endProcess)

// Error Handler
app.use(async (ctx, next) => {
  try {
    await next()
  } catch (err) {
    if(err.status == 500) ctx.append('Connection', 'close')  // append connection close header
    let body = err.message
    ctx.status = Number.isInteger(err.status) ? err.status : 500
    ctx.body = body
  }
})

const prepareServer = async () => {
  let routeEntryFiles = await new Promise((resolve, reject) => {
    // folder in routes/ presents a API route
    require("glob")("routes/*", (err, files) => {
      if(err) reject(err)
      resolve(files)
    })
  })

  await Promise.all(routeEntryFiles.map( item => {
    let result = item.split('/').pop()
    getRouteList[result] = require(`../../routes/${result}`)  // will include default index.js
    return getRouteList[result]()
    .then(route => {
      app.use(route.routes())
    })
  }))

  startServer()
}

const startServer = () => {
  // 404 handler here (if we goes to this last route, it means 404)
  app.use(ctx => {
    ctx.status = 404
    ctx.body = stringTable.error.httpError['404']
  })

  server = app.listen(agileConfig.get('AGILE_NBI_PORT'), () => {
    console.log(`server listening port ${agileConfig.get('AGILE_NBI_PORT')}`)
  })
}

prepareServer()
