var common, device, http_request, cache;

common = require('../common');
device = require('../device');
http_request = require('../http_request');
cache = require('../cache');
logger = require('../logger');

is98DeviceType = function (rpcReq) {
    var ref, m, len;
    ref = rpcReq.parameterList;
    for (m = 0, len = ref.length; m < len; m++) {
      p = ref[m];
      //console.log("[is98DeviceType] p[0] = " + p[0])
      if(p[0] === "InternetGatewayDevice.DeviceSummary" ){
        return true;
      }
    }
    return false;
};

getRootNode = function (parameter_name) {
    var ret  = "NULL";
    try {
      var index = parameter_name.indexOf("Device.");
      switch(index) {
      case 0:
        ret  = "Device";
        break;
      case 15:
        ret  = "InternetGatewayDevice";
        break;
      case -1:
        default:
        ret  = "NULL";
      }
    } catch (ex) {
      //console.log("license-check.js: getRootNode(), Exception=\n"+ex);
      ret  = "NULL";
    } finally {
     // console.log("license-check.js: getRootNode(), ret="+ret);
    }
    return ret;
  };

  findRootNode = function (parameterList) {
    var ret  = "NULL";
    try {
      // Ref: soap.js# function parameterValueList(xml)
      if (parameterList && parameterList.length > 0 ) {
        ret = getRootNode(parameterList[0][0]);
      }
    } catch (ex) {
     // console.log("license-check.js: findRootNode(), Exception=\n"+ex);
      ret  = "NULL";
    } finally {
     // console.log("license-check.js: findRootNode(), ret="+ret);
    }
    return ret;
  };

addDefaultDataModel098ToSession = function (sessionContext, rpc){
  var toClear = null;
  var timestamp = 1;
  var is098 = true;

  if (rpc.cpeRequest != null && rpc.cpeRequest.name === 'Inform') {
    is098 = is98DeviceType(rpc.cpeRequest);
  }
  if (rpc.cpeResponse && rpc.cpeResponse.parameterList)
  {
    datamodel = findRootNode(rpc.cpeResponse.parameterList);
    if(datamodel === "Device")
      is098 =  false;
  }
  //console.log("[addDefaultDataModel098ToSession] DeviceID is98 Data in sessionContext to be "+ is098)
  p = [
    ['DeviceID', 'is98'], timestamp, {
      object: [timestamp, 0],
      writable: [timestamp, 0],
      value: [timestamp, [is098, 'xsd:boolean']]
    }
  ]
  toClear = device.set(sessionContext.deviceData, p[0], p[1], p[2], toClear);
  return is098;

}

isDataModel098FromSession = function (sessionContext, rpc){
    var ref1, p, ref2, ref3;
    var ret = false;
   ref1 = sessionContext.deviceData.paths.find(['DeviceID', 'is98'], false, true);
   if(ref1.length)
   {
      for (l = 0, len = ref1.length; l < len; l++) {
          p = ref1[l];
          if((ref2 = sessionContext.deviceData.attributes.get(p)) != null)
          {
              ref3 = ref2.value
              ret = ref3[1][0]
          }
      }
    }
    else
    {
      //console.log("[isDataModel098FromSession] No DeviceID is98 Data in sessionContext")
      ret = addDefaultDataModel098ToSession(sessionContext, rpc)
    }
    return ret;
}


notMatchDataModelParamters = function (sessionContext, rpc) {
   var datamodel, sessiondatatype098 = false, informtype098, error_msg;
   sessiondatatype098 = isDataModel098FromSession(sessionContext, rpc) ;
   //console.log("[datamode-check.js] ["+sessionContext.deviceId+"] sessiondatatype098 = "+ sessiondatatype098);
   if (rpc.cpeRequest != null && rpc.cpeRequest.name === 'Inform') {
      informtype098 = is98DeviceType(rpc.cpeRequest);
      //console.log("[datamode-check.js] ["+sessionContext.deviceId+"] informtype098 = "+ informtype098);
      if((sessiondatatype098 === true) && (informtype098 === true))
        return false;
      else if((sessiondatatype098 === false) && (informtype098 === false))
        return false;
      else
      {
        error_msg = 'Inform as ' + (informtype098 ? "TR098" : "TR181") + " but Current DataModel is " + (sessiondatatype098 ? "TR098" : "TR181");
        logger.accessInfo({
          message: error_msg,
          sessionContext: sessionContext
        });
        return true;
      }
    }
    if (rpc.cpeResponse && rpc.cpeResponse.parameterList)
    {
      datamodel = findRootNode(rpc.cpeResponse.parameterList);
      //console.log("[datamode-check.js] ["+sessionContext.deviceId+"] datamodel = "+ datamodel + " sessiondatatype098 = "+sessiondatatype098);
      if((sessiondatatype098 === true) && (datamodel === "InternetGatewayDevice"))
        return false;
      else if((sessiondatatype098 === false) && (datamodel === "Device"))
        return false;
      else if(datamodel === "NULL")
        return false;
      else
      {
        error_msg = 'Response Parameters RootDevice as ' + datamodel + " but Current DataModel is " + (sessiondatatype098 ? "TR098" : "TR181");
        logger.accessInfo({
          message: error_msg,
          sessionContext: sessionContext
        });
        return true;
      }
    }
    return false;
};


var proc_value_change_mesh_topology = function (sessionContext) {
  var isNew = sessionContext.new, hasChagned = false, p, ref1, ref2, node, cnt = 0;
  var nodes = [
    "Device.DeviceInfo.X_TP_DevName",
    "Device.WiFi.MultiAP.APDeviceNumberOfEntries",
    "Device.WiFi.MultiAP.APDevice.*.*",
    "Device.X_TP_RE_MAPPING.*.X_TP_DevName",
    "Device.X_TP_RE_MAPPING.*.Status.*"
  ];

  try {
    if (!isNew) {
      for (var i = 0; i < nodes.length && cnt < 1; i++) {
        node = nodes[i], hasChagned = false;
        ref1 = sessionContext.deviceData.paths.find(node.split('.'), false, true);
        if (ref1 && ref1.length) {
          for (var j = 0; j < ref1.length && cnt < 1; j++) {
            p = ref1[j];
            ref2 = sessionContext.deviceData.attributes.get(p);
            if (ref2 && ref2.value && ref2.value.length >= 2 && ref2.value[1] && ref2.value[1][1]) {
              cnt++;
              //console.log('  ' + p.join(".") + ' = ' + ref2.value[1]);
            }
          }
        }
      }
    }
  } catch (exx) {
    console.log(exx);
    cnt = nodes.length;
  }

  if (isNew || cnt > 0) {
    var toClear = null;
    var timestamp = Date.now();
    p = [
      ['DeviceID', 'enable_update_mesh_topology'], timestamp, {
        object: [timestamp, 0],
        writable: [timestamp, 0],
        value: [timestamp, [true, 'xsd:boolean']]
      }
    ];
    device.set(sessionContext.deviceData, p[0], p[1], p[2], toClear);
  }
}

var find_diagName = function (deviceId) {
  return new Promise((resolve, reject) => {
    if (http_request && http_request.send_post_diaglock_request) {
      try {
        http_request.send_post_diaglock_request(deviceId, "isLocked", function (result) {
          // * Get Diagnostic Name
          result_json = JSON.parse(result);
          diagName = result_json.result;
          resolve(diagName);
        });
      } catch (ex) {
        resolve("");
      }
    } else {
      resolve("");
    }
  });
}

var proc_diagnostics_complete = function(sessionContext) {
  find_diagName(sessionContext.deviceId).then((diagName)=>{
    //console.log('proc_diagnostics_complete: diagName = ', diagName);
    if (diagName && diagName != "") {
      var toClear = null;
      var timestamp = Date.now();
      var p = [
        ['DeviceID', 'diagName'], timestamp, {
          object: [timestamp, 0],
          writable: [timestamp, 0],
          value: [timestamp, [diagName, 'xsd:string']]
        }
      ];
      
      device.set(sessionContext.deviceData, p[0], p[1], p[2], toClear);
    }
  });
}

var proc_connection_request = function(sessionContext) {
  return cache.del(`${sessionContext.deviceId}_xmpp_request_sent`);
}

var proc_inform_event = function (sessionContext, rpc) {
  try {
    if (rpc.cpeRequest != null && rpc.cpeRequest.name === 'Inform') {
      var ref1 = sessionContext.deviceData.paths.find(['Events', '*'], false, true), l=0, len=0; p;
      if (ref1 && ref1.length) {
        for (l = 0, len = ref1.length; l < len; l++) {
          p = ref1[l];
          //console.log('proc_inform_event: '+p[1]);
          switch(p[1]) {
            case "4_VALUE CHANGE":
              proc_value_change_mesh_topology(sessionContext);
              break;
            case "8_DIAGNOSTICS COMPLETE":
              proc_diagnostics_complete(sessionContext);
              break;
            case "6_CONNECTION REQUEST":
              proc_connection_request(sessionContext);
              break;
            default:
          }
        }
      }
    }
  } catch (ex) {
    console.log('proc_inform_event: err=', ex);
  }
};

exports.is98DeviceType = is98DeviceType
exports.notMatchDataModelParamters = notMatchDataModelParamters;
exports.proc_inform_event = proc_inform_event;
