'use strict'

const exec = require('child_process').exec
const spawn = require('child_process').spawn
const fs = require('fs')

const LOG_FILE = "/var/log/genieacs/acsapps.log"
const enable_logfile = true
const enable_debug = true

class AgileACSApps {
  static exec(cmd, options, callback) {
    exec(cmd, options, callback)
  }

  static spawn(cmd, args, options) {
    spawn(cmd, args, options)
  }

  static generateSafeCmdArgs(input) {
    return `"${input.replace(/\\/g, '\\\\').replace(/"/g, '\\"')}"`
  }

  // args allows string or string array
  static genCmdStringWithSafeArgs(cmd, args) {
    let result = cmd
    if(!args) return result
    if(typeof(args) === "string") {
      return `${cmd} ${AgileACSApps.generateSafeCmdArgs(args)}`
    }
    args.forEach(item => {
      result = `${result} ${AgileACSApps.generateSafeCmdArgs(item)}`
    })
    return result
  }

  // args allows string or string array
  static execCmdWithSafeArgs(cmd, args, options) {
    return new Promise((res, rej) => {
      let combinedCmd = AgileACSApps.genCmdStringWithSafeArgs(cmd, args)
      exec(combinedCmd, options, (err, stdout, stderr) => {
        if(err) {
          err.stdout = stdout
          err.stderr = stderr
          return rej(err)
        }
        res({ stdout, stderr })
      })
    })
  }

  static log(logFrom, message) {
    let log_str = `[${logFrom}] ${new Date()}: ${message}`
    if(enable_debug) console.log(log_str)
    if(enable_logfile) fs.appendFile(LOG_FILE, `${log_str}\n`)
  }
}

module.exports = AgileACSApps
