/*
 * Copyright 2019-2029 Agile ACS
 *
 * This file is part of Agile ACS
 *   the config is defined in config-ext/agile-config.json
 */

'use strict'

let path = require('path')
const SETTING = require("../../config-ext/agile-config.json")

let agileConfig = {}

/**
 * convert the data type of the given val. 
 * @param {*} val 
 * @param {*} type 
 */
let convert_value = function (val, type) {
  var ref;
  var ret = null;
  switch (type) {
    case 'int':
      ret = Number(val);
      break;
    case 'boolean':
      ret = (ref = String(val).trim().toLowerCase()) === 'true' || ref === 'on' || ref === 'yes' || ref === '1';
      break;
    case 'string':
      ret = String(val);
      break;
    case 'path':
      if (val) {
        ret = path.resolve(val);
      } else {
        ret = '';
      }
      break;
  }
  return ret;
};

/**
 * Set agile config via key, val. If the given system is true, set into process.env with name="AgileACS"+key 
 * @param {*} k 
 * @param {*} v 
 * @param {*} system 
 */
let setValue = function (k, v, system) {
  agileConfig[k] = v;
  if (system) {
    var name = "AgileACS_" + k;
    if ( process.env[name] == null ) {
      process.env[name] = v;
    }
  }
}

/**
 * init agile config
 */
let init_config = function () {
  var k;
  for (k in SETTING ) {
    var system = SETTING[k]["system"];
    var type = SETTING[k]["type"];
    var value = SETTING[k]["value"];
    setValue(k, convert_value(value, type), system);
  }
}

let get = function (key, formalize) {
  return (formalize) ? agileConfig[key.replace("-", "_").toUpperCase()] :agileConfig[key];
};

init_config();

exports.get = get;

function getRedisConfig() {
  return {
    redisHost: get('REDIS_HOST'),
    redisPort: get('REDIS_PORT'),
    redisDb: get('REDIS_DB'),
    maxCacheTtl: get('MAX_CACHE_TTL')
  }
}

exports.getRedisConfig = getRedisConfig
