'use strict'

const fs = require('fs')

const MongoClass = require('./mongo-class')
const agileConfig = require('./agile-config')
const stringTable = require('./string-table.json')

class AgileFeatureSet {
  constructor() {
    this.dbTplink = new MongoClass(agileConfig.get(stringTable.url.dbTplink))
  }

  /**
   * clean function
   */
  destruct() {
    this.dbTplink.disconnectDb()
  }

  /**
   * Check if current feature set is pro
   * @returns {Promise<boolean|string>}
   */
  async isProFeatureEnabled() {
    return new Promise((res, rej) => {
      fs.access('/opt/agileacs/tmp/xj4j32l4ejvu045j6vup', fs.constants.F_OK, err => {
        if(err) return rej('license file not found')
        fs.readFile('/opt/agileacs/tmp/xj4j32l4ejvu045j6vup', 'utf8', async (err, data) => {
          if(err) return rej('read license file failed')
          if(data.trim().toLocaleLowerCase() === 'lite') return res(false)
          res(true)
        })
      })
    })
  }

  /**
   * Check if a feature is pro feature or not
   * @param {string} feature feature to check
   * @returns {boolean}
   */
  async isProFeature(feature) {
    let result = await this.dbTplink.findOne(stringTable.collection.proFeatures, {_id: 'pro_features'})
    if(result && result[feature] !== undefined) return result[feature]
    return false
  }
}

module.exports = AgileFeatureSet
