'use strict'

const AgileCacheClass = require('tplib/agile-cache')
const agileConfig = require('tplib/agile-config')

const { insert_diag_timer, remove_diag_timer, disconnectDB } = require('./diag_timer')

const agileCache = new AgileCacheClass(agileConfig.getRedisConfig())
let reset_timmer = false

function parse_json_obj(json_str) {
  let ret = {}
  try {
    ret = JSON.parse((json_str) ? json_str: "{}")
  } catch(err){
    console.log(err)
    ret = {}
  }
  return ret
}

async function get_udp_echo_ttl(diagTask, deviceId, resultObj, obj, callback) {
  if (reset_timmer) {
    return // seems that reset_timer is always false
    /*
    if (diagTask == "UDPEchoConfig" && cache.expire && cache.ttl) {
      return cache.ttl(deviceId + "_diaglock", function (err, ttl) {
        if (err) {
          return callback(resultObj)
        }
        //console.log(deviceId+', '+diagTask + ', ttl = ' + ttl )
        var offset = 60
        if ((ttl - 0) > 0 && (ttl - 0) < (60 * 5 - offset) ) {
          return cache.expire(deviceId + "_diaglock", 60 * 5, function (err) {
            timer.insert_diag_timer(deviceId, diagTask, (60 * 5 + offset))
            return callback(resultObj)
          })
        }
        return callback(resultObj)
      })
    } else {
      return callback(resultObj)
    }
    */
  } else {
    if (diagTask === 'UDPEchoConfig') {
      try {
        let ttl = await agileCache.ttlPromise(`${deviceId}_diaglock`)
        if(obj && obj.timeout && Number.parseInt(obj.timeout) > 0 && Number.parseInt(ttl) > 0) resultObj['ttl'] = ttl
      } catch(err) {
        console.log(err)
      }
    }
    if(callback) return callback(resultObj)
    return resultObj
  }
}

async function getDiagnosticLock(deviceId, controllerId, callback) {
  let result= { result: '', message: '' }
  if(controllerId && controllerId != '') {
    try {
      let json_data = parse_json_obj(await agileCache.getPromise(`${controllerId}_diaglock`))
      let diagTask = json_data.name
      if(diagTask) {
        let resultObj = { result: diagTask, message: `Controller ID: ${controllerId}`}
        result = await get_udp_echo_ttl(diagTask, deviceId, resultObj, json_data)
      } else {
        let json_data = parse_json_obj(await agileCache.getPromise(`${deviceId}_diaglock`))
        let diagTask = json_data.name
        if(!diagTask) result = { result: '', message: '' }
        else result = { result: diagTask, message: '' }
      }
    } catch(err) {
      console.log(err)
      result = { result: '', message: '' }
    }
  } else {
    try {
      let json_data = parse_json_obj(await agileCache.getPromise(`${deviceId}_diaglock`))
      let diagTask = json_data.name
      if(!diagTask) {
        result = { result: '', message: '' }
        remove_diag_timer(deviceId)
      } else {
        let resultObj = { result: diagTask, message: '' }
        result = await get_udp_echo_ttl(diagTask, deviceId, resultObj, json_data)
      }
    } catch(err) {
      remove_diag_timer(deviceId)
      result = { result: '', message: '' }
    }
  }
  if(callback) return callback(result)
  return result
}

async function saveDiagnosticLock(deviceId, diagName, timeout, callback) {
  timeout = Number.parseInt(timeout)
  if(timeout < 60) timeout = 60

  let obj = { "name": diagName }, result
  if (diagName === 'UDPEchoConfig') obj['timeout'] = timeout

  try {
    let json_data = JSON.stringify(obj)
    await agileCache.setPromise(`${deviceId}_diaglock`, json_data, timeout)
    insert_diag_timer(deviceId, diagName, timeout)
    result = { result: 'locked', message: '' }

  } catch(err) {
    result = { result: 'error', message: 'locked failed' }
    remove_diag_timer(deviceId)
  }
  if(callback) return callback(result)
  return result
}

async function destoryDiagnosticLock(deviceId, callback) {
  let result
  try {
    await agileCache.delPromise(`${deviceId}_diaglock`)
    result = { result: 'unlocked', message: '' }
  } catch(err) {
    result= { result: 'error', message: 'unlocked failed'}
  }
  if(callback) return callback(result)
  return result
}

async function disconnectCache() {
  await agileCache.disconnect()
  disconnectDB()
}

module.exports = {
  getDiagnosticLock,
  saveDiagnosticLock,
  destoryDiagnosticLock,
  disconnectCache
}
