'use strict'

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

let subrouteList = {}

route.use(require('koa-bodyparser')())

let { agileConfig, stringTable, MongoClass } = require('tplib')
let tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))

// default route for this prefix
route.get('/', async (ctx) => {
  try {
    let res = await dbFindActivateReport()
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})

route.post('/check_network', async (ctx) => {
  try {
    let res = await checkNetwork()
    ctx.body = res
  } catch (err) {
    ctx.throw(400)
  }
})

route.post('/save_info', async (ctx) => {
  try {
    let activateReport
    if (ctx.request.body)
      activateReport = ctx.request.body
    await dbSaveInfo(activateReport)
    ctx.body = { msg: "success" }
  } catch (err) {
    ctx.throw(400)
  }
})

let dbFindActivateReport = async () => {
  return tpdb.queryResource(stringTable.collection.activateReport)
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

let checkNetwork = async () => {
  return new Promise((resolve, reject) => {
    let result, msg
    let cmd = "node /opt/genieacs/bin/agileacs_activate_report.js update activation activation";
    require("child_process").exec(cmd, function (err, stdout, stderr) {
      if (err) {
        reject(err)
      } else if (stderr) {
        reject(stderr)
      }
      if (stdout.match(/0/)) {
        msg = "fail";
      } else if (stdout.match(/1/)) {
        msg = "success";
      } else {
        msg = "error";
      }
      result = {
        msg: msg
      }
      resolve(result)
    });
  })

}

let dbSaveInfo = async (activateReport) => {
  return tpdb.updateOne(stringTable.collection.activateReport, { "_id": "service" }, { $set: activateReport })
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    tpdb.disconnectDb()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  let subEntryFiles = null

  // find subroute files
  try {
    subEntryFiles = await new Promise((resolve, reject) => {
      require("glob")(`${__dirname}/*.sub.js`, (err, files) => {
        if (err) reject(err)
        resolve(files)
      })
    })
  } catch (err) {
    // return route without subroute
    return route
  }

  // add subroute to route
  subEntryFiles.forEach(item => {
    let filename = item.split('/').pop()
    let subroute = require(`./${filename}`)
    subrouteList[filename] = subroute
    route.use(subroute.routes(), subroute.allowedMethods())
  })

  return route
}

module.exports = getRoute
