'use strict'

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

const { AgileACSApps } = require('tplib')

let subrouteList = {}

// debug flag
const DBG_APPLICATION = false

// acsapps command
const STATUS_CMD1 = "cat /tmp/mongodb_tmp_status && touch /tmp/mongodb_tmp_status_timestamp && diff <(ls -l /tmp/mongodb_tmp_status) <(cat /tmp/mongodb_tmp_status_timestamp) >/dev/null 2>&1;echo $?"
const STATUS_CMD2 = "ls -l /tmp/mongodb_tmp_status > /tmp/mongodb_tmp_status_timestamp"

function dbg_console_req(ctx) {
  if (DBG_APPLICATION) {
    console.log(`Now we are at ${ctx.req.url}`)
    console.log(ctx.method + " " + ctx.path)
    console.log(ctx.request.body)
    console.log(Object.keys(ctx.request.body).length)
  }
}

function dbg_console_res(ctx, cmd) {
  if (DBG_APPLICATION) {
    if (cmd) console.log('cmd= ' + cmd)
    console.log(ctx.response)
  }
}

function http_result(status_code, msg) {
  return { 'code': status_code, 'message': msg }
}

function generic_spawn(cmd, options) {
  return new Promise((resolve, reject) => {
    try {
      let cmdarray = STATUS_CMD2.split(' ');
      //require('child_process').spawn(cmdarray.shift(), cmdarray, options)
      AgileACSApps.spawn(cmdarray.shift(), cmdarray, options)
      resolve('200 OK')
    }
    catch (e) {
      reject(e)
    }
  })
    .then((r) => http_result(200, r))
    .catch((e) => { http_result(400, e) })
}

function generic_exec(ctx, cmd, options) {
  return new Promise((resolve, reject) => {
    //require('child_process').exec(cmd, options, (err, stdout, stderr) => {
    AgileACSApps.exec(cmd, options, (err, stdout, stderr) => {
      var status
      // if (err) reject(http_result(202, err))
      if (stderr) reject(http_result(404, '404 Not Found'))
      stdout = stdout.replace('\n', '');
      status = stdout.match(/[a-z]+/)[0];
      if (status === "ready") {
        if (stdout.match(/[0-9]+/)[0] === "0") {
          status = "no change";
        }
        else{
          resolve(http_result(100, status))
        }
      }
      resolve(http_result(200, status))
    })
  })
    .then(result => {
      ctx.status = result['code']
      ctx.body = JSON.stringify({ "status": result['message'] })
    })
    .catch((error) => {
      ctx.status = error['code']
      ctx.body = error['message']
    })
}

route.use(require('koa-bodyparser')())

// '/applicatin/status'
route.get('/status', async ctx => {
  dbg_console_req(ctx)

  await generic_exec(ctx, STATUS_CMD1, { "shell": "/bin/bash" })
  if (ctx.status == 100) await generic_spawn(STATUS_CMD2, { "shell": "/bin/bash" })
  ctx.status = 200
   
  dbg_console_res(ctx, STATUS_CMD1)
})

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  let subEntryFiles = null

  // find subroute files
  try {
    subEntryFiles = await new Promise((resolve, reject) => {
      require("glob")(`${__dirname}/*.sub.js`, (err, files) => {
        if (err) reject(err)
        resolve(files)
      })
    })
  } catch (err) {
    // return route without subroute
    return route
  }

  // add subroute to route
  subEntryFiles.forEach(item => {
    let filename = item.split('/').pop()
    let subroute = require(`./${filename}`)
    subrouteList[filename] = subroute
    route.use(subroute.routes(), subroute.allowedMethods())
  })

  return route
}

module.exports = getRoute
