'use strict'

const koarouter = require('koa-router')

const AgileACSApps = require('tplib/agile-acsapps')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

let subrouteList = {}

// include general purpose body parser here
route.use(require('koa-bodyparser')())

/**
 * get back file list
 * expect ctx.request.body = { mount_point: '' }
 */
route.post('/filelist', async ctx => {
  try {
    // when testing, just check the command string
    if(process.env.NODE_ENV === 'test') {
      ctx.body = { result: AgileACSApps.genCmdStringWithSafeArgs('/opt/agileacs/bin/usb-storage.sh', [
        'get-list',
        ctx.request.body.mount_point
      ])}
      return
    }
    let { stdout, stderr } = await AgileACSApps.execCmdWithSafeArgs('/opt/agileacs/bin/usb-storage.sh', [
      'get-list',
      ctx.request.body.mount_point
    ])
    if(stderr) throw new Error(stderr)
    ctx.body = { result: stdout.trim().split("\n") }
    ctx.status = 200
  } catch(err) {
    console.log(err)
    response202(ctx)
  }
})

/**
 * handles backup
 * expect ctx.request.body = {action: [mount_point, filename]}
 */
route.post('/', ctx => {
  try {
    // when testing, just check the command string
    if(process.env.NODE_ENV === 'test') {
      ctx.body = { backup: AgileACSApps.genCmdStringWithSafeArgs('/usr/bin/nice', [
        '-n19',
        '/opt/agileacs/bin/agileacs-backup.sh',
        ...ctx.request.body.action
      ]),
      progress: AgileACSApps.genCmdStringWithSafeArgs('/opt/agileacs/bin/agileacs-backup-progress.sh', ctx.request.body.action[0])}
      return
    }
    // seems this api do not need to wait command response, no async/await here
    AgileACSApps.execCmdWithSafeArgs('/usr/bin/nice', [
      '-n19',
      '/opt/agileacs/bin/agileacs-backup.sh',
      ...ctx.request.body.action
    ])
    AgileACSApps.execCmdWithSafeArgs('/opt/agileacs/bin/agileacs-backup-progress.sh', ctx.request.body.action[0])
    ctx.status = 200
  } catch(err) {
    console.log(err)
    response202(ctx)
  }
})

function response202(ctx) {
  ctx.set('Cache-Control', 'no-cache')
  ctx.status = 202
}

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }
  return route
}

module.exports = getRoute
