'use strict'

const koarouter = require('koa-router')
const { ProxyHandler } = require('tplib')
const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

const proxyHandler = new ProxyHandler()

let subrouteList = {}

route.use(require('koa-bodyparser')())

// proxy needs to leave body un-parsed
route.use(proxyHandler.streamBodyParser)

let { agileConfig, stringTable, MongoClass } = require('tplib')
let geniedb = new MongoClass(agileConfig.get(stringTable.url.dbGenie))

// default route for this prefix
route.get('/', async (ctx) => {
  try {
    let res = await getCsmenus(ctx.request.query['query'])
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})

route.put('/:item_id', async (ctx) => {
  let csmenu = ctx.request.body
  csmenu._id = ctx.params['item_id']
  try {
    let res = await saveCsmenus(csmenu)
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})

route.delete('/:item_id', async (ctx) => {
  let csmenu = {'_id': ctx.params['item_id']}
  try {
    let res = await removeCsmenus(csmenu)
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})

let getCsmenus = async (query) => {
  return geniedb.queryResource(stringTable.collection.csmenus, query)
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

let saveCsmenus = async (csmenu) => {
  return geniedb.save(stringTable.collection.csmenus, csmenu)
  .then(res => {
    return res
  })
  .catch(err => {
    return err
  })
}

let removeCsmenus = async (csmenu) => {
  return geniedb.remove(stringTable.collection.csmenus, csmenu)
  .then(res => {
    return res
  })
  .catch(err => {
    return err
  })
}

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    geniedb.disconnectDb()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  let subEntryFiles = null

  // find subroute files
  try {
    subEntryFiles = await new Promise((resolve, reject) => {
      require("glob")(`${__dirname}/*.sub.js`, (err, files) => {
        if (err) reject(err)
        resolve(files)
      })
    })
  } catch (err) {
    // return route without subroute
    return route
  }

  // add subroute to route
  subEntryFiles.forEach(item => {
    let filename = item.split('/').pop()
    let subroute = require(`./${filename}`)
    subrouteList[filename] = subroute
    route.use(subroute.routes(), subroute.allowedMethods())
  })

  return route
}

module.exports = getRoute
