'use strict'

const AgileCryptor = require('tplib/agile-cryptor')
const MongoClass = require('tplib/mongo-class')
const agileConfig = require('tplib/agile-config')
const stringTable = require('tplib/string-table.json')
const { getDiagnosticLock, destoryDiagnosticLock, disconnectCache } = require('tplib/diag_lock_util')

const DEFAULT_RE_MAPPING_INSTANCE_MAX_VAL = 6
const ENCRYPT_ENABLE = agileConfig.get('ENCRYPT_ENABLE')
const EXTENSION_FIELDS = ['_homeId', '_zoneId', '_cellPhone', '_cpf']
const agileCryptor = new AgileCryptor(ENCRYPT_ENABLE, agileConfig.getRedisConfig())
const dbGenie = new MongoClass(agileConfig.get(stringTable.url.dbGenie))
const dbTplink = new MongoClass(agileConfig.get(stringTable.url.dbTplink))
const dbTmp = new MongoClass(agileConfig.get(stringTable.url.dbTmp))

function create_new_mesh_links(deviceId, workMode, device) {
  let match, update, ref1, ref2
  switch (workMode) {
    case 'CAP':
      ref1 = find_mesh_re_macs(device)
      if (ref1.re_macs.length == 0) return Promise.resolve()

      match = {
        '$or': []
        // , "Device.X_TP_MESH_WIFI.WorkMode._value": "RE"
      }
      ref1.re_macs.forEach(function (item) {
        match['$or'].push({ "Device.Ethernet.Link.1.MACAddress._value": item })
      })
      update = { '$set': { "_mesh.type": "Satellite", "_mesh.cap": deviceId } }

      return dbGenie.update(stringTable.collection.devices, { "_id": deviceId }, { "$set": { "_mesh.max_index": ref1.max } }, { multi: false }).then(() => {
        return dbGenie.update(stringTable.collection.devices, match, update, { multi: true }).then(() => {
          return dbTmp.update(stringTable.collection.tmpdevices, match, update, { multi: true }).then(() => {
            return getDiagnosticLock(deviceId, "").then(diagName => {
              if (diagName == 'Mesh') return destoryDiagnosticLock(deviceId)
              return
            }).catch((err) => {
              console.log(err)
              return
            })
          }).catch((err) => {
            console.log(err)
            return
          })
        }).catch(() => {
          return
        })
      }).catch(() => {
        return
      })
    case 'RE':
      return dbGenie.aggregate(stringTable.collection.devices, [{ '$group': { '_id': {}, 'max': { '$max': '$_mesh.max_index' }}}])
      .toArray()
      .then(out => {
        let upper_limit = (out.length > 0 && out[0].max) ? out[0].max : DEFAULT_RE_MAPPING_INSTANCE_MAX_VAL

        ref1 = device['Device']['Ethernet']['Link']['1']['MACAddress']['_value'], ref2 = []
        for (let i = 1; i <= upper_limit; i++) {
          var obj = {}, path = "Device.X_TP_RE_MAPPING." + i + ".X_TP_ReMac._value"
          obj[path] = ref1
          ref2.push(obj)
        }
        match = { '$or': ref2 }
        //console.log('match = ', match)

        return dbGenie.find(stringTable.collection.devices, match, { '_id': 1, "Device.X_TP_RE_MAPPING._timestamp": 1 })
        .sort({ "Device.X_TP_RE_MAPPING._timestamp": -1 })
        .limit(1)
        .toArray()
        .then((res) => {
          if (res.length != 1) return

          let controllerId = res[0]["_id"], match = { "_id": deviceId }, update = { "$set": { "_mesh.cap": controllerId }}
          return dbGenie.update(stringTable.collection.devices, match, update, { multi: false })
          .then(() => {
            return dbTmp.update(stringTable.collection.tmpdevices, match, update, { multi: false })
            .catch((err) => {
              console.log(err)
              return
            })
          }).catch(() => {
            return
          })
        }).catch(() => {
          return
        })
      })
      .catch((err) => {
        console.log(err)
        return
      })
    default:
      return Promise.resolve()
  }
}

function find_mesh_re_macs(dm) {
  let ret = { "max": DEFAULT_RE_MAPPING_INSTANCE_MAX_VAL, "re_macs": [] }
  try {
    if (dm) {
      var mapping = dm.Device.X_TP_RE_MAPPING
      var m = 0
      for (let i in mapping) {
        if (!isNaN(i) && i*1 > m) {
          m = i*1
        }
        for (let f in mapping[i]) {
          if (f == "X_TP_ReMac") {
            if (mapping[i][f]._value) {
              ret.re_macs.push(mapping[i][f]._value)
            }
          }
        }
      }
      if (m > 0) {
        ret.max = m
      }
    }
  } catch (ex) {
    console.log(ex)
    ret = { "max": DEFAULT_RE_MAPPING_INSTANCE_MAX_VAL, "re_macs": [] }
  }
  //console.log('find_mesh_re_macs = ', ret)
  return ret
}

function get_ip(cr_url) {
  return cr_url.match(
      /(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]):/g
  )[0].slice(0,-1)
}

function padding(num, length) {
  var len = (num + '').length
  var diff = length - len
  if (diff > 0) return Array(diff + 1).join('0') + num
  return num
}

function remove_old_mesh_links(deviceId) {
  return dbGenie.find(stringTable.collection.devices,
    { '_id': deviceId, "Device.X_TP_MESH_WIFI.WorkMode._value": { '$exists': true } },
    { '_id': 1, "_mesh": 1, "Device.X_TP_MESH_WIFI.WorkMode": 1, "Device.Ethernet.Link.1.MACAddress": 1, "Device.X_TP_RE_MAPPING": 1 }
  )
  .then(res => {
    if(res.length != 1) return

    let match, update, workMode = res[0].Device.X_TP_MESH_WIFI.WorkMode._value
    switch(workMode) {
    case 'CAP':
      match = { "_id": { "$ne": deviceId }, "_mesh.cap": deviceId }
      update = { '$unset': { "_mesh.cap": 1 }}
      break
    case 'RE':
      return create_new_mesh_links(deviceId, workMode, res[0]).catch((err) => {
        console.log(err)
        return
      })
    default:
      return
    }

    return dbGenie.update(stringTable.collection.devices, match, update, { multi: true })
    .then(() => {
      return dbTmp.update(stringTable.collection.tmpdevices, match, update, { multi: true })
      .then(()=>{
        return create_new_mesh_links(deviceId, workMode, res[0])
        .catch((err) => {
          console.log(err)
          return
        })
      })
      .catch((err) => {
        console.log(err)
        return
      })
    }).catch((err) => {
      console.log(err)
      return
    })
  })
  .catch(err => {
    console.log(err)
    return
  })
}

function tmpdb_update_parser(isNew, update, has_mesh_updated) {
  let _update = { '$set': {}, '$unset': {}, '$pull': {}, '$addToSet': {} }
  for (let k in update) {
    for (let kk in update[k]) {
      if (has_mesh_updated && has_mesh_updated['mode'] == '0' && /Device\.WiFi\.MultiAP\.APDevice\..+\._value/.test(kk)) {
        has_mesh_updated['mode'] = '1'
      }
      if(!isNew) {
        switch (kk) {
        case "_ipaddress":
        case "_lastInform":
        case "_username":
        case "_tags":
        case "_mesh":
        case "_mesh.type":
        case "_mesh.cap":
        case "Device.DeviceInfo.SoftwareVersion._value":
        //case "Device.DeviceInfo.SoftwareVersion._timestamp":
        case "Device.XMPP.Connection.1.Status._value": // eslint-disable-line
        //case "Device.XMPP.Connection.1.Status._timestamp":
        case "Device.ManagementServer.ConnectionRequestURL._value": // eslint-disable-line
        //case "Device.ManagementServer.ConnectionRequestURL._timestamp":
        case "InternetGatewayDevice.DeviceInfo.SoftwareVersion._value": // eslint-disable-line
        //case "InternetGatewayDevice.DeviceInfo.SoftwareVersion._timestamp":
        case "InternetGatewayDevice.XMPP.Connection.1.Status._value": // eslint-disable-line
        //case "InternetGatewayDevice.XMPP.Connection.1.Status._timestamp":
        case "InternetGatewayDevice.ManagementServer.ConnectionRequestURL._value": // eslint-disable-line
        //case "InternetGatewayDevice.ManagementServer.ConnectionRequestURL._timestamp":
        case "Device.X_TP_MESH_WIFI.WorkMode._value": // eslint-disable-line
          _update[k][kk] =  update[k][kk]
          break
        }
      } else {
        switch (kk) {
        case "_ipaddress":
          _update[k]["IP"] =  update[k][kk]
          _update[k][kk] =  update[k][kk]
          break
        case "_tags":
          _update[k][kk] =  update[k][kk]
          break
        case "_lastInform":
          _update[k][kk] =  update[k][kk]
        case "_username":
          _update[k][kk] =  update[k][kk]
          break
        case "_mesh":
        case "_mesh.type":
        case "_mesh.cap":
          _update[k][kk] =  update[k][kk]
          break
        case "Device._object":
        case "InternetGatewayDevice._object":
          _update[k][kk] =  update[k][kk]
          break
        case "Device.DeviceInfo.SerialNumber._value":
        case "InternetGatewayDevice.DeviceInfo.SerialNumber._value":
          _update[k]["SerialNumber"] =  update[k][kk]
          _update[k][kk] =  update[k][kk]
          break
        case "Device.DeviceInfo.ProductClass._value":
        case "InternetGatewayDevice.DeviceInfo.ProductClass._value":
          _update[k]["ProductClass"] =  update[k][kk]
          _update[k][kk] =  update[k][kk]
          break
        case "Device.DeviceInfo.ModelName._value":
        case "InternetGatewayDevice.DeviceInfo.ModelName._value":
          _update[k]["ModelName"] =  update[k][kk]
          _update[k][kk] =  update[k][kk]
          break
        case "Device.DeviceInfo.SoftwareVersion._value":
        case "InternetGatewayDevice.DeviceInfo.SoftwareVersion._value":
          _update[k]["SwVers"] =  update[k][kk]
          _update[k][kk] =  update[k][kk]
          break
        case "Device.DeviceInfo.HardwareVersion._value":
        case "InternetGatewayDevice.DeviceInfo.HardwareVersion._value":
          _update[k]["HwVers"] =  update[k][kk]
          _update[k][kk] =  update[k][kk]
          break
        case "Device.Ethernet.Link.1.MACAddress._value":
        case "InternetGatewayDevice.LANDevice.1.LANEthernetInterfaceConfig.1.MACAddress._value":
          _update[k]["MAC"] =  update[k][kk]
          _update[k][kk] =  update[k][kk]
          break
        case "Device.XMPP.Connection.1.Status._value":
        case "InternetGatewayDevice.XMPP.Connection.1.Status._value":
          _update[k]["XmppStatus"] =  update[k][kk]
          _update[k][kk] =  update[k][kk]
          break
        case "Device.ManagementServer.ConnectionRequestURL._value":
        case "InternetGatewayDevice.ManagementServer.ConnectionRequestURL._value":
          _update[k][kk] =  update[k][kk]
          break
        case "Device.X_TP_MESH_WIFI.WorkMode._value":
          _update[k][kk] =  update[k][kk]
          break
        }
      }
    }
  }
  //console.log("tmpdb_update_parser _update = " +JSON.stringify(_update))
  return _update
}

function destructDeviceLib() {
  disconnectCache()
  agileCryptor.disconnectRedis()
  dbGenie.disconnectDb()
  dbTplink.disconnectDb()
  dbTmp.disconnectDb()
}

async function create_ipaddress_field (data) {
  var res;

  res = await dbGenie.find(stringTable.collection.devices,
    {"_id":data['deviceId'],
     "_ipaddress":{"$exists":true}
    });

  if (res.length == 0){data['update']['$set']['_ipaddress'] = '' }
}

function get_ip_from_url(update) {
  var cr_url = undefined, ip, ip_string

  try {
    // from original version, the if statement isn't typo
    // eslint-disable-next-line
    if (cr_url = update['$set']['InternetGatewayDevice.ManagementServer.ConnectionRequestURL._value'] || update['$set']['Device.ManagementServer.ConnectionRequestURL._value']) {
      ip = get_ip(cr_url)
      ip_string = []

      ip.split(".").forEach(function (item) {
        ip_string.push(padding(item, 3))
      })

      update['$set']["_ipaddress"] = ip_string.join(".")
    }
  } catch(err) {
    console.log(err)
  }
  return update
}


async function get_username_from_pppoe(data) {

  let index_max = 96;
  let pppoe_path = '';
  let pppoe_username = '';
  let found_flag = 0;
  let current_username = '';
  let WANDevice_idx, WANConnDevice_idx, WANPPPConn_idx, PPP_idx;
  let update = data['update'];
  try {
    var res = await dbGenie.find(stringTable.collection.devices,
      { '_id': data['deviceId'], "_username": { '$exists': true }})
    if(res && res.length > 0 ) {
      if(res[0]['_username'])
        current_username = res[0]['_username']
    }

    // only update username when no _username data exist. Or it will be upgrade by Update_Username provision
    if(current_username === ''){
      //"InternetGatewayDevice.WANDevice.{i}.WANConnectionDevice.{i}.WANPPPConnection.{i}.Username";
      if(update['$set']['InternetGatewayDevice.WANDevice._timestamp']) {
        for (WANDevice_idx=1; WANDevice_idx<=index_max; WANDevice_idx++) {
          for (WANConnDevice_idx=1; WANConnDevice_idx<=index_max; WANConnDevice_idx++) {
            for(WANPPPConn_idx=1; WANPPPConn_idx<=index_max; WANPPPConn_idx++) {
              pppoe_path = "InternetGatewayDevice.WANDevice." + WANDevice_idx + ".WANConnectionDevice." + WANConnDevice_idx + ".WANPPPConnection." + WANPPPConn_idx + ".Username._value";
              //console.log("[get_username_from_pppoe]  try to find [" + pppoe_path+ "] ")
              if(update['$set'][pppoe_path])
              {
                  pppoe_username = update['$set'][pppoe_path]
                  found_flag=1;
                  break;
              }
            } // * WANPPPConn_idx
            if(found_flag) break;
          } // * WANConnDevice_idx
          if(found_flag) break;
        }  // * WANDevice_idx
      }
      //"Device.PPP.Interface.{i}.Username";
      if(update['$set']['Device.WANDevice.PPP.Interface._timestamp']) {
        for (PPP_idx=1; PPP_idx<=index_max; PPP_idx++) {
          pppoe_path = "Device.WANDevice.PPP.Interface." + PPP_idx + ".Username._value";
          //console.log("[get_username_from_pppoe]  try to find [" + pppoe_path+ "] ")
          if(update['$set'][pppoe_path])
          {
            pppoe_username = update['$set'][pppoe_path]
            found_flag=1;
            break;
          }
        }  // * PPP_idx
      }

      if(found_flag === 1 ) {
        console.log("[get_username_from_pppoe] found pppoe_username = [" + pppoe_username+ "] from path["+ pppoe_path+ "]")
        update['$set']['_username'] = pppoe_username
      }
    }
  } catch(err) {
    console.log(err)
  }
  return update
}

function handle_mesh_topology(data) {
  var deviceId = data['deviceId'], update = data['update'], ref1
  if (!update) return

  // from original version, the if statement isn't typo
  // eslint-disable-next-line
  if (ref1 = update['$set']['Device.X_TP_MESH_WIFI.WorkMode._value']) {
    switch (ref1) {
    case "CAP":
      update['$set']['_mesh.type'] = "Controller"
      update['$set']['_mesh.cap'] = deviceId
      break
    case "RE":
      update['$set']['_mesh.type'] = "Satellite"
      if (!update['$unset'])
        update['$unset'] = {}
      update['$unset']['_mesh.cap'] = 1
      break
    default:
      if (!update['$unset'])
        update['$unset'] = {}
      update['$unset']['_mesh'] = 1
    }
  }
}

 async function pre_processing (update) {
  for (let k in update) {
    for (let kk in update[k]) {
      var splite = kk.split('.')
      let store_password = false
      switch (kk) {
        case "InternetGatewayDevice.ManagementServer.URL._value":
        case "InternetGatewayDevice.ManagementServer.Username._value":
        case "InternetGatewayDevice.ManagementServer.Password._value":
        case "InternetGatewayDevice.ManagementServer.ConnectionRequestURL._value":
        case "InternetGatewayDevice.ManagementServer.UDPConnectionRequestAddress._value":
        case "InternetGatewayDevice.ManagementServer.ConnectionRequestUsername._value":
        case "InternetGatewayDevice.ManagementServer.ConnectionRequestPassword._value":
        case "Device.ManagementServer.URL._value":
        case "Device.ManagementServer.Username._value":
        case "Device.ManagementServer.Password._value":
        case "Device.ManagementServer.ConnectionRequestURL._value":
        case "Device.ManagementServer.UDPConnectionRequestAddress._value":
        case "Device.ManagementServer.ConnectionRequestUsername._value":
        case "Device.ManagementServer.ConnectionRequestPassword._value":
        case "Device.XMPP.Connection.1.Username._value":
        case "Device.XMPP.Connection.1.Password._value":
        case "Device.XMPP.Connection.1.Domain._value":
        case "Device.XMPP.Connection.1.Resource._value":
        case "Device.XMPP.Connection.1.Server.1.ServerAddress._value":
        //case "InternetGatewayDevice.ManagementServer.PeriodicInformInterval._value": //For test
          store_password = true
          update[k][kk] = ENCRYPT_ENABLE ? await agileCryptor.encrypt(update[k][kk]) : update[k][kk]
          update[k]["_encrypt"] = 1
          break
      }
      switch (splite[splite.length-1]) {
      case "_lastInform":
      case "_lastBoot":
      case "_lastBootstrap":
      case "_registered":
      case "_timestamp":
        update[k][kk] =  new Date(update[k][kk])
        break
      case "_value":
        switch (splite[splite.length-2]) {
          case "PreSharedKey":
          case "KeyPassphrase":
          case "WEPKey":
          case "PIN":
          case "EncryptionKey":
          case "RadiusSecret":
          case "SecondaryRadiusSecret":
          case "STUNPassword":
          case "Password":
          case "NetworkPassword":
          case "ConfigPassword":
          case "FileTransferPassword":
            if(!store_password)
              update[k][kk] = ""
            break
        }
        break
      }
    }
  }
}

function update_device_db(data) {
  var isNew = data['isNew']
  var deviceId = data['deviceId']
  var update = data['update']
  if(!isNew) delete update['$set']['_lastInform']

  for (let k in update) {
    if (k === '$addToSet') {
      for (let kk in update[k]) {
        if (update[k][kk]['$each'] && update[k][kk]['$each'].length === 0) {
          delete update[k][kk]
        }
      }
    } else if (k === '$pull') {
      for (let kk in update[k]) {
        if (update[k][kk]['$in'] && update[k][kk]['$in'].length === 0) {
          delete update[k][kk]
        }
      }
    }
    if (Object.keys(update[k]).length === 0) {
      delete update[k]
    }
  }

  return new Promise((resolve, reject) => {
    if (Object.keys(update).length !== 0) {
      dbGenie.update( stringTable.collection.devices, {
        '_id': deviceId
      }, update, {
        upsert: isNew
      }, function(err, result) {
        if(err) reject(err)
        if (!err && result.result.n !== 1) reject(new Error("Device " + deviceId + " not found in database"))
        if (data['update2']) {
          let update2 = data['update2']
          dbGenie.update(stringTable.collection.devices, {
            '_id': deviceId
          }, update2, function(err){
            if(err) reject(err)
            resolve()
          })
        }
        resolve(deviceId)
      })
    }
    else
      resolve()
  })
}

function update_deviceExt_db(data) {
  var isNew = data['isNew']
  var deviceId = data['deviceId']
  var match = { '_id': deviceId }
  var update
  var set_default_empty = true

  return new Promise((resolve) => {
    dbTplink.find(stringTable.collection.devicesExt, match, function(err, res){
      if (err || !res) return resolve()

      if (res.length == 0) {
        for (let i = 0; i < EXTENSION_FIELDS.length; i++) {
          let field = EXTENSION_FIELDS[i]
          if (!update) update = { '$set': {} }
          update['$set'][field] = ""
        }
        if (set_default_empty) {
          Promise.all([
            dbTplink.update(stringTable.collection.devicesExt, match, update, { 'upsert': true }),
            dbGenie.update(stringTable.collection.devices, match, update),
            dbTmp.update(stringTable.collection.tmpdevices, match, update)
          ])
          .then(() => resolve())
        } else resolve()
      } else {
        if (isNew) {
          for (let i = 0; i < EXTENSION_FIELDS.length; i++) {
            var field = EXTENSION_FIELDS[i]
            if (res[0][field]) {
              if (!update) update = { '$set': {} }
              update['$set'][field] = res[0][field]
            }
          }

          dbGenie.find(stringTable.collection.devices, match)
          .then((results) => {
            if (results.length == 1) {
              var diff = 0
              for (var i = 0; i < EXTENSION_FIELDS.length; i++) {
                var field = EXTENSION_FIELDS[i]
                if (!results[0][field] || results[0][field] != update['$set'][field] ) {
                  diff++
                }
              }
              if (diff) {
                Promise.all([
                  dbGenie.update(stringTable.collection.devices, match, update),
                  dbTmp.update(stringTable.collection.tmpdevices, match, update)
                ])
                .then(() => resolve())
              }
            }
          })
        } else {
          resolve()
        }
      }
    })
  })
}

function update_old_mesh_links(deviceId) {
  return remove_old_mesh_links(deviceId).catch((err) => {
    console.log(err)
    return
  })
}

function update_tmpdevice_db(data) {
  var isNew = data['isNew']
  var deviceId = data['deviceId']
  var update = data['update']
  var cache_update = {}
  var cache_update2 = {}
  var has_mesh_updated = {'mode': '0'}
  cache_update = tmpdb_update_parser(isNew, update, has_mesh_updated)
  data['has_mesh_updated'] = has_mesh_updated
  if(data['update2']) cache_update2 = tmpdb_update_parser(isNew,  data['update2'])
  for (let k in cache_update) {
    if (k === '$addToSet') {
      for (let kk in cache_update[k]) {
        if (cache_update[k][kk]['$each'] && cache_update[k][kk]['$each'].length === 0) {
          delete cache_update[k][kk]
        }
      }
    } else if (k === '$pull') {
      for (let kk in cache_update[k]) {
        if (cache_update[k][kk]['$in'] && cache_update[k][kk]['$in'].length === 0) {
          delete cache_update[k][kk]
        }
      }
    }
    if (Object.keys(cache_update[k]).length === 0) {
      delete cache_update[k]
    }
  }
  for (let k in cache_update2) {
    if (k === '$addToSet') {
      for (let kk in cache_update2[k]) {
        if (cache_update2[k][kk]['$each'] && cache_update2[k][kk]['$each'].length === 0) {
          delete cache_update2[k][kk]
        }
      }
    } else if (k === '$pull') {
      for (let kk in cache_update2[k]) {
        if (cache_update2[k][kk]['$in'] && cache_update2[k][kk]['$in'].length === 0) {
          delete cache_update2[k][kk]
        }
      }
    }
    if (Object.keys(cache_update2[k]).length === 0) {
      delete cache_update2[k]
    }
  }
  return new Promise((resolve, reject) => {
    if (Object.keys(cache_update).length !== 0) {
      dbTmp.update(stringTable.collection.tmpdevices, {'_id': deviceId }, cache_update, {upsert: isNew }, function(err, result) {
        if(err) reject(err)
        if (!err && result.result.n !== 1) reject(new Error("Device " + deviceId + " not found in tmp database"))
        if (Object.keys(cache_update2).length !== 0) {
          dbTmp.update(stringTable.collection.tmpdevices, { '_id': deviceId }, cache_update2,
          function(err){
            if(err) reject(err)
            resolve()
          })
        }
        resolve(deviceId)
      })
    }
    else
      resolve()
  })
}

module.exports = {
  destructDeviceLib,
  get_ip_from_url,
  create_ipaddress_field,
  handle_mesh_topology,
  get_username_from_pppoe,
  pre_processing,
  update_device_db,
  update_deviceExt_db,
  update_old_mesh_links,
  update_tmpdevice_db
}
