'use strict'

const koarouter = require('koa-router')

const subroute = new koarouter()
const subroutepath = __filename.split('/').pop().split('.')[0]

const {
  get_ip_from_url, pre_processing, handle_mesh_topology, update_tmpdevice_db, update_device_db,
  update_deviceExt_db, update_old_mesh_links, destructDeviceLib, create_ipaddress_field, get_username_from_pppoe
} = require('./cwmp_device_lib.js')

// include general purpose body parser here
subroute.use(require('koa-bodyparser')())

subroute.post(`/${subroutepath}`, async ctx => {
  let data = ctx.request.body
  if(Object.keys(data).length === 0) return ctx.throw(400, 'No body in post data')
  if(data['update']) data['update'] = get_ip_from_url(data['update'])
  if(data['update']) data['update'] = await get_username_from_pppoe(data)
  await pre_processing(data['update'])
  handle_mesh_topology(data)

  if ( !data['update']['$set']['_ipaddress'] )
    await create_ipaddress_field(data);

  try {
    await Promise.all([update_tmpdevice_db(data), update_device_db(data)])
    let post_processors = [update_deviceExt_db(data)]
    let has_mesh_updated = (data && data['has_mesh_updated'] && data['has_mesh_updated']['mode'] == "1") ? true : false
    if (has_mesh_updated) post_processors.push(update_old_mesh_links(data['deviceId']))
    await Promise.all(post_processors)
    ctx.status = 200
  } catch(err) {
    console.log(err)
    ctx.status = 404
    if(err.message)
      ctx.res.statusMessage = err.message
    else
    ctx.res.statusMessage =  'update error'
  }
})

subroute.destruct = () => {
  destructDeviceLib()
}

module.exports = subroute
