'use strict'

const koarouter = require('koa-router')
const bodyParser = require('koa-bodyparser')

const subroute = new koarouter()
const subroutepath = __filename.split('/').pop().split('.')[0]

const { AgileACSApps } = require('tplib')

// debug flag
const DBG_ACSAPPS = false

// acsapps command
const ACSAPPS_CMD = '/opt/agileacs/bin/acsutils_devices'

// use general body parser here
subroute.use(bodyParser())

function dbg_console_req(ctx) {
  if (DBG_ACSAPPS) {
    console.log(`Now we are at ${ctx.req.url}`)
    console.log(ctx.method + " " + ctx.path)
    if (ctx.request.body) {
      console.log(ctx.request.body)
      console.log(typeof ctx.request.body)
    }
  }
}

function dbg_console_res(ctx, cmd) {
  if (DBG_ACSAPPS) {
    if (cmd) console.log('cmd= ' + cmd)
    console.log(ctx.response)
  }
}

function pack_act_set(act, args = '') {
  return ACSAPPS_CMD + " " + act + " " + args
}

function http_result(status_code, msg) {
  return { 'code': status_code, 'message': msg }
}

function generic_exec(ctx, cmd) {
  return new Promise((resolve, reject) => {
    AgileACSApps.exec(cmd, (err, stdout, stderr) => {
      if (err) reject(http_result(202, err))
      if (stderr) reject(http_result(202, stderr))
      stdout = stdout.replace('\n', '')
      resolve(http_result(200, stdout))
    })
  })
    .then(result => {
      ctx.status = result['code']
      ctx.body = result['message']
    })
    .catch((error) => {
      ctx.status = error['code']
      ctx.body = error['message']
    })
}

// post '/devices/acsapps', action['remove']
subroute.post(`/${subroutepath}/license`, async ctx => {
  dbg_console_req(ctx)
  if (ctx.request.body) {
    let data = ctx.request.body
    let args = typeof data.data == 'string' ? data.data : JSON.stringify(data.data)
    let cmd = pack_act_set(data.action, "'" + args + "'")
    await generic_exec(ctx, cmd)
    dbg_console_res(ctx, cmd)
  }
  else {
    return ctx.throw(404, 'No HTTP Body')
  }
})

subroute.all(`/${subroutepath}`, ctx => {
  // request related
  console.log('acsapps.all ' + ctx.method)
  ctx.set('Allow', 'POST')
  ctx.throw(405)
  return
})

subroute.destruct = () => {}

module.exports = subroute
