"use strict";

const fs = require('fs')

let common = require('tplib/common');

let { agileConfig, stringTable, MongoClass, AgileCacheClass, AgileCryptor, AgileACSApps } = require('tplib')


let geniedb = new MongoClass(agileConfig.get(stringTable.url.dbGenie))
let tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))
let tmpdb = new MongoClass(agileConfig.get(stringTable.url.dbTmp))
let cache = new AgileCacheClass(agileConfig.getRedisConfig())
let agileCryptor = new AgileCryptor(agileConfig.get('ENCRYPT_ENABLE'), agileConfig.getRedisConfig())

const ENCRYPT_ENABLE = agileConfig.get("ENCRYPT_ENABLE");
const enable_logfile=true;
const enable_debug=false;
const DEVICE_LOG_FILE="/var/log/genieacs/acsapps.log";

function log_device(prefix) {
  var current_time = new Date();
  var log_str = "[device.js] "+ current_time + ": " + prefix; 
  if(enable_debug)
    console.log(log_str);
  if(enable_logfile) {
    fs.appendFile(DEVICE_LOG_FILE, log_str+"\n", (err) => {
      console.log(err);
    });
  }
}

async function decrypt_node(device, path){
  let splite = path.split('.');
  let ref;
  ref = device;
  for(let i=0;i<splite.length;i++){
    if(!ref[splite[i]]) break;
    if(splite[i]=="_value"){
      try {
        ref[splite[i]] = await agileCryptor.decrypt(ref[splite[i]]);
      }
      catch(e){
        log_device("[decrypt_all] "+ e);
      }
      break;
    }
    ref = ref[splite[i]];
  }
}

async function decrypt_all(device) {
  let encrypt_data = [
    "InternetGatewayDevice.ManagementServer.URL._value",
    "InternetGatewayDevice.ManagementServer.Username._value",
    "InternetGatewayDevice.ManagementServer.Password._value",
    "InternetGatewayDevice.ManagementServer.ConnectionRequestURL._value",
    "InternetGatewayDevice.ManagementServer.UDPConnectionRequestAddress._value",
    "InternetGatewayDevice.ManagementServer.ConnectionRequestUsername._value",
    "InternetGatewayDevice.ManagementServer.ConnectionRequestPassword._value",
    "Device.ManagementServer.URL._value",
    "Device.ManagementServer.Username._value",
    "Device.ManagementServer.Password._value",
    "Device.ManagementServer.ConnectionRequestURL._value",
    "Device.ManagementServer.UDPConnectionRequestAddress._value",
    "Device.ManagementServer.ConnectionRequestUsername._value",
    "Device.ManagementServer.ConnectionRequestPassword._value",
    "Device.XMPP.Connection.1.Username._value",
    "Device.XMPP.Connection.1.Password._value",
    "Device.XMPP.Connection.1.Domain._value",
    "Device.XMPP.Connection.1.Resource._value",
    "Device.XMPP.Connection.1.Server.1.ServerAddress._value",
    //"InternetGatewayDevice.ManagementServer.PeriodicInformInterval._value" //For test
  ];
  if(ENCRYPT_ENABLE){
    for(let k in encrypt_data){
      await decrypt_node(device, encrypt_data[k]);
    }
  }
}

function get_tmpdb_lastInform(device) {
  return tmpdb.find(stringTable.collection.devices, {'_id': device["_id"]},{'_lastInform': 1})
  .then((res) => {
    device["_lastInform"] = res[0]["_lastInform"]
    return device;
  })
  .catch((err) => {
    return err
  })
}

async function deleteDevice(deviceId) {
  // fault tolerance, for %20 issue
  let xmppdata = await Promise.all([
    tpdb.findOne(stringTable.collection.xmpp, { _id: deviceId }),
    tpdb.findOne(stringTable.collection.xmpp, { _id: decodeURI(deviceId) }),
  ])

  // unregister xmpp first
  await Promise.all(
    xmppdata.map(async item => {
      if(!item || !item.username ) return
      try {
        await AgileACSApps.execCmdWithSafeArgs('/opt/ejabberd-19.05/bin/ejabberdctl', [
          'unregister',
          item.username,
          'agileacs.com'
        ])
      } catch(error) {
        // just log here, do not break delete device process
        console.log(`error when unrigister xmpp client, xmpp server down?`)
      }
    })
  )

  // delete additional information of this device
  await Promise.all([
    geniedb.remove(stringTable.collection.tasks, { device: deviceId }),
    geniedb.remove(stringTable.collection.faults, { '_id': { '$regex': "^" + (common.escapeRegExp(deviceId)) + "\\:" }}),
    geniedb.remove(stringTable.collection.operations, { '_id': { '$regex': "^" + (common.escapeRegExp(deviceId)) + "\\:" }}),
    tpdb.remove(stringTable.collection.xmpp, { '_id': deviceId }),
    tpdb.remove(stringTable.collection.xmpp, { '_id': decodeURI(deviceId) }),
    cache.delPromise(deviceId + '_tasks_faults_operations'),
  ])

  // delete device in mongodb
  await Promise.all([
    geniedb.remove(stringTable.collection.devices, { '_id': deviceId }),
    tmpdb.remove(stringTable.collection.tmpdevices, { '_id': deviceId }),
    tpdb.remove(stringTable.collection.devicesExt, { '_id': deviceId }),
    delete_mesh_links_after_del_dev(deviceId),
  ])
}

function delete_mesh_links_after_del_dev(deviceId) {
  var match = {'_mesh.cap': deviceId};
  var update = { '$unset': {'_mesh.cap':1}};
  var options = {'multi': true};
  return geniedb.update(stringTable.collection.devices, match, update, options)
  .then(()=>{
    return tmpdb.update(stringTable.collection.devices, match, update, options)
    .catch(()=>{
      return;
    });
  }).catch(()=>{
    return;
  });
}


function device_proc_edit(body) {
  let result = { result: '', message: ''}
  let data;

  if (!body.length) {
    result = { result: '404', message: 'No body in post data' }
    return result
  }

  data = JSON.parse(body);
  if ( !data._id && data._id == "" ) {
    result = { result: '404', message: 'The given data is wrong' }
    return result
  }
  
  let match = { '_id': data._id }, update, options = { 'multi': false };
  let fields = EXTENSION_FIELDS;
  for (let i = 0; i < fields.length; i++) {
    let field = fields[i];

    if ( !update ) {
      update = { '$set': {} };
    }
    update['$set'][field] = (data[field]) ? data[field] : "";
  }
  
  let update_devices_ext = function(match, update, options) {
    tpdb.find(stringTable.collection.devicesExt, match, (err, results) => {
      if(err) {
        result = { result: '404', message: 'No data in table : ' + stringTable.collection.devicesExt }
        return result
      }
      let _options = {};
      if(results.length == 0) {
        _options['upsert'] = true;
      }
      return tpdb.update(stringTable.collection.devicesExt, match, update, _options);
    })
  }

  if (update) {
    Promise.all([
      geniedb.update(stringTable.collection.devices, match, update, options),
      update_devices_ext(match, update, options),
      tmpdb.update(stringTable.collection.tmpdevices, match, update, options)

    ])
    .then(() => {
        result = { result: '200', message: '' }
        return result
    })
    .catch((err) => {
      result = { result: '404', message: 'update error' }
      return result
    })
  } else {
    result = { result: '200', message: '' }
    return result
  }
}

function destructDeviceUtil() {
  geniedb.disconnectDb()
  tpdb.disconnectDb()
  tmpdb.disconnectDb()
  cache.disconnect()
  agileCryptor.disconnectRedis()
}

module.exports = {
  destructDeviceUtil,
  decrypt_all,
  get_tmpdb_lastInform,
  device_proc_edit,
  deleteDevice
}
