'use strict'

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

const { agileConfig, stringTable, MongoClass } = require('tplib')
const DevicesMongoClass = require('./class/devices-mongo-class')

let devicesdb = new DevicesMongoClass(agileConfig.get(stringTable.url.dbGenie))
let tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))
let tmpdb = new MongoClass(agileConfig.get(stringTable.url.dbTmp))

let subrouteList = {}

route.use(require('koa-bodyparser')())

const devices = require('./devices_util')
const { deleteDevice } = devices

// DELETE Mehtod for Devices
route.delete('/:device_id', async (ctx) => {
  try {
    await deleteDevice(ctx.params.device_id)
    Object.assign(ctx, { status: 200, body: '' })
  } catch (error) {
    ctx.throw(400)
  }
})

// GET Mehtod for Devices
route.get('/', async (ctx) => {
  try {
    let res = await dbFildDevices(ctx.request.query, ctx.request.method);
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})

// HEAD Mehtod for Devices
route.head('/', async (ctx) => {
  try {
    let res = await dbFildDevices(ctx.request.query, ctx.request.method);
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})

// PUT Mehtod for Devices (/devices/edit_info)
route.put('/edit_info', async (ctx) => {
  try {
    if (!ctx.request.length) {
      ctx.throw(400)
    }
    let device_setting = {}
    let device_id = {}

    device_id._id = ctx.request.body['_id']
    device_setting = {$set:{
      '_homeId': ctx.request.body['_homeId'],
      '_zoneId': ctx.request.body['_zoneId'],
      '_cellPhone': ctx.request.body['_cellPhone'],
      '_cpf': ctx.request.body['_cpf']
    }};

    await dbUpdateDevices(device_id, device_setting)
    ctx.status = 200
    ctx.body = ""
  } catch (err) {
    ctx.throw(400)
  }
})

let dbFildDevices = async (queryParts, method='GET') => {
  let query, projection, sort, skip, limit;

  if(queryParts.query != null) {
    query = JSON.parse(queryParts.query);
    // query = queryParts.query;
  } else {
    query = null;
  }

  if(queryParts.projection != null) {
    projection = queryParts.projection;
  } else {
    projection = null;
  }

  if(queryParts.sort != null) {
    sort = queryParts.sort
  } else {
    sort = null;
  }

  if(queryParts.skip != null) {
    skip = queryParts.skip;
  } else {
    skip = null;
  }

  if(queryParts.limit != null) {
    limit = queryParts.limit;
  } else {
    limit = null;
  }

  return devicesdb.queryProcResource(stringTable.collection.devices, query, projection, sort, skip, limit, method)
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

let dbUpdateDevices = async (filter, update) => {
  return tpdb.updateOne(stringTable.collection.devicesExt, filter, update)
    .then(res => {
      return devicesdb.updateOne(stringTable.collection.devices, filter, update)
      .then(res => {
        return tmpdb.updateOne(stringTable.collection.tmpdevices, filter, update)
          .then(res => {
            return res
          })
          .catch(err => {
            return err
          })
      })
      .catch(err => {
        return err
      })
    })
    .catch(err => {
      return err
    })
}

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    devicesdb.disconnectDb()
    tmpdb.disconnectDb()
    tpdb.disconnectDb()
    devices.destructDeviceUtil()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  let subEntryFiles = null

  // find subroute files
  try {
    subEntryFiles = await new Promise((resolve, reject) => {
      require("glob")(`${__dirname}/*.sub.js`, (err, files) => {
        if (err) reject(err)
        resolve(files)
      })
    })
  } catch (err) {
    // return route without subroute
    return route
  }

  // add subroute to route
  subEntryFiles.forEach(item => {
    let filename = item.split('/').pop()
    let subroute = require(`./${filename}`)
    subrouteList[filename] = subroute
    route.use(subroute.routes(), subroute.allowedMethods())
  })

  return route
}

module.exports = getRoute
