'use strict'

const koarouter = require('koa-router')

const MongoClass = require('tplib/mongo-class')
const agileConfig = require('tplib/agile-config')
const stringTable = require('tplib/string-table.json')

const subroute = new koarouter()
const subroutepath = __filename.split('/').pop().split('.')[0]

const dbGenie = new MongoClass(agileConfig.get(stringTable.url.dbGenie))
const dbTmp = new MongoClass(agileConfig.get(stringTable.url.dbTmp))

subroute.use(require('koa-bodyparser')())

subroute.post(`/:devicdId/${subroutepath}/:tag`, async ctx => {
  await handleTag(ctx)
})

subroute.delete(`/:devicdId/${subroutepath}/:tag`, async ctx => {
  await handleTag(ctx)
})

/**
 * Update Tag Handler
 * @param {object} ctx http context
 */
async function handleTag(ctx) {
  const actionList = { post: '$addToSet', delete: '$pull' }
  const update = {}
  update[actionList[ctx.method.toLowerCase()]] = { '_tags': ctx.params['tag'] }

  try {
    await dbGenie.update(stringTable.collection.devices,
      { '_id': ctx.params['devicdId'] },
      update,
      { safe: true })
    await dbTmp.update(stringTable.collection.tmpdevices,
      { '_id': ctx.params['devicdId'] },
      update,
      { safe: true })
    ctx.status = 200
  } catch(err) {
    ctx.throw(400)
  }
}

subroute.destruct = () => {
  dbGenie.disconnectDb()
  dbTmp.disconnectDb()
}

module.exports = subroute
