'use strict'

const koarouter = require('koa-router')

const { getDiagnosticLock, saveDiagnosticLock, destoryDiagnosticLock, disconnectCache } = require('tplib/diag_lock_util')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

let subrouteList = {}

// include general purpose body parser here
route.use(require('koa-bodyparser')())

// default route for this prefix
route.post('/:id', async ctx => {
  let deviceId = ctx.params['id']
  let task = ctx.request.body
  if(!task || Object.keys(task).length === 0) {
    Object.assign(ctx, { status:200, body: 'No any data.' })
  } else if(!task.action) {
    Object.assign(ctx, { status:200, body: 'No action is specified.' })
  } else {
    let result = {}
    let controller_id = task.controller_id || null
    switch(task.action) {
    case 'lockDiag':
      if(!task.diagName || !task.timeout) {
        return response2xx(ctx, 200, 'error', `No diagName or timeout is specified.`)
      }
      result = await saveDiagnosticLock(deviceId, task.diagName, task.timeout)
      ctx.body = result
      break
    case 'unlockDiag':
      ctx.body = await destoryDiagnosticLock(deviceId)
      break
    case 'isLocked':
      ctx.body = await getDiagnosticLock(deviceId, controller_id)
      break
    default:
      response2xx(ctx, 200, 'error', `Invalid action [${task.action}]`)
    }
  }
})

function response2xx(ctx, status, result, message) {
  return Object.assign(ctx, { status, body: { result, message }})
}

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    disconnectCache()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  return route
}

module.exports = getRoute
