'use strict'

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

const AgileACSApps = require('tplib/agile-acsapps')
const AgileCacheClass = require('tplib/agile-cache')
const agileConfig = require('tplib/agile-config')

// let dbTplink = new MongoClass(agileConfig.get(stringTable.url.dbTplink))
const agileCache = new AgileCacheClass(agileConfig.getRedisConfig())

let subrouteList = {}

// include general purpose body parser here
route.use(require('koa-bodyparser')())

route.post('/ttl', async ctx => {
  if(!ctx.request.rawBody || Object.keys(ctx.request.body) === 0) {
    return ctx.throw(415)
  }
  let { username } = ctx.request.body
  if(!username) return ctx.throw(415)
  try {
    let ttl = await agileCache.ttlPromise(`${username}_verification`)
    if(ttl === -2) throw new Error('not found')
    ctx.body = { ttl }
  } catch(e) {
    ctx.set('Cache-Control', 'no-cache')
    ctx.throw(404)
  }
})

route.post('/verification', async ctx => {
  if(!ctx.request.rawBody || Object.keys(ctx.request.body) === 0) {
    return ctx.throw(415)
  }
  let { action, data } = ctx.request.body
  ctx.set('Cache-Control', 'no-cache')
  try {
    let { stdout, stderr } = await AgileACSApps.execCmdWithSafeArgs('/opt/agileacs/bin/acs_sessions', [action, data])
    if(stderr) {
      return Object.assign(ctx, { status: 202, body: stderr })
    }
    ctx.set('Content-Type', 'text/plain')
    ctx.body = stdout.replace('\n', '')
  } catch(e) {
    ctx.status = 202
  }
})

route.post('/verify', async ctx => {
  if(!ctx.request.rawBody || Object.keys(ctx.request.body) === 0) {
    return ctx.throw(415)
  }
  let { username, vcode } = ctx.request.body
  if(!username) ctx.throw(415)
  try {
    let result = await agileCache.getPromise(`${username}_verification`)
    if(!result) throw new Error('not found')
    if(result === vcode) {
      await agileCache.delPromise(`${username}_verification`)
      await agileCache.delPromise(`${username}_verification_freq`)
      ctx.status = 200
    } else {
      let freq = await agileCache.getPromise(`${username}_verification_freq`)
      let retryTimes = freq ? Number.parseInt(freq)+1 : 1
      if(retryTimes >= 3) {
        await agileCache.delPromise(`${username}_verification`)
        await agileCache.delPromise(`${username}_verification_freq`)
        Object.assign(ctx, { status: 202, body: 'Verification has been more than three times, please export verification info again.' })
      } else {
        await agileCache.setPromise(`${username}_verification_freq`, retryTimes, -1)
        Object.assign(ctx, { status: 202, body: `Already wrong ${retryTimes} time(s), max 3 times, please try again.` })
      }
    }
  } catch(e) {
    ctx.set('Cache-Control', 'no-cache')
    ctx.throw(404)
  }
})

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    agileCache.disconnect()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }
  return route
}

module.exports = getRoute
