'use strict'

let { agileConfig, stringTable, MongoClass } = require('tplib')

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

let subrouteList = {}

let tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))

route.use(require('koa-bodyparser')())

route.get('/', async ctx => {
    try {
    let query
    if (ctx.request.query) {
      query = ctx.request.query['query']
    }
    let res = await dbFindLicenses(query)
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})


route.all('/', ctx => {
  ctx.set('Allow', 'GET')
  ctx.throw(405)
  return
})


let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    tpdb.disconnectDb()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  let subEntryFiles = null

  try {
    subEntryFiles = await new Promise((resolve, reject) => {
      require("glob")(`${__dirname}/*.sub.js`, (err, files) => {
        if(err) reject(err)
        resolve(files)
      })
    })
  } catch(err) {
    return route
  }

  subEntryFiles.forEach( item => {
    let filename = item.split('/').pop()
    let subroute = require(`./${filename}`)
    subrouteList[filename] = subroute
    route.use(subroute.routes(), subroute.allowedMethods())
  })

  return route
}

let dbFindLicenses = async (query) => {
  return tpdb.queryResource(stringTable.collection.totalLicense, query)
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

module.exports = getRoute
