'use strict'

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

const { agileConfig, stringTable, MongoClass, AgileACSApps, AgileCryptor } = require('tplib')
const tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))
const ac = new AgileCryptor(agileConfig.get('ENCRYPT_ENABLE'), agileConfig.getRedisConfig())

let subrouteList = {}

route.use(require('koa-bodyparser')())

// handler for query resource /notification
route.get('/', async ctx => {
  let dbQuery = ctx.query['query'] ? ctx.query['query'] : null
  try {
    let result = await tpdb.queryResource(stringTable.collection.notification, dbQuery)
    ctx.set('total', result['total'])
    let resJsonData = JSON.parse(result['result'])[0] // need parse array first, to avoid strange \n issue
    if(resJsonData.settings.encrypt) {
      resJsonData.settings.username = await ac.decrypt(resJsonData.settings.username)
      resJsonData.settings.password = await ac.decrypt(resJsonData.settings.password)
    }
    ctx.body = `[\n${JSON.stringify(resJsonData)}\n]` // restore to string array with \n
  } catch(err) {
    ctx.throw(400)
  }
})

// send test mail
route.post('/test', async ctx => {
  let args = JSON.stringify(ctx.request.body.settings)
  let safeArgs = AgileACSApps.generateSafeCmdArgs(args)
  let cmd = `node /opt/genieacs/bin/mail.js 000 ${safeArgs}`
  if(process.env.NODE_ENV === 'test')
    cmd = `node ${require('path').join(__dirname, '../../mail.js')} 000 ${safeArgs}`
  await new Promise((res, rej) => {
    AgileACSApps.exec(cmd, (err, stdout) => {
      if(err) rej()
      stdout = stdout.replace('\n', '')
      if(stdout.match(/Email sent.+OK/)) res('Email Sent Successfully')
      else res('Email Sent failed.' + stdout)
    })
  })
  .then(result => ctx.body = result)
  .catch(() => ctx.throw(400))
})

// update mail setting
route.post('/mail', async ctx=> {
  let notification = ctx.request.body
  if(notification.settings && agileConfig.get('ENCRYPT_ENABLE')) {
    // encrypt user setting
    try {
      notification.settings.username = await ac.encrypt(notification.settings.username)
      notification.settings.password = await ac.encrypt(notification.settings.password)
      if(!notification.settings.encrypt) notification.settings.encrypt = 1
    } catch(e) {
      return ctx.throw(400, e)
    }
  }

  await tpdb.updateOne(stringTable.collection.notification, {'_id': 'mail'}, { $set: notification })
    .then(() => ctx.status = 200)
    .catch(() => ctx.throw(400))
})

let response405 = ctx => {
  ctx.set('Allow', 'POST')
  ctx.throw(405)
}
route.all('/test', response405)
route.all('/mail', response405)

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    ac.disconnectRedis()
    tpdb.disconnectDb()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  return route
}

module.exports = getRoute
