'use strict'

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

let subrouteList = {}

route.use(require('koa-bodyparser')())

let { agileConfig, stringTable, MongoClass } = require('tplib')
let geniedb = new MongoClass(agileConfig.get(stringTable.url.dbGenie))

// GET Mehtod for Profiles
route.get('/', async (ctx) => {
  let query = null
  let projection = null
  let skip = null
  let limit = null
  let sort = null
  try {
    if (ctx.request.query) {
      query = ctx.request.query['query']
      projection = ctx.request.query['projection']
      skip = ctx.request.query['skip']
      limit = ctx.request.query['limit']
      sort = ctx.request.query['sort']
    }
    let res = await dbFindProfiles(query, projection, sort, skip, limit)
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})

// PUT Mehtod for Profiles
route.put('/:profile_id', async (ctx) => {
  try {
    let profile = ctx.request.body
    let profile_id = ctx.params.profile_id
    profile._id = profile_id
    if (!ctx.request.length) {
      ctx.throw(400)
    }
    await dbSaveProfiles(profile)
    ctx.status = 200
    ctx.body = ""
  } catch (err) {
    ctx.throw(400)
  }
})

// DELETE Mehtod for Profiles
route.delete('/:profile_id', async (ctx) => {
  try {
    let profile_id = ctx.params.profile_id
    let selector = { '_id': profile_id }
    await dbDeleteProfiles(selector)
    ctx.status = 200
    ctx.body = ""
  } catch (err) {
    ctx.throw(400)
  }
})

let dbFindProfiles = async (query, projection, sort, skip, limit) => {
  return geniedb.queryResource(stringTable.collection.profiles, query, projection, sort, skip, limit)
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

let dbSaveProfiles = async (body) => {
  return geniedb.save(stringTable.collection.profiles, body)
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

let dbDeleteProfiles = async (selector) => {
  return geniedb.remove(stringTable.collection.profiles, selector)
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    geniedb.disconnectDb()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  let subEntryFiles = null

  // find subroute files
  try {
    subEntryFiles = await new Promise((resolve, reject) => {
      require("glob")(`${__dirname}/*.sub.js`, (err, files) => {
        if (err) reject(err)
        resolve(files)
      })
    })
  } catch (err) {
    // return route without subroute
    return route
  }

  // add subroute to route
  subEntryFiles.forEach(item => {
    let filename = item.split('/').pop()
    let subroute = require(`./${filename}`)
    subrouteList[filename] = subroute
    route.use(subroute.routes(), subroute.allowedMethods())
  })

  return route
}

module.exports = getRoute
