'use strict'

const koarouter = require('koa-router')
const { ProxyHandler } = require('tplib')
const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

let subrouteList = {}

const proxyHandler = new ProxyHandler()

let { agileConfig, stringTable } = require('tplib')
let ProvisionsMongoClass = require('./class/provisions-mongo-class')

let geniedb = new ProvisionsMongoClass(agileConfig.get(stringTable.url.dbGenie))

route.use(require('koa-bodyparser')())

route.get('/', async (ctx) => {
  let query = null
  let projection = null
  let skip = null
  let limit = null
  let sort = null
  try {
    if (ctx.request.query) {
      query = ctx.request.query['query']
      projection = ctx.request.query['projection']
      skip = ctx.request.query['skip']
      limit = ctx.request.query['limit']
      sort = ctx.request.query['sort']
    }
    let res = await dbFindProvisions(query, projection, sort, skip, limit)
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})

route.put('/:id', proxyHandler.handler)
route.delete('/:id', proxyHandler.handler)
route.all('/', proxyHandler.handler)

let dbFindProvisions = async (query, projection, sort, skip, limit) => {
  return geniedb.queryResourceProvisions(stringTable.collection.provisions, query, projection, sort, skip, limit)
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    geniedb.disconnectDb()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  let subEntryFiles = null

  try {
    subEntryFiles = await new Promise((resolve, reject) => {
      require("glob")(`${__dirname}/*.sub.js`, (err, files) => {
        if (err) reject(err)
        resolve(files)
      })
    })
  } catch (err) {
    return route
  }

  subEntryFiles.forEach(item => {
    let filename = item.split('/').pop()
    let subroute = require(`./${filename}`)
    subrouteList[filename] = subroute
    route.use(subroute.routes(), subroute.allowedMethods())
  })

  return route
}

module.exports = getRoute
