'use strict'

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

let { agileConfig, stringTable, MongoClass } = require('tplib')
let tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))

let subrouteList = {}

route.use(require('koa-bodyparser')())

// GET Mehtod for ReportSetting
route.get('/', async (ctx) => {
  try {
    let query = null
    if (ctx.request.query) {
      query = ctx.request.query['query']
    }

    let res = await dbFindReportSetting(query)
    ctx.set('total', res['total'])
    ctx.body = res['result']
  } catch (err) {
    ctx.throw(400)
  }
})

// POST Mehtod for ReportSetting
route.post('/:reportDay', async (ctx) => {
  try {
    let reportDay = null
    if (ctx.params.reportDay) {
      reportDay = ctx.params.reportDay
    }
    await saveReportSetting(reportDay)
    ctx.status = 200
    ctx.body = ""
  } catch (err) {
    ctx.throw(400)
  }
})

let dbFindReportSetting = async (query) => {
  return tpdb.queryResource(stringTable.collection.reportSetting, query)
    .then(res => {
      return res
    })
    .catch(err => {
      return err
    })
}

let saveReportSetting = async (reportDay) => {
  return new Promise((resolve, reject) => {
    require('child_process').exec("/opt/agileacs/bin/acs_report_setting.sh " + reportDay, function (err, stdout, stderr) {
      if (err) {
        reject("err" + err)
      } else if (stderr) {
        reject("stderr" + stderr)
      } else {
        tpdb.update(stringTable.collection.reportSetting, { "_id": "report_setting" }, { $set: { "reportday": reportDay } })
          .then(res => {
            resolve(res)
          })
          .catch(err => {
            reject(err)
          })
      }
    });
  })
}

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    tpdb.disconnectDb()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  let subEntryFiles = null

  // find subroute files
  try {
    subEntryFiles = await new Promise((resolve, reject) => {
      require("glob")(`${__dirname}/*.sub.js`, (err, files) => {
        if (err) reject(err)
        resolve(files)
      })
    })
  } catch (err) {
    // return route without subroute
    return route
  }

  // add subroute to route
  subEntryFiles.forEach(item => {
    let filename = item.split('/').pop()
    let subroute = require(`./${filename}`)
    subrouteList[filename] = subroute
    route.use(subroute.routes(), subroute.allowedMethods())
  })

  return route
}

module.exports = getRoute
