'use strict'

const koarouter = require('koa-router')

const subroute = new koarouter()
const subroutepath = __filename.split('/').pop().split('.')[0]

const { agileConfig, stringTable, MongoClass, AgileACSApps, AgileCryptor } = require('tplib')
const tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))

// debug flag
const DBG_CEIP = false
const CEIP_TEST_CMD = '/opt/agileacs/bin/agileacs_ceip_report_check.sh'
// action
const ACT_SAVE_INFO = 'save_info'

function dbg_console_req(ctx) {
  if (DBG_CEIP) {
    console.log(`Now we are at ${ctx.req.url}`)
    console.log(ctx.method + " " + ctx.path)
    console.log(ctx.request.body)
    console.log(Object.keys(ctx.request.body).length)
  }
}

function dbg_console_res(ctx, cmd) {
  if (DBG_CEIP) {
    if (cmd) console.log('cmd= ' + cmd)
    console.log(ctx.response)
  }
}

function http_result(status_code, msg) {
  return { 'code': status_code, 'message': msg }
}

function generic_exec(ctx, cmd) {
  return new Promise((resolve, reject) => {
    AgileACSApps.exec(cmd, (err, stdout, stderr) => {
      if (err) reject(http_result(202, err))
      if (stderr) reject(http_result(202, stderr))
      if (stdout) {
        stdout = stdout.replace('\n', '')
        resolve(http_result(200, stdout))
      }
      else
      reject(http_result(202, 'Unknown Status'))
    })
  })
    .then(result => {
      ctx.status = result['code']
      ctx.body = result['message']
    })
    .catch((error) => {
      ctx.status = error['code']
      ctx.body = error['message']
    })
}

subroute.use(require('koa-bodyparser')())

// '/system/ceip'
subroute.get(`/${subroutepath}`, async ctx => {
  try {
    dbg_console_req(ctx)
    let result = await tpdb.queryResource(stringTable.collection.activateReport, null)
    ctx.set('total', result['total'])
    ctx.body = result['result']
    dbg_console_res(ctx, null)
  } catch (err) {
    ctx.throw(400)
  }
})

subroute.post(`/${subroutepath}`, async ctx => {
  dbg_console_req(ctx)

  let action, ceipSetting
  try {
    action = ctx.request.body.action
    ceipSetting = ctx.request.body.data
    if (action != ACT_SAVE_INFO) {
      ctx.throw(400)
    }
  } catch (e) {
    return ctx.throw(400, e)
  }

  await tpdb.updateOne(stringTable.collection.activateReport, { "_id": "service" }, { $set: ceipSetting })
    .then(() => ctx.status = 200)
    .catch(() => ctx.throw(400))

  dbg_console_res(ctx, null)
})


subroute.put(`/${subroutepath}`, async ctx => {
  dbg_console_req(ctx)
  await generic_exec(ctx, CEIP_TEST_CMD)
  dbg_console_res(ctx, null)
})

let resp_ceip_405 = ctx => {
  console.log('resp_ceip_405 ' + ctx.method)
  ctx.set('Allow', 'GET, PUT, POST')
  ctx.throw(405)
}
subroute.all(`/${subroutepath}`, resp_ceip_405)

subroute.destruct = () => {
  tpdb.disconnectDb()
}

module.exports = subroute
