'use strict'

const koarouter = require('koa-router')

const subroute = new koarouter()
const subroutepath = __filename.split('/').pop().split('.')[0]

const { agileConfig, stringTable, MongoClass, AgileACSApps, AgileCryptor } = require('tplib')
const tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))

// debug flag
const DBG_CPE = false

function dbg_console_req(ctx) {
  if (DBG_CPE) {
    console.log(`Now we are at ${ctx.req.url}`)
    console.log(ctx.method + " " + ctx.path)
    if (ctx.request.body) {
      console.log(ctx.request.body)
      console.log(typeof ctx.request.body)
      //console.log(Object.keys(ctx.request.body).length)
    }
  }
}

function dbg_console_res(ctx, cmd) {
  if (DBG_CPE) {
    if (cmd) console.log('cmd= ' + cmd)
    console.log(ctx.response)
  }
}

subroute.use(require('koa-bodyparser')())

// '/system/cpe'
subroute.get(`/${subroutepath}`, async ctx => {
  dbg_console_req(ctx)
  try {
    let result = await tpdb.queryResource(stringTable.collection.cpeSetting, null)
    ctx.set('total', result['total'])
    ctx.body = result['result']
  } catch (err) {
    ctx.throw(400)
  }
  dbg_console_res(ctx, null)
})

subroute.post(`/${subroutepath}`, async ctx => {
  let cpeSetting = {}

  dbg_console_req(ctx)
  try {
    let informinterval = ctx.request.body.informinterval
    let intervaloffset = ctx.request.body.intervaloffset

    if (isNaN(parseInt(informinterval)) || isNaN(parseInt(intervaloffset))) {
      ctx.throw(400)
    }
    if( parseInt(informinterval) > 129600 || parseInt(informinterval) < 7200 ){
      ctx.status = 400
      ctx.body = "CPE inform interval must between 7200 ~ 129600"
      return
    }
    else if(parseInt(intervaloffset) > parseInt(informinterval/2) || parseInt(intervaloffset) < parseInt(informinterval/24)){
        ctx.status = 400
        ctx.body = "CPE inform interval must between " + parseInt(informinterval/24) + " ~ " + parseInt(informinterval/2)
        return
    }
    cpeSetting.informinterval = informinterval
    cpeSetting.intervaloffset = intervaloffset
  } catch (e) {
    return ctx.throw(400, e)
  }

  await tpdb.updateOne(stringTable.collection.cpeSetting, { '_id': 'cpesetting' }, { $set: cpeSetting })
    .then(() => ctx.status = 200)
    .catch(() => ctx.throw(400))
  dbg_console_res(ctx, null)
})

let resp_cpe_405 = ctx => {
  console.log('resp_cpe_405 ' + ctx.method)
  ctx.set('Allow', 'GET, POST')
  ctx.throw(405)
}
subroute.all(`/${subroutepath}`, resp_cpe_405)

subroute.destruct = () => {
  tpdb.disconnectDb()
}

module.exports = subroute
