'use strict'

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

// const { agileConfig, stringTable, MongoClass, AgileACSApps, AgileCryptor } = require('tplib')
let subrouteList = {}

function dump_ctx(ctx) {
  // request related
  console.log('header-name: ' + ctx.get('header-name'))   // get Header, case insensitive
  console.log('method: ' + ctx.method)                    // get method
  console.log('url: ' + ctx.url)                          // get url, result is like '/routesample?name=klinsman'
  console.log('path: ' + ctx.path)                        // get path, result is like '/routesample'
  console.log(ctx.querystring)                            // get original query string, like 'name=klinsman'
  console.log(ctx.query)                                  // get query string, parsed as JSON object, like { name: 'klinsman' }
  console.log('request.body: ' + ctx.request.body)        // post body, json object (form data is parsed to json, too)

  // host
  console.log('origin: ' + ctx.origin)      // http://localhost:3000
  console.log('host: ' + ctx.host)          // localhost:3000
  console.log('hostname: ' + ctx.hostname)  // localhost  
}

route.use(require('koa-bodyparser')())

// koa routing
route.all('/', ctx => {

  dump_ctx(ctx)
  // we can throw status code, error message and additional information like the following
  // but the final error response sent depends on what we do in our default error handler
  ctx.throw(400, 'Not Found', { additional: 'additional message' })
  /* eslint-enable */
})

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }

  let subEntryFiles = null

  // find subroute files
  try {
    subEntryFiles = await new Promise((resolve, reject) => {
      require("glob")(`${__dirname}/*.sub.js`, (err, files) => {
        if (err) reject(err)
        resolve(files)
      })
    })
  } catch (err) {
    // return route without subroute
    return route
  }

  // add subroute to route
  subEntryFiles.forEach(item => {
    let filename = item.split('/').pop()
    let subroute = require(`./${filename}`)
    subrouteList[filename] = subroute
    route.use(subroute.routes(), subroute.allowedMethods())
  })

  return route
}

module.exports = getRoute
