'use strict'

const koarouter = require('koa-router')

const subroute = new koarouter()
const subroutepath = __filename.split('/').pop().split('.')[0]

const { agileConfig, stringTable, MongoClass, AgileACSApps, AgileCryptor } = require('tplib')
const tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))

// debug flag
const DBG_MODEL = false

function dbg_console_req(ctx) {
    if (DBG_MODEL) {
        console.log(`Now we are at ${ctx.req.url}`)
        console.log(ctx.method + " " + ctx.path)
        console.log(ctx.request.body)
        console.log(Object.keys(ctx.request.body).length)
    }
}

function dbg_console_res(ctx, cmd) {
    if (DBG_MODEL) {
        if (cmd) console.log('cmd= ' + cmd)
        console.log(ctx.response)
    }
}

subroute.use(require('koa-bodyparser')())

// '/system/models'
subroute.get(`/${subroutepath}`, async ctx => {
    dbg_console_req(ctx)
    let params = ctx.request.query
    let query = params.query ? params.query : null
    let limit = params.limit ? params.limit : null
    let skip = params.skip ? params.skip : null
    let sort = params.sort ? params.sort : null
    try {
        let result = await tpdb.queryResource(stringTable.collection.model, query, null, sort, skip, limit)
        ctx.set('total', result['total'])
        ctx.body = result['result']
    } catch (err) {
        ctx.status = 400
        ctx.message = '404 Not Found'
    }
    dbg_console_res(ctx, null)
})

let resp_model_405 = ctx => {
    console.log('resp_model_405 ' + ctx.method)
    ctx.set('Allow', 'GET')
    ctx.throw(405)
}
subroute.all(`/${subroutepath}`, resp_model_405)

subroute.destruct = () => {
  tpdb.disconnectDb()
}

module.exports = subroute
