'use strict'

const koarouter = require('koa-router')

const subroute = new koarouter()
const subroutepath = __filename.split('/').pop().split('.')[0]

const { agileConfig, stringTable, MongoClass, AgileACSApps, AgileCryptor } = require('tplib')

// debug flag
const DBG_USAGE = false

// acsapps command
const ACSAPPS_CMD = '/opt/agileacs/bin/getSystemUsage.sh'

// action
const ACT_CPU = 'cpu'
const ACT_MEMORY = 'memory'
const ACT_STORAGE = 'storage'

// HTTP error message
const HS_MSG_UNKNOW_ERROR = 'Unknow error'

function dbg_console_req(ctx) {
  if (DBG_USAGE) {
    console.log(`Now we are at ${ctx.req.url}`)
    console.log(ctx.method + " " + ctx.path)
    console.log(ctx.request.body)
    console.log(Object.keys(ctx.request.body).length)
  }
}

function dbg_console_res(ctx, cmd) {
  if (DBG_USAGE) {
    if (cmd) console.log('cmd= ' + cmd)
    console.log(ctx.response)
  }
}

function pack_act_set(act, op = '', args = '') {
  return ACSAPPS_CMD + " " + act + " " + op + " " + args
}

function http_result(status_code, msg) {
  return { 'code': status_code, 'message': msg }
}

function generic_exec(ctx, cmd) {
  return new Promise((resolve, reject) => {
    AgileACSApps.exec(cmd, (err, stdout, stderr) => {
      if (err) reject(http_result(202, err))
      if (stderr) reject(http_result(202, stderr))
      stdout = stdout.replace('\n', '');
      resolve(http_result(200, stdout))
    })
  })
    .then(result => {
      ctx.status = result['code']
      ctx.body = result['message']
    })
    .catch((error) => {
      ctx.status = error['code']
      ctx.body = error['message']
    })
}

function usage_cpu(ctx) {
  if (ctx.status == 200) {
    let data = ctx.body
    let systemUsage = {
      usage: '0%'
    }
    data = data.replace("\n", '');
    systemUsage['usage'] = data;
    ctx.body = JSON.stringify(systemUsage);
  }
}

function usage_memory(ctx) {
  if (ctx.status == 200) {
    let data = ctx.body
    let systemUsage = {
      usage: '0%',
      used: 0,
      available: 0
    };
    data = data.replace("\n", '');
    data = data.split(',');
    systemUsage['used'] = data[0];
    systemUsage['available'] = data[1];
    systemUsage['usage'] = data[2];
    ctx.body = JSON.stringify(systemUsage);
  }
}

function usage_storage(ctx) {
  if (ctx.status == 200) {
    let data = ctx.body
    let systemUsage = {
      usage: '0%',
      used: 0,
      available: 0
    };
    data = data.replace("\n", '');
    data = data.split(',');
    systemUsage['total'] = data[0];
    systemUsage['used'] = data[1];
    systemUsage['usage'] = data[2];
    ctx.body = JSON.stringify(systemUsage);
  }
}

subroute.use(require('koa-bodyparser')())

// '/system/usage/cpu'
subroute.post(`/${subroutepath}/cpu`, async ctx => {
  dbg_console_req(ctx)
  let cmd = pack_act_set(ACT_CPU)
  await generic_exec(ctx, cmd)
  usage_cpu(ctx)
  dbg_console_res(ctx, cmd)
})

// '/system/usage/memory'
subroute.post(`/${subroutepath}/memory`, async ctx => {
  dbg_console_req(ctx)
  let cmd = pack_act_set(ACT_MEMORY)
  await generic_exec(ctx, cmd)
  usage_memory(ctx)
  dbg_console_res(ctx, cmd)
})

// '/system/usage/storage'
subroute.post(`/${subroutepath}/storage`, async ctx => {
  dbg_console_req(ctx)
  let cmd = pack_act_set(ACT_STORAGE)
  await generic_exec(ctx, cmd)
  usage_storage(ctx)
  dbg_console_res(ctx, cmd)
})

let resp_usage_405 = ctx => {
  console.log('resp_usage_405 ' + ctx.method)
  ctx.set('Allow', 'POST')
  ctx.throw(405)
}
subroute.all(`/${subroutepath}`, resp_usage_405)
subroute.all(`/${subroutepath}/cpu`, resp_usage_405)
subroute.all(`/${subroutepath}/memory`, resp_usage_405)
subroute.all(`/${subroutepath}/storage`, resp_usage_405)

subroute.destruct = () => {}

module.exports = subroute
