'use strict'

const fs = require('fs')
const koarouter = require('koa-router')

const subroute = new koarouter()
const subroutepath = __filename.split('/').pop().split('.')[0]

const AgileACSApps = require('tplib/agile-acsapps')
const MongoClass = require('tplib/mongo-class')

const agileConfig = require('tplib/agile-config')
const stringTable = require('tplib/string-table.json')

const dbTplink = new MongoClass(agileConfig.get(stringTable.url.dbTplink))

const XMPP_LOCK_FILE = '/tmp/xmppservice'
const XMPP_LOCKED = 'XMPPLocked'

// get xmpp settings
subroute.get(`/${subroutepath}`, async ctx => {
  let response = { 'xmpp': 'disable', 'xmpp_keepalive_interval': 300, 'xmppstatus': 'stopped' }
  try {
    let result = await Promise.all([
      dbTplink.findOne(stringTable.collection.sysSetting, { _id: 'service' }, { xmpp: 1, xmpp_keepalive_interval: 1 }),
      AgileACSApps.execCmdWithSafeArgs('/opt/agileacs/bin/xmpp-tool.sh', ['server', 'status'])
    ])
    if(result[0] && result[0].xmpp !== undefined ) Object.assign(response, result[0])
    response.xmppstatus = result[1].stdout.includes('running') ? 'running' : 'stopped'
  } catch(err) {
    // console.log(err)
  }
  ctx.body = response
})

// modify xmpp settings
subroute.post(`/${subroutepath}`, async ctx => {
  let dataToSet = {}
  dataToSet.xmpp_keepalive_interval = ctx.request.body.xmpp_keepalive_interval !== undefined ? ctx.request.body.xmpp_keepalive_interval: 300
  try{
    // check if service xmpp script is done
    await new Promise((res, rej) => {
      fs.access(XMPP_LOCK_FILE, fs.constants.F_OK, (err) => {
        if(err) return res() // file not exist, xmpp service is not running
        rej(new Error(XMPP_LOCKED)) // no error => file exists => xmpp service is busy
      })
    })

    if(ctx.request.body.xmpp == 'enable') {
      // enable xmpp
      dataToSet.xmpp = 'enable'
      await dbTplink.updateOne(stringTable.collection.sysSetting, { _id: 'service' }, { $set: dataToSet })
      AgileACSApps.execCmdWithSafeArgs('service', ['xmpp', 'start'])
    } else {
      // disable xmpp
      dataToSet.xmpp = 'disable'
      await dbTplink.updateOne(stringTable.collection.sysSetting, { _id: 'service' }, { $set: dataToSet })
      AgileACSApps.execCmdWithSafeArgs('service', ['xmpp', 'stop'])
    }
  } catch(err) {
    console.log(err)
    if(err.message === XMPP_LOCKED) {
      return ctx.status = 405
    }
  }
  ctx.status = 200
})

subroute.destruct = () => {
  dbTplink.disconnectDb()
}

module.exports = subroute
