'use strict'

const koarouter = require('koa-router')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

const { agileConfig, stringTable, MongoClass, AgileACSApps } = require('tplib')
const tpdb = new MongoClass(agileConfig.get(stringTable.url.dbTplink))

let subrouteList = {}

route.use(require('koa-bodyparser')())

route.get('/', async ctx => {
  let dbQuery = ctx.query['query'] ? ctx.query['query'] : null
  try {
    let result = await tpdb.queryResource(stringTable.collection.troubleShooting, dbQuery)
    ctx.set('total', result['total'])
    ctx.body = result['result']
  } catch(err) {
    ctx.throw(400)
  }
})

route.post('/remote_assistance', ctx => {
  let ra = Object.assign({}, ctx.request.body, { _id: 'remote_assistance' })
  let args
  ra.enable === 'on' ? args = ["start", ra.ipAddr] : args = ["stop"]

  AgileACSApps.spawn('/opt/agileacs/etc/agileacs_remote_assistance_setup.sh', args)

  return tpdb.save(stringTable.collection.troubleShooting, ra)
  .then(() => ctx.status = 200)
  .catch(err => {
    console.log(err)
    ctx.status = 400
  })
})

// return 405 for all other methods
route.all('/remote_assistance', ctx => {
  ctx.set('Allow', 'POST')
  ctx.throw(405)
})

let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    tpdb.disconnectDb()
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }
  return route
}

module.exports = getRoute
