'use strict'

const koarouter = require('koa-router')

const AgileACSApps = require('tplib/agile-acsapps')

const prefix = '/' + __dirname.split('/').pop()  // use folder name as route path
const route = new koarouter({ prefix })

let subrouteList = {}

// include general purpose body parser here
route.use(require('koa-bodyparser')())

/**
 * get usb info
 */
route.get('/', async ctx => {
  try {
    let specialReplace = 'TPINlVaiNklIDTAg' // strange string to prevent conflict
    let { stdout, stderr } = await AgileACSApps.execCmdWithSafeArgs('/opt/agileacs/bin/usb-storage.sh', 'get-usage')
    if(stderr && process.env.NODE_ENV !== 'test') throw new Error(stderr)
    // clear '\' and replace with specialReplace
    // do this because we may have multiple usb storages, we can not sure which one contains '\'
    let body = JSON.parse(stdout.trim().replace(/\\/g, specialReplace))
    // find specialReplace
    Object.keys(body).forEach(item => {
      if(body[item].USBStorage && body[item].USBStorage.includes(specialReplace)) {
        // retrieve size information first
        let sizeString = body[item].USBStorage.substring(body[item].USBStorage.lastIndexOf('('))
        body[item].USBStorage = 'Storage Media [Invalid Name] ' + sizeString
      }
    })
    Object.assign(ctx, { status: 200, body })
  } catch(err) {
    console.log(err)
    response202(ctx)
  }
})

function response202(ctx) {
  ctx.set('Cache-Control', 'no-cache')
  ctx.status = 202
}

// import other subroutes and return result
let getRoute = async (destruct) => {
  // destruct function
  if(destruct) {
    for(let i in subrouteList) {
      subrouteList[i].destruct()
    }
    return
  }
  return route
}

module.exports = getRoute
